#include <stdio.h>
#include <apollo/base.h>
#include <apollo/gpr.h>
#include <tml/tml_c_types.h>
#include <tml/tml_dp_graph.hxx>

extern long create_bitmap();
extern void enable_all_keys();

main()
{
    tml_c_model      display_model;
    tml_c_view_port  display_viewport;
    tml_c_format     display_format;
    C_MODEL_POS      start_display;
    long             bitmap_desc;
    char             unob;
    gpr_$event_t     ev_type;
    char             ev_value = '\0';
    gpr_$position_t  ret_coord;
    status_$t        status;

    /*********************************************/
    long         dx, dy;    /* size of viewport */
    C_SCREEN_XY  inq_pos;   /* any (x,y) coordinate on the screen */
    tml_c_phead  last_page; /* used for going to bottom of file */
    tml_c_cline  last_line; /* used for going to bottom of file */
    /**********************************************/

    /* initialize TMLib */
    tml_init();

    /* associate all viewport objects with the GPR graphic system */
    tml_c_tml_dp_set_dflt_vpftbl(tml_c_tml_dp_get_gpr_vpftbl());

    /* create a model from the ASCII file test_file */
    display_model = tml_c_read_uasc_path("test_file", 9);
    if (!display_model)
        exit (1);

    /* allocate a viewport */
    display_viewport = view_port_class_alloc();
    /* create a graphics display area, */
    /*   using a routine that depends on the graphics system */
    bitmap_desc = create_bitmap();

    /* assign the display area to the viewport */
    view_port_set_handle(display_viewport, &bitmap_desc);

    /* create a format */
    /* attach it to the viewport */
    view_port_attach_format(display_viewport, 0, 0, 0);
    /* get a handle for the format created by the last call */
    display_format = view_port_get_format(display_viewport);
    /* attach the model to the format that is associated with the viewport */
    format_attach(display_format, display_model);

    /* fill viewport with text; first character placed at pixel (0,0) */
    tml_dp_damage_repair(view_port_get_handle(display_viewport));

    /* enable keyboard */
    enable_all_keys();

    /*****************************************/

    while (ev_value != 'e') { /* typing 'e' exits the program */

        unob = gpr_$event_wait(&ev_type, (unsigned char*) &ev_value,
                               &ret_coord, &status);

        switch (ev_value) {

            case ('u'):
                /* move viewport up the conceptual page */
                /*   half the viewport height */
                /* get height of viewport -- it may have changed */
                dy = view_port_get_hi(display_viewport);
                /* find character at top of viewport */
                view_port_xy_2_mpos(display_viewport,
                                    &zero_zero, &start_display);
                /* if character found, display it at halfway down the screen */
                if (start_display.in_node) {
                    inq_pos.at_x = 0;
                    inq_pos.at_y = dy/2;
                    view_port_draw_at(display_viewport, &start_display,
                                      &inq_pos);
                    }
                break;

            case ('d'):
                /* move the viewport down the conceptual page */
                /*   half the viewport height */
                /* get height of viewport -- it may have changed */
                dy = view_port_get_hi(display_viewport);
                /* find the character at pixel (0,dy/2) */
                inq_pos.at_x = 0;
                inq_pos.at_y = dy/2;
                view_port_xy_2_mpos(display_viewport, &inq_pos, &start_display);
                /* if character found, display it at pixel (0,0) */
                if (start_display.in_node)
                    view_port_draw_at(display_viewport,
                                      &start_display, &zero_zero);
                break;


            case ('l'):
                /* move left on the conceptual page */
                /*   half the viewport width */
                /* get width of viewport */
                dx = view_port_get_width(display_viewport);
                /* find the character at pixel (0,0) */
                view_port_xy_2_mpos(display_viewport, &zero_zero,
                                    &start_display);
                /* if character found, display it at pixel (dx/2,0) */
                if (start_display.in_node) {
                     /* get character location */
                     if (0 == view_port_mpos_2_xy(display_viewport,
                                                  &start_display, &inq_pos, 0))
                         break;
                     /* move half a viewport left */
                     inq_pos.at_x += (dx/2);
                     view_port_draw_at(display_viewport, &start_display,
                                       &inq_pos);
                     }
                break;

            case ('r'):
                /* move right on the conceptual page */
                /*   half the viewport width */
                /* get width of viewport */
                dx = view_port_get_width(display_viewport);
                /* find character at pixel (0,0) */
                view_port_xy_2_mpos(display_viewport, &zero_zero,
                                    &start_display);
                /* if character found, display it at pixel (-dx/2,0) */
                if (start_display.in_node) {
                     /* get character location */
                     if (0 == view_port_mpos_2_xy(display_viewport,
                                                  &start_display,
                                                  &inq_pos, 0))
                         break;
                     /* move half a viewport right */
                     inq_pos.at_x -= (dx/2);
                     view_port_draw_at(display_viewport, &start_display,
                                       &inq_pos);
                     }
                break;

            case ('t'):
                /* move to display top of file */
                /* get root node of model */
                /* get first model position */
                start_display.in_node = model_get_root(display_model);
                start_display.char_index = 0;
                model_pos_get_next(&start_display, &start_display);
                /* if character found, display it at pixel (0,0) */
                if (start_display.in_node)
                    view_port_draw_at(display_viewport,
                                      &start_display, &zero_zero);
                break;

            case ('b'):
                /* move to display bottom of file */
                /* get root node of model */
                /* get last node dominated by root */
                start_display.in_node =
                  node_get_last( model_get_root(display_model) );
                /* quit if no node found */
                if (!start_display.in_node) break;
                /* get index of last character in last node */
                start_display.char_index =
                  node_get_max_index(start_display.in_node);
                /* find conceptual page containiong that model position */
                last_page = format_mpos_2_phead(display_format, &start_display);
                /* quit if no page found */
                if (!last_page) break;
                /* find last cline on last page */
                last_line = phead_mpos_2_cline(last_page, &start_display);
                /* quit if no line found */
                if (!last_line) break;
                /* set up a screen_xy for bottom of page */
                /*   at height of last line */
                inq_pos.at_x = 0;
                inq_pos.at_y = view_port_get_hi(display_viewport)
                               - cline_get_hi(last_line);
                view_port_draw(display_viewport, last_line, &inq_pos);
                break;

            default:
                break;
        }
    }
    /* release dynamically allocated storage */
    view_port_free( display_viewport);
    format_free( display_format);
    model_free( display_model);
    /*****************************************/
}
