#include <stdio.h>
#include <apollo/base.h>
/*************************************************************/
#include <apollo/gpr.h>
#include <apollo/kbd.h>
/*************************************************************/

#include <tml/tml_c_types.h>
#include <tml/tml_dp_graph.hxx>

extern long create_bitmap();

/*************************************************************/
extern void enable_mouse();
extern void enable_all_keys();
/*************************************************************/

main()
{
    tml_c_model      display_model;
    tml_c_view_port  display_viewport;
    tml_c_format     display_format;
    C_MODEL_POS      start_display;
    long             bitmap_desc;
    int              c;
    long             dx, dy;    /* size of viewport */
    C_SCREEN_XY      inq_pos;   /* any (x,y) coordinate on the screen */

/**************************************************************/
    tml_c_v_cursor   the_cursor;
    gpr_$event_t     ev_type;
    char             ev_value;
    gpr_$position_t  ret_coord;
    status_$t        status;
    char             unob;
    C_SCREEN_XY      curs_loc;
    C_MODEL_POS      curs_pos;
    tml_c_cline      old_top_line;
    tml_c_cline      new_top_line;
    tml_c_cline      first_line;
    tml_c_phead      first_page;
    C_XY_COORD       at_coord;
/*************************************************************/

    /* initialize TMLib */
    tml_init();

    /* associate all viewport objects with the GPR graphic system */
    tml_c_tml_dp_set_dflt_vpftbl(tml_c_tml_dp_get_gpr_vpftbl());

    /* create a model from the ASCII file test_file */
    display_model = tml_c_read_uasc_path("test_file", 9);
    if (!display_model)
        exit (1);

    /* allocate a viewport */
    display_viewport = view_port_class_alloc();

    /* create a display area */
    bitmap_desc = create_bitmap();

    /* assign the display area to the viewport */
    view_port_set_handle(display_viewport, &bitmap_desc);

    /* create a format */
    /* attach it to the viewport */
    view_port_attach_format(display_viewport, 0, 0, 0);
    /* get a handle for the format created by the last call */
    display_format = view_port_get_format(display_viewport);
    /* attach model to format that is associated with viewport */
    format_attach(display_format, display_model);

    /* display top of file */
    tml_dp_damage_repair((void*) view_port_get_handle(display_viewport));

/*********************************************************************/
    /* allocate cursor */
    /* attach cursor to viewport at first character */
    the_cursor = v_cursor_class_alloc();
    if (!the_cursor) {
        fprintf(stderr, "unable to allocate a cursor\n");
        exit(1);
        }

    view_port_attach_cursor(display_viewport, the_cursor, &zero_zero);
    v_cursor_enable(the_cursor, 0);
    enable_mouse();
    enable_all_keys();

    while (1) {
        unob = gpr_$event_wait(&ev_type, (unsigned char*) &ev_value,
                               &ret_coord, &status);
        switch(ev_type) {
            case(gpr_$buttons):
                /* mouse event */
                switch(ev_value) {
                    case (KBD_$M1D):
                        /* mouse button 1 down */
                        /* move TMLib cursor to where pointing cursor was */
                        /*   at click */
                        inq_pos.at_x = ret_coord.x_coord;
                        inq_pos.at_y = ret_coord.y_coord;
                        v_cursor_xy_set(the_cursor, &inq_pos);
                        break;

                    default:
                        break;
                    }
                break;

            case (gpr_$keystroke):
                switch(ev_value) {
                    case('\215'):
                        /* up-box-arrow key: scroll up half a viewport */
                        /* get (x,y) position of cursor in viewport */
                        v_cursor_get_raw_location(the_cursor, &curs_loc);
                        /* get height of viewport -- it may have changed */
                        dy = view_port_get_hi(display_viewport);
                        /* get top line of viewport */
                        old_top_line =
                          view_pos_get_cline(view_port_get_top_line(
                                             display_viewport));
                        /* find the character if any at top of viewport */
                        view_port_xy_2_mpos(display_viewport,
                                            &zero_zero, &start_display);
                        /* if a character was found */
                        /*   display it halfway down the screen */
                        if (start_display.in_node) {
                            view_port_mpos_2_xy(display_viewport,
                                                &start_display, &inq_pos, 0);
                            inq_pos.at_y = dy/2;
                            view_port_draw_at(display_viewport,
                                              &start_display, &inq_pos);
                            /* if cursor character not on screen, */
                            /*   set cursor at position it had before scroll */
                            v_cursor_get_model_pos(the_cursor, &curs_pos);
                            if (!view_port_mpos_2_vpos(display_viewport,
                                                       &curs_pos))
                                v_cursor_xy_set(the_cursor, &curs_loc);
                            }
                        /* if no movement and line above exists, */
                        /*   go to top of file */
                        /* get new top line of viewport */
                        new_top_line =
                          view_pos_get_cline(view_port_get_top_line(
                                             display_viewport));
                        if (new_top_line &&
                          (new_top_line == old_top_line) &&
                          (cline_get_prior(new_top_line))) {
                            /* no movement and prior line exists */
                            /* get first position in model */
                            start_display.in_node =
                              model_get_root(format_get_model(display_format));
                            start_display.char_index = 0;
                            model_pos_get_next(&start_display, &start_display);
                            /* get and correct position of first position */
                            /*   on its conceptual page */
                            first_page =
                              format_mpos_2_phead(display_format,
                                                  &start_display);
                            first_line =
                              phead_mpos_2_cline(first_page, &start_display);
                            cline_mpos_2_xy(first_line,
                                            &start_display, &at_coord);
                            inq_pos.at_x = at_coord.x_coord
                              + view_port_get_hoffset(display_viewport);
                            inq_pos.at_y = 0;
                            view_port_draw_at(display_viewport,
                                              &start_display, &inq_pos);
                            /* if cursor character not on screen, */
                            /*   set cursor at position it had before scroll */
                            v_cursor_get_model_pos(the_cursor, &curs_pos);
                            if (!view_port_mpos_2_vpos(display_viewport,
                                                       &curs_pos))
                                v_cursor_xy_set(the_cursor, &curs_loc);
                            }
                        break;

                    case ('\217'):
                        /* down-box arrow key: scroll down half a viewport */
                        /* get (x,y) position of cursor in viewport */
                        v_cursor_get_raw_location(the_cursor, &curs_loc);
                        /* get height of viewport -- it may have changed */
                        dy = view_port_get_hi(display_viewport);
                        /* find character halfway down and furthest left */
                        inq_pos.at_x = 0;
                        inq_pos.at_y = dy/2;
                        view_port_xy_2_mpos(display_viewport,
                                            &inq_pos, &start_display);
                        /* if character found, display at top of viewport */
                        if (start_display.in_node) {
                            /* get true location of character */
                            view_port_mpos_2_xy(display_viewport,
                                                &start_display, &inq_pos, 0);
                            inq_pos.at_y = 0;
                            /* display at new coordinates */
                            view_port_draw_at(display_viewport,
                                              &start_display, &inq_pos);
                            /* if cursor character not on screen, */
                            /*   set cursor at position it had before scroll */
                            v_cursor_get_model_pos(the_cursor, &curs_pos);
                            if (! view_port_mpos_2_vpos(display_viewport,
                                                        &curs_pos))
                                v_cursor_xy_set(the_cursor, &curs_loc);
                            }
                        break;

                    case('\207'):
                        /* left-box arrow key: */
                        /*   scroll left 1/4 viewport width */
                        /* get (x,y) position of cursor in viewport */
                        v_cursor_get_raw_location(the_cursor, &curs_loc);
                        /* get width of viewport -- it may have changed */
                        dx = view_port_get_width(display_viewport);
                        /* find character at pixel (0,0) */
                        view_port_xy_2_mpos(display_viewport,
                                            &zero_zero, &start_display);
                        if (start_display.in_node) {
                            /* get (x,y) of topmost character */
                            view_port_mpos_2_xy(display_viewport,
                                                &start_display, &inq_pos, 0);
                            /* increment x coordinate by 1/4 viewport width */
                            inq_pos.at_x += dx/4;
                            /* draw first character at new coordinates */
                            view_port_draw_at(display_viewport,
                                              &start_display, &inq_pos);
                            /* if cursor character not on screen, */
                            /*   set cursor at position it had before scroll */
                            v_cursor_get_caged_location(the_cursor, &inq_pos);
                            if (inq_pos.at_x > dx)
                                v_cursor_xy_set(the_cursor, &curs_loc);
                            }
                        break;

                    case('\211'):
                        /* right-box arrow key: */
                        /*   scroll right 1/4 viewport width */
                        /* get (x,y) position of cursor */
                        v_cursor_get_raw_location(the_cursor, &curs_loc);
                        /* get width of viewport -- it may have changed */
                        dx = view_port_get_width(display_viewport);
                        /* find character at pixel (0,0) */
                        view_port_xy_2_mpos(display_viewport,
                                            &zero_zero, &start_display);
                        if (start_display.in_node) {
                            /* get (x,y) of topmost character */
                            view_port_mpos_2_xy(display_viewport,
                                                &start_display, &inq_pos, 0);
                            /* decrement x coordinate by 1/4 viewport width */
                            inq_pos.at_x -= dx/4;
                            /* draw first character at new coordinates */
                            view_port_draw_at(display_viewport,
                                              &start_display, &inq_pos);
                            /* if cursor character not on screen, */
                            /*   set cursor at position it had before scroll */
                            v_cursor_get_caged_location(the_cursor, &inq_pos);
                            if (inq_pos.at_x < 0)
                                v_cursor_xy_set(the_cursor, &curs_loc);
                            }
                        break;

                    case('\224'):
                        /* exit key: exit the program */
                        /* release dynamically allocated storage */
                        /* freeing viewport frees cursor */
                        view_port_free(display_viewport);
                        /* delete format */
                        format_free(display_format);
                        /* delete model */
                        model_free(display_model);
                        exit(0);

                    default:
                        break;
                }
                break;

            default:
                break;
            }
        }
}/*********************************************************************/

