#include <stdio.h>
#include <apollo/base.h>
/***********************************************************************/
#include <apollo/gpr.h>
#include <apollo/kbd.h>
/***********************************************************************/
#include <tml/tml_objects.hxx>
#include <tml/tml_functions.hxx>
#include <tml/tml_dp_graph.hxx>

#ifdef __cplusplus
extern "C" {
    long create_bitmap();
    void enable_mouse();
    void enable_all_keys();
};
#endif

extern long create_bitmap();
extern void enable_mouse();
extern void enable_all_keys();

/***********************************************************************/
extern void enable_mouse();
extern void enable_all_keys();
/***********************************************************************/

main()
{
    model     *display_model;
    view_port *display_viewport;
    format    *display_format;
    model_pos  start_display;
    long       bitmap_desc;
    long       dx, dy;              /* size of viewport */
    screen_xy  inq_pos;             /* any (x,y) coordinate on the screen */
/***************************************************************************/
    v_cursor        *the_cursor;
    gpr_$event_t     ev_type;
    char             ev_value;
    gpr_$position_t  ret_coord;
    status_$t        status;
    char             unob;
    screen_xy        curs_loc;
    model_pos        curs_pos;
    cline           *old_top_line;
    cline           *new_top_line;
    cline           *first_line;
    phead           *first_page;
    xy_coord         at_coord;
/***************************************************************************/

    /* associate all viewport objects with the GPR graphic system */
    tml_dp_set_dflt_vpftbl(tml_dp_get_gpr_vpftbl());

    /* create a model from the ASCII file test_file */
    display_model = read_uasc("test_file", 9);

    /* allocate a viewport */
    display_viewport = new view_port;

    /* create a display area */
    bitmap_desc = create_bitmap();

    /* assign the display area to the viewport */
    display_viewport->set_handle(&bitmap_desc);

    /* create a format */
    /* attach it to the viewport */
    display_viewport->attach((format*) 0);
    /* get a handle for the format created by the last call */
    display_format = display_viewport->get_format();
    /* attach model to format that is associated with viewport */
    display_format->attach(display_model);

    /* display top of file */
    tml_dp_damage_repair(display_viewport->get_handle());

/***************************************************************************/
    /* allocate cursor */
    /* attach cursor to viewport at first character */
    the_cursor = new v_cursor;
    if (!the_cursor) {
        fprintf(stderr, "unable to allocate a cursor\n");
        exit(1);
        }

    display_viewport->attach(the_cursor, zero_zero);
    the_cursor->enable();

    /* enable the mouse and keyboard */
    enable_mouse();
    enable_all_keys();

    while (1) {
        unob = gpr_$event_wait(&ev_type, (unsigned char*) &ev_value,
                               &ret_coord, &status);
        switch(ev_type) {
            case (gpr_$buttons):
                /* mouse event */
                switch(ev_value) {
                    case(KBD_$M1D):
                        /* mouse button 1 down */
                        /* move TMLib cursor to where pointing cursor was */
                        /*   at click */
                        inq_pos.at_x = ret_coord.x_coord;
                        inq_pos.at_y = ret_coord.y_coord;
                        the_cursor->set(inq_pos);
                        break;

                    default:
                        break;
                    }
                break;

            case (gpr_$keystroke):
                switch(ev_value) {
                    case ('\215'):
                        /* up-box-arrow key: scroll up half a viewport */
                        /* get (x,y) position of cursor in viewport */
                        curs_loc = the_cursor->get_raw_loc();
                        /* get height of viewport -- it may have changed */
                        dy = display_viewport->get_hi();
                        /* get top line in viewport */
                        old_top_line =
                          display_viewport->get_top_line()->get_cline();
                        /* find the character if any at top of viewport */
                        start_display = display_viewport->xy_2_mpos(zero_zero);
                        /* if a character was found */
                        /*   display it halfway down the screen */
                        if (start_display.in_node) {
                            inq_pos =
                              display_viewport->mpos_2_xy(start_display);
                            inq_pos.at_y = dy/2;
                            display_viewport->draw(start_display, inq_pos);
                            /* if cursor character not on screen, */
                            /*   set cursor at position it had before scroll */
                            curs_pos = the_cursor->get_model_pos();
                            if (!display_viewport->mpos_2_vpos(curs_pos))
                                the_cursor->set(curs_loc);
                            }
                        /* if no movement and line above exists, */
                        /*   go to top of file */
                        /* get new top line of viewport */
                        new_top_line =
                          display_viewport->get_top_line()->get_cline();
                        if (new_top_line &&
                          (new_top_line == old_top_line) &&
                          (new_top_line->get_prior())) {
                            /* no movement and prior text line exists */
                            /* get first position in model */
                            start_display.in_node =
                              display_format->get_model()->get_root();
                            start_display.char_index = 0;
                            start_display = start_display.get_next();
                            /* get and correct position of first position */
                            /*   on its conceptual page */
                            first_page =
                              display_format->mpos_2_phead(start_display);
                            first_line =
                              first_page->mpos_2_cline(start_display);
                            at_coord = first_line->mpos_2_xy(start_display);
                            inq_pos.at_x = at_coord.x_coord
                              + display_viewport->get_hoffset();
                            inq_pos.at_y = 0;
                            display_viewport->draw(start_display, inq_pos);
                            /* if cursor character not on screen, */
                            /*   set cursor at position it had before scroll */
                            curs_pos = the_cursor->get_model_pos();
                            if (!display_viewport->mpos_2_vpos(curs_pos))
                                the_cursor->set(curs_loc);
                            }
                        break;

                    case ('\217'):
                        /* down-box arrow key: scroll down half a viewport */
                        /* get (x,y) position of cursor in viewport */
                        curs_loc = the_cursor->get_raw_loc();
                        /* get height of viewport -- it may have changed */
                        dy = display_viewport->get_hi();
                        /* find character halfway down and furthest left */
                        inq_pos.at_x = 0;
                        inq_pos.at_y = dy/2;
                        start_display = display_viewport->xy_2_mpos(inq_pos);
                        /* if character found, display at top of viewport */
                        if (start_display.in_node) {
                            /* get true location of character */
                            inq_pos =
                              display_viewport->mpos_2_xy(start_display);
                            inq_pos.at_y = 0;
                            /* display at new coordinates */
                            display_viewport->draw(start_display, inq_pos);
                            /* if cursor character not on screen, */
                            /*   set cursor at position it had before scroll */
                            curs_pos = the_cursor->get_model_pos();
                            if (!display_viewport->mpos_2_vpos(curs_pos))
                                the_cursor->set(curs_loc);
                            }
                        break;

                    case ('\207'):
                        /* left-box arrow key: */
                        /*   scroll left 1/4 viewport width */
                        /* get (x,y) position of cursor in viewport */
                        curs_loc = the_cursor->get_raw_loc();
                        /* get width of viewport -- it may have changed */
                        dx = display_viewport->get_width();
                        /* find character at pixel (0,0) */
                        start_display = display_viewport->xy_2_mpos(zero_zero);
                        if (start_display.in_node) {
                            /* get (x,y) of topmost character */
                            inq_pos =
                              display_viewport->mpos_2_xy(start_display);
                            /* increment x coordinate by 1/4 viewport width */
                            inq_pos.at_x += dx/4;
                            /* display at new coordinates */
                            display_viewport->draw(start_display, inq_pos);
                            /* if cursor character not on screen, */
                            /*   set cursor at position it had before scroll */
                            inq_pos = the_cursor->get_caged_loc();
                            if (inq_pos.at_x > dx)
                                the_cursor->set(curs_loc);
                            }
                        break;

                    case ('\211'):
                        /* right-box arrow key: */
                        /*   scroll right 1/4 viewport width */
                        /* get (x,y) position of cursor */
                        curs_loc = the_cursor->get_raw_loc();
                        /* get width of viewport -- it may have changed */
                        dx = display_viewport->get_width();
                        /* find character at pixel (0,0) */
                        start_display = display_viewport->xy_2_mpos(zero_zero);
                        if (start_display.in_node) {
                            /* get (x,y) of topmost character */
                            inq_pos =
                              display_viewport->mpos_2_xy(start_display);
                            /* decrement x coordinate by 1/4 viewport width */
                            inq_pos.at_x -= dx/4;
                            /* display at new coordinates */
                            display_viewport->draw(start_display, inq_pos);
                            /* if cursor character not on screen, */
                            /*   set cursor at position it had before scroll */
                            inq_pos = the_cursor->get_caged_loc();
                            if (inq_pos.at_x < 0)
                                the_cursor->set(curs_loc);
                            }
                        break;

                    case ('\224'):
                        /* exit key: exit the program */
                        /* release dynamically allocated storage */
                        /* freeing viewport frees cursor */
                        delete display_viewport;
                        /* delete format */
                        delete display_format;
                        /* delete model */
                        delete display_model;
                        /* exit */
                        exit (0);

                    default:
                        break;
                }
                break;
            default:
                break;
            }

        }
/***************************************************************************/
}

