{ gpr_pas_example_1b.pas }
PROGRAM gpr_pas_example_1b;

%include '/sys/ins/base.ins.pas';
%include '/sys/ins/gpr.ins.pas';
%include '/usr/include/tml/tml.ins.pas';

PROCEDURE create_bitmap(OUT return_bitmap : integer32); EXTERN;
PROCEDURE enable_all_keys; EXTERN;
PROCEDURE tml_dp_damage_repair(viewport_handle : tml_c_bitmap_id); EXTERN;

VAR
    zero              :integer32            :=  0;
    filename          :array[0..8] of char;
    filename_length   :integer32;
    display_model     :TML_C_MODEL;
    display_viewport  :TML_C_VIEW_PORT;
    display_format    :TML_C_FORMAT;
    start_display     :C_MODEL_POS;
    p_start_display   :PTR_TO_C_MODEL_POS   := addr(start_display);
    bitmap            :integer32;
    unob              :boolean;
    ev_type           :gpr_$event_t;
    ev_value          :char                 := '0';
    ret_coord         :gpr_$position_t;
    status            :status_$t;

{***************************************************************}
    dy                :INTEGER32;           { viewport height }
    dx                :INTEGER32;           { viewport width }
    inq_pos           :C_SCREEN_XY;         { any x,y coordinate on the screen }
    p_inq_pos         :PTR_TO_C_SCREEN_XY   := addr(inq_pos);

    last_page         : TML_C_PHEAD;        { used for going to bottom of file }
    last_line         : TML_C_CLINE;        { used for going to bottom of file }
    dummy_view_pos    :TML_C_VIEW_POS;
    dummy_integer     :INTEGER32;
    dummy_format      :TML_C_FORMAT;
{***************************************************************}

BEGIN

{ initialize TMLLIB }
tml_init;

{ associate all viewport objects with the GPR graphic system }
tml_c_tml_dp_set_dflt_vpftbl(tml_c_tml_dp_get_gpr_vpftbl());

{ create a model from the ASCII file test_file }
filename := 'test_file';
filename_length := SIZEOF(filename);
display_model := tml_c_read_uasc_path(filename[0], filename_length);

{ If a model is created successfully, continue }
IF (display_model <> NIL) THEN
   BEGIN
   { allocate a viewport }
   display_viewport := view_port_class_alloc;

   { create a graphics display area
    use a routine that depends on the graphics system }
   create_bitmap(bitmap);

   { assign the display area to the viewport }
   dummy_integer :=
     view_port_set_handle(display_viewport,
                          tml_c_bitmap_id(addr(bitmap)));

   { create a format attach it to the viewport }
   dummy_format := NIL;
   dummy_integer:=
     view_port_attach_format(display_viewport,
                             dummy_format, zero, zero);

   { get a handle for the format created by the last call }
   display_format := view_port_get_format(display_viewport);

   { attach the model to the format which is associated with the viewport }
   format_attach(display_format, display_model);

   start_display.in_node := model_get_root(display_model);

   { do nothing is no character found }
   IF (start_display.in_node <> NIL) THEN
      BEGIN
      start_display.char_index := 0;

      { display top of file }
      tml_dp_damage_repair(view_port_get_handle(display_viewport));

      { enable keyboard }
      enable_all_keys();

{***************************************************************}
      WHILE (ev_value <> 'e') DO
      { typing 'e' exits the program }

         BEGIN

         unob := gpr_$event_wait(ev_type, ev_value, ret_coord, status);

         CASE ev_value OF

         'u' :
            BEGIN
            { move the viewport up the conceptual page
             half the viewport height }
            { get height of viewport -- it may have changed }
            dy := view_port_get_hi(display_viewport);
            { find character at top of viewport }
            p_start_display :=
              view_port_xy_2_mpos(display_viewport,
                                  zero_zero, start_display);
            { if character found, display it halfway down the screen }
            IF (start_display.in_node <> NIL) THEN
               BEGIN
               inq_pos.at_x := 0;
               inq_pos.at_y := dy div 2;
               view_port_draw_at(display_viewport, start_display,
                                 inq_pos);
               END;
            END;

         'd' :
            BEGIN
            { move viewport down the conceptual page
             half the viewport height }
            { get height of viewport -- it may have changed }
            dy := view_port_get_hi(display_viewport);
            { find character at pixel (0,dy/2) }
            inq_pos.at_x := 0;
            inq_pos.at_y := dy div 2;
            p_start_display :=
              view_port_xy_2_mpos(display_viewport,
                                  inq_pos, start_display);
            { if character found, display it at pixel (0,0 }
            IF (start_display.in_node <> NIL) THEN
               view_port_draw_at(display_viewport, start_display,
                                 zero_zero);
            END;

         'l' :
            BEGIN
            { move left on the conceptual page
             half the viewport width }
            { get width of viewport }
            dx := view_port_get_width(display_viewport);
            { find character at pixel (0,0) }
            p_start_display :=
              view_port_xy_2_mpos(display_viewport,
                                  zero_zero, start_display);
            { if character found, display it at pixel (dx/2,0) }
            IF (start_display.in_node <> NIL) THEN
               BEGIN
               { get true character location }
               p_inq_pos :=
                 view_port_mpos_2_xy(display_viewport,
                                     start_display, inq_pos, NIL);
               { move half a viewport left }
               inq_pos.at_x := inq_pos.at_x + (dx div 2);
               view_port_draw_at(display_viewport, start_display, inq_pos);
               END;
            END;

         'r' :
             BEGIN
             { move right on the conceptual page half the viewport width }
             { get width of viewport }
             dx := view_port_get_width(display_viewport);
             { find character at pixel (0,0) }
             p_start_display :=
               view_port_xy_2_mpos(display_viewport,
                                   zero_zero, start_display);
             { if character found, display it at pixel (-dx/2,0) }
             IF (start_display.in_node <> NIL) THEN
                BEGIN
                { get location of character }
                p_inq_pos :=
                  view_port_mpos_2_xy(display_viewport,
                                      start_display, inq_pos, NIL);
                { move half a viewport right }
                inq_pos.at_x := inq_pos.at_x - (dx div 2);
                view_port_draw_at(display_viewport, start_display,
                                  inq_pos);
                END;
             END;

         't' :
            BEGIN
            { move display top of file }
            { get root node of model, then move to next model position }
            start_display.in_node := model_get_root(display_model);
            start_display.char_index := 0;
            p_start_display :=
              model_pos_get_next(start_display, start_display);
            { if character found, display it at pixel (0,0) }
            IF (start_display.in_node <> NIL) THEN
                view_port_draw_at(display_viewport, start_display,
                                  zero_zero);
            END;

         'b' :
            BEGIN
            { move to display end of file }
            { get root node of model and last node dominated by root }
            start_display.in_node :=
              node_get_last(model_get_root(display_model));
            { quit if no character found }
            IF (start_display.in_node <> NIL) THEN
               BEGIN
               { get index of last character for that node }
               start_display.char_index :=
                 node_get_max_index(start_display.in_node);
               { find conceptual page the model position is on }
               last_page :=
                 format_mpos_2_phead(display_format, start_display);
               { quit if page not found }
               IF (last_page <> NIL) THEN
                  BEGIN
                  { find last cline on last page }
                  last_line :=
                    phead_mpos_2_cline(last_page, start_display);
                  { quit if line not found }
                  IF (last_line <> NIL) THEN
                     BEGIN
                     { set up a screen_xy for the bottom of file
                      at height of last line }
                     inq_pos.at_x := 0;
                     inq_pos.at_y :=
                       view_port_get_hi(display_viewport)
                       - cline_get_hi(last_line);
                     view_port_draw(display_viewport, last_line, inq_pos);
                     END;

                 END;

              END;

            END;

         END;

         END;

      END;

   { release dynamically allocated storage }
   view_port_free( display_viewport);
   format_free( display_format);
   model_free( display_model);

   END;
{***************************************************************}

END.

