#include <stdio.h>
#include <tml/tml_objects.hxx>
#include <tml/tml_functions.hxx>

    /*****************************************/
#include <dlg/Xlib.h>
#include <dlg/Xutil.h>
#include <tml/tml_dp_x_graph.h>
#include <tml/tml_dp_graph.hxx> /* graphics system call profiles */

extern window_info* make_window(long mask_selection);
extern char         x_exam_getchar(Display *display,
                                   view_port *display_viewport);

main()
{
    model     *display_model;
    view_port *display_viewport;
    format    *display_format;
    model_pos  start_display;
    long       mask_selection = KeyPressMask |       /* Ask XWindows for */
                                ExposureMask |       /* notice of key, */
                                StructureNotifyMask; /* expose, and window */
                                                     /* size and location */
                                                     /* change events */
    /***************************************/
    window_info   *window_desc;
    /***************************************/

    /*****************************************/
    int        c;
    long       dx, dy;    /* size of viewport */
    screen_xy  inq_pos;   /* any (x,y) coordinate on the screen */
    phead     *last_page; /* used for going to bottom of file */
    cline     *last_line; /* used for going to bottom of file */
    /*****************************************/

    /***************************************/
    /* set up XWindows */
    /* get information about display, etc. */
    window_desc = make_window(mask_selection);
    if (!window_desc) {
        fprintf(stderr, "unable to initialize X\n");
        exit(1);
        }
    /* set font display */
    /* MUST BE DONE BEFORE CREATING A MODEL */
    tml_dp_set_font_display(window_desc->display);
    /***************************************/

    /* create a model from the ASCII file test_file */
    display_model = read_uasc("test_file", 9);

    /* allocate a viewport */
    display_viewport = new view_port;

    /* assign the display area to the viewport */
    display_viewport->set_handle(window_desc);

    /* create a format and attach it to the viewport */
    display_viewport->attach((format*) 0);
    /* get a handle for the format created by the last call */
    display_format = display_viewport->get_format();
    /* attach the model to the format that is associated with the viewport */
    display_format->attach(display_model);

    /*****************************************/
    /* typing 'e' exits the program */
    while ('e' !=
      (c = x_exam_getchar(window_desc->display, display_viewport))) {
        switch (c) {
            case 'u':
                /* move viewport up the conceptual page */
                /*   half the viewport height */
                /* get height of viewport -- it may have changed */
                dy = display_viewport->get_hi();
                /* find character at top of viewport */
                start_display = display_viewport->xy_2_mpos(zero_zero);
                /* if character found, display it halfway down the screen */
                if (start_display.in_node) {
                    inq_pos.at_x = 0;
                    inq_pos.at_y = dy/2;
                    display_viewport->draw(start_display, inq_pos);
                    }
                break;

            case 'd':
                /* move viewport down the conceptual page */
                /*   half the viewport height */
                /* get height of viewport -- it may have changed */
                dy = display_viewport->get_hi();
                /* find character at pixel (0,dy/2) */
                inq_pos.at_x = 0;
                inq_pos.at_y = dy/2;
                start_display = display_viewport->xy_2_mpos(inq_pos);
                /* if character found, display it at pixel (0,0) */
                if (start_display.in_node)
                    display_viewport->draw(start_display, zero_zero);
                break;

            case 'l':
                /* move left on the conceptual page */
                /*  half the viewport width */
                /* get width of viewport */
                dx = display_viewport->get_width();
                /* find character at pixel (0,0) */
                start_display = display_viewport->xy_2_mpos(zero_zero);
                /* if character found, display it at pixel (dx/2,0) */
                if (start_display.in_node) {
                    /* get true character location */
                    inq_pos = display_viewport->mpos_2_xy(start_display);
                    /* move half a viewport left */
                    inq_pos.at_x += (dx/2);
                    display_viewport->draw(start_display, inq_pos);
                    }
                break;

            case 'r':
                /* move right on the conceptual page */
                /*   half the viewport width */
                /* get width of viewport */
                dx = display_viewport->get_width();
                /* find character at pixel (0,0) */
                start_display = display_viewport->xy_2_mpos(zero_zero);
                /* if character found, display it at pixel (-dx/2,0) */
                if (start_display.in_node) {
                    /* get true character location */
                    inq_pos = display_viewport->mpos_2_xy(start_display);
                    /* move half a viewport right */
                    inq_pos.at_x -= (dx/2);
                    display_viewport->draw(start_display, inq_pos);
                    }
                break;

            case 't':
                /* move to display top of file */
                /* get root node of model, and move to next model position */
                start_display.in_node = display_model->get_root();
                start_display.char_index = 0;
                start_display = start_display.get_next();
                /* if character found, display it at pixel (0,0) */
                if (start_display.in_node)
                    display_viewport->draw(start_display, zero_zero);
                break;

            case 'b':
                /* move to display end of file */
                /* get root node of model and last node dominated by root */
                start_display.in_node = display_model->get_root()->get_last();
                /* quit if no character found */
                if (!start_display.in_node) break;
                /* get index of last character for that node */
                start_display.char_index =
                  start_display.in_node->get_max_index();
                /* find conceptual page containing that model position */
                last_page = display_format->mpos_2_phead(start_display);
                /* quit if no page found */
                if (!last_page) break;
                /* find last cline on last_page */
                last_line = last_page->mpos_2_cline(start_display);
                /* quit if no line found */
                if (!last_line) break;
                /* set up a screen_xy for the bottom of the last page */
                /*   at the height of the last line */
                inq_pos.at_x = 0;
                inq_pos.at_y = display_viewport->get_hi() - last_line->get_hi();
                display_viewport->draw(last_line, inq_pos);
                break;

            default:
                break;
        }
    }
    /* release dynamically allocated storage */
    delete display_viewport;
    /*****************************************/
}
