#include <stdio.h>
#include <tml/tml_objects.hxx>
#include <tml/tml_functions.hxx>

    /*****************************************/
#include <dlg/Xlib.h>
#include <dlg/Xutil.h>
#include <tml/tml_dp_x_graph.h>
#include <tml/tml_dp_graph.hxx> /* graphics system call profiles */

/**************************************************************/
/* construct model */
/* attach properties */
model *example_read(char *file_name, int f_len)
{
    model    *return_model; /* the model created */
    FILE     *file_id;
    char     *open_type = "r";
    node     *current_node;
    gnode    *current_gnode,
             *last_gnode;
    property *font_prop,
             *margin_prop,
             *wrap_prop;
    char     *char_buf;
    int       buf_size,
              chars_back,
              add_font_prop;
    long      chars_attached;

    /* open file from which model will be made */
    file_id = fopen(file_name, open_type);
    if (!file_id) {
        fprintf(stderr, "unable to open file\n");
        return 0;
        }

    /* allocate model */
    return_model = new model;
    if (!return_model) {
        fprintf(stderr, "unable to create model\n");
        return 0;
        }

    /* make root mode for model */
    current_node = return_model->make_root();
    if (!current_node) {
        fprintf(stderr, "unable to create root node\n");
        return 0;
        }

    /* make a character wrap property */
    /* associate it with the root node */
    /* enables character wrap for entire document */
    /*   except where overridden, for a subtree in */
    /*   the model, by a different wrap property */
    /*   attached to some other node */
    wrap_prop = make_wrap_prop(char_wrap, return_model);
    if (!wrap_prop) {
        fprintf(stderr, "unable to make wrap prop\n");
        return 0;
        }
    /* connect wrap property to root node */
    connect_prop(wrap_prop, current_node);

    /* make a margin property */
    /* use 40-pixel left and right margin */
    /* no top or bottom margin */
    margin_prop = make_margin_prop(40, 40, 0, 0, return_model);
    if (!margin_prop) {
        fprintf(stderr, "unable to make margin property \n");
        return 0;
        }
    /* connect margin property to root node */
    connect_prop(margin_prop, current_node);

    /* make a terminal node */
    current_gnode = new gnode(return_model);
    if (!current_gnode) {
        fprintf(stderr, "unable to create node\n");
        return 0;
        }
    /* connect it as child of root */
    current_node->connect_node(current_gnode, n_down);

    /* make a font property */
    font_prop = make_font_prop("8x13bold", 8, return_model);
    if (!font_prop) {
        fprintf(stderr, "failed in making font property\n");
        exit(0);
        }
    /* connect font property to first terminal node */
    /* overrides, for this node only (because it is a */
    /*   terminal node) the default font property value */
    /*   associated with each model */
    connect_prop(font_prop, current_gnode);

    /* allocate space for reading characters */
    buf_size = 256;
    char_buf = new char[buf_size];
    if (!char_buf) {
        fprintf(stderr, "unable to allocate character buffer\n");
        return 0;
        }

    /* read 256 bytes from file */
    chars_back = fread(char_buf, 1, buf_size, file_id);
    /* associate them with the newly-created node */
    chars_attached = current_gnode->attach_data(char_buf, chars_back);
    /* check for read error or error in attaching data */
    if (chars_attached != chars_back) {
        fprintf(stderr, "unable to attach all characters to a node\n");
        delete return_model;
        return 0;
        }

    /* remember node just created */
    last_gnode = current_gnode;
    /* check for more characters in file */
    if (chars_back < buf_size)
        return return_model;
    /* file-reading loop */
    /* read characters from file */
    /* create a new terminal node */
    /* associate characters with newly-created node */
    /* connect new node to last node created */
    /* remember newly-created node as last created */
    /* change font on alternate nodes */
    add_font_prop = 0;
    for(chars_back = fread(char_buf, 1, buf_size, file_id);
      chars_back;
      chars_back = fread(char_buf, 1, buf_size, file_id)) {

        /* create a new terminal node */
        current_gnode = new gnode(return_model);

        /* quit if allocation fails */
        if (!current_gnode) {
            fprintf(stderr, "unable to alloc node\n");
            return return_model;
            }

        /* associate characters read with newly-created node */
        chars_attached = current_gnode->attach_data(char_buf, chars_back);
        if (chars_attached != chars_back) {
            fprintf(stderr, "unable to attach all characters to a node\n");
            delete return_model;
            return 0;
            }

        /* attach font property to alternate nodes */
        if (add_font_prop) {
            connect_prop(font_prop, current_gnode);
            add_font_prop = 0;
            }
        else
            add_font_prop = 1;

        /* connect new node to last node created */
        last_gnode->connect_node(current_gnode, n_right);
        /* remember newly-created node as last created */
        last_gnode = current_gnode;
        }

    /* close file */
    chars_back = fclose(file_id);

    /* release dynamically allocated storage */
    delete char_buf;

    /* return model */
    return return_model;
}
/**************************************************************/

extern window_info* make_window(long mask_selection);
extern char         x_exam_getchar(Display *display,
                                   view_port *display_viewport);

main()
{
    model     *display_model;
    view_port *display_viewport;
    format    *display_format;
    model_pos  start_display;
    long       mask_selection = KeyPressMask |       /* Ask XWindows for */
                                ExposureMask |       /* notice of key, */
                                StructureNotifyMask; /* expose, and window */
                                                     /* size and location */
                                                     /* change events */
    /***************************************/
    window_info   *window_desc;
    /***************************************/

    int        c;
    long       dx, dy;    /* size of viewport */
    screen_xy  inq_pos;   /* any (x,y) coordinate on the screen */
    phead     *last_page; /* used for going to bottom of file */
    cline     *last_line; /* used for going to bottom of file */


    /***************************************/
    /* set up XWindows */
    /* get information about display, etc. */
    window_desc = make_window(mask_selection);
    if (!window_desc) {
        fprintf(stderr, "unable to initialize X\n");
        exit(1);
        }
    /* set font display */
    /* MUST BE DONE BEFORE CREATING A MODEL */
    tml_dp_set_font_display(window_desc->display);
    /***************************************/

    /**************************************************/
    /* create a model from the ASCII file test_file */
    display_model = example_read("test_file", 9);
    /**************************************************/

    /* allocate a viewport */
    display_viewport = new view_port;

    /* assign the display area to the viewport */
    display_viewport->set_handle(window_desc);

    /* create a format */
    /* attach it to the viewport */
    display_viewport->attach((format*) 0);
    /* get a handle for the format created by the last call */
    display_format = display_viewport->get_format();
    /* attach the model to the format that is associated with the viewport */
    display_format->attach(display_model);

    /* typing 'e' exits the program */
    while ('e' !=
      (c = x_exam_getchar(window_desc->display, display_viewport))) {
        switch (c) {
            case 'u':
                /* move viewport up the conceptual page */
                /*   half the viewport height */
                /* get height of viewport -- it may have changed */
                dy = display_viewport->get_hi();
                /* find character at top of viewport */
                start_display = display_viewport->xy_2_mpos(zero_zero);
                /* if a character was found */
                /*   display it halfway down the screen */
                if (start_display.in_node) {
                    /* get true character location */
                    inq_pos = display_viewport->mpos_2_xy(start_display);
                    inq_pos.at_y = dy/2;
                    display_viewport->draw(start_display, inq_pos);
                    }
                break;

            case 'd':
                /* move the viewport down the conceptual page */
                /*   half the viewport height */
                /* get height of viewport -- it may have changed */
                dy = display_viewport->get_hi();
                /* find the character at pixel (0,dy/2) */
                inq_pos.at_x = 0;
                inq_pos.at_y = dy/2;
                start_display = display_viewport->xy_2_mpos(inq_pos);
                /* if a character was found, display it */
                /*   at top of screen */
                if (start_display.in_node) {
                    /* get true character location */
                    inq_pos = display_viewport->mpos_2_xy(start_display);
                    inq_pos.at_y = 0;
                    display_viewport->draw(start_display, inq_pos);
                    }
                break;

            case 'l':
                /* move left on the conceptual page half the viewport width */
                /* get width of viewport */
                dx = display_viewport->get_width();
                /* find the character if any at pixel (0,0) */
                start_display = display_viewport->xy_2_mpos(zero_zero);
                /* if a character was found, redisplay */
                if (start_display.in_node) {
                    /* get true character location */
                    inq_pos = display_viewport->mpos_2_xy(start_display);
                    /* move half a viewport left */
                    inq_pos.at_x += (dx/2);
                    display_viewport->draw(start_display, inq_pos);
                    }
                break;

            case 'r':
                /* move right on conceptual page half the viewport width */
                /* get width of viewport */
                dx = display_viewport->get_width();
                /* find the character if any at pixel (0,0) */
                start_display = display_viewport->xy_2_mpos(zero_zero);
                /* if a character was found, redisplay */
                if (start_display.in_node) {
                    /* get true character location */
                    inq_pos = display_viewport->mpos_2_xy(start_display);
                    /* move half a viewport right */
                    inq_pos.at_x -= (dx/2);
                    display_viewport->draw(start_display, inq_pos);
                    }
                break;

            case 't':
                /* move to display top of file */
                /* get root node of model */
                /*   and move to first model position */
                start_display.in_node = display_model->get_root();
                start_display.char_index = 0;
                start_display = start_display.get_next();
                /* if a character was found, display it at pixel (0,0) */
                if (start_display.in_node)
                    display_viewport->draw(start_display, zero_zero);
                break;

            case 'b':
                /* move to display end of file */
                /* get root node of model */
                /* get last node dominated by root */
                start_display.in_node = display_model->get_root()->get_last();
                /* quit if no character found */
                if (!start_display.in_node) break;
                /* get index of last character for that node */
                start_display.char_index =
                  start_display.in_node->get_max_index();
                /* find the conceptual page containing the model position */
                last_page = display_format->mpos_2_phead(start_display);
                /* quit if no page found */
                if (!last_page) break;
                /* find the last cline for the last page */
                last_line = last_page->mpos_2_cline(start_display);
                /* quit if no line found */
                if (!last_line) break;
                /* set up a screen_xy for the bottom of the page */
                /*   at the height of the last line */
                inq_pos.at_x = 0;
                inq_pos.at_y = display_viewport->get_hi() - last_line->get_hi();
                display_viewport->draw(last_line, inq_pos);
                break;

            default:
                break;
        }
    }
    /* release dynamically allocated storage */
    delete display_viewport;
    delete display_model;
}
