(*************************************************************************************
    csm.ins.pas -- header file including Pascal language interface for the object 
      definitions and assorted type declarations for the Character Set Manager (CSM)

    (c) Copyright Apollo Computer Inc., 1990 (unpublished)
     --------------------------------------------------------------------
    |THE FOLLOWING PROGRAMS ARE THE SOLE PROPERTY OF APOLLO COMPUTER INC.|
    |      AND CONTAIN ITS PROPRIETARY AND CONFIDENTIAL INFORMATION.     |
     --------------------------------------------------------------------
**************************************************************************************)

(******************************************************************************
    TYPEs for C++ classes

**********************************************************************************)

TYPE csm_c_csm_character       = UNIV_PTR;
TYPE csm_c_char_range          = UNIV_PTR;
TYPE csm_c_char_range_set      = UNIV_PTR;
TYPE csm_c_collation_map       = UNIV_PTR;
TYPE csm_c_char_set            = UNIV_PTR;
TYPE csm_c_alpha_char_set      = UNIV_PTR;
TYPE csm_c_syll_frag_char_set  = UNIV_PTR;
TYPE csm_c_union_char_set      = UNIV_PTR;
TYPE csm_c_char_set_heap       = UNIV_PTR;
TYPE csm_c_cs_heap             = UNIV_PTR;
TYPE csm_c_csm_string          = UNIV_PTR;
TYPE csm_c_kmp_node            = UNIV_PTR;
TYPE csm_c_xregister           = UNIV_PTR;
TYPE csm_c_char_func_tbl_entry = UNIV_PTR;
TYPE csm_c_char_func_tbl       = UNIV_PTR;
TYPE tml_c_gc_hash_tbl         = UNIV_PTR;
TYPE tml_c_tml_ext_type        = UNIV_PTR;
TYPE tml_c_endian              = UNIV_PTR;

(******************************************************************************
   TYPEs and instances for extensibility

******************************************************************************)
(**********************************************************************************
    Preloaded character set types
**********************************************************************************)

TYPE tml_c_tml_type            = UNIV_PTR;
TYPE tml_c_cs_type             = UNIV_PTR;

VAR tml_char_set               : EXTERN tml_c_cs_type;
VAR tml_alpha_char_set         : EXTERN tml_c_cs_type;
VAR tml_syll_frag_char_set     : EXTERN tml_c_cs_type;

(******************************************************************************
    constants used in the classes

******************************************************************************)

CONST MAX_BYTES_IN_CHAR        = 4294967295;
CONST MAX_CHARS_IN_SET         = 4294967295;


(******************************************************************************
    Enumerated types

******************************************************************************)

(**********************************************************************************
    Enumerated and defined constants
**********************************************************************************)

(* ENUMERATED CLASS OF DIRECTIONS FOR TEXT RENDERING *)

(* c?_l?_?== <direction_to_next_character>_  ? -> { r==right, l==left, u==up, d==down };            *)
(*           <direction_to_next_line>_       ? -> { r==right, l==left, u==up, d==down };            *)
(*           <direction_of_next_line>        ? -> { i==identical, b==bustrophedon (alternating) };  *)
(* In bustrophedon the c? determines direction of first line                                        *)

TYPE csm_c_char_set_dir        = INTEGER32;

CONST csd_undefined            = -1;
CONST cl_ll_i                  = 0;
CONST cl_lr_i                  = 1;
CONST cl_lu_i                  = 2;
CONST cl_ld_i                  = 3;
CONST cl_ll_b                  = 4;
CONST cl_lr_b                  = 5;
CONST cl_lu_b                  = 6;
CONST cl_ld_b                  = 7;
CONST cr_ll_i                  = 8;
CONST cr_lr_i                  = 9;
CONST cr_lu_i                  = 10;
CONST cr_ld_i                  = 11;
CONST cr_ll_b                  = 12;
CONST cr_lr_b                  = 13;
CONST cr_lu_b                  = 14;
CONST cr_ld_b                  = 15;
CONST cu_ll_i                  = 16;
CONST cu_lr_i                  = 17;
CONST cu_lu_i                  = 18;
CONST cu_ld_i                  = 19;
CONST cu_ll_b                  = 20;
CONST cu_lr_b                  = 21;
CONST cu_lu_b                  = 22;
CONST cu_ld_b                  = 23;
CONST cd_ll_i                  = 24;
CONST cd_lr_i                  = 25;
CONST cd_lu_i                  = 26;
CONST cd_ld_i                  = 27;
CONST cd_ll_b                  = 28;
CONST cd_lr_b                  = 29;
CONST cd_lu_b                  = 30;
CONST cd_ld_b                  = 31;

(* ENUMERATED CLASS OF SIMPLE DIRECTIONS *)

(* enum lrud_dir      { lrud_undefined = -1,     *)
(*                      left, right, up, down }; *)
CONST lrud_left                = 0;
CONST lrud_right               = 1;
CONST lrud_up                  = 2;
CONST lrud_down                = 3;

(* ENUMERATED CLASS OF C LANGUAGE ESCAPE CHARACTERS *)
(* enum esc_char {esc_alert,        esc_backspace,       esc_formfeed,       *)
(*                esc_newline,      esc_carriage_return, esc_horizontal_tab, *)
(*                esc_vertical_tab, esc_backslash,       esc_question_mark,  *)
(*                esc_single_quote, esc_double_quote,    num_esc_chars};     *)
CONST esc_alert                = 0;
CONST esc_backspace            = 1;
CONST esc_formfeedesc_newline  = 2;
CONST esc_carriage_return      = 3;
CONST esc_horizontal_tabesc_vertical_tab = 4;
CONST esc_backslash            = 5;
CONST esc_question_markesc_single_quote = 6;
CONST esc_double_quote         = 7;
CONST num_esc_chars            = 8;

(* ENUMERATED CLASS OF ALTERNATIVE DEFINITIONS FOR POSIX BASIC                     *)
(*   AND EXTENDED REGULAR EXPRESSION CHARACTERS                                    *)
(* enum re_char {re_asterisk,         re_backslash,     re_circumflex,       *)
(*               re_close_brace,      re_close_bracket, re_close_parenthesis,*)
(*               re_colon,            re_comma,         re_dollar_sign,      *)
(*               re_eight,            re_equals_sign,   re_five,             *)
(*               re_four,             re_hyphen,        re_nine,             *)
(*               re_one,              re_open_brace,    re_open_bracket,     *)
(*               re_open_parenthesis, re_period,        re_plus_sign,        *)
(*               re_question_mark,    re_seven,         re_six,              *)
(*               re_three,            re_two,           re_vertical_line,    *)
(*               re_zero,                                                    *)
(*               num_re_chars};                                              *)
CONST re_asterisk              = 0;
CONST re_backslash             = 1;
CONST re_circumflex            = 2;
CONST re_close_brace           = 3;
CONST re_close_bracket         = 4;
CONST re_close_parenthesis     = 5;
CONST re_colon                 = 6;
CONST re_comma                 = 7;
CONST re_dollar_sign           = 8;
CONST re_eight                 = 9;
CONST re_equals_sign           = 10;
CONST re_five                  = 11;
CONST re_four                  = 12;
CONST re_hyphen                = 13;
CONST re_nine                  = 14;
CONST re_one                   = 15;
CONST re_open_brace            = 16;
CONST re_open_bracket          = 17;
CONST re_open_parenthesis      = 18;
CONST re_period                = 19;
CONST re_plus_sign             = 20;
CONST re_question_mark         = 21;
CONST re_seven                 = 22;
CONST re_six                   = 23;
CONST re_three                 = 24;
CONST re_two                   = 25;
CONST re_vertical_line         = 26;
CONST re_zero                  = 27;
CONST num_re_chars             = 28;

(**************************************************************************************
    Magic numbers for 32-bit architectures
    The CSM considers its host machine TML_MAGIC_3210
***************************************************************************************)

CONST TML_MAGIC_0123           = 168757262;
CONST TML_MAGIC_1032           = 252317192;
CONST TML_MAGIC_2301           = 135137807;
CONST TML_MAGIC_3210           = 235409162;

(******************************************************************************
    Structures used for general purposes with allocator and
    deallocator for each where necessary
******************************************************************************)

(******************************************************************************
    CSM Objects

******************************************************************************)

(* CLASS CSM_CHARACTER *)

    (* allocate a csm_character from free store *)

FUNCTION csm_character_class_alloc_1 : csm_c_csm_character; OPTIONS(C_PARAM, EXTERN);

    (* ... initialized for a given character set *)

FUNCTION csm_character_class_alloc_2(
        for_char_set : csm_c_char_set
    ) : csm_c_csm_character; OPTIONS(C_PARAM, EXTERN);

    (* ... and a given terminated string value *)

FUNCTION csm_character_class_alloc_3(
    VAR val_start : CHAR;
        for_char_set : csm_c_char_set
    ) : csm_c_csm_character; OPTIONS(C_PARAM, EXTERN);

    (* ... or a nonterminated string value *)

FUNCTION csm_character_class_alloc_4(
    VAR val_start : CHAR;
        len : INTEGER32;
        for_char_set : csm_c_char_set
    ) : csm_c_csm_character; OPTIONS(C_PARAM, EXTERN);

    (* ... or a nonterminated value and length in CHARACTERS *)

FUNCTION csm_character_class_alloc_5(
    VAR val_start : CHAR;
        len : INTEGER32;
        collation_order : INTEGER32;
        for_char_set : csm_c_char_set
    ) : csm_c_csm_character; OPTIONS(C_PARAM, EXTERN);

    (* deallocate a csm_character *)

PROCEDURE csm_character_free(
        csm_char_p : csm_c_csm_character
    ); OPTIONS(C_PARAM, EXTERN);

    (* initialize to argument values *)

PROCEDURE csm_character_init(
        csm_char_p : csm_c_csm_character;
    VAR val_start : CHAR;
        len : INTEGER32;
        for_char_set : csm_c_char_set;
        index : INTEGER32
    ); OPTIONS(C_PARAM, EXTERN);

    (* return string value *)

TYPE PTR_TO_CHAR               = ^CHAR;

FUNCTION csm_character_get_value(
        csm_char_p : csm_c_csm_character
    ) : PTR_TO_CHAR; OPTIONS(C_PARAM, EXTERN);

    (* return string value length in CHARACTERS *)

FUNCTION csm_character_get_num_chars(
        csm_char_p : csm_c_csm_character
    ) : INTEGER32; OPTIONS(C_PARAM, EXTERN);

    (* return a given byte of the string value *)

FUNCTION csm_character_get_byte_at(
        csm_char_p : csm_c_csm_character;
        byte_index : INTEGER32
    ) : PTR_TO_CHAR; OPTIONS(C_PARAM, EXTERN);

    (* returns collation value *)

FUNCTION csm_character_get_collation(
        csm_char_p : csm_c_csm_character
    ) : INTEGER32; OPTIONS(C_PARAM, EXTERN);

    (* return associated character set *)

FUNCTION csm_character_get_char_set(
        csm_char_p : csm_c_csm_character
    ) : csm_c_char_set; OPTIONS(C_PARAM, EXTERN);

    (* set collation value *)

PROCEDURE csm_character_set_collation(
        csm_char_p : csm_c_csm_character;
        new_order : INTEGER32
    ); OPTIONS(C_PARAM, EXTERN);


(* CLASS CHAR_RANGE *)

    (* allocate a char_range from free store *)

FUNCTION char_range_class_alloc_1 : csm_c_char_range; OPTIONS(C_PARAM, EXTERN);

    (* ... initialized with terminated strings for range endpoints *)

FUNCTION char_range_class_alloc_2(
    VAR lo_start : CHAR;
    VAR hi_start : CHAR;
        for_char_set : csm_c_char_set;
        lo_map : INTEGER32;
        hi_map : INTEGER32
    ) : csm_c_char_range; OPTIONS(C_PARAM, EXTERN);

    (* ... or nonterminated strings *)

FUNCTION char_range_class_alloc_3(
    VAR lo_start : CHAR;
        lo_len : INTEGER32;
    VAR hi_start : CHAR;
        hi_len : INTEGER32;
        for_char_set : csm_c_char_set;
        lo_map : INTEGER32;
        hi_map : INTEGER32
    ) : csm_c_char_range; OPTIONS(C_PARAM, EXTERN);

    (* deallocate a char_range *)

PROCEDURE char_range_free(
        range_p : csm_c_char_range
    ); OPTIONS(C_PARAM, EXTERN);

    (* return high csm_character in range *)

FUNCTION char_range_get_high(
        range_p : csm_c_char_range
    ) : csm_c_csm_character; OPTIONS(C_PARAM, EXTERN);

    (* return low csm_character in range *)

FUNCTION char_range_get_low(
        range_p : csm_c_char_range
    ) : csm_c_csm_character; OPTIONS(C_PARAM, EXTERN);

    (* return collation of high csm_character *)

FUNCTION char_range_get_high_index(
        range_p : csm_c_char_range
    ) : INTEGER32; OPTIONS(C_PARAM, EXTERN);

    (* return collation of low csm_character *)

FUNCTION char_range_get_low_index(
        range_p : csm_c_char_range
    ) : INTEGER32; OPTIONS(C_PARAM, EXTERN);

    (* return number of character in range *)

FUNCTION char_range_get_size(
        range_p : csm_c_char_range
    ) : INTEGER32; OPTIONS(C_PARAM, EXTERN);

    (* return whether range contains a given (terminated) character *)

FUNCTION char_range_contains_char(
        range_p : csm_c_char_range;
    VAR candidate_start : CHAR
    ) : INTEGER32; OPTIONS(C_PARAM, EXTERN);

    (* return whether range contains a given collation value *)

FUNCTION char_range_contains_coll(
        range_p : csm_c_char_range;
        collation : INTEGER32
    ) : INTEGER32; OPTIONS(C_PARAM, EXTERN);

    (* return whether range contains a given (nonterminated) character *)

FUNCTION char_range_contains_nt_char(
        range_p : csm_c_char_range;
    VAR candidate_start : CHAR;
        len : INTEGER32
    ) : INTEGER32; OPTIONS(C_PARAM, EXTERN);


(* CLASS CHAR_RANGE_SET *)

    (* allocate a char_range_set from free store *)

FUNCTION char_range_set_class_alloc : csm_c_char_range_set; OPTIONS(C_PARAM, EXTERN);

    (* deallocate a char_range_set *)

PROCEDURE char_range_set_free(
        range_set_p : csm_c_char_range_set
    ); OPTIONS(C_PARAM, EXTERN);

    (* return whether a range set contains a given (terminated) character *)

FUNCTION char_range_set_contain(
        range_p : csm_c_char_range;
    VAR candidate_start : CHAR
    ) : INTEGER32; OPTIONS(C_PARAM, EXTERN);

    (* return the ith range in the set *)

FUNCTION char_range_set_get_range(
        range_p : csm_c_char_range;
        i : INTEGER32
    ) : csm_c_char_range; OPTIONS(C_PARAM, EXTERN);

    (* return the array of ranges in the set *)

FUNCTION char_range_set_get_ranges(
        range_p : csm_c_char_range
    ) : csm_c_char_range; OPTIONS(C_PARAM, EXTERN);

    (* return the number of ranges in the set *)

FUNCTION char_range_set_get_num_ranges(
        range_p : csm_c_char_range
    ) : INTEGER32; OPTIONS(C_PARAM, EXTERN);

    (* initialize the set with a range array of a given size *)

PROCEDURE char_range_set_make_ranges(
        range_p : csm_c_char_range;
        new_num : INTEGER32
    ); OPTIONS(C_PARAM, EXTERN);

    (* initialize a given range in the set *)

PROCEDURE char_range_set_init_range(
        range_p : csm_c_char_range;
        slot : INTEGER32;
    VAR lo_start : CHAR;
    VAR hi_start : CHAR;
        for_char_set : csm_c_char_set;
        spec_size : INTEGER32;
        lo_map : INTEGER32;
        hi_map : INTEGER32
    ); OPTIONS(C_PARAM, EXTERN);

    (* initialize a given range in the set from a range argument *)

PROCEDURE char_range_set_set_range_1(
        range_p : csm_c_char_range;
        from_range : csm_c_char_range;
        for_char_set : csm_c_char_set;
        slot : INTEGER32
    ); OPTIONS(C_PARAM, EXTERN);

    (* initialize a given range for the low and high terminated strings *)

PROCEDURE char_range_set_set_range_2(
        range_p : csm_c_char_range;
    VAR lo_start : CHAR;
    VAR hi_start : CHAR;
        for_char_set : csm_c_char_set;
        slot : INTEGER32;
        lo_map : INTEGER32;
        hi_map : INTEGER32
    ); OPTIONS(C_PARAM, EXTERN);

    (* initialize a given range for the low and high nonterminated strings *)

PROCEDURE char_range_set_set_range_3(
        range_p : csm_c_char_range;
    VAR lo_start : CHAR;
        lo_len : INTEGER32;
    VAR hi_start : CHAR;
        hi_len : INTEGER32;
        for_char_set : csm_c_char_set;
        slot : INTEGER32;
        lo_map : INTEGER32;
        hi_map : INTEGER32
    ); OPTIONS(C_PARAM, EXTERN);

    (* return whether the range contains a given character *)

FUNCTION char_range_set_lookup_char(
        range_p : csm_c_char_range;
    VAR sought_start : CHAR
    ) : INTEGER32; OPTIONS(C_PARAM, EXTERN);

    (* return whether the range contains a given collation *)

FUNCTION char_range_set_lookup_coll(
        range_p : csm_c_char_range;
        collation : INTEGER32;
    VAR c_start : CHAR
    ) : PTR_TO_CHAR; OPTIONS(C_PARAM, EXTERN);

    (* given a character in one range, return the identically indexed character in the other range *)

FUNCTION char_range_set_corresponding(
        range_p : csm_c_char_range;
        other_set : csm_c_char_range_set;
    VAR c_start : CHAR
    ) : PTR_TO_CHAR; OPTIONS(C_PARAM, EXTERN);


(* CLASS COLLATION_MAP *)

    (* allocate a collation_map from free store *)

FUNCTION collation_map_class_alloc_1 : csm_c_collation_map; OPTIONS(C_PARAM, EXTERN);

    (* ... initialized with the values of the given arguments *)

FUNCTION collation_map_class_alloc_2(
        sequences : csm_c_char_range;
        num_sequences : INTEGER32;
        for_char_set : csm_c_char_set;
        start_at : INTEGER32
    ) : csm_c_collation_map; OPTIONS(C_PARAM, EXTERN);

    (* deallocate a collation_map *)

PROCEDURE collation_map_free(
        map_p : csm_c_collation_map
    ); OPTIONS(C_PARAM, EXTERN);

    (* return the map from multibyte characters to collation values *)

FUNCTION collation_map_get_many_to_one(
        map_p : csm_c_collation_map
    ) : csm_c_kmp_node; OPTIONS(C_PARAM, EXTERN);

    (* initialize the member for equivalence class collation *)
    (*   map must have been made with csm_c_collation_map_class_alloc_2 *)

PROCEDURE collation_map_make_secondaries(
        map_p : csm_c_collation_map;
        sequences : csm_c_char_range;
        num_sequences : INTEGER32
    ); OPTIONS(C_PARAM, EXTERN);

    (* initialize the member for one-to-many collation *)
    (*   map must have been made with csm_c_collation_map_class_alloc_2 *)

PROCEDURE collation_map_make_one_to_many(
        map_p : csm_c_collation_map;
        sequences : csm_c_char_range;
        num_sequences : INTEGER32
    ); OPTIONS(C_PARAM, EXTERN);

    (* initialize the member for regular expression substitution *)
    (*   map must have been made with csm_c_collation_map_class_alloc_2 *)

PROCEDURE collation_map_make_substitutes(
        map_p : csm_c_collation_map;
    VAR find : PTR_TO_CHAR;
    VAR sub : PTR_TO_CHAR;
        num_subs : INTEGER32
    ); OPTIONS(C_PARAM, EXTERN);

    (* return length of single- or multicharacter collation sequence beginning at sought;  *)
    (*   advances sought; reports primary collation only *)

FUNCTION collation_map_lookup(
        map_p : csm_c_collation_map;
    VAR sought : PTR_TO_CHAR
    ) : INTEGER32; OPTIONS(C_PARAM, EXTERN);

    (* compare two terminated strings according to data in map *)

FUNCTION collation_map_compare(
        map_p : csm_c_collation_map;
    VAR s1_start : CHAR;
    VAR s2_start : CHAR
    ) : INTEGER32; OPTIONS(C_PARAM, EXTERN);

    (* compare two nonterminated strings according to data in map *)

FUNCTION collation_map_ncompare(
        map_p : csm_c_collation_map;
    VAR s1_start : CHAR;
    VAR s2_start : CHAR;
        n : INTEGER32
    ) : INTEGER32; OPTIONS(C_PARAM, EXTERN);


(* CLASS CHAR_SET *)

    (* allocate a char_set from free store *)

FUNCTION char_set_class_alloc(
        per_char_size : INTEGER32
    ) : csm_c_char_set; OPTIONS(C_PARAM, EXTERN);

    (* deallocate a char_set *)

PROCEDURE char_set_free(
        cs_p : csm_c_char_set
    ); OPTIONS(C_PARAM, EXTERN);

    (* make another instance of the same type of character set object *)

FUNCTION char_set_make_another(
        cs_p : csm_c_char_set;
        c_size : INTEGER32
    ) : csm_c_char_set; OPTIONS(C_PARAM, EXTERN);

    (* return whether the character set uses a zero terminator, has one-byte characters, *)
    (*   and does not use the high bit in any character *)

FUNCTION char_set_is_std(
        cs_p : csm_c_char_set
    ) : INTEGER32; OPTIONS(C_PARAM, EXTERN);

    (* string utility functions like ... *)

    (* ... strcat                *)

FUNCTION char_set_cs_strcat(
        cs_p : csm_c_char_set;
    VAR s1_start : CHAR;
    VAR s2_start : CHAR
    ) : PTR_TO_CHAR; OPTIONS(C_PARAM, EXTERN);

    (* ... strncat               *)

FUNCTION char_set_cs_strncat(
        cs_p : csm_c_char_set;
    VAR s1_start : CHAR;
    VAR s2_start : CHAR;
        n : INTEGER32
    ) : PTR_TO_CHAR; OPTIONS(C_PARAM, EXTERN);

    (* ... strcmp                *)

FUNCTION char_set_cs_strcmp(
        cs_p : csm_c_char_set;
    VAR s1_start : CHAR;
    VAR s2_start : CHAR
    ) : INTEGER32; OPTIONS(C_PARAM, EXTERN);

    (* ... strncmp               *)

FUNCTION char_set_cs_strncmp(
        cs_p : csm_c_char_set;
    VAR s1_start : CHAR;
    VAR s2_start : CHAR;
        n : INTEGER32
    ) : INTEGER32; OPTIONS(C_PARAM, EXTERN);

    (* ... strcpy                *)

FUNCTION char_set_cs_strcpy(
        cs_p : csm_c_char_set;
    VAR s1_start : CHAR;
    VAR s2_start : CHAR
    ) : PTR_TO_CHAR; OPTIONS(C_PARAM, EXTERN);

    (* ... strncpy               *)

FUNCTION char_set_cs_strncpy(
        cs_p : csm_c_char_set;
    VAR s1_start : CHAR;
    VAR s2_start : CHAR;
        n : INTEGER32
    ) : PTR_TO_CHAR; OPTIONS(C_PARAM, EXTERN);

    (* ... strlen                *)

FUNCTION char_set_cs_strlen(
        cs_p : csm_c_char_set;
    VAR s_start : CHAR
    ) : INTEGER32; OPTIONS(C_PARAM, EXTERN);

    (* ... index (sys5 strchr)   *)

FUNCTION char_set_cs_index(
        cs_p : csm_c_char_set;
    VAR s_start : CHAR;
    VAR c_start : CHAR
    ) : PTR_TO_CHAR; OPTIONS(C_PARAM, EXTERN);

    (* ... rindex (sys5 strrchr) *)

FUNCTION char_set_cs_rindex(
        cs_p : csm_c_char_set;
    VAR s_start : CHAR;
    VAR c_start : CHAR
    ) : PTR_TO_CHAR; OPTIONS(C_PARAM, EXTERN);

    (* return type field *)

FUNCTION char_set_get_type(
        cs_p : csm_c_char_set
    ) : tml_c_cs_type; OPTIONS(C_PARAM, EXTERN);

    (* return string giving name of object *)

FUNCTION char_set_get_name(
        cs_p : csm_c_char_set
    ) : PTR_TO_CHAR; OPTIONS(C_PARAM, EXTERN);

    (* return structure giving information that coordinates with XConsortium Character Set Registry *)

FUNCTION char_set_get_xreg(
        cs_p : csm_c_char_set
    ) : csm_c_xregister; OPTIONS(C_PARAM, EXTERN);

    (* return bytes per character *)

FUNCTION char_set_get_char_size(
        cs_p : csm_c_char_set
    ) : INTEGER32; OPTIONS(C_PARAM, EXTERN);

    (* return number of character in set *)

FUNCTION char_set_get_num_chars(
        cs_p : csm_c_char_set
    ) : INTEGER32; OPTIONS(C_PARAM, EXTERN);

    (* return reference count on char_set *)

FUNCTION char_set_get_ref_count(
        cs_p : csm_c_char_set
    ) : INTEGER32; OPTIONS(C_PARAM, EXTERN);

    (* return predominate direction associated with char_set *)

FUNCTION char_set_get_predom_direc(
        cs_p : csm_c_char_set
    ) : csm_c_char_set_dir; OPTIONS(C_PARAM, EXTERN);

    (* return direction to next character *)

FUNCTION char_set_get_dir_to_char(
        cs_p : csm_c_char_set;
    VAR c_start : CHAR
    ) : INTEGER32; OPTIONS(C_PARAM, EXTERN);

    (* return direction to next line *)

FUNCTION char_set_get_dir_to_line(
        cs_p : csm_c_char_set
    ) : INTEGER32; OPTIONS(C_PARAM, EXTERN);

    (* return direction of next line *)

FUNCTION char_set_get_dir_of_line(
        cs_p : csm_c_char_set;
        dir : INTEGER32
    ) : INTEGER32; OPTIONS(C_PARAM, EXTERN);

    (* get next char_set on queue *)

FUNCTION char_set_get_next(
        cs_p : csm_c_char_set
    ) : csm_c_char_set; OPTIONS(C_PARAM, EXTERN);

    (* get pointer to collation map member *)

FUNCTION char_set_get_collate_map(
        cs_p : csm_c_char_set
    ) : csm_c_collation_map; OPTIONS(C_PARAM, EXTERN);

    (* get ranges of legal character values *)

FUNCTION char_set_get_legal_ranges(
        cs_p : csm_c_char_set
    ) : csm_c_char_range_set; OPTIONS(C_PARAM, EXTERN);

    (* get ranges of alphabetic characters *)

FUNCTION char_set_get_alpha_ranges(
        cs_p : csm_c_char_set
    ) : csm_c_char_range_set; OPTIONS(C_PARAM, EXTERN);

    (* get ranges of digit characters *)

FUNCTION char_set_get_digit_ranges(
        cs_p : csm_c_char_set
    ) : csm_c_char_range_set; OPTIONS(C_PARAM, EXTERN);

    (* get ranges of hexadecimal digit characters *)

FUNCTION char_set_get_xdigit_ranges(
        cs_p : csm_c_char_set
    ) : csm_c_char_range_set; OPTIONS(C_PARAM, EXTERN);

    (* get ranges of space characters *)

FUNCTION char_set_get_space_ranges(
        cs_p : csm_c_char_set
    ) : csm_c_char_range_set; OPTIONS(C_PARAM, EXTERN);

    (* get ranges of punctuation characters *)

FUNCTION char_set_get_punct_ranges(
        cs_p : csm_c_char_set
    ) : csm_c_char_range_set; OPTIONS(C_PARAM, EXTERN);

    (* get ranges of printable characters *)

FUNCTION char_set_get_print_ranges(
        cs_p : csm_c_char_set
    ) : csm_c_char_range_set; OPTIONS(C_PARAM, EXTERN);

    (* get ranges of graphical characters *)

FUNCTION char_set_get_graph_ranges(
        cs_p : csm_c_char_set
    ) : csm_c_char_range_set; OPTIONS(C_PARAM, EXTERN);

    (* get ranges of control characters *)

FUNCTION char_set_get_cntrl_ranges(
        cs_p : csm_c_char_set
    ) : csm_c_char_range_set; OPTIONS(C_PARAM, EXTERN);

    (* get terminator for this character set *)

PROCEDURE char_set_get_terminator(
        cs_p : csm_c_char_set;
    VAR c_start : CHAR
    ); OPTIONS(C_PARAM, EXTERN);

    (* return whether terminator is all zero bytes *)

FUNCTION char_set_zero_term(
        cs_p : csm_c_char_set
    ) : INTEGER32; OPTIONS(C_PARAM, EXTERN);

    (* return whether any characters use the high bit *)

FUNCTION char_set_hibit_is_clear(
        cs_p : csm_c_char_set
    ) : INTEGER32; OPTIONS(C_PARAM, EXTERN);

    (* set the type field *)

PROCEDURE char_set_set_type(
        cs_p : csm_c_char_set;
        new_cs_type : tml_c_cs_type
    ); OPTIONS(C_PARAM, EXTERN);

    (* set the name field *)

PROCEDURE char_set_set_name(
        cs_p : csm_c_char_set;
    VAR new_name_start : CHAR
    ); OPTIONS(C_PARAM, EXTERN);

    (* set the structure that coordinates with the XConsortium Character Set Registry *)

PROCEDURE char_set_set_xreg(
        cs_p : csm_c_char_set;
        new_xreg : csm_c_xregister
    ); OPTIONS(C_PARAM, EXTERN);

    (* set the number of bytes per character *)

PROCEDURE char_set_set_char_size(
        cs_p : csm_c_char_set;
        new_char_size : INTEGER32
    ); OPTIONS(C_PARAM, EXTERN);

    (* set the number of characters in the set *)

PROCEDURE char_set_set_num_chars(
        cs_p : csm_c_char_set;
        new_num_chars : INTEGER32
    ); OPTIONS(C_PARAM, EXTERN);

    (* set the reference counter *)

PROCEDURE char_set_set_ref_count(
        cs_p : csm_c_char_set;
        new_ref_count : INTEGER32
    ); OPTIONS(C_PARAM, EXTERN);

    (* increment the reference counter *)

PROCEDURE char_set_incr_ref_count(
        cs_p : csm_c_char_set
    ); OPTIONS(C_PARAM, EXTERN);

    (* decrement the reference counter *)

PROCEDURE char_set_decr_ref_count(
        cs_p : csm_c_char_set
    ); OPTIONS(C_PARAM, EXTERN);

    (* set the indicator of predominate rendering direction *)

PROCEDURE char_set_set_predom_direc(
        cs_p : csm_c_char_set;
        new_direc : csm_c_char_set_dir
    ); OPTIONS(C_PARAM, EXTERN);

    (* set the indicator of direction to next character *)

PROCEDURE char_set_set_char_direc(
        cs_p : csm_c_char_set;
        new_direc : INTEGER32
    ); OPTIONS(C_PARAM, EXTERN);

    (* set the pointer to next char_set in queue *)

PROCEDURE char_set_set_next(
        cs_p : csm_c_char_set;
        new_next : csm_c_char_set
    ); OPTIONS(C_PARAM, EXTERN);

    (* set the collation map *)

PROCEDURE char_set_set_map(
        cs_p : csm_c_char_set;
        new_map : csm_c_collation_map
    ); OPTIONS(C_PARAM, EXTERN);

    (* set the terminator *)

PROCEDURE char_set_set_null(
        cs_p : csm_c_char_set;
    VAR new_null_start : CHAR
    ); OPTIONS(C_PARAM, EXTERN);

    (* initialize escape characters to UNIX standard *)

PROCEDURE char_set_init_esc_chars(
        cs_p : csm_c_char_set;
    VAR esc_chars : PTR_TO_CHAR;
        spec_size : INTEGER32
    ); OPTIONS(C_PARAM, EXTERN);

    (* prepend a queue of char_set objects at the head of the queue on which this char_set resides *)

FUNCTION char_set_prepend(
        cs_p : csm_c_char_set;
        new_sets : csm_c_char_set
    ) : csm_c_char_set; OPTIONS(C_PARAM, EXTERN);

    (* append a queue of char_set objects at the tail of the queue on which this char_set resides *)

FUNCTION char_set_append(
        cs_p : csm_c_char_set;
        new_sets : csm_c_char_set
    ) : csm_c_char_set; OPTIONS(C_PARAM, EXTERN);

    (* detach an individual char_set object (first argument) from a queue of char_set objects *)

FUNCTION char_set_detach(
        cs_p : csm_c_char_set;
        from_vec : csm_c_char_set
    ) : csm_c_char_set; OPTIONS(C_PARAM, EXTERN);

    (* return nonzero if character at c is the terminator, zero otherwise *)

FUNCTION char_set_is_null(
        cs_p : csm_c_char_set;
    VAR c_start : CHAR
    ) : INTEGER32; OPTIONS(C_PARAM, EXTERN);

    (* return nonzero if both characters are terminators, zero otherwise *)

FUNCTION char_set_both_null(
        cs_p : csm_c_char_set;
    VAR c1_start : CHAR;
    VAR c2_start : CHAR
    ) : INTEGER32; OPTIONS(C_PARAM, EXTERN);

    (* for comparisons by COLLATING CHARACTERS of strings in this character set *)

    (* ... both strings must be null-terminated - like strcmp() *)

FUNCTION char_set_char_arr_cmp(
        cs_p : csm_c_char_set;
    VAR s1_start : CHAR;
    VAR s2_start : CHAR
    ) : INTEGER32; OPTIONS(C_PARAM, EXTERN);

    (* ... neither string need be null-terminated - like strncmp() *)

FUNCTION char_set_char_arr_ncmp(
        cs_p : csm_c_char_set;
    VAR s1_start : CHAR;
    VAR s2_start : CHAR;
        n : INTEGER32
    ) : INTEGER32; OPTIONS(C_PARAM, EXTERN);

    (* sort strings, ignoring given prefixes *)

PROCEDURE char_set_ignored_prefix_sort(
        cs_p : csm_c_char_set;
    VAR strings : PTR_TO_CHAR;
        num_strings : INTEGER32;
    VAR prefixes : PTR_TO_CHAR;
        num_prefixes : INTEGER32
    ); OPTIONS(C_PARAM, EXTERN);

    (* return the numeric value of a character in an long *)

FUNCTION csm_c_char_set_as_ulong(
        cs_p : csm_c_char_set;
    VAR s_start : CHAR
    ) : INTEGER32; OPTIONS(C_PARAM, EXTERN);

    (* advance the given character address to the beginning of the next character *)

PROCEDURE char_set_advance(
        cs_p : csm_c_char_set;
    VAR c : PTR_TO_CHAR
    ); OPTIONS(C_PARAM, EXTERN);

    (* decrement the given character address by the size of a character *)

PROCEDURE char_set_retreat(
        cs_p : csm_c_char_set;
    VAR c : PTR_TO_CHAR
    ); OPTIONS(C_PARAM, EXTERN);

    (* return whether character at c is instantiated in this set *)

FUNCTION char_set_is_char(
        cs_p : csm_c_char_set;
    VAR c_start : CHAR
    ) : INTEGER32; OPTIONS(C_PARAM, EXTERN);

    (* character type classification funtions like ... *)

    (* ... isalpha *)

FUNCTION char_set_cs_isalpha(
        cs_p : csm_c_char_set;
    VAR c_start : CHAR
    ) : INTEGER32; OPTIONS(C_PARAM, EXTERN);

    (* ... isdigit *)

FUNCTION char_set_cs_isdigit(
        cs_p : csm_c_char_set;
    VAR c_start : CHAR
    ) : INTEGER32; OPTIONS(C_PARAM, EXTERN);

    (* ... isxdigit *)

FUNCTION char_set_cs_isxdigit(
        cs_p : csm_c_char_set;
    VAR c_start : CHAR
    ) : INTEGER32; OPTIONS(C_PARAM, EXTERN);

    (* ... isalnum *)

FUNCTION char_set_cs_isalnum(
        cs_p : csm_c_char_set;
    VAR c_start : CHAR
    ) : INTEGER32; OPTIONS(C_PARAM, EXTERN);

    (* ... isspace *)

FUNCTION char_set_cs_isspace(
        cs_p : csm_c_char_set;
    VAR c_start : CHAR
    ) : INTEGER32; OPTIONS(C_PARAM, EXTERN);

    (* ... ispunct *)

FUNCTION char_set_cs_ispunct(
        cs_p : csm_c_char_set;
    VAR c_start : CHAR
    ) : INTEGER32; OPTIONS(C_PARAM, EXTERN);

    (* ... isprint *)

FUNCTION char_set_cs_isprint(
        cs_p : csm_c_char_set;
    VAR c_start : CHAR
    ) : INTEGER32; OPTIONS(C_PARAM, EXTERN);

    (* ... isgraph *)

FUNCTION char_set_cs_isgraph(
        cs_p : csm_c_char_set;
    VAR c_start : CHAR
    ) : INTEGER32; OPTIONS(C_PARAM, EXTERN);

    (* ... iscntrl *)

FUNCTION char_set_cs_iscntrl(
        cs_p : csm_c_char_set;
    VAR c_start : CHAR
    ) : INTEGER32; OPTIONS(C_PARAM, EXTERN);

    (* check for identity of character at c with C language escape character ... *)

    (* ... alert *)

FUNCTION char_set_is_alert(
        cs_p : csm_c_char_set;
    VAR c_start : CHAR
    ) : INTEGER32; OPTIONS(C_PARAM, EXTERN);

    (* ... backspace *)

FUNCTION char_set_is_backspace(
        cs_p : csm_c_char_set;
    VAR c_start : CHAR
    ) : INTEGER32; OPTIONS(C_PARAM, EXTERN);

    (* ... formfeed *)

FUNCTION char_set_is_formfeed(
        cs_p : csm_c_char_set;
    VAR c_start : CHAR
    ) : INTEGER32; OPTIONS(C_PARAM, EXTERN);

    (* ... newline *)

FUNCTION char_set_is_newline(
        cs_p : csm_c_char_set;
    VAR c_start : CHAR
    ) : INTEGER32; OPTIONS(C_PARAM, EXTERN);

    (* ... carriage return *)

FUNCTION char_set_is_cr(
        cs_p : csm_c_char_set;
    VAR c_start : CHAR
    ) : INTEGER32; OPTIONS(C_PARAM, EXTERN);

    (* ... horizontal tab *)

FUNCTION char_set_is_htab(
        cs_p : csm_c_char_set;
    VAR c_start : CHAR
    ) : INTEGER32; OPTIONS(C_PARAM, EXTERN);

    (* ... vertical tab *)

FUNCTION char_set_is_vtab(
        cs_p : csm_c_char_set;
    VAR c_start : CHAR
    ) : INTEGER32; OPTIONS(C_PARAM, EXTERN);

    (* ... backslash *)

FUNCTION char_set_is_backslash(
        cs_p : csm_c_char_set;
    VAR c_start : CHAR
    ) : INTEGER32; OPTIONS(C_PARAM, EXTERN);

    (* ... question mark (used in C Language triglyphs) *)

FUNCTION char_set_is_question(
        cs_p : csm_c_char_set;
    VAR c_start : CHAR
    ) : INTEGER32; OPTIONS(C_PARAM, EXTERN);

    (* ... single quote *)

FUNCTION char_set_is_squote(
        cs_p : csm_c_char_set;
    VAR c_start : CHAR
    ) : INTEGER32; OPTIONS(C_PARAM, EXTERN);

    (* ... double quote *)

FUNCTION char_set_is_dquote(
        cs_p : csm_c_char_set;
    VAR c_start : CHAR
    ) : INTEGER32; OPTIONS(C_PARAM, EXTERN);

    (* copy into the character at c the escape character ... *)

    (* ... alert *)

PROCEDURE char_set_get_alert(
        cs_p : csm_c_char_set;
    VAR c_start : CHAR
    ); OPTIONS(C_PARAM, EXTERN);

    (* ... backspace *)

PROCEDURE char_set_get_backspace(
        cs_p : csm_c_char_set;
    VAR c_start : CHAR
    ); OPTIONS(C_PARAM, EXTERN);

    (* ... formfeed *)

PROCEDURE char_set_get_formfeed(
        cs_p : csm_c_char_set;
    VAR c_start : CHAR
    ); OPTIONS(C_PARAM, EXTERN);

    (* ... newline *)

PROCEDURE char_set_get_newline(
        cs_p : csm_c_char_set;
    VAR c_start : CHAR
    ); OPTIONS(C_PARAM, EXTERN);

    (* ... carriage return *)

PROCEDURE char_set_get_cr(
        cs_p : csm_c_char_set;
    VAR c_start : CHAR
    ); OPTIONS(C_PARAM, EXTERN);

    (* ... horizontal tab *)

PROCEDURE char_set_get_htab(
        cs_p : csm_c_char_set;
    VAR c_start : CHAR
    ); OPTIONS(C_PARAM, EXTERN);

    (* ... vertical tab *)

PROCEDURE char_set_get_vtab(
        cs_p : csm_c_char_set;
    VAR c_start : CHAR
    ); OPTIONS(C_PARAM, EXTERN);

    (* ... backslash *)

PROCEDURE char_set_get_backslash(
        cs_p : csm_c_char_set;
    VAR c_start : CHAR
    ); OPTIONS(C_PARAM, EXTERN);

    (* ... question mark (used in C Language triglyphs) *)

PROCEDURE char_set_get_question(
        cs_p : csm_c_char_set;
    VAR c_start : CHAR
    ); OPTIONS(C_PARAM, EXTERN);

    (* ... single quote *)

PROCEDURE char_set_get_squote(
        cs_p : csm_c_char_set;
    VAR c_start : CHAR
    ); OPTIONS(C_PARAM, EXTERN);

    (* ... double quote *)

PROCEDURE char_set_get_dquote(
        cs_p : csm_c_char_set;
    VAR c_start : CHAR
    ); OPTIONS(C_PARAM, EXTERN);

    (* get origin of array of regular expression characters *)

FUNCTION char_set_get_regexp_chars(
        cs_p : csm_c_char_set
    ) : csm_c_csm_character; OPTIONS(C_PARAM, EXTERN);

    (* set origin of array of regular expression characters *)

PROCEDURE char_set_set_regexp_chars(
        cs_p : csm_c_char_set;
        new_re_chars : csm_c_csm_character
    ); OPTIONS(C_PARAM, EXTERN);


(* CLASS ALPHA_CHAR_SET *)

    (* allocate an alpha_char_set from free store *)

FUNCTION alpha_char_set_class_alloc(
        per_char_size : INTEGER32
    ) : csm_c_alpha_char_set; OPTIONS(C_PARAM, EXTERN);

    (* deallocate an alpha_char_set *)

PROCEDURE alpha_char_set_class_free(
        acs_p : csm_c_alpha_char_set
    ); OPTIONS(C_PARAM, EXTERN);

    (* return whether character at c is upper case *)

FUNCTION alpha_char_set_cs_isupper(
        acs_p : csm_c_alpha_char_set;
    VAR c_start : CHAR
    ) : INTEGER32; OPTIONS(C_PARAM, EXTERN);

    (* return whether character at c is lower case *)

FUNCTION alpha_char_set_cs_islower(
        acs_p : csm_c_alpha_char_set;
    VAR c_start : CHAR
    ) : INTEGER32; OPTIONS(C_PARAM, EXTERN);

    (* return lower case character that corresponds to upper case *)
    (*   character at c. Undefined if c is not upper case. *)

FUNCTION alpha_char_set_cs_tolower(
        acs_p : csm_c_alpha_char_set;
    VAR c_start : CHAR
    ) : PTR_TO_CHAR; OPTIONS(C_PARAM, EXTERN);

    (* return upper case character that corresponds to lower case *)
    (*   character at c. Undefined if c is not lower case. *)

FUNCTION alpha_char_set_cs_toupper(
        acs_p : csm_c_alpha_char_set;
    VAR c_start : CHAR
    ) : PTR_TO_CHAR; OPTIONS(C_PARAM, EXTERN);

    (* become a character set parallel to ISO8859-1 standard at this character size *)

PROCEDURE alpha_char_set_become_iso(
        acs_p : csm_c_alpha_char_set
    ); OPTIONS(C_PARAM, EXTERN);

    (* become a character set parallel to ASCII standard at this character size *)

PROCEDURE alpha_char_set_become_ascii(
        acs_p : csm_c_alpha_char_set
    ); OPTIONS(C_PARAM, EXTERN);


(* CLASS SYLL_FRAG_CHAR_SET *)

    (* class used to translate East-Indian and Korean *)
    (*   phonemic input to scripted syllables *)

    (* allocate a syll_frag_char_set from free store *)

FUNCTION syll_frag_char_set_class_alloc(
        per_char_size : INTEGER32
    ) : csm_c_syll_frag_char_set; OPTIONS(C_PARAM, EXTERN);

    (* deallocate a syll_frag_char_set *)

PROCEDURE syll_frag_char_set_free(
        sfcs_p : csm_c_syll_frag_char_set
    ); OPTIONS(C_PARAM, EXTERN);

    (* return nonzero if character at start is a legal multichar sequence, 0 otherwise *)

FUNCTION syll_frag_char_set_is_syll(
        sfcs_p : csm_c_syll_frag_char_set;
    VAR start_start : CHAR
    ) : INTEGER32; OPTIONS(C_PARAM, EXTERN);

    (* if the character at the address pointed to by start is a not valid syllable, *)
    (*   returns MAX_CHARS_IN_SET. Otherwise returns the appropriate syllable *)
    (*   identifier from the output character set and advances start to point to *)
    (*   the address of the beginning of the next syllable. *)

FUNCTION syll_frag_char_set_get_syll(
        sfcs_p : csm_c_syll_frag_char_set;
    VAR start : PTR_TO_CHAR
    ) : INTEGER32; OPTIONS(C_PARAM, EXTERN);

    (* returns length of syllable at start *)

FUNCTION syll_frag_char_set_get_syll_len(
        sfcs_p : csm_c_syll_frag_char_set;
    VAR start_start : CHAR
    ) : INTEGER32; OPTIONS(C_PARAM, EXTERN);


(* CLASS UNION_CHAR_SET *)

    (* allocate a union_char_set from free store *)

FUNCTION union_char_set_class_alloc_1 : csm_c_union_char_set; OPTIONS(C_PARAM, EXTERN);

    (* allocate a union_char_set initialized with the given char_set vector *)

FUNCTION union_char_set_class_alloc_2(
        new_char_sets : csm_c_char_set
    ) : csm_c_union_char_set; OPTIONS(C_PARAM, EXTERN);

    (* deallocate a union_char_set *)

PROCEDURE union_char_set_free(
        ucs_p : csm_c_union_char_set
    ); OPTIONS(C_PARAM, EXTERN);

    (* get character set vector *)

FUNCTION union_char_set_get_sets(
        ucs_p : csm_c_union_char_set
    ) : csm_c_char_set; OPTIONS(C_PARAM, EXTERN);

    (* get number of character sets in vector *)

FUNCTION union_char_set_get_num_sets(
        ucs_p : csm_c_union_char_set
    ) : INTEGER32; OPTIONS(C_PARAM, EXTERN);

    (* get next union_char_set on queue *)

FUNCTION union_char_set_get_next(
        ucs_p : csm_c_union_char_set
    ) : csm_c_union_char_set; OPTIONS(C_PARAM, EXTERN);

    (* get reference count *)

FUNCTION union_char_set_get_ref_count(
        ucs_p : csm_c_union_char_set
    ) : INTEGER32; OPTIONS(C_PARAM, EXTERN);

    (* set the vector of character sets *)

PROCEDURE union_char_set_set_sets(
        ucs_p : csm_c_union_char_set;
        new_sets : csm_c_char_set
    ); OPTIONS(C_PARAM, EXTERN);

    (* set the number of character sets in the vector *)

PROCEDURE union_char_set_set_num_sets(
        ucs_p : csm_c_union_char_set;
        new_num : INTEGER32
    ); OPTIONS(C_PARAM, EXTERN);

    (* set the next union_char_set in the queue *)

PROCEDURE union_char_set_set_next(
        ucs_p : csm_c_union_char_set;
        new_next : csm_c_union_char_set
    ); OPTIONS(C_PARAM, EXTERN);

    (* set the reference count *)

PROCEDURE union_char_set_set_ref_count(
        ucs_p : csm_c_union_char_set;
        new_ref_count : INTEGER32
    ); OPTIONS(C_PARAM, EXTERN);

    (* increment the reference count *)

PROCEDURE union_char_set_incr_ref_count(
        ucs_p : csm_c_union_char_set
    ); OPTIONS(C_PARAM, EXTERN);

    (* decrement the reference count *)

PROCEDURE union_char_set_decr_ref_count(
        ucs_p : csm_c_union_char_set
    ); OPTIONS(C_PARAM, EXTERN);

    (* prepend the vector of union_char_set objects at new_unions on this queue of union_char_sets *)

FUNCTION union_char_set_prepend(
        ucs_p : csm_c_union_char_set;
        new_unions : csm_c_union_char_set
    ) : csm_c_union_char_set; OPTIONS(C_PARAM, EXTERN);

    (* append the vector of union_char_set objects at new_unions to this queue of union_char_sets *)

FUNCTION union_char_set_append(
        ucs_p : csm_c_union_char_set;
        new_unions : csm_c_union_char_set
    ) : csm_c_union_char_set; OPTIONS(C_PARAM, EXTERN);

    (* detach this union_char_set from the vector beginning with from_vec *)

FUNCTION union_char_set_detach(
        ucs_p : csm_c_union_char_set;
        from_vec : csm_c_union_char_set
    ) : csm_c_union_char_set; OPTIONS(C_PARAM, EXTERN);

    (* prepend the vector of char_set objects at new_sets onto the vector of *)
    (*   char_set objects associated with this union_char_set *)

FUNCTION union_char_set_prepend_char_set(
        ucs_p : csm_c_union_char_set;
        new_sets : csm_c_char_set
    ) : csm_c_char_set; OPTIONS(C_PARAM, EXTERN);

    (* append the vector of char_set objects at new_sets to the vector of *)
    (*   char_set objects associated with this union_char_set *)

FUNCTION union_char_set_append_char_set(
        ucs_p : csm_c_union_char_set;
        new_sets : csm_c_char_set
    ) : csm_c_char_set; OPTIONS(C_PARAM, EXTERN);

    (* detach a char_set object from the vector of char_set objects associated with this union_char_set *)

FUNCTION union_char_set_detach_char_set(
        ucs_p : csm_c_union_char_set;
        old_set : csm_c_char_set
    ) : csm_c_char_set; OPTIONS(C_PARAM, EXTERN);

    (* return the first char_set on the vector that includes the character at c of size char_size *)

FUNCTION union_char_set_contains_any(
        ucs_p : csm_c_union_char_set;
    VAR c_start : CHAR;
        char_size : INTEGER32
    ) : csm_c_char_set; OPTIONS(C_PARAM, EXTERN);

    (* return whether the char_set at vector position char_set_index contains the character at c *)

FUNCTION union_char_set_contains_ask(
        ucs_p : csm_c_union_char_set;
        char_set_index : INTEGER32;
    VAR c_start : CHAR
    ) : INTEGER32; OPTIONS(C_PARAM, EXTERN);

    (* return nonzero if this is a default union_char_set (ISO8859-1) with no "next" *)

FUNCTION union_char_set_default_only(
        ucs_p : csm_c_union_char_set
    ) : INTEGER32; OPTIONS(C_PARAM, EXTERN);

    (* save the CSM objects associated with the vector of union_char_set objects at ucs_p *)
    (*   in human-readable form to file indicated by file handle *)

FUNCTION union_char_set_write_ucs_config_file(
        ucs_p : csm_c_union_char_set;
        file_handle : UNIV_PTR
    ) : INTEGER32; OPTIONS(C_PARAM, EXTERN);

    (* prepare the CSM objects associated with the vector of union_char_set objects at ucs_p *)
    (*   for writing to file in binary format *)

TYPE PTR_TO_csm_c_cs_heap      = ^csm_c_cs_heap;

TYPE PTR_TO_tml_c_gc_hash_tbl  = ^tml_c_gc_hash_tbl;

TYPE PTR_TO_tml_c_tml_ext_type = ^tml_c_tml_ext_type;

PROCEDURE union_char_set_prepare_ucs_binary(
        ucs_p : csm_c_union_char_set;
    VAR hash_tbl : PTR_TO_tml_c_gc_hash_tbl;
    VAR heaps : PTR_TO_csm_c_cs_heap;
    VAR types : PTR_TO_tml_c_tml_ext_type
    ); OPTIONS(C_PARAM, EXTERN);

    (* save the CSM objects associated with the vector of union_char_set objects at ucs_p *)
    (*   in binary form to file indicated by file_handle *)

FUNCTION union_char_set_write_ucs_binary_file(
        ucs_p : csm_c_union_char_set;
        file_handle : UNIV_PTR;
    VAR hash_tbl : PTR_TO_tml_c_gc_hash_tbl;
    VAR heaps : PTR_TO_csm_c_cs_heap;
    VAR types : PTR_TO_tml_c_tml_ext_type
    ) : INTEGER32; OPTIONS(C_PARAM, EXTERN);

(* CLASS CS_HEAP *)

    (* allocate a cs_heap from free store. The argument is an application-dependent structure *)

FUNCTION cs_heap_class_alloc(
        my_endian : tml_c_endian
    ) : csm_c_cs_heap; OPTIONS(C_PARAM, EXTERN);
PROCEDURE cs_heap_free(
        cs_heap_p : csm_c_cs_heap
    ); OPTIONS(C_PARAM, EXTERN);

(*************************************************************************************
    Declarations of general functions to read character sets recorded
      in binary and human-readable format respectively
*************************************************************************************)

    (* create union_char_sets from binary file indicated by file handle *)

TYPE PTR_TO_tml_c_endian       = ^tml_c_endian;

FUNCTION csm_c_read_union_char_sets(
        file_handle : UNIV_PTR;
    VAR hash_tbl : PTR_TO_tml_c_gc_hash_tbl;
    VAR types : PTR_TO_tml_c_tml_ext_type;
    VAR their_endian : PTR_TO_tml_c_endian
    ) : csm_c_union_char_set; OPTIONS(C_PARAM, EXTERN);

    (* create union_char_sets from human-readable file indicated by file handle *)

FUNCTION csm_c_read_config_file_union_char_sets(
        file_handle : UNIV_PTR
    ) : csm_c_union_char_set; OPTIONS(C_PARAM, EXTERN);

(*************************************************************************************
    Declarations of general functions for comparison of strings
      in two different character sets whose collation maps have been
      coordinated
*************************************************************************************)

    (* compare strings from two different character sets, each of which is *)
    (*   terminated with respect to its own character set. The comparison is *)
    (*   made with respect to the collation values associated with the *)
    (*   characters in their respective char_set objects. *)

FUNCTION csm_c_char_arr_cmp(
    VAR c1 : csm_c_csm_character;
    VAR c2 : csm_c_csm_character
    ) : INTEGER32; OPTIONS(C_PARAM, EXTERN);

    (* compare up to n characters of strings from two different character sets, *)
    (*   neither of which need be terminated. The comparison is *)
    (*   made with respect to the collation values associated with the *)
    (*   characters in their respective char_set objects. *)

FUNCTION csm_c_char_arr_ncmp(
    VAR c1 : csm_c_csm_character;
    VAR c2 : csm_c_csm_character;
        n : INTEGER32
    ) : INTEGER32; OPTIONS(C_PARAM, EXTERN);

(*************************************************************************************
    Declarations of some general functions for sorting
*************************************************************************************)

    (* set or reset a character set pointer to be used for csm_cmp_for_qsort() *)

PROCEDURE csm_c_set_csm_char_set_for_qsort(
        new_char_set : csm_c_char_set
    ); OPTIONS(C_PARAM, EXTERN);

    (* comparison function for use with UNIX qsort() when character sets are involved *)

FUNCTION csm_c_csm_cmp_for_qsort(
    VAR sp1 : PTR_TO_CHAR;
    VAR sp2 : PTR_TO_CHAR
    ) : INTEGER32; OPTIONS(C_PARAM, EXTERN);

(**********************************************************************************
    Objects used to implement Knuth-Morris-Pratt algorithm
**********************************************************************************)

(* CLASS KMP_NODE *)
    
    (* no constructor is supplied - use csm_c_kmp_fsm() *)

    (* deallocate the tree rooted at kmp_node_p *)

PROCEDURE kmp_node_free(
        kmp_node_p : csm_c_kmp_node
    ); OPTIONS(C_PARAM, EXTERN);

    (* if the string at sought matches the kmp_node, return its identifier *)
    (*   and advance sought beyond end of match; *)
    (* otherwise return MAX_CHARS_IN_SET leaving sought undefined *)

FUNCTION kmp_node_lookup(
        kmp_node_p : csm_c_kmp_node;
    VAR sought : PTR_TO_CHAR
    ) : INTEGER32; OPTIONS(C_PARAM, EXTERN);

    (* incremental lookup -- call this function repeatedly with a new character at s. Initial call takes *)
    (*   these argument values: "kmp_node_p" and "next_node" should be root of a KMP FSM, and "s", a pointer to a *)
    (*   (multibyte) character. On successive calls supply at "s" the next character to be considered. *)
    (*   The "num_chars" argument need not be initialized, but logically should be initialized to zero. *)
    (*   Function returns MAX_CHARS_IN_SET unless a successful string match is completed. On a successful *)
    (*   string match, the function returns the value associated with the successful match, sets "next_node" *)
    (*   to NULL if no longer match is possible, and returns in "num_chars" the number of characters *)
    (*   matched. If "next_node" is non-NULL, a match is still possible. The shortest possible match is *)
    (*   reported, but if a longer match is possible, "next_node" will be non-NULL. Note that successive *)
    (*   calls may yield a match, but that the length matched may be less than the number of characters *)
    (*   examined. For example, a KMP FSM that was designed to recognize "asdf" would yield a successful *)
    (*   match on "asdasdf", but only the last four characters would constitute the match. *)

TYPE PTR_TO_csm_c_kmp_node     = ^csm_c_kmp_node;

FUNCTION kmp_node_incr_lookup(
        kmp_node_p : csm_c_kmp_node;
    VAR next_node : PTR_TO_csm_c_kmp_node;
    VAR num_chars : INTEGER32;
    VAR s_start : CHAR
    ) : INTEGER32; OPTIONS(C_PARAM, EXTERN);


(*************************************************************************************
    FUNCTIONS TO OBTAIN OBJECTS FOR STRING SEARCH
*************************************************************************************)

    (* return a string search object *)

FUNCTION csm_c_kmp_fsm(
    VAR strings : PTR_TO_CHAR;
        num_strings : INTEGER32;
    VAR collations : INTEGER32;
        for_char_set : csm_c_char_set
    ) : csm_c_kmp_node; OPTIONS(C_PARAM, EXTERN);

(*************************************************************************************
    FUNCTIONS TO SUPPORT ARCHITECTURE NEUTRALITY
*************************************************************************************)

    (* return an endian object appropriate for the given magic number *)

FUNCTION csm_c_get_their_endian(
        their_magic_number : INTEGER32
    ) : tml_c_endian; OPTIONS(C_PARAM, EXTERN);

    (* return an endian object appropriate for the host machine *)
 
FUNCTION csm_c_get_my_endian : tml_c_endian; OPTIONS(C_PARAM, EXTERN);

