/*************************************************************************************
    csm_c_types.h
    header file including C language interface for the object definitions and
      assorted type declarations for the Character Set Manager (CSM)

    (c) Copyright Apollo Computer Inc., 1990 (unpublished)
     --------------------------------------------------------------------
    |THE FOLLOWING PROGRAMS ARE THE SOLE PROPERTY OF APOLLO COMPUTER INC.|
    |      AND CONTAIN ITS PROPRIETARY AND CONFIDENTIAL INFORMATION.     |
     --------------------------------------------------------------------
**************************************************************************************/

#ifndef csm_c_types_h
#define csm_c_types_h

#ifdef __STDC__

/* FROM HERE TO "#else", declarations for ANSI C programmers                 */
/**********************************************************************************
    typedefs for C++ classes

**********************************************************************************/

typedef void *csm_c_csm_character;
typedef void *csm_c_char_range;
typedef void *csm_c_char_range_set;
typedef void *csm_c_collation_map;
typedef void *csm_c_char_set;
typedef void *csm_c_alpha_char_set;
typedef void *csm_c_syll_frag_char_set;
typedef void *csm_c_union_char_set;
typedef void *csm_c_char_set_heap;
typedef void *csm_c_cs_heap;
typedef void *csm_c_csm_string;
typedef void *csm_c_kmp_node;
typedef void *csm_c_xregister;
typedef void *csm_c_char_func_tbl_entry;
typedef void *csm_c_char_func_tbl;
typedef void *tml_c_gc_hash_tbl;
typedef void *tml_c_tml_ext_type;
typedef void *tml_c_endian;

/******************************************************************************
   typedefs and instances for extensibility

******************************************************************************/
/**********************************************************************************
    Preloaded character set types
**********************************************************************************/

typedef void* tml_c_tml_type;
typedef void* tml_c_cs_type;

extern tml_c_cs_type* tml_char_set;
extern tml_c_cs_type* tml_alpha_char_set;
extern tml_c_cs_type* tml_syll_frag_char_set;

/******************************************************************************
    constants used in the classes

******************************************************************************/

#define MAX_BYTES_IN_CHAR  4294967295
#define MAX_CHARS_IN_SET   4294967295


/******************************************************************************
    Enumerated types

******************************************************************************/

/**********************************************************************************
    Enumerated and defined constants
**********************************************************************************/

/* ENUMERATED CLASS OF DIRECTIONS FOR TEXT RENDERING */

/* c?_l?_?== <direction_to_next_character>_  ? -> { r==right, l==left, u==up, d==down };            */
/*           <direction_to_next_line>_       ? -> { r==right, l==left, u==up, d==down };            */
/*           <direction_of_next_line>        ? -> { i==identical, b==bustrophedon (alternating) };  */
/* In bustrophedon the c? determines direction of first line                                        */

typedef long csm_c_char_set_dir;

#define csd_undefined -1
#define cl_ll_i        0
#define cl_lr_i        1
#define cl_lu_i        2
#define cl_ld_i        3
#define cl_ll_b        4
#define cl_lr_b        5
#define cl_lu_b        6
#define cl_ld_b        7
#define cr_ll_i        8
#define cr_lr_i        9
#define cr_lu_i       10
#define cr_ld_i       11
#define cr_ll_b       12
#define cr_lr_b       13
#define cr_lu_b       14
#define cr_ld_b       15
#define cu_ll_i       16
#define cu_lr_i       17
#define cu_lu_i       18
#define cu_ld_i       19
#define cu_ll_b       20
#define cu_lr_b       21
#define cu_lu_b       22
#define cu_ld_b       23
#define cd_ll_i       24
#define cd_lr_i       25
#define cd_lu_i       26
#define cd_ld_i       27
#define cd_ll_b       28
#define cd_lr_b       29
#define cd_lu_b       30
#define cd_ld_b       31

/* ENUMERATED CLASS OF SIMPLE DIRECTIONS */

/* enum lrud_dir      { lrud_undefined = -1,     */
/*                      left, right, up, down }; */
#define lrud_left   0
#define lrud_right  1
#define lrud_up     2
#define lrud_down   3

/* ENUMERATED CLASS OF C LANGUAGE ESCAPE CHARACTERS */
/* enum esc_char {esc_alert,        esc_backspace,       esc_formfeed,       */
/*                esc_newline,      esc_carriage_return, esc_horizontal_tab, */
/*                esc_vertical_tab, esc_backslash,       esc_question_mark,  */
/*                esc_single_quote, esc_double_quote,    num_esc_chars};     */
#define esc_alert                           0
#define esc_backspace                       1
#define esc_formfeedesc_newline             2
#define esc_carriage_return                 3
#define esc_horizontal_tabesc_vertical_tab  4
#define esc_backslash                       5
#define esc_question_markesc_single_quote   6
#define esc_double_quote                    7
#define num_esc_chars                       8

/* ENUMERATED CLASS OF ALTERNATIVE DEFINITIONS FOR POSIX BASIC                     */
/*   AND EXTENDED REGULAR EXPRESSION CHARACTERS                                    */
/* enum re_char {re_asterisk,         re_backslash,     re_circumflex,       */
/*               re_close_brace,      re_close_bracket, re_close_parenthesis,*/
/*               re_colon,            re_comma,         re_dollar_sign,      */
/*               re_eight,            re_equals_sign,   re_five,             */
/*               re_four,             re_hyphen,        re_nine,             */
/*               re_one,              re_open_brace,    re_open_bracket,     */
/*               re_open_parenthesis, re_period,        re_plus_sign,        */
/*               re_question_mark,    re_seven,         re_six,              */
/*               re_three,            re_two,           re_vertical_line,    */
/*               re_zero,                                                    */
/*               num_re_chars};                                              */
#define re_asterisk           0
#define re_backslash          1
#define re_circumflex         2
#define re_close_brace        3
#define re_close_bracket      4
#define re_close_parenthesis  5
#define re_colon              6
#define re_comma              7
#define re_dollar_sign        8
#define re_eight              9
#define re_equals_sign       10
#define re_five              11
#define re_four              12
#define re_hyphen            13
#define re_nine              14
#define re_one               15
#define re_open_brace        16
#define re_open_bracket      17
#define re_open_parenthesis  18
#define re_period            19
#define re_plus_sign         20
#define re_question_mark     21
#define re_seven             22
#define re_six               23
#define re_three             24
#define re_two               25
#define re_vertical_line     26
#define re_zero              27
#define num_re_chars         28

/**************************************************************************************
    Magic numbers for 32-bit architectures
    The CSM considers its host machine TML_MAGIC_3210
***************************************************************************************/

#define TML_MAGIC_0123  0x0A0F080E      /* low order  byte on left;  low order  word on left */
#define TML_MAGIC_1032  0x0F0A0E08      /* high order byte on left;  low order  word on left */
#define TML_MAGIC_2301  0x080E0A0F      /* low order  byte on left;  high order word on left */
#define TML_MAGIC_3210  0x0E080F0A      /* high order byte on left;  high order word on left */

/******************************************************************************
    Structures used for general purposes with allocator and
    deallocator for each where necessary
******************************************************************************/

/******************************************************************************
    CSM Objects

******************************************************************************/

/* CLASS CSM_CHARACTER */

    /* allocate a csm_character from free store */

extern csm_c_csm_character csm_character_class_alloc_1(void);

    /* ... initialized for a given character set */

extern csm_c_csm_character csm_character_class_alloc_2(csm_c_char_set for_char_set);

    /* ... and a given terminated string value */

extern csm_c_csm_character csm_character_class_alloc_3(unsigned char* val, csm_c_char_set for_char_set);

    /* ... or a nonterminated string value */

extern csm_c_csm_character csm_character_class_alloc_4(unsigned char* val, unsigned long len, csm_c_char_set for_char_set);

    /* ... or a nonterminated value and length in CHARACTERS */

extern csm_c_csm_character csm_character_class_alloc_5(unsigned char* val, unsigned long len, unsigned long collation_order, csm_c_char_set for_char_set);

    /* deallocate a csm_character */

extern void                csm_character_free(csm_c_csm_character csm_char_p);

    /* initialize to argument values */

extern void                csm_character_init(csm_c_csm_character csm_char_p, unsigned char* val, unsigned long len, csm_c_char_set for_char_set, unsigned long index);

    /* return string value */

extern unsigned char*      csm_character_get_value(csm_c_csm_character csm_char_p);

    /* return string value length in CHARACTERS */

extern unsigned long       csm_character_get_num_chars(csm_c_csm_character csm_char_p);

    /* return a given byte of the string value */

extern unsigned char*      csm_character_get_byte_at(csm_c_csm_character csm_char_p, long byte_index);

    /* returns collation value */

extern unsigned long       csm_character_get_collation(csm_c_csm_character csm_char_p);

    /* return associated character set */

extern csm_c_char_set      csm_character_get_char_set(csm_c_csm_character csm_char_p);

    /* set collation value */

extern void                csm_character_set_collation(csm_c_csm_character csm_char_p, unsigned long new_order);


/* CLASS CHAR_RANGE */

    /* allocate a char_range from free store */

extern csm_c_char_range char_range_class_alloc_1(void);

    /* ... initialized with terminated strings for range endpoints */

extern csm_c_char_range    char_range_class_alloc_2(unsigned char* lo, unsigned char* hi, csm_c_char_set for_char_set, unsigned long lo_map, unsigned long hi_map);

    /* ... or nonterminated strings */

extern csm_c_char_range    char_range_class_alloc_3(unsigned char* lo, unsigned long lo_len, unsigned char* hi, unsigned long hi_len, csm_c_char_set for_char_set, unsigned long lo_map, unsigned long hi_map);

    /* deallocate a char_range */

extern void                char_range_free(csm_c_char_range range_p);

    /* return high csm_character in range */

extern csm_c_csm_character char_range_get_high(csm_c_char_range range_p);

    /* return low csm_character in range */

extern csm_c_csm_character char_range_get_low(csm_c_char_range range_p);

    /* return collation of high csm_character */

extern unsigned long       char_range_get_high_index(csm_c_char_range range_p);

    /* return collation of low csm_character */

extern unsigned long       char_range_get_low_index(csm_c_char_range range_p);

    /* return number of character in range */

extern unsigned long       char_range_get_size(csm_c_char_range range_p);

    /* return whether range contains a given (terminated) character */

extern long                char_range_contains_char(csm_c_char_range range_p, unsigned char* candidate);

    /* return whether range contains a given collation value */

extern long                char_range_contains_coll(csm_c_char_range range_p, unsigned long collation);

    /* return whether range contains a given (nonterminated) character */

extern long                char_range_contains_nt_char(csm_c_char_range range_p, unsigned char* candidate, unsigned long len);


/* CLASS CHAR_RANGE_SET */

    /* allocate a char_range_set from free store */

extern csm_c_char_range_set char_range_set_class_alloc(void);

    /* deallocate a char_range_set */

extern void                 char_range_set_free(csm_c_char_range_set range_set_p);

    /* return whether a range set contains a given (terminated) character */

extern long                 char_range_set_contain(csm_c_char_range range_p, unsigned char* candidate);

    /* return the ith range in the set */

extern csm_c_char_range     char_range_set_get_range(csm_c_char_range range_p, unsigned long i);

    /* return the array of ranges in the set */

extern csm_c_char_range     char_range_set_get_ranges(csm_c_char_range range_p);

    /* return the number of ranges in the set */

extern unsigned long        char_range_set_get_num_ranges(csm_c_char_range range_p);

    /* initialize the set with a range array of a given size */

extern void                 char_range_set_make_ranges(csm_c_char_range range_p, long new_num);

    /* initialize a given range in the set */

extern void                 char_range_set_init_range(csm_c_char_range range_p, unsigned long slot, unsigned char* lo, unsigned char* hi, csm_c_char_set for_char_set, unsigned long spec_size, unsigned long lo_map, unsigned long hi_map);

    /* initialize a given range in the set from a range argument */

extern void                 char_range_set_set_range_1(csm_c_char_range range_p, csm_c_char_range from_range, csm_c_char_set for_char_set, unsigned long slot);

    /* initialize a given range for the low and high terminated strings */

extern void                 char_range_set_set_range_2(csm_c_char_range range_p, unsigned char* lo, unsigned char* hi, csm_c_char_set for_char_set, unsigned long slot, unsigned long lo_map, unsigned long hi_map);

    /* initialize a given range for the low and high nonterminated strings */

extern void                 char_range_set_set_range_3(csm_c_char_range range_p, unsigned char* lo, unsigned long lo_len, unsigned char* hi, unsigned long hi_len, csm_c_char_set for_char_set, unsigned long slot, unsigned long lo_map, unsigned long hi_map);

    /* return whether the range contains a given character */

extern unsigned long        char_range_set_lookup_char(csm_c_char_range range_p, unsigned char* sought);

    /* return whether the range contains a given collation */

extern unsigned char*       char_range_set_lookup_coll(csm_c_char_range range_p, unsigned long collation, unsigned char* c);

    /* given a character in one range, return the identically indexed character in the other range */

extern unsigned char*       char_range_set_corresponding(csm_c_char_range range_p, csm_c_char_range_set other_set, unsigned char* c);


/* CLASS COLLATION_MAP */

    /* allocate a collation_map from free store */

extern csm_c_collation_map collation_map_class_alloc_1(void);

    /* ... initialized with the values of the given arguments */

extern csm_c_collation_map collation_map_class_alloc_2(csm_c_char_range sequences, unsigned long num_sequences, csm_c_char_set for_char_set, unsigned long start_at);

    /* deallocate a collation_map */

extern void                collation_map_free(csm_c_collation_map map_p);

    /* return the map from multibyte characters to collation values */

extern csm_c_kmp_node      collation_map_get_many_to_one(csm_c_collation_map map_p);

    /* initialize the member for equivalence class collation */
    /*   map must have been made with csm_c_collation_map_class_alloc_2 */

extern void                collation_map_make_secondaries(csm_c_collation_map map_p, csm_c_char_range sequences, unsigned long num_sequences);

    /* initialize the member for one-to-many collation */
    /*   map must have been made with csm_c_collation_map_class_alloc_2 */

extern void                collation_map_make_one_to_many(csm_c_collation_map map_p, csm_c_char_range sequences, unsigned long num_sequences);

    /* initialize the member for regular expression substitution */
    /*   map must have been made with csm_c_collation_map_class_alloc_2 */

extern void                collation_map_make_substitutes(csm_c_collation_map map_p, char** find, char** sub, unsigned long num_subs);

    /* return length of single- or multicharacter collation sequence beginning at sought;  */
    /*   advances sought; reports primary collation only */

extern unsigned long       collation_map_lookup(csm_c_collation_map map_p, unsigned char** sought);

    /* compare two terminated strings according to data in map */

extern long                collation_map_compare(csm_c_collation_map map_p, unsigned char* s1, unsigned char* s2);

    /* compare two nonterminated strings according to data in map */

extern long                collation_map_ncompare(csm_c_collation_map map_p, unsigned char* s1, unsigned char* s2, unsigned long n);


/* CLASS CHAR_SET */

    /* allocate a char_set from free store */

extern csm_c_char_set       char_set_class_alloc(unsigned long per_char_size);

    /* deallocate a char_set */

extern void                 char_set_free(csm_c_char_set cs_p);

    /* make another instance of the same type of character set object */

extern csm_c_char_set       char_set_make_another(csm_c_char_set cs_p, unsigned long c_size);

    /* return whether the character set uses a zero terminator, has one-byte characters, */
    /*   and does not use the high bit in any character */

extern unsigned long        char_set_is_std(csm_c_char_set cs_p);

    /* string utility functions like ... */

    /* ... strcat                */

extern unsigned char*       char_set_cs_strcat(csm_c_char_set cs_p, unsigned char* s1, unsigned char* s2);

    /* ... strncat               */

extern unsigned char*       char_set_cs_strncat(csm_c_char_set cs_p, unsigned char* s1, unsigned char* s2, long n);

    /* ... strcmp                */

extern long                 char_set_cs_strcmp(csm_c_char_set cs_p, unsigned char* s1, unsigned char* s2);

    /* ... strncmp               */

extern long                 char_set_cs_strncmp(csm_c_char_set cs_p, unsigned char* s1, unsigned char* s2, long n);

    /* ... strcpy                */

extern unsigned char*       char_set_cs_strcpy(csm_c_char_set cs_p, unsigned char* s1, unsigned char* s2);

    /* ... strncpy               */

extern unsigned char*       char_set_cs_strncpy(csm_c_char_set cs_p, unsigned char* s1, unsigned char* s2, long n);

    /* ... strlen                */

extern long                 char_set_cs_strlen(csm_c_char_set cs_p, unsigned char* s);

    /* ... index (sys5 strchr)   */

extern unsigned char*       char_set_cs_index(csm_c_char_set cs_p, unsigned char* s, unsigned char* c);

    /* ... rindex (sys5 strrchr) */

extern unsigned char*       char_set_cs_rindex(csm_c_char_set cs_p, unsigned char* s, unsigned char* c);

    /* return type field */

extern tml_c_cs_type        char_set_get_type(csm_c_char_set cs_p);

    /* return string giving name of object */

extern unsigned char*       char_set_get_name(csm_c_char_set cs_p);

    /* return structure giving information that coordinates with XConsortium Character Set Registry */

extern csm_c_xregister      char_set_get_xreg(csm_c_char_set cs_p);

    /* return bytes per character */

extern unsigned long        char_set_get_char_size(csm_c_char_set cs_p);

    /* return number of character in set */

extern unsigned long        char_set_get_num_chars(csm_c_char_set cs_p);

    /* return reference count on char_set */

extern unsigned long        char_set_get_ref_count(csm_c_char_set cs_p);

    /* return predominate direction associated with char_set */

extern csm_c_char_set_dir   char_set_get_predom_direc(csm_c_char_set cs_p);

    /* return direction to next character */

extern long                 char_set_get_dir_to_char(csm_c_char_set cs_p, unsigned char* c);

    /* return direction to next line */

extern long                 char_set_get_dir_to_line(csm_c_char_set cs_p);

    /* return direction of next line */

extern long                 char_set_get_dir_of_line(csm_c_char_set cs_p, long dir);

    /* get next char_set on queue */

extern csm_c_char_set       char_set_get_next(csm_c_char_set cs_p);

    /* get pointer to collation map member */

extern csm_c_collation_map  char_set_get_collate_map(csm_c_char_set cs_p);

    /* get ranges of legal character values */

extern csm_c_char_range_set char_set_get_legal_ranges(csm_c_char_set cs_p);

    /* get ranges of alphabetic characters */

extern csm_c_char_range_set char_set_get_alpha_ranges(csm_c_char_set cs_p);

    /* get ranges of digit characters */

extern csm_c_char_range_set char_set_get_digit_ranges(csm_c_char_set cs_p);

    /* get ranges of hexadecimal digit characters */

extern csm_c_char_range_set char_set_get_xdigit_ranges(csm_c_char_set cs_p);

    /* get ranges of space characters */

extern csm_c_char_range_set char_set_get_space_ranges(csm_c_char_set cs_p);

    /* get ranges of punctuation characters */

extern csm_c_char_range_set char_set_get_punct_ranges(csm_c_char_set cs_p);

    /* get ranges of printable characters */

extern csm_c_char_range_set char_set_get_print_ranges(csm_c_char_set cs_p);

    /* get ranges of graphical characters */

extern csm_c_char_range_set char_set_get_graph_ranges(csm_c_char_set cs_p);

    /* get ranges of control characters */

extern csm_c_char_range_set char_set_get_cntrl_ranges(csm_c_char_set cs_p);

    /* get terminator for this character set */

extern void                 char_set_get_terminator(csm_c_char_set cs_p, unsigned char* c);

    /* return whether terminator is all zero bytes */

extern unsigned long        char_set_zero_term(csm_c_char_set cs_p);

    /* return whether any characters use the high bit */

extern unsigned long        char_set_hibit_is_clear(csm_c_char_set cs_p);

    /* set the type field */

extern void                 char_set_set_type(csm_c_char_set cs_p, tml_c_cs_type new_cs_type);

    /* set the name field */

extern void                 char_set_set_name(csm_c_char_set cs_p, unsigned char* new_name);

    /* set the structure that coordinates with the XConsortium Character Set Registry */

extern void                 char_set_set_xreg(csm_c_char_set cs_p, csm_c_xregister new_xreg);

    /* set the number of bytes per character */

extern void                 char_set_set_char_size(csm_c_char_set cs_p, unsigned long new_char_size);

    /* set the number of characters in the set */

extern void                 char_set_set_num_chars(csm_c_char_set cs_p, unsigned long new_num_chars);

    /* set the reference counter */

extern void                 char_set_set_ref_count(csm_c_char_set cs_p, unsigned long new_ref_count);

    /* increment the reference counter */

extern void                 char_set_incr_ref_count(csm_c_char_set cs_p);

    /* decrement the reference counter */

extern void                 char_set_decr_ref_count(csm_c_char_set cs_p);

    /* set the indicator of predominate rendering direction */

extern void                 char_set_set_predom_direc(csm_c_char_set cs_p, csm_c_char_set_dir new_direc);

    /* set the indicator of direction to next character */

extern void                 char_set_set_char_direc(csm_c_char_set cs_p, long new_direc);

    /* set the pointer to next char_set in queue */

extern void                 char_set_set_next(csm_c_char_set cs_p, csm_c_char_set new_next);

    /* set the collation map */

extern void                 char_set_set_map(csm_c_char_set cs_p, csm_c_collation_map new_map);

    /* set the terminator */

extern void                 char_set_set_null(csm_c_char_set cs_p, unsigned char* new_null);

    /* initialize escape characters to UNIX standard */

extern void                 char_set_init_esc_chars(csm_c_char_set cs_p, unsigned char** esc_chars, long spec_size);

    /* prepend a queue of char_set objects at the head of the queue on which this char_set resides */

extern csm_c_char_set       char_set_prepend(csm_c_char_set cs_p, csm_c_char_set new_sets);

    /* append a queue of char_set objects at the tail of the queue on which this char_set resides */

extern csm_c_char_set       char_set_append(csm_c_char_set cs_p, csm_c_char_set new_sets);

    /* detach an individual char_set object (first argument) from a queue of char_set objects */

extern csm_c_char_set       char_set_detach(csm_c_char_set cs_p, csm_c_char_set from_vec);

    /* return nonzero if character at c is the terminator, zero otherwise */

extern long                 char_set_is_null(csm_c_char_set cs_p, unsigned char* c);

    /* return nonzero if both characters are terminators, zero otherwise */

extern long                 char_set_both_null(csm_c_char_set cs_p, unsigned char* c1, unsigned char* c2);

    /* for comparisons by COLLATING CHARACTERS of strings in this character set */

    /* ... both strings must be null-terminated - like strcmp() */

extern long                 char_set_char_arr_cmp(csm_c_char_set cs_p, unsigned char* s1, unsigned char* s2);

    /* ... neither string need be null-terminated - like strncmp() */

extern long                 char_set_char_arr_ncmp(csm_c_char_set cs_p, unsigned char* s1, unsigned char* s2, unsigned long n);

    /* sort strings, ignoring given prefixes */

extern void                 char_set_ignored_prefix_sort(csm_c_char_set cs_p, char** strings, unsigned long num_strings, char** prefixes, unsigned long num_prefixes);

    /* return the numeric value of a character in an unsigned long */

extern unsigned long  csm_c_char_set_as_ulong(csm_c_char_set cs_p, unsigned char* s);

    /* advance the given character address to the beginning of the next character */

extern void                 char_set_advance(csm_c_char_set cs_p, unsigned char** c);

    /* decrement the given character address by the size of a character */

extern void                 char_set_retreat(csm_c_char_set cs_p, unsigned char** c);

    /* return whether character at c is instantiated in this set */

extern long                 char_set_is_char(csm_c_char_set cs_p, unsigned char* c);

    /* character type classification funtions like ... */

    /* ... isalpha */

extern long                 char_set_cs_isalpha(csm_c_char_set cs_p, unsigned char* c);

    /* ... isdigit */

extern long                 char_set_cs_isdigit(csm_c_char_set cs_p, unsigned char* c);

    /* ... isxdigit */

extern long                 char_set_cs_isxdigit(csm_c_char_set cs_p, unsigned char* c);

    /* ... isalnum */

extern long                 char_set_cs_isalnum(csm_c_char_set cs_p, unsigned char* c);

    /* ... isspace */

extern long                 char_set_cs_isspace(csm_c_char_set cs_p, unsigned char* c);

    /* ... ispunct */

extern long                 char_set_cs_ispunct(csm_c_char_set cs_p, unsigned char* c);

    /* ... isprint */

extern long                 char_set_cs_isprint(csm_c_char_set cs_p, unsigned char* c);

    /* ... isgraph */

extern long                 char_set_cs_isgraph(csm_c_char_set cs_p, unsigned char* c);

    /* ... iscntrl */

extern long                 char_set_cs_iscntrl(csm_c_char_set cs_p, unsigned char* c);

    /* check for identity of character at c with C language escape character ... */

    /* ... alert */

extern long                 char_set_is_alert(csm_c_char_set cs_p, unsigned char* c);

    /* ... backspace */

extern long                 char_set_is_backspace(csm_c_char_set cs_p, unsigned char* c);

    /* ... formfeed */

extern long                 char_set_is_formfeed(csm_c_char_set cs_p, unsigned char* c);

    /* ... newline */

extern long                 char_set_is_newline(csm_c_char_set cs_p, unsigned char* c);

    /* ... carriage return */

extern long                 char_set_is_cr(csm_c_char_set cs_p, unsigned char* c);

    /* ... horizontal tab */

extern long                 char_set_is_htab(csm_c_char_set cs_p, unsigned char* c);

    /* ... vertical tab */

extern long                 char_set_is_vtab(csm_c_char_set cs_p, unsigned char* c);

    /* ... backslash */

extern long                 char_set_is_backslash(csm_c_char_set cs_p, unsigned char* c);

    /* ... question mark (used in C Language triglyphs) */

extern long                 char_set_is_question(csm_c_char_set cs_p, unsigned char* c);

    /* ... single quote */

extern long                 char_set_is_squote(csm_c_char_set cs_p, unsigned char* c);

    /* ... double quote */

extern long                 char_set_is_dquote(csm_c_char_set cs_p, unsigned char* c);

    /* copy into the character at c the escape character ... */

    /* ... alert */

extern void                 char_set_get_alert(csm_c_char_set cs_p, unsigned char* c);

    /* ... backspace */

extern void                 char_set_get_backspace(csm_c_char_set cs_p, unsigned char* c);

    /* ... formfeed */

extern void                 char_set_get_formfeed(csm_c_char_set cs_p, unsigned char* c);

    /* ... newline */

extern void                 char_set_get_newline(csm_c_char_set cs_p, unsigned char* c);

    /* ... carriage return */

extern void                 char_set_get_cr(csm_c_char_set cs_p, unsigned char* c);

    /* ... horizontal tab */

extern void                 char_set_get_htab(csm_c_char_set cs_p, unsigned char* c);

    /* ... vertical tab */

extern void                 char_set_get_vtab(csm_c_char_set cs_p, unsigned char* c);

    /* ... backslash */

extern void                 char_set_get_backslash(csm_c_char_set cs_p, unsigned char* c);

    /* ... question mark (used in C Language triglyphs) */

extern void                 char_set_get_question(csm_c_char_set cs_p, unsigned char* c);

    /* ... single quote */

extern void                 char_set_get_squote(csm_c_char_set cs_p, unsigned char* c);

    /* ... double quote */

extern void                 char_set_get_dquote(csm_c_char_set cs_p, unsigned char* c);

    /* get origin of array of regular expression characters */

extern csm_c_csm_character char_set_get_regexp_chars(csm_c_char_set cs_p);

    /* set origin of array of regular expression characters */

extern void                char_set_set_regexp_chars(csm_c_char_set cs_p, csm_c_csm_character new_re_chars);


/* CLASS ALPHA_CHAR_SET */

    /* allocate an alpha_char_set from free store */

extern csm_c_alpha_char_set alpha_char_set_class_alloc(unsigned long per_char_size);

    /* deallocate an alpha_char_set */

extern void                 alpha_char_set_class_free(csm_c_alpha_char_set acs_p);

    /* return whether character at c is upper case */

extern long           alpha_char_set_cs_isupper(csm_c_alpha_char_set acs_p, unsigned char* c);

    /* return whether character at c is lower case */

extern long           alpha_char_set_cs_islower(csm_c_alpha_char_set acs_p, unsigned char* c);

    /* return lower case character that corresponds to upper case */
    /*   character at c. Undefined if c is not upper case. */

extern unsigned char* alpha_char_set_cs_tolower(csm_c_alpha_char_set acs_p, unsigned char* c);

    /* return upper case character that corresponds to lower case */
    /*   character at c. Undefined if c is not lower case. */

extern unsigned char* alpha_char_set_cs_toupper(csm_c_alpha_char_set acs_p, unsigned char* c);

    /* become a character set parallel to ISO8859-1 standard at this character size */

extern void           alpha_char_set_become_iso(csm_c_alpha_char_set acs_p);    

    /* become a character set parallel to ASCII standard at this character size */

extern void           alpha_char_set_become_ascii(csm_c_alpha_char_set acs_p);


/* CLASS SYLL_FRAG_CHAR_SET */

    /* class used to translate East-Indian and Korean */
    /*   phonemic input to scripted syllables */

    /* allocate a syll_frag_char_set from free store */

extern csm_c_syll_frag_char_set syll_frag_char_set_class_alloc(unsigned long per_char_size);

    /* deallocate a syll_frag_char_set */

extern void                     syll_frag_char_set_free(csm_c_syll_frag_char_set sfcs_p);

    /* return nonzero if character at start is a legal multichar sequence, 0 otherwise */

extern unsigned long            syll_frag_char_set_is_syll(csm_c_syll_frag_char_set sfcs_p, unsigned char* start);

    /* if the character at the address pointed to by start is a not valid syllable, */
    /*   returns MAX_CHARS_IN_SET. Otherwise returns the appropriate syllable */
    /*   identifier from the output character set and advances start to point to */
    /*   the address of the beginning of the next syllable. */

extern unsigned long            syll_frag_char_set_get_syll(csm_c_syll_frag_char_set sfcs_p, unsigned char** start);

    /* returns length of syllable at start */

extern unsigned long            syll_frag_char_set_get_syll_len(csm_c_syll_frag_char_set sfcs_p, unsigned char* start);


/* CLASS UNION_CHAR_SET */

    /* allocate a union_char_set from free store */

extern csm_c_union_char_set union_char_set_class_alloc_1(void);

    /* allocate a union_char_set initialized with the given char_set vector */

extern csm_c_union_char_set union_char_set_class_alloc_2(csm_c_char_set new_char_sets);

    /* deallocate a union_char_set */

extern void                 union_char_set_free(csm_c_union_char_set ucs_p);

    /* get character set vector */

extern csm_c_char_set       union_char_set_get_sets(csm_c_union_char_set ucs_p);

    /* get number of character sets in vector */

extern unsigned long        union_char_set_get_num_sets(csm_c_union_char_set ucs_p);

    /* get next union_char_set on queue */

extern csm_c_union_char_set union_char_set_get_next(csm_c_union_char_set ucs_p);

    /* get reference count */

extern unsigned long        union_char_set_get_ref_count(csm_c_union_char_set ucs_p);

    /* set the vector of character sets */

extern void                 union_char_set_set_sets(csm_c_union_char_set ucs_p, csm_c_char_set new_sets);

    /* set the number of character sets in the vector */

extern void                 union_char_set_set_num_sets(csm_c_union_char_set ucs_p, unsigned long new_num);

    /* set the next union_char_set in the queue */

extern void                 union_char_set_set_next(csm_c_union_char_set ucs_p, csm_c_union_char_set new_next);

    /* set the reference count */

extern void                 union_char_set_set_ref_count(csm_c_union_char_set ucs_p, unsigned long new_ref_count);

    /* increment the reference count */

extern void                 union_char_set_incr_ref_count(csm_c_union_char_set ucs_p);

    /* decrement the reference count */

extern void                 union_char_set_decr_ref_count(csm_c_union_char_set ucs_p);

    /* prepend the vector of union_char_set objects at new_unions on this queue of union_char_sets */

extern csm_c_union_char_set union_char_set_prepend(csm_c_union_char_set ucs_p, csm_c_union_char_set new_unions);

    /* append the vector of union_char_set objects at new_unions to this queue of union_char_sets */

extern csm_c_union_char_set union_char_set_append(csm_c_union_char_set ucs_p, csm_c_union_char_set new_unions);

    /* detach this union_char_set from the vector beginning with from_vec */

extern csm_c_union_char_set union_char_set_detach(csm_c_union_char_set ucs_p, csm_c_union_char_set from_vec);

    /* prepend the vector of char_set objects at new_sets onto the vector of */
    /*   char_set objects associated with this union_char_set */

extern csm_c_char_set       union_char_set_prepend_char_set(csm_c_union_char_set ucs_p, csm_c_char_set new_sets);

    /* append the vector of char_set objects at new_sets to the vector of */
    /*   char_set objects associated with this union_char_set */

extern csm_c_char_set       union_char_set_append_char_set(csm_c_union_char_set ucs_p, csm_c_char_set new_sets);

    /* detach a char_set object from the vector of char_set objects associated with this union_char_set */

extern csm_c_char_set       union_char_set_detach_char_set(csm_c_union_char_set ucs_p, csm_c_char_set old_set);

    /* return the first char_set on the vector that includes the character at c of size char_size */

extern csm_c_char_set       union_char_set_contains_any(csm_c_union_char_set ucs_p, unsigned char* c, unsigned long char_size);

    /* return whether the char_set at vector position char_set_index contains the character at c */

extern long                 union_char_set_contains_ask(csm_c_union_char_set ucs_p, unsigned long char_set_index, unsigned char* c);

    /* return nonzero if this is a default union_char_set (ISO8859-1) with no "next" */

extern long                 union_char_set_default_only(csm_c_union_char_set ucs_p);

    /* save the CSM objects associated with the vector of union_char_set objects at ucs_p */
    /*   in human-readable form to file indicated by file handle */

extern long                 union_char_set_write_ucs_config_file(csm_c_union_char_set ucs_p, void *file_handle);

    /* prepare the CSM objects associated with the vector of union_char_set objects at ucs_p */
    /*   for writing to file in binary format */

extern void                 union_char_set_prepare_ucs_binary(csm_c_union_char_set ucs_p, tml_c_gc_hash_tbl** hash_tbl, csm_c_cs_heap** heaps, tml_c_tml_ext_type** types);

    /* save the CSM objects associated with the vector of union_char_set objects at ucs_p */
    /*   in binary form to file indicated by file_handle */

extern long                 union_char_set_write_ucs_binary_file(csm_c_union_char_set ucs_p, void *file_handle, tml_c_gc_hash_tbl** hash_tbl, csm_c_cs_heap** heaps, tml_c_tml_ext_type** types);

/* CLASS CS_HEAP */

    /* allocate a cs_heap from free store. The argument is an application-dependent structure */

extern csm_c_cs_heap cs_heap_class_alloc(tml_c_endian my_endian);
extern void          cs_heap_free(csm_c_cs_heap cs_heap_p);

/*************************************************************************************
    Declarations of general functions to read character sets recorded
      in binary and human-readable format respectively
*************************************************************************************/

    /* create union_char_sets from binary file indicated by file handle */

extern csm_c_union_char_set csm_c_read_union_char_sets(void *file_handle, tml_c_gc_hash_tbl** hash_tbl, tml_c_tml_ext_type** types, tml_c_endian** their_endian);

    /* create union_char_sets from human-readable file indicated by file handle */

extern csm_c_union_char_set csm_c_read_config_file_union_char_sets(void *file_handle);

/*************************************************************************************
    Declarations of general functions for comparison of strings
      in two different character sets whose collation maps have been
      coordinated
*************************************************************************************/

    /* compare strings from two different character sets, each of which is */
    /*   terminated with respect to its own character set. The comparison is */
    /*   made with respect to the collation values associated with the */
    /*   characters in their respective char_set objects. */

extern long csm_c_char_arr_cmp(csm_c_csm_character* c1, csm_c_csm_character* c2);

    /* compare up to n characters of strings from two different character sets, */
    /*   neither of which need be terminated. The comparison is */
    /*   made with respect to the collation values associated with the */
    /*   characters in their respective char_set objects. */

extern long csm_c_char_arr_ncmp(csm_c_csm_character* c1, csm_c_csm_character* c2, unsigned long n);

/*************************************************************************************
    Declarations of some general functions for sorting
*************************************************************************************/

    /* set or reset a character set pointer to be used for csm_cmp_for_qsort() */

extern void csm_c_set_csm_char_set_for_qsort(csm_c_char_set new_char_set);

    /* comparison function for use with UNIX qsort() when character sets are involved */

extern int csm_c_csm_cmp_for_qsort(char** sp1, char** sp2);

/**********************************************************************************
    Objects used to implement Knuth-Morris-Pratt algorithm
**********************************************************************************/

/* CLASS KMP_NODE */
    
    /* no constructor is supplied - use csm_c_kmp_fsm() */

    /* deallocate the tree rooted at kmp_node_p */

extern void           kmp_node_free(csm_c_kmp_node kmp_node_p);

    /* if the string at sought matches the kmp_node, return its identifier */
    /*   and advance sought beyond end of match; */
    /* otherwise return MAX_CHARS_IN_SET leaving sought undefined */

extern unsigned long  kmp_node_lookup(csm_c_kmp_node kmp_node_p, unsigned char** sought);

    /* incremental lookup -- call this function repeatedly with a new character at s. Initial call takes */
    /*   these argument values: "kmp_node_p" and "next_node" should be root of a KMP FSM, and "s", a pointer to a */
    /*   (multibyte) character. On successive calls supply at "s" the next character to be considered. */
    /*   The "num_chars" argument need not be initialized, but logically should be initialized to zero. */
    /*   Function returns MAX_CHARS_IN_SET unless a successful string match is completed. On a successful */
    /*   string match, the function returns the value associated with the successful match, sets "next_node" */
    /*   to NULL if no longer match is possible, and returns in "num_chars" the number of characters */
    /*   matched. If "next_node" is non-NULL, a match is still possible. The shortest possible match is */
    /*   reported, but if a longer match is possible, "next_node" will be non-NULL. Note that successive */
    /*   calls may yield a match, but that the length matched may be less than the number of characters */
    /*   examined. For example, a KMP FSM that was designed to recognize "asdf" would yield a successful */
    /*   match on "asdasdf", but only the last four characters would constitute the match. */

extern unsigned long  kmp_node_incr_lookup(csm_c_kmp_node kmp_node_p, csm_c_kmp_node** next_node, unsigned long* num_chars, unsigned char* s);


/*************************************************************************************
    FUNCTIONS TO OBTAIN OBJECTS FOR STRING SEARCH
*************************************************************************************/

    /* return a string search object */

extern csm_c_kmp_node csm_c_kmp_fsm(unsigned char** strings, unsigned long num_strings, unsigned long* collations, csm_c_char_set for_char_set);

/*************************************************************************************
    FUNCTIONS TO SUPPORT ARCHITECTURE NEUTRALITY
*************************************************************************************/

    /* return an endian object appropriate for the given magic number */

extern tml_c_endian csm_c_get_their_endian(long their_magic_number);

    /* return an endian object appropriate for the host machine */
 
extern tml_c_endian csm_c_get_my_endian(void);

#else

/* FROM HERE TO "#endif", declarations for programmers using vanilla C       */
/**********************************************************************************
    typedefs for C++ classes

**********************************************************************************/

typedef char *csm_c_csm_character;
typedef char *csm_c_char_range;
typedef char *csm_c_char_range_set;
typedef char *csm_c_collation_map;
typedef char *csm_c_char_set;
typedef char *csm_c_alpha_char_set;
typedef char *csm_c_syll_frag_char_set;
typedef char *csm_c_union_char_set;
typedef char *csm_c_char_set_heap;
typedef char *csm_c_cs_heap;
typedef char *csm_c_csm_string;
typedef char *csm_c_kmp_node;
typedef char *csm_c_xregister;
typedef char *csm_c_char_func_tbl_entry;
typedef char *csm_c_char_func_tbl;
typedef char *tml_c_gc_hash_tbl;
typedef char *tml_c_tml_ext_type;
typedef char *tml_c_endian;

/******************************************************************************
   typedefs and instances for extensibility

******************************************************************************/
/**********************************************************************************
    Preloaded character set types
**********************************************************************************/

typedef char *tml_c_tml_type;
typedef char *tml_c_cs_type;

extern tml_c_cs_type* tml_char_set;
extern tml_c_cs_type* tml_alpha_char_set;
extern tml_c_cs_type* tml_syll_frag_char_set;

/******************************************************************************
    constants used in the classes

******************************************************************************/

#define MAX_BYTES_IN_CHAR  4294967295
#define MAX_CHARS_IN_SET   4294967295


/******************************************************************************
    Enumerated types

******************************************************************************/

/**********************************************************************************
    Enumerated and defined constants
**********************************************************************************/

/* ENUMERATED CLASS OF DIRECTIONS FOR TEXT RENDERING */

/* c?_l?_?== <direction_to_next_character>_  ? -> { r==right, l==left, u==up, d==down };            */
/*           <direction_to_next_line>_       ? -> { r==right, l==left, u==up, d==down };            */
/*           <direction_of_next_line>        ? -> { i==identical, b==bustrophedon (alternating) };  */
/* In bustrophedon the c? determines direction of first line                                        */

typedef long csm_c_char_set_dir;

#define csd_undefined -1
#define cl_ll_i        0
#define cl_lr_i        1
#define cl_lu_i        2
#define cl_ld_i        3
#define cl_ll_b        4
#define cl_lr_b        5
#define cl_lu_b        6
#define cl_ld_b        7
#define cr_ll_i        8
#define cr_lr_i        9
#define cr_lu_i       10
#define cr_ld_i       11
#define cr_ll_b       12
#define cr_lr_b       13
#define cr_lu_b       14
#define cr_ld_b       15
#define cu_ll_i       16
#define cu_lr_i       17
#define cu_lu_i       18
#define cu_ld_i       19
#define cu_ll_b       20
#define cu_lr_b       21
#define cu_lu_b       22
#define cu_ld_b       23
#define cd_ll_i       24
#define cd_lr_i       25
#define cd_lu_i       26
#define cd_ld_i       27
#define cd_ll_b       28
#define cd_lr_b       29
#define cd_lu_b       30
#define cd_ld_b       31

/* ENUMERATED CLASS OF SIMPLE DIRECTIONS */

/* enum lrud_dir      { lrud_undefined = -1,     */
/*                      left, right, up, down }; */
#define lrud_left   0
#define lrud_right  1
#define lrud_up     2
#define lrud_down   3

/* ENUMERATED CLASS OF C LANGUAGE ESCAPE CHARACTERS */
/* enum esc_char {esc_alert,        esc_backspace,       esc_formfeed,       */
/*                esc_newline,      esc_carriage_return, esc_horizontal_tab, */
/*                esc_vertical_tab, esc_backslash,       esc_question_mark,  */
/*                esc_single_quote, esc_double_quote,    num_esc_chars};     */
#define esc_alert                           0
#define esc_backspace                       1
#define esc_formfeedesc_newline             2
#define esc_carriage_return                 3
#define esc_horizontal_tabesc_vertical_tab  4
#define esc_backslash                       5
#define esc_question_markesc_single_quote   6
#define esc_double_quote                    7
#define num_esc_chars                       8

/* ENUMERATED CLASS OF ALTERNATIVE DEFINITIONS FOR POSIX BASIC                     */
/*   AND EXTENDED REGULAR EXPRESSION CHARACTERS                                    */
/* enum re_char {re_asterisk,         re_backslash,     re_circumflex,       */
/*               re_close_brace,      re_close_bracket, re_close_parenthesis,*/
/*               re_colon,            re_comma,         re_dollar_sign,      */
/*               re_eight,            re_equals_sign,   re_five,             */
/*               re_four,             re_hyphen,        re_nine,             */
/*               re_one,              re_open_brace,    re_open_bracket,     */
/*               re_open_parenthesis, re_period,        re_plus_sign,        */
/*               re_question_mark,    re_seven,         re_six,              */
/*               re_three,            re_two,           re_vertical_line,    */
/*               re_zero,                                                    */
/*               num_re_chars};                                              */
#define re_asterisk           0
#define re_backslash          1
#define re_circumflex         2
#define re_close_brace        3
#define re_close_bracket      4
#define re_close_parenthesis  5
#define re_colon              6
#define re_comma              7
#define re_dollar_sign        8
#define re_eight              9
#define re_equals_sign       10
#define re_five              11
#define re_four              12
#define re_hyphen            13
#define re_nine              14
#define re_one               15
#define re_open_brace        16
#define re_open_bracket      17
#define re_open_parenthesis  18
#define re_period            19
#define re_plus_sign         20
#define re_question_mark     21
#define re_seven             22
#define re_six               23
#define re_three             24
#define re_two               25
#define re_vertical_line     26
#define re_zero              27
#define num_re_chars         28

/**************************************************************************************
    Magic numbers for 32-bit architectures
    The CSM considers its host machine TML_MAGIC_3210
***************************************************************************************/

#define TML_MAGIC_0123  0x0A0F080E      /* low order  byte on left;  low order  word on left */
#define TML_MAGIC_1032  0x0F0A0E08      /* high order byte on left;  low order  word on left */
#define TML_MAGIC_2301  0x080E0A0F      /* low order  byte on left;  high order word on left */
#define TML_MAGIC_3210  0x0E080F0A      /* high order byte on left;  high order word on left */

/******************************************************************************
    Structures used for general purposes with allocator and
    deallocator for each where necessary
******************************************************************************/

/******************************************************************************
    CSM Objects

******************************************************************************/

/* CLASS CSM_CHARACTER */

    /* allocate a csm_character from free store */
    /* ARGS:  */

extern csm_c_csm_character csm_character_class_alloc_1();

    /* ... initialized for a given character set */
    /* ARGS: csm_c_char_set for_char_set */

extern csm_c_csm_character csm_character_class_alloc_2();

    /* ... and a given terminated string value */
    /* ARGS: unsigned char* val, csm_c_char_set for_char_set */

extern csm_c_csm_character csm_character_class_alloc_3();

    /* ... or a nonterminated string value */
    /* ARGS: unsigned char* val, unsigned long len, csm_c_char_set for_char_set */

extern csm_c_csm_character csm_character_class_alloc_4();

    /* ... or a nonterminated value and length in CHARACTERS */
    /* ARGS: unsigned char* val, unsigned long len, unsigned long collation_order, csm_c_char_set for_char_set */

extern csm_c_csm_character csm_character_class_alloc_5();

    /* deallocate a csm_character */
    /* ARGS: csm_c_csm_character csm_char_p */

extern                 csm_character_free();

    /* initialize to argument values */
    /* ARGS: csm_c_csm_character csm_char_p, unsigned char* val, unsigned long len, csm_c_char_set for_char_set, unsigned long index */

extern                 csm_character_init();

    /* return string value */
    /* ARGS: csm_c_csm_character csm_char_p */

extern unsigned char*      csm_character_get_value();

    /* return string value length in CHARACTERS */
    /* ARGS: csm_c_csm_character csm_char_p */

extern unsigned long       csm_character_get_num_chars();

    /* return a given byte of the string value */
    /* ARGS: csm_c_csm_character csm_char_p, long byte_index */

extern unsigned char*      csm_character_get_byte_at();

    /* returns collation value */
    /* ARGS: csm_c_csm_character csm_char_p */

extern unsigned long       csm_character_get_collation();

    /* return associated character set */
    /* ARGS: csm_c_csm_character csm_char_p */

extern csm_c_char_set      csm_character_get_char_set();

    /* set collation value */
    /* ARGS: csm_c_csm_character csm_char_p, unsigned long new_order */

extern                 csm_character_set_collation();


/* CLASS CHAR_RANGE */

    /* allocate a char_range from free store */
    /* ARGS:  */

extern csm_c_char_range char_range_class_alloc_1();

    /* ... initialized with terminated strings for range endpoints */
    /* ARGS: unsigned char* lo, unsigned char* hi, csm_c_char_set for_char_set, unsigned long lo_map, unsigned long hi_map */

extern csm_c_char_range    char_range_class_alloc_2();

    /* ... or nonterminated strings */
    /* ARGS: unsigned char* lo, unsigned long lo_len, unsigned char* hi, unsigned long hi_len, csm_c_char_set for_char_set, unsigned long lo_map, unsigned long hi_map */

extern csm_c_char_range    char_range_class_alloc_3();

    /* deallocate a char_range */
    /* ARGS: csm_c_char_range range_p */

extern                 char_range_free();

    /* return high csm_character in range */
    /* ARGS: csm_c_char_range range_p */

extern csm_c_csm_character char_range_get_high();

    /* return low csm_character in range */
    /* ARGS: csm_c_char_range range_p */

extern csm_c_csm_character char_range_get_low();

    /* return collation of high csm_character */
    /* ARGS: csm_c_char_range range_p */

extern unsigned long       char_range_get_high_index();

    /* return collation of low csm_character */
    /* ARGS: csm_c_char_range range_p */

extern unsigned long       char_range_get_low_index();

    /* return number of character in range */
    /* ARGS: csm_c_char_range range_p */

extern unsigned long       char_range_get_size();

    /* return whether range contains a given (terminated) character */
    /* ARGS: csm_c_char_range range_p, unsigned char* candidate */

extern long                char_range_contains_char();

    /* return whether range contains a given collation value */
    /* ARGS: csm_c_char_range range_p, unsigned long collation */

extern long                char_range_contains_coll();

    /* return whether range contains a given (nonterminated) character */
    /* ARGS: csm_c_char_range range_p, unsigned char* candidate, unsigned long len */

extern long                char_range_contains_nt_char();


/* CLASS CHAR_RANGE_SET */

    /* allocate a char_range_set from free store */
    /* ARGS:  */

extern csm_c_char_range_set char_range_set_class_alloc();

    /* deallocate a char_range_set */
    /* ARGS: csm_c_char_range_set range_set_p */

extern                  char_range_set_free();

    /* return whether a range set contains a given (terminated) character */
    /* ARGS: csm_c_char_range range_p, unsigned char* candidate */

extern long                 char_range_set_contain();

    /* return the ith range in the set */
    /* ARGS: csm_c_char_range range_p, unsigned long i */

extern csm_c_char_range     char_range_set_get_range();

    /* return the array of ranges in the set */
    /* ARGS: csm_c_char_range range_p */

extern csm_c_char_range     char_range_set_get_ranges();

    /* return the number of ranges in the set */
    /* ARGS: csm_c_char_range range_p */

extern unsigned long        char_range_set_get_num_ranges();

    /* initialize the set with a range array of a given size */
    /* ARGS: csm_c_char_range range_p, long new_num */

extern                  char_range_set_make_ranges();

    /* initialize a given range in the set */
    /* ARGS: csm_c_char_range range_p, unsigned long slot, unsigned char* lo, unsigned char* hi, csm_c_char_set for_char_set, unsigned long spec_size, unsigned long lo_map, unsigned long hi_map */

extern                  char_range_set_init_range();

    /* initialize a given range in the set from a range argument */
    /* ARGS: csm_c_char_range range_p, csm_c_char_range from_range, csm_c_char_set for_char_set, unsigned long slot */

extern                  char_range_set_set_range_1();

    /* initialize a given range for the low and high terminated strings */
    /* ARGS: csm_c_char_range range_p, unsigned char* lo, unsigned char* hi, csm_c_char_set for_char_set, unsigned long slot, unsigned long lo_map, unsigned long hi_map */

extern                  char_range_set_set_range_2();

    /* initialize a given range for the low and high nonterminated strings */
    /* ARGS: csm_c_char_range range_p, unsigned char* lo, unsigned long lo_len, unsigned char* hi, unsigned long hi_len, csm_c_char_set for_char_set, unsigned long slot, unsigned long lo_map, unsigned long hi_map */

extern                  char_range_set_set_range_3();

    /* return whether the range contains a given character */
    /* ARGS: csm_c_char_range range_p, unsigned char* sought */

extern unsigned long        char_range_set_lookup_char();

    /* return whether the range contains a given collation */
    /* ARGS: csm_c_char_range range_p, unsigned long collation, unsigned char* c */

extern unsigned char*       char_range_set_lookup_coll();

    /* given a character in one range, return the identically indexed character in the other range */
    /* ARGS: csm_c_char_range range_p, csm_c_char_range_set other_set, unsigned char* c */

extern unsigned char*       char_range_set_corresponding();


/* CLASS COLLATION_MAP */

    /* allocate a collation_map from free store */
    /* ARGS:  */

extern csm_c_collation_map collation_map_class_alloc_1();

    /* ... initialized with the values of the given arguments */
    /* ARGS: csm_c_char_range sequences, unsigned long num_sequences, csm_c_char_set for_char_set, unsigned long start_at */

extern csm_c_collation_map collation_map_class_alloc_2();

    /* deallocate a collation_map */
    /* ARGS: csm_c_collation_map map_p */

extern                 collation_map_free();

    /* return the map from multibyte characters to collation values */
    /* ARGS: csm_c_collation_map map_p */

extern csm_c_kmp_node      collation_map_get_many_to_one();

    /* initialize the member for equivalence class collation */
    /*   map must have been made with csm_c_collation_map_class_alloc_2 */
    /* ARGS: csm_c_collation_map map_p, csm_c_char_range sequences, unsigned long num_sequences */

extern                 collation_map_make_secondaries();

    /* initialize the member for one-to-many collation */
    /*   map must have been made with csm_c_collation_map_class_alloc_2 */
    /* ARGS: csm_c_collation_map map_p, csm_c_char_range sequences, unsigned long num_sequences */

extern                 collation_map_make_one_to_many();

    /* initialize the member for regular expression substitution */
    /*   map must have been made with csm_c_collation_map_class_alloc_2 */
    /* ARGS: csm_c_collation_map map_p, char** find, char** sub, unsigned long num_subs */

extern                 collation_map_make_substitutes();

    /* return length of single- or multicharacter collation sequence beginning at sought;  */
    /*   advances sought; reports primary collation only */
    /* ARGS: csm_c_collation_map map_p, unsigned char** sought */

extern unsigned long       collation_map_lookup();

    /* compare two terminated strings according to data in map */
    /* ARGS: csm_c_collation_map map_p, unsigned char* s1, unsigned char* s2 */

extern long                collation_map_compare();

    /* compare two nonterminated strings according to data in map */
    /* ARGS: csm_c_collation_map map_p, unsigned char* s1, unsigned char* s2, unsigned long n */

extern long                collation_map_ncompare();


/* CLASS CHAR_SET */

    /* allocate a char_set from free store */
    /* ARGS: unsigned long per_char_size */

extern csm_c_char_set       char_set_class_alloc();

    /* deallocate a char_set */
    /* ARGS: csm_c_char_set cs_p */

extern                  char_set_free();

    /* make another instance of the same type of character set object */
    /* ARGS: csm_c_char_set cs_p, unsigned long c_size */

extern csm_c_char_set       char_set_make_another();

    /* return whether the character set uses a zero terminator, has one-byte characters, */
    /*   and does not use the high bit in any character */
    /* ARGS: csm_c_char_set cs_p */

extern unsigned long        char_set_is_std();

    /* string utility functions like ... */

    /* ... strcat                */
    /* ARGS: csm_c_char_set cs_p, unsigned char* s1, unsigned char* s2 */

extern unsigned char*       char_set_cs_strcat();

    /* ... strncat               */
    /* ARGS: csm_c_char_set cs_p, unsigned char* s1, unsigned char* s2, long n */

extern unsigned char*       char_set_cs_strncat();

    /* ... strcmp                */
    /* ARGS: csm_c_char_set cs_p, unsigned char* s1, unsigned char* s2 */

extern long                 char_set_cs_strcmp();

    /* ... strncmp               */
    /* ARGS: csm_c_char_set cs_p, unsigned char* s1, unsigned char* s2, long n */

extern long                 char_set_cs_strncmp();

    /* ... strcpy                */
    /* ARGS: csm_c_char_set cs_p, unsigned char* s1, unsigned char* s2 */

extern unsigned char*       char_set_cs_strcpy();

    /* ... strncpy               */
    /* ARGS: csm_c_char_set cs_p, unsigned char* s1, unsigned char* s2, long n */

extern unsigned char*       char_set_cs_strncpy();

    /* ... strlen                */
    /* ARGS: csm_c_char_set cs_p, unsigned char* s */

extern long                 char_set_cs_strlen();

    /* ... index (sys5 strchr)   */
    /* ARGS: csm_c_char_set cs_p, unsigned char* s, unsigned char* c */

extern unsigned char*       char_set_cs_index();

    /* ... rindex (sys5 strrchr) */
    /* ARGS: csm_c_char_set cs_p, unsigned char* s, unsigned char* c */

extern unsigned char*       char_set_cs_rindex();

    /* return type field */
    /* ARGS: csm_c_char_set cs_p */

extern tml_c_cs_type        char_set_get_type();

    /* return string giving name of object */
    /* ARGS: csm_c_char_set cs_p */

extern unsigned char*       char_set_get_name();

    /* return structure giving information that coordinates with XConsortium Character Set Registry */
    /* ARGS: csm_c_char_set cs_p */

extern csm_c_xregister      char_set_get_xreg();

    /* return bytes per character */
    /* ARGS: csm_c_char_set cs_p */

extern unsigned long        char_set_get_char_size();

    /* return number of character in set */
    /* ARGS: csm_c_char_set cs_p */

extern unsigned long        char_set_get_num_chars();

    /* return reference count on char_set */
    /* ARGS: csm_c_char_set cs_p */

extern unsigned long        char_set_get_ref_count();

    /* return predominate direction associated with char_set */
    /* ARGS: csm_c_char_set cs_p */

extern csm_c_char_set_dir   char_set_get_predom_direc();

    /* return direction to next character */
    /* ARGS: csm_c_char_set cs_p, unsigned char* c */

extern long                 char_set_get_dir_to_char();

    /* return direction to next line */
    /* ARGS: csm_c_char_set cs_p */

extern long                 char_set_get_dir_to_line();

    /* return direction of next line */
    /* ARGS: csm_c_char_set cs_p, long dir */

extern long                 char_set_get_dir_of_line();

    /* get next char_set on queue */
    /* ARGS: csm_c_char_set cs_p */

extern csm_c_char_set       char_set_get_next();

    /* get pointer to collation map member */
    /* ARGS: csm_c_char_set cs_p */

extern csm_c_collation_map  char_set_get_collate_map();

    /* get ranges of legal character values */
    /* ARGS: csm_c_char_set cs_p */

extern csm_c_char_range_set char_set_get_legal_ranges();

    /* get ranges of alphabetic characters */
    /* ARGS: csm_c_char_set cs_p */

extern csm_c_char_range_set char_set_get_alpha_ranges();

    /* get ranges of digit characters */
    /* ARGS: csm_c_char_set cs_p */

extern csm_c_char_range_set char_set_get_digit_ranges();

    /* get ranges of hexadecimal digit characters */
    /* ARGS: csm_c_char_set cs_p */

extern csm_c_char_range_set char_set_get_xdigit_ranges();

    /* get ranges of space characters */
    /* ARGS: csm_c_char_set cs_p */

extern csm_c_char_range_set char_set_get_space_ranges();

    /* get ranges of punctuation characters */
    /* ARGS: csm_c_char_set cs_p */

extern csm_c_char_range_set char_set_get_punct_ranges();

    /* get ranges of printable characters */
    /* ARGS: csm_c_char_set cs_p */

extern csm_c_char_range_set char_set_get_print_ranges();

    /* get ranges of graphical characters */
    /* ARGS: csm_c_char_set cs_p */

extern csm_c_char_range_set char_set_get_graph_ranges();

    /* get ranges of control characters */
    /* ARGS: csm_c_char_set cs_p */

extern csm_c_char_range_set char_set_get_cntrl_ranges();

    /* get terminator for this character set */
    /* ARGS: csm_c_char_set cs_p, unsigned char* c */

extern                  char_set_get_terminator();

    /* return whether terminator is all zero bytes */
    /* ARGS: csm_c_char_set cs_p */

extern unsigned long        char_set_zero_term();

    /* return whether any characters use the high bit */
    /* ARGS: csm_c_char_set cs_p */

extern unsigned long        char_set_hibit_is_clear();

    /* set the type field */
    /* ARGS: csm_c_char_set cs_p, tml_c_cs_type new_cs_type */

extern                  char_set_set_type();

    /* set the name field */
    /* ARGS: csm_c_char_set cs_p, unsigned char* new_name */

extern                  char_set_set_name();

    /* set the structure that coordinates with the XConsortium Character Set Registry */
    /* ARGS: csm_c_char_set cs_p, csm_c_xregister new_xreg */

extern                  char_set_set_xreg();

    /* set the number of bytes per character */
    /* ARGS: csm_c_char_set cs_p, unsigned long new_char_size */

extern                  char_set_set_char_size();

    /* set the number of characters in the set */
    /* ARGS: csm_c_char_set cs_p, unsigned long new_num_chars */

extern                  char_set_set_num_chars();

    /* set the reference counter */
    /* ARGS: csm_c_char_set cs_p, unsigned long new_ref_count */

extern                  char_set_set_ref_count();

    /* increment the reference counter */
    /* ARGS: csm_c_char_set cs_p */

extern                  char_set_incr_ref_count();

    /* decrement the reference counter */
    /* ARGS: csm_c_char_set cs_p */

extern                  char_set_decr_ref_count();

    /* set the indicator of predominate rendering direction */
    /* ARGS: csm_c_char_set cs_p, csm_c_char_set_dir new_direc */

extern                  char_set_set_predom_direc();

    /* set the indicator of direction to next character */
    /* ARGS: csm_c_char_set cs_p, long new_direc */

extern                  char_set_set_char_direc();

    /* set the pointer to next char_set in queue */
    /* ARGS: csm_c_char_set cs_p, csm_c_char_set new_next */

extern                  char_set_set_next();

    /* set the collation map */
    /* ARGS: csm_c_char_set cs_p, csm_c_collation_map new_map */

extern                  char_set_set_map();

    /* set the terminator */
    /* ARGS: csm_c_char_set cs_p, unsigned char* new_null */

extern                  char_set_set_null();

    /* initialize escape characters to UNIX standard */
    /* ARGS: csm_c_char_set cs_p, unsigned char** esc_chars, long spec_size */

extern                  char_set_init_esc_chars();

    /* prepend a queue of char_set objects at the head of the queue on which this char_set resides */
    /* ARGS: csm_c_char_set cs_p, csm_c_char_set new_sets */

extern csm_c_char_set       char_set_prepend();

    /* append a queue of char_set objects at the tail of the queue on which this char_set resides */
    /* ARGS: csm_c_char_set cs_p, csm_c_char_set new_sets */

extern csm_c_char_set       char_set_append();

    /* detach an individual char_set object (first argument) from a queue of char_set objects */
    /* ARGS: csm_c_char_set cs_p, csm_c_char_set from_vec */

extern csm_c_char_set       char_set_detach();

    /* return nonzero if character at c is the terminator, zero otherwise */
    /* ARGS: csm_c_char_set cs_p, unsigned char* c */

extern long                 char_set_is_null();

    /* return nonzero if both characters are terminators, zero otherwise */
    /* ARGS: csm_c_char_set cs_p, unsigned char* c1, unsigned char* c2 */

extern long                 char_set_both_null();

    /* for comparisons by COLLATING CHARACTERS of strings in this character set */

    /* ... both strings must be null-terminated - like strcmp() */
    /* ARGS: csm_c_char_set cs_p, unsigned char* s1, unsigned char* s2 */

extern long                 char_set_char_arr_cmp();

    /* ... neither string need be null-terminated - like strncmp() */
    /* ARGS: csm_c_char_set cs_p, unsigned char* s1, unsigned char* s2, unsigned long n */

extern long                 char_set_char_arr_ncmp();

    /* sort strings, ignoring given prefixes */
    /* ARGS: csm_c_char_set cs_p, char** strings, unsigned long num_strings, char** prefixes, unsigned long num_prefixes */

extern                  char_set_ignored_prefix_sort();

    /* return the numeric value of a character in an unsigned long */
    /* ARGS: csm_c_char_set cs_p, unsigned char* s */

extern unsigned long  csm_c_char_set_as_ulong();

    /* advance the given character address to the beginning of the next character */
    /* ARGS: csm_c_char_set cs_p, unsigned char** c */

extern                  char_set_advance();

    /* decrement the given character address by the size of a character */
    /* ARGS: csm_c_char_set cs_p, unsigned char** c */

extern                  char_set_retreat();

    /* return whether character at c is instantiated in this set */
    /* ARGS: csm_c_char_set cs_p, unsigned char* c */

extern long                 char_set_is_char();

    /* character type classification funtions like ... */

    /* ... isalpha */
    /* ARGS: csm_c_char_set cs_p, unsigned char* c */

extern long                 char_set_cs_isalpha();

    /* ... isdigit */
    /* ARGS: csm_c_char_set cs_p, unsigned char* c */

extern long                 char_set_cs_isdigit();

    /* ... isxdigit */
    /* ARGS: csm_c_char_set cs_p, unsigned char* c */

extern long                 char_set_cs_isxdigit();

    /* ... isalnum */
    /* ARGS: csm_c_char_set cs_p, unsigned char* c */

extern long                 char_set_cs_isalnum();

    /* ... isspace */
    /* ARGS: csm_c_char_set cs_p, unsigned char* c */

extern long                 char_set_cs_isspace();

    /* ... ispunct */
    /* ARGS: csm_c_char_set cs_p, unsigned char* c */

extern long                 char_set_cs_ispunct();

    /* ... isprint */
    /* ARGS: csm_c_char_set cs_p, unsigned char* c */

extern long                 char_set_cs_isprint();

    /* ... isgraph */
    /* ARGS: csm_c_char_set cs_p, unsigned char* c */

extern long                 char_set_cs_isgraph();

    /* ... iscntrl */
    /* ARGS: csm_c_char_set cs_p, unsigned char* c */

extern long                 char_set_cs_iscntrl();

    /* check for identity of character at c with C language escape character ... */

    /* ... alert */
    /* ARGS: csm_c_char_set cs_p, unsigned char* c */

extern long                 char_set_is_alert();

    /* ... backspace */
    /* ARGS: csm_c_char_set cs_p, unsigned char* c */

extern long                 char_set_is_backspace();

    /* ... formfeed */
    /* ARGS: csm_c_char_set cs_p, unsigned char* c */

extern long                 char_set_is_formfeed();

    /* ... newline */
    /* ARGS: csm_c_char_set cs_p, unsigned char* c */

extern long                 char_set_is_newline();

    /* ... carriage return */
    /* ARGS: csm_c_char_set cs_p, unsigned char* c */

extern long                 char_set_is_cr();

    /* ... horizontal tab */
    /* ARGS: csm_c_char_set cs_p, unsigned char* c */

extern long                 char_set_is_htab();

    /* ... vertical tab */
    /* ARGS: csm_c_char_set cs_p, unsigned char* c */

extern long                 char_set_is_vtab();

    /* ... backslash */
    /* ARGS: csm_c_char_set cs_p, unsigned char* c */

extern long                 char_set_is_backslash();

    /* ... question mark (used in C Language triglyphs) */
    /* ARGS: csm_c_char_set cs_p, unsigned char* c */

extern long                 char_set_is_question();

    /* ... single quote */
    /* ARGS: csm_c_char_set cs_p, unsigned char* c */

extern long                 char_set_is_squote();

    /* ... double quote */
    /* ARGS: csm_c_char_set cs_p, unsigned char* c */

extern long                 char_set_is_dquote();

    /* copy into the character at c the escape character ... */

    /* ... alert */
    /* ARGS: csm_c_char_set cs_p, unsigned char* c */

extern                  char_set_get_alert();

    /* ... backspace */
    /* ARGS: csm_c_char_set cs_p, unsigned char* c */

extern                  char_set_get_backspace();

    /* ... formfeed */
    /* ARGS: csm_c_char_set cs_p, unsigned char* c */

extern                  char_set_get_formfeed();

    /* ... newline */
    /* ARGS: csm_c_char_set cs_p, unsigned char* c */

extern                  char_set_get_newline();

    /* ... carriage return */
    /* ARGS: csm_c_char_set cs_p, unsigned char* c */

extern                  char_set_get_cr();

    /* ... horizontal tab */
    /* ARGS: csm_c_char_set cs_p, unsigned char* c */

extern                  char_set_get_htab();

    /* ... vertical tab */
    /* ARGS: csm_c_char_set cs_p, unsigned char* c */

extern                  char_set_get_vtab();

    /* ... backslash */
    /* ARGS: csm_c_char_set cs_p, unsigned char* c */

extern                  char_set_get_backslash();

    /* ... question mark (used in C Language triglyphs) */
    /* ARGS: csm_c_char_set cs_p, unsigned char* c */

extern                  char_set_get_question();

    /* ... single quote */
    /* ARGS: csm_c_char_set cs_p, unsigned char* c */

extern                  char_set_get_squote();

    /* ... double quote */
    /* ARGS: csm_c_char_set cs_p, unsigned char* c */

extern                  char_set_get_dquote();

    /* get origin of array of regular expression characters */
    /* ARGS: csm_c_char_set cs_p */

extern csm_c_csm_character char_set_get_regexp_chars();

    /* set origin of array of regular expression characters */
    /* ARGS: csm_c_char_set cs_p, csm_c_csm_character new_re_chars */

extern                 char_set_set_regexp_chars();


/* CLASS ALPHA_CHAR_SET */

    /* allocate an alpha_char_set from free store */
    /* ARGS: unsigned long per_char_size */

extern csm_c_alpha_char_set alpha_char_set_class_alloc();

    /* deallocate an alpha_char_set */
    /* ARGS: csm_c_alpha_char_set acs_p */

extern                  alpha_char_set_class_free();

    /* return whether character at c is upper case */
    /* ARGS: csm_c_alpha_char_set acs_p, unsigned char* c */

extern long           alpha_char_set_cs_isupper();

    /* return whether character at c is lower case */
    /* ARGS: csm_c_alpha_char_set acs_p, unsigned char* c */

extern long           alpha_char_set_cs_islower();

    /* return lower case character that corresponds to upper case */
    /*   character at c. Undefined if c is not upper case. */
    /* ARGS: csm_c_alpha_char_set acs_p, unsigned char* c */

extern unsigned char* alpha_char_set_cs_tolower();

    /* return upper case character that corresponds to lower case */
    /*   character at c. Undefined if c is not lower case. */
    /* ARGS: csm_c_alpha_char_set acs_p, unsigned char* c */

extern unsigned char* alpha_char_set_cs_toupper();

    /* become a character set parallel to ISO8859-1 standard at this character size */
    /* ARGS: csm_c_alpha_char_set acs_p */

extern            alpha_char_set_become_iso();    

    /* become a character set parallel to ASCII standard at this character size */
    /* ARGS: csm_c_alpha_char_set acs_p */

extern            alpha_char_set_become_ascii();


/* CLASS SYLL_FRAG_CHAR_SET */

    /* class used to translate East-Indian and Korean */
    /*   phonemic input to scripted syllables */

    /* allocate a syll_frag_char_set from free store */
    /* ARGS: unsigned long per_char_size */

extern csm_c_syll_frag_char_set syll_frag_char_set_class_alloc();

    /* deallocate a syll_frag_char_set */
    /* ARGS: csm_c_syll_frag_char_set sfcs_p */

extern                      syll_frag_char_set_free();

    /* return nonzero if character at start is a legal multichar sequence, 0 otherwise */
    /* ARGS: csm_c_syll_frag_char_set sfcs_p, unsigned char* start */

extern unsigned long            syll_frag_char_set_is_syll();

    /* if the character at the address pointed to by start is a not valid syllable, */
    /*   returns MAX_CHARS_IN_SET. Otherwise returns the appropriate syllable */
    /*   identifier from the output character set and advances start to point to */
    /*   the address of the beginning of the next syllable. */
    /* ARGS: csm_c_syll_frag_char_set sfcs_p, unsigned char** start */

extern unsigned long            syll_frag_char_set_get_syll();

    /* returns length of syllable at start */
    /* ARGS: csm_c_syll_frag_char_set sfcs_p, unsigned char* start */

extern unsigned long            syll_frag_char_set_get_syll_len();


/* CLASS UNION_CHAR_SET */

    /* allocate a union_char_set from free store */
    /* ARGS:  */

extern csm_c_union_char_set union_char_set_class_alloc_1();

    /* allocate a union_char_set initialized with the given char_set vector */
    /* ARGS: csm_c_char_set new_char_sets */

extern csm_c_union_char_set union_char_set_class_alloc_2();

    /* deallocate a union_char_set */
    /* ARGS: csm_c_union_char_set ucs_p */

extern                  union_char_set_free();

    /* get character set vector */
    /* ARGS: csm_c_union_char_set ucs_p */

extern csm_c_char_set       union_char_set_get_sets();

    /* get number of character sets in vector */
    /* ARGS: csm_c_union_char_set ucs_p */

extern unsigned long        union_char_set_get_num_sets();

    /* get next union_char_set on queue */
    /* ARGS: csm_c_union_char_set ucs_p */

extern csm_c_union_char_set union_char_set_get_next();

    /* get reference count */
    /* ARGS: csm_c_union_char_set ucs_p */

extern unsigned long        union_char_set_get_ref_count();

    /* set the vector of character sets */
    /* ARGS: csm_c_union_char_set ucs_p, csm_c_char_set new_sets */

extern                  union_char_set_set_sets();

    /* set the number of character sets in the vector */
    /* ARGS: csm_c_union_char_set ucs_p, unsigned long new_num */

extern                  union_char_set_set_num_sets();

    /* set the next union_char_set in the queue */
    /* ARGS: csm_c_union_char_set ucs_p, csm_c_union_char_set new_next */

extern                  union_char_set_set_next();

    /* set the reference count */
    /* ARGS: csm_c_union_char_set ucs_p, unsigned long new_ref_count */

extern                  union_char_set_set_ref_count();

    /* increment the reference count */
    /* ARGS: csm_c_union_char_set ucs_p */

extern                  union_char_set_incr_ref_count();

    /* decrement the reference count */
    /* ARGS: csm_c_union_char_set ucs_p */

extern                  union_char_set_decr_ref_count();

    /* prepend the vector of union_char_set objects at new_unions on this queue of union_char_sets */
    /* ARGS: csm_c_union_char_set ucs_p, csm_c_union_char_set new_unions */

extern csm_c_union_char_set union_char_set_prepend();

    /* append the vector of union_char_set objects at new_unions to this queue of union_char_sets */
    /* ARGS: csm_c_union_char_set ucs_p, csm_c_union_char_set new_unions */

extern csm_c_union_char_set union_char_set_append();

    /* detach this union_char_set from the vector beginning with from_vec */
    /* ARGS: csm_c_union_char_set ucs_p, csm_c_union_char_set from_vec */

extern csm_c_union_char_set union_char_set_detach();

    /* prepend the vector of char_set objects at new_sets onto the vector of */
    /*   char_set objects associated with this union_char_set */
    /* ARGS: csm_c_union_char_set ucs_p, csm_c_char_set new_sets */

extern csm_c_char_set       union_char_set_prepend_char_set();

    /* append the vector of char_set objects at new_sets to the vector of */
    /*   char_set objects associated with this union_char_set */
    /* ARGS: csm_c_union_char_set ucs_p, csm_c_char_set new_sets */

extern csm_c_char_set       union_char_set_append_char_set();

    /* detach a char_set object from the vector of char_set objects associated with this union_char_set */
    /* ARGS: csm_c_union_char_set ucs_p, csm_c_char_set old_set */

extern csm_c_char_set       union_char_set_detach_char_set();

    /* return the first char_set on the vector that includes the character at c of size char_size */
    /* ARGS: csm_c_union_char_set ucs_p, unsigned char* c, unsigned long char_size */

extern csm_c_char_set       union_char_set_contains_any();

    /* return whether the char_set at vector position char_set_index contains the character at c */
    /* ARGS: csm_c_union_char_set ucs_p, unsigned long char_set_index, unsigned char* c */

extern long                 union_char_set_contains_ask();

    /* return nonzero if this is a default union_char_set (ISO8859-1) with no "next" */
    /* ARGS: csm_c_union_char_set ucs_p */

extern long                 union_char_set_default_only();

    /* save the CSM objects associated with the vector of union_char_set objects at ucs_p */
    /*   in human-readable form to file indicated by file handle */
    /* ARGS: csm_c_union_char_set ucs_p, char *file_handle */

extern long                 union_char_set_write_ucs_config_file();

    /* prepare the CSM objects associated with the vector of union_char_set objects at ucs_p */
    /*   for writing to file in binary format */
    /* ARGS: csm_c_union_char_set ucs_p, tml_c_gc_hash_tbl** hash_tbl, csm_c_cs_heap** heaps, tml_c_tml_ext_type** types */

extern                  union_char_set_prepare_ucs_binary();

    /* save the CSM objects associated with the vector of union_char_set objects at ucs_p */
    /*   in binary form to file indicated by file_handle */
    /* ARGS: csm_c_union_char_set ucs_p, char *file_handle, tml_c_gc_hash_tbl** hash_tbl, csm_c_cs_heap** heaps, tml_c_tml_ext_type** types */

extern long                 union_char_set_write_ucs_binary_file();

/* CLASS CS_HEAP */

    /* allocate a cs_heap from free store. The argument is an application-dependent structure */
    /* ARGS: tml_c_endian my_endian */

    /* ARGS: csm_c_cs_heap cs_heap_p */

extern           cs_heap_free();

/*************************************************************************************
    Declarations of general functions to read character sets recorded
      in binary and human-readable format respectively
*************************************************************************************/

    /* create union_char_sets from binary file indicated by file handle */
    /* ARGS: char *file_handle, tml_c_gc_hash_tbl** hash_tbl, tml_c_tml_ext_type** types, tml_c_endian** their_endian */

extern csm_c_union_char_set csm_c_read_union_char_sets();

    /* create union_char_sets from human-readable file indicated by file handle */
    /* ARGS: char *file_handle */

extern csm_c_union_char_set csm_c_read_config_file_union_char_sets();

/*************************************************************************************
    Declarations of general functions for comparison of strings
      in two different character sets whose collation maps have been
      coordinated
*************************************************************************************/

    /* compare strings from two different character sets, each of which is */
    /*   terminated with respect to its own character set. The comparison is */
    /*   made with respect to the collation values associated with the */
    /*   characters in their respective char_set objects. */
    /* ARGS: csm_c_csm_character* c1, csm_c_csm_character* c2 */

extern long csm_c_char_arr_cmp();

    /* compare up to n characters of strings from two different character sets, */
    /*   neither of which need be terminated. The comparison is */
    /*   made with respect to the collation values associated with the */
    /*   characters in their respective char_set objects. */
    /* ARGS: csm_c_csm_character* c1, csm_c_csm_character* c2, unsigned long n */

extern long csm_c_char_arr_ncmp();

/*************************************************************************************
    Declarations of some general functions for sorting
*************************************************************************************/

    /* set or reset a character set pointer to be used for csm_cmp_for_qsort() */
    /* ARGS: csm_c_char_set new_char_set */

extern  csm_c_set_csm_char_set_for_qsort();

    /* comparison function for use with UNIX qsort() when character sets are involved */
    /* ARGS: char** sp1, char** sp2 */

extern int csm_c_csm_cmp_for_qsort();

/**********************************************************************************
    Objects used to implement Knuth-Morris-Pratt algorithm
**********************************************************************************/

/* CLASS KMP_NODE */
    
    /* no constructor is supplied - use csm_c_kmp_fsm() */

    /* deallocate the tree rooted at kmp_node_p */
    /* ARGS: csm_c_kmp_node kmp_node_p */

extern            kmp_node_free();

    /* if the string at sought matches the kmp_node, return its identifier */
    /*   and advance sought beyond end of match; */
    /* otherwise return MAX_CHARS_IN_SET leaving sought undefined */
    /* ARGS: csm_c_kmp_node kmp_node_p, unsigned char** sought */

extern unsigned long  kmp_node_lookup();

    /* incremental lookup -- call this function repeatedly with a new character at s. Initial call takes */
    /*   these argument values: "kmp_node_p" and "next_node" should be root of a KMP FSM, and "s", a pointer to a */
    /*   (multibyte) character. On successive calls supply at "s" the next character to be considered. */
    /*   The "num_chars" argument need not be initialized, but logically should be initialized to zero. */
    /*   Function returns MAX_CHARS_IN_SET unless a successful string match is completed. On a successful */
    /*   string match, the function returns the value associated with the successful match, sets "next_node" */
    /*   to NULL if no longer match is possible, and returns in "num_chars" the number of characters */
    /*   matched. If "next_node" is non-NULL, a match is still possible. The shortest possible match is */
    /*   reported, but if a longer match is possible, "next_node" will be non-NULL. Note that successive */
    /*   calls may yield a match, but that the length matched may be less than the number of characters */
    /*   examined. For example, a KMP FSM that was designed to recognize "asdf" would yield a successful */
    /*   match on "asdasdf", but only the last four characters would constitute the match. */
    /* ARGS: csm_c_kmp_node kmp_node_p, csm_c_kmp_node** next_node, unsigned long* num_chars, unsigned char* s */

extern unsigned long  kmp_node_incr_lookup();


/*************************************************************************************
    FUNCTIONS TO OBTAIN OBJECTS FOR STRING SEARCH
*************************************************************************************/

    /* return a string search object */
    /* ARGS: unsigned char** strings, unsigned long num_strings, unsigned long* collations, csm_c_char_set for_char_set */

extern csm_c_kmp_node csm_c_kmp_fsm();

/*************************************************************************************
    FUNCTIONS TO SUPPORT ARCHITECTURE NEUTRALITY
*************************************************************************************/

    /* return an endian object appropriate for the given magic number */
    /* ARGS: long their_magic_number */

extern tml_c_endian csm_c_get_their_endian();

    /* return an endian object appropriate for the host machine */
    /* ARGS:  */

extern tml_c_endian csm_c_get_my_endian();

#endif

#endif /* csm_c_types_h */
