/**************************************************************************************
    tml_c_types.h
    header file including C language interface for the object definitions and
      assorted type declarations for the portable version of the text management
      library

    (c) Copyright Apollo Computer Inc., 1987 (unpublished)
     --------------------------------------------------------------------
    |THE FOLLOWING PROGRAMS ARE THE SOLE PROPERTY OF APOLLO COMPUTER INC.|
    |      AND CONTAIN ITS PROPRIETARY AND CONFIDENTIAL INFORMATION.     |
     --------------------------------------------------------------------
**************************************************************************************/

#ifndef tml_c_types_h

#define tml_c_types_h

#ifndef csm_c_types_h
#include <tml/csm_c_types.h>
#endif

#ifndef tml_c_structs_h
#include <tml/tml_c_structs.h>
#endif

#ifdef __STDC__

/* FROM HERE TO "#else", declarations for ANSI C programmers                 */
/******************************************************************************
   typedefs for C++ classes

******************************************************************************/


typedef void *tml_c_node;

typedef void *tml_c_bnode;

typedef void *tml_c_gnode;

typedef void *tml_c_draw_mark;

typedef void *tml_c_model;

typedef void *tml_c_prop_vec;

typedef void *tml_c_property;

typedef void *tml_c_argument;

typedef void *tml_c_history;

typedef void *tml_c_prop_func_ent;

typedef void *tml_c_format;

typedef void *tml_c_cline;

typedef void *tml_c_tline;

typedef void *tml_c_phead;

typedef void *tml_c_font_tbl;

typedef void *tml_c_view_pos;

typedef void *tml_c_v_cursor;

typedef void *tml_c_view_port;

typedef void *tml_c_bitmap_id;

typedef void *tml_c_vpftbl;

typedef void *tml_c_grafsys;

/******************************************************************************
   typedefs and instances for extensibility

******************************************************************************/


typedef void *node_type;

typedef void *p_index;

typedef void *arg_type;

extern node_type undefined_node, bobj, branch, gobj, pgobj, leaf, is_free;

extern p_index undefined_prop, tml_wrap_prop, tml_margin_prop,
               tml_font_prop, tml_inv_prop, tml_pgsize_prop,
               tml_header_prop, tml_footer_prop, tml_form_feed_prop,
               tml_char_set_prop, tml_union_char_set_prop, tml_tab_prop;

extern arg_type int_val_arg, float_val_arg, string_val_arg, no_val_arg,
                subtree_val_arg, char_set_val_arg, union_char_set_val_arg;

/******************************************************************************
    constants used in the classes

******************************************************************************/


/* BOOLEAN REFERENCES FOR PROPERTY VALUES */


#define tml_true_prop 1

#define tml_false_prop 0
    


/******************************************************************************
    Enumerated types

******************************************************************************/

/* ENUMERATED CLASS OF DIRECTIONS FROM ONE NODE TO ANOTHER */
/* enum node_dir {n_up, n_right, n_left, n_down}; */
#define n_up    0
#define n_right 1
#define n_left  2
#define n_down  3
typedef long node_dir;



/* ENUMERATED CLASS OF TYPES OF VALUES FOR TML VERSION 1 CLASS ARGUMENT */
/* enum arg_type_v1 {int_val, float_val, string_val, no_val}; */
#define int_val    0
#define float_val  1
#define string_val 2
#define no_val     3
typedef long arg_type_v1;



/* ENUMERATED CLASS OF TYPES OF CONCEPTUAL LINES */
/* enum  cl_type {cl_free, cl_text, cl_graphic, cl_blank, cl_gutter, cl_other}; */
#define cl_free    0
#define cl_text    1
#define cl_graphic 2
#define cl_blank   3
#define cl_gutter  4
#define cl_other   5
typedef long cl_type;



/* ENUMERATED CLASS OF THE DIRECTIONS THE CURSOR CAN MOVE */
/* enum vp_dir {vp_right, vp_left, vp_up, vp_down,
                vp_begin_line, vp_end_line, vp_top, vp_bottom}; */
#define vp_right      0
#define vp_left       1
#define vp_up         2
#define vp_down       3
#define vp_begin_line 4
#define vp_end_line   5
#define vp_top        6
#define vp_bottom     7
typedef long vp_dir;



/* ENUMERATED CLASS OF PRINTING ORIENTATIONS */
/* enum orientation {portrait, landscape} */
#define portrait      0
#define landscape     1
typedef long orientation;



/* ENUMERATED CLASS OF MARGIN LOCATIONS */
/* enum marg_loc {top_marg, bottom_marg, left_marg, right_marg}; */
#define top_marg      0
#define bottom_marg   1
#define left_marg     2
#define right_marg    3
typedef long marg_loc;



/* ENUMERATED CLASS OF WRAP PROPERTY TYPES */
/* enum wrap_type {no_wrap, char_wrap, word_wrap}; */
#define no_wrap       0
#define char_wrap     1
#define word_wrap     2
typedef long wrap_type;



/******************************************************************************
    Special initializer for users of TML with other languages
******************************************************************************/

extern void tml_init(void);



/******************************************************************************
    Generic function pointer
******************************************************************************/

typedef void* (*GEN_FUNCP)(...);



/******************************************************************************
    Structures used for general purposes with allocator and
    deallocator for each where necessary
******************************************************************************/


/* STRUCTURE USED TO CAPTURE CONTENTS OF A MEMBER OF CLASS MODEL_POS */
/* DEFINED IN tml_c_structs.h */

    /* allocate a C_MODEL_POS */

extern C_MODEL_POS *c_model_pos_alloc(void);

    /* deallocate a C_MODEL_POS */

extern void         c_model_pos_free(C_MODEL_POS *this_c_model_pos);



/* STRUCTURE USED TO CAPTURE CONTENTS OF A MEMBER OF CLASS MODEL_RANGE */
/* DEFINED IN tml_c_structs.h */

    /* allocate a C_MODEL_RANGE */

extern C_MODEL_RANGE *c_model_range_alloc(void);

    /* deallocate a C_MODEL_RANGE */

extern void           c_model_range_free(C_MODEL_RANGE *this_c_model_range);



/* STRUCTURE USED TO CAPTURE CONTENTS OF A MEMBER OF CLASS MARK_LIST */
/* DEFINED IN tml_c_structs.h */

    /* allocate a C_MARK_LIST */

extern C_MARK_LIST *c_mark_list_alloc(void);

    /* deallocate a C_MARK_LIST */

extern void         c_mark_list_free(C_MARK_LIST *this_c_mark_list);



/* STRUCTURE USED TO CAPTURE CONTENTS OF A MEMBER OF CLASS XY_COORD */
/* DEFINED IN tml_c_structs.h */
 
    /* allocate a C_XY_COORD */

extern C_XY_COORD *c_xy_coord_alloc(void);

    /* deallocate a C_XY_COORD */

extern void        c_xy_coord_free(C_XY_COORD *this_c_xy_coord);



/* STRUCTURE USED TO CAPTURE CONTENTS OF A MEMBER OF CLASS SCREEN_XY */
/* DEFINED IN tml_c_structs.h */

    /* UPPER LEFT CORNER OF A VIEWPORT */

    extern C_SCREEN_XY zero_zero;

    /* allocate a C_SCREEN_XY */

extern C_SCREEN_XY *c_screen_xy_alloc(void);

    /* deallocate a C_SCREEN_XY */

extern void         c_screen_xy_free(C_SCREEN_XY *this_c_screen_xy);



/* STRUCTURE USED TO SPECIFY PRINTING PARAMETERS */
/* DEFINED IN tml_c_structs.h */

    /* allocate a C_PRINT_DB */

extern C_PRINT_DB *c_print_db_alloc(void);

    /* deallocate a C_PRINT_DB */

extern void        c_print_db_free(C_PRINT_DB *this_c_print_db);



/* STRUCTURE FOR CAPTURING THE CONTENTS OF A MEMBER OF CLASS SEARCH_ENV */
/* DEFINED IN tml_c_structs.h */

    /* allocate a C_SEARCH_ENV */

extern C_SEARCH_ENV *c_search_env_alloc(void);

    /* deallocate a C_SEARCH_ENV */

extern void          c_search_env_free(C_SEARCH_ENV *this_search_env);




/******************************************************************************
    Objects used in the modelling component of tml

******************************************************************************/


/* CLASS MODEL_POS */

    /* allocate a model_pos from free store */

extern C_MODEL_POS *model_pos_class_alloc(void);


    /* deallocate a model_pos */

extern void model_pos_free(C_MODEL_POS *this_model_pos);


    /* return 1 if first position is same as second, 0 otherwise */

extern long model_pos_eq(C_MODEL_POS* this_model_pos, C_MODEL_POS* comp_pos);


    /* return 1 if first position is not the same as the second, 0 otherwise */

extern long model_pos_neq(C_MODEL_POS* this_model_pos, C_MODEL_POS* comp_pos);


    /* return 1 if first position is less than second, 0 otherwise */

extern long model_pos_lt(C_MODEL_POS* this_model_pos, C_MODEL_POS* comp_pos);


    /* return 1 if first position is less than or equal to second, */
    /* 0 otherwise */

extern long model_pos_le(C_MODEL_POS* this_model_pos, C_MODEL_POS* comp_pos);


    /* return 1 if first position is greater than second, 0 otherwise */

extern long model_pos_gt(C_MODEL_POS* this_model_pos, C_MODEL_POS* comp_pos);


    /* return 1 if first position is greater than or equal to second, */
    /* 0 otherwise */

extern long model_pos_ge(C_MODEL_POS* this_model_pos, C_MODEL_POS* comp_pos);


    /* second C_MODEL_POS* argument points to caller-supplied storage */
    /* a pointer to the next formattable model position is written into the */
    /* second argument. If all goes well, the return value equals the second */
    /* argument. */

extern C_MODEL_POS *model_pos_get_next(C_MODEL_POS* this_model_pos, C_MODEL_POS* next_pos);


    /* second C_MODEL_POS* argument points to caller-supplied storage */
    /* a pointer to the prior formattable model position is written into the */
    /* second argument. If all goes well, the return value equals the second */
    /* argument. */

extern C_MODEL_POS *model_pos_get_prior(C_MODEL_POS* this_model_pos, C_MODEL_POS* prior_pos);


    /* mark the page header associated with the argument model position */
    /* as needing to be reformatted */

extern void model_pos_mark_changed(C_MODEL_POS *this_model_pos);


    /* returns zero if the node at which the model position is set is not */
    /* formattable or if the model position's node member is NULL. */
    /* Otherwise, the nonzero return value depends on the node's type. */

extern long model_pos_can_format(C_MODEL_POS* this_model_pos);


    /*redraws every line on which the model pos occurs (if visible). Assumes */
    /* that the change does not alter the format of the conceptual page*/

extern void model_pos_in_place_redraw(C_MODEL_POS *this_model_pos);




/* FOR CLASS MODEL_RANGE */


    /* allocate a model_range from free store */

extern C_MODEL_RANGE *model_range_class_alloc(void);


    /* deallocate a model_range */

extern void model_range_free(C_MODEL_RANGE *model_range_p);


    /* set the start member of the argument model_range to the model */
    /* position specified by new_start */

extern void model_range_set_start(C_MODEL_RANGE *model_range_p, C_MODEL_POS *new_start);


    /* set the end member of the argument model_range to the model */
    /* position specified by new_end */

extern void model_range_set_end(C_MODEL_RANGE *model_range_p, C_MODEL_POS *new_end);


    /* get the start member of a model range */
    /* Second argument points to user-supplied storage. If all goes well, */
    /* the return value equals the second argument. */

extern C_MODEL_POS *model_range_get_start(C_MODEL_RANGE *model_range_p, C_MODEL_POS *model_pos_p);


    /* get the end member of a model range */
    /* Second argument points to user-supplied storage. If all goes well, */
    /* the return value equals the second argument. */

extern C_MODEL_POS *model_range_get_end(C_MODEL_RANGE *model_range_p, C_MODEL_POS *model_pos_p);


    /* isolate the argument model_range into a subtree */
    /* Return a pointer to the root of the subtree or NULL if the */
    /* model_range is bogus */

extern void *model_range_isolate(C_MODEL_RANGE *model_range_p);


    /* return 1 if the argument model_range contains target_pos, */
    /* 0 otherwise. */

extern long model_range_contains(C_MODEL_RANGE *model_range_p, C_MODEL_POS *target_pos);




/* FOR CLASS NODE */


    /* no allocator for class node is provided since class node is a */
    /* container class. Use the allocator appropriate for the subclass */
    /* desired. */

    /* deallocate a node: dissociate it from its properties. If it is */
    /* associated with a model, remove it from the model and make it the */
    /* head of the free list for its subclass of node. */

extern void node_free(tml_c_node this_node);



    /* FUNCTIONS ALLOWING INQUIRY AND SETTING OF THE DATA CONTAINED */
    /* IN THE CLASS */


    /* get model associated with tml_c_node node */

extern tml_c_model node_get_model(tml_c_node this_node);


    /* set model member of tml_c_node to tml_c_model */

extern void node_set_model(tml_c_node this_node, tml_c_model new_model);


    /* get node type member of tml_c_node - one of enum node_type */

extern node_type node_get_type(tml_c_node this_node);


    /* set node type member of tml_c_node to node_type specified by node_type */

extern void node_set_type(tml_c_node this_node, node_type this_type);


    /* find the level of tml_c_node in the tree by counting */

extern long node_get_level(tml_c_node this_node);


    /* get the one-based sibling order (leftmost is 1) of tml_c_node */

extern long node_get_sib_order(tml_c_node this_node);


    /* get the property vector associated with tml_c_node */

extern tml_c_prop_vec node_get_props(tml_c_node this_node);


    /* get the model_range spanned by the subtree rooted at node */

extern C_MODEL_RANGE *node_get_span(tml_c_node this_node, C_MODEL_RANGE *model_range_p);



    /* FUNCTIONS FOR PLACING AND MOVING NODES IN A TREE */


    /* connect 2nd node to 1st in direction specified - one of enum node_dir */

extern tml_c_node node_connect_node(tml_c_node this_node, tml_c_node to_connect, node_dir dir);


    /* take a node out of a tree */

extern void node_detach(tml_c_node this_node);


    /* move 1st node to place in direction specified from 2nd node */

extern tml_c_node node_move(tml_c_node this_node, tml_c_node move_to, node_dir dir);



    /* FUNCTIONS DEALING WITH PROPERTIES OF NODES */


    /* attach a property vector to a node */

extern void node_prop_attach(tml_c_node this_node, tml_c_prop_vec add_pvec);


    /* detach a node's property vector from it */

extern void node_prop_detach(tml_c_node this_node);


    /* deactivate all the properties of a node */

extern void node_exit_props(tml_c_node this_node);


    /* activate all the properties of a node */

extern void node_enter_props(tml_c_node this_node);



    /* VIRTUAL FUNCTIONS--ACTUAL IMPLEMENTATION MAY DIFFER */
    /* FOR VARIOUS SUB-CLASSES */


    /* does this node contain formattable information? 1==yes, 0==no */

extern long node_can_format(tml_c_node this_node);


    /* for text nodes (gnodes) give number of characters dominated */

extern long node_get_max_index(tml_c_node this_node);



    /* THE FOLLOWING VIRTUAL FUNCTIONS ARE ILL-DEFINED */
    /* FOR ALL BUT THE DERIVED CLASSES */


    /* return pointer to formattable contents dominated by this node */

extern void* node_get_contents(tml_c_node this_node);


    /* get new copy of node in model specified; if model is zero, */
    /* use same model as node is associated with */

extern tml_c_node node_copy(tml_c_node this_node, tml_c_model to_model);


    /* copy the subtree below node to model specified; if model is zero, */
    /* use same model as node is associated with */

extern tml_c_node node_copy_all(tml_c_node this_node, tml_c_model to_model);


    /* duplicate this node in its model; use same property vector as */
    /* original node */

extern tml_c_node node_dup(tml_c_node this_node);


    /* duplicate, as above, the subtree at node in its model */

extern tml_c_node node_dup_all(tml_c_node this_node);


    /* split nodes for insertion; implementation varies with node type */
    /* for gnode: split at text index specified by long; put portion to */
    /* left of that index into one node; put portion starting at that */
    /* index into another node; create an empty node; make these three */
    /* nodes siblings in order left-empty-right. */

extern tml_c_node node_split(tml_c_node this_node, long split_point);



    /* NAVIGATION FUNCTIONS */


    /* get next node in traversal order; direction traveled from returned */
    /* in node_dir* argument if returning from lower level */

extern tml_c_node node_get_next(tml_c_node this_node, node_dir* dir);


    /* get previous node in traversal order; direction traveled from returned */
    /* in node_dir* argment if returning from lower level */

extern tml_c_node node_get_prev(tml_c_node this_node, node_dir* dir);


    /* get the node in the specified direction (one of node_dir) from */
    /* node argument */

extern tml_c_node node_get_node(tml_c_node this_node, node_dir dir);


    /* make the first node's pointer in direction specified by node_dir*/
    /* (one of node_dir) point to second node */

extern void node_set_node(tml_c_node this_node, tml_c_node to_node, node_dir dir);


    /* get deepest rightmost node in subtree which has this node as root */
    /* or rightmost sibling of this node if this node is childless */

extern tml_c_node node_get_last(tml_c_node this_node);



    /* FUNCTIONS MEANINGFUL ONLY FOR CERTAIN SUBCLASSES */


    /* MEANINGFUL ONLY FOR GNODES AND OTHER TYPES HANDLING CHARACTER DATA */


    /* associate data_count bytes of data at data_start with this_node. */
    /* Returns 0 if applied to a nonterminal type of node, otherwise */
    /* dissociates node from any data it currently dominates, and copies */
    /* data at data_start into new data space. New data space is allocated */
    /* from node's associated model if any, otherwise from free space. */

extern long node_attach_data(tml_c_node this_node, void *data_start, long data_count);

    /* insert add_count characters of add_data after character at */
    /* after_index in goo associated with this_node. Returns model position */
    /* of last character inserted. C_MODEL_POS argument points to */
    /* user-supplied storage. */

extern C_MODEL_POS *node_insert(tml_c_node this_node, long after_index, char *add_data, long add_count, C_MODEL_POS *model_pos_p);

    /* delete delete_num characters at delete_index from goo associated with */
    /* this_node. Return model position of node == 0 and char_index == -1. */
    /* (Implementations of this function for appropriate subclasses can */
    /* return a good model position.) */
    /* C_MODEL_POS argument points to user-supplied storage. */

extern C_MODEL_POS *node_del_cont(tml_c_node this_node, long delete_index, long delete_num, C_MODEL_POS *model_pos_p);


    /* overstrike, with rep_with, over_num characters of goo associated with */
    /* this_node, starting at begin_index. Returns model position of */
    /* node == 0 and char_index == -1. */
    /* (Implementations of this function for appropriate subclasses can */
    /* return a good model position.) */
    /* C_MODEL_POS argument points to user-supplied storage. */

extern C_MODEL_POS *node_overstrike(tml_c_node this_node, long begin_index, char *rep_with, long over_num, C_MODEL_POS *model_pos_p);



    /* FORMATTING FUNCTIONS */


    /* get the height of the bounding box for information to be formatted */
    /* out of this node */

extern long node_get_format_hi(tml_c_node this_node, tml_c_format in_format);


    /* return conceptual line associated with the given node under control */
    /* of the given format. Formatting is to start at a point (e.g. */
    /* character index) in the formattable information (e.g. text string) */
    /* dominated by the node. This point is specified by the first long */
    /* argument. The second and third long arguments give the height and */
    /* width, respectively, of the space remaining on the current conceptual */
    /* page. The final long argument indicates whether this is the first time */
    /* this node has been formatted (i.e. whether formatting or reformatting */
    /* is being performed.) */

extern tml_c_cline node_format_line(tml_c_node this_node, tml_c_format in_format, long start_index, long space_hi, long space_wide, long first_concep_pixel);


    /* like node_format_line, but it formats BACKWARD from the point specified. */

extern tml_c_cline node_tamrof_line(tml_c_node this_node, tml_c_format in_format, long start_index, long space_hi, long space_wide, long first_concep_pixel);




/* FOR CLASS BNODE */

    /* -- DERIVED CLASS FOR NON-TERMINAL NODES IN THE MODEL STRUCTURE TREE */


    /* allocate a bnode in the given model. If the argument is zero, */
    /*   allocate from free space. */

extern tml_c_bnode bnode_class_alloc(tml_c_model to_model);


    /* deallocate a bnode */

extern void bnode_free(tml_c_bnode this_bnode);




/* FOR CLASS GNODE */

    /* -- DERIVED CLASS FOR A TERMINAL OBJECT WHICH POINTS TO */
    /* CHARACTERS IN THE GOO HEAP */


    /* allocate a gnode in the given model. If the argument is zero, */
    /*   allocate from free space. */

extern tml_c_gnode gnode_class_alloc(tml_c_model to_model);


    /* deallocate a gnode */

extern void gnode_free(tml_c_gnode this_gnode);


    /* after this call, the argument gnode dominates text equivalent to the */
    /* null-terminated string specified by the argument char*. The gnode's */
    /* copy doesn't include the null terminator */

extern void gnode_gets(tml_c_gnode this_gnode, char *s);


    /* get the non-null-terminated string dominated by the node */

extern void* gnode_get_contents(tml_c_gnode this_gnode);


    /* get the number of characters in the non-null-terminated string */
    /* dominated by the node */

extern long gnode_get_char_count(tml_c_gnode this_gnode);


    /* get a null-terminated copy of the string dominated by the node */

extern char* gnode_get_0t_contents(tml_c_gnode this_gnode);


    /* set the character count of a gnode */

extern void gnode_set_char_count(tml_c_gnode gnode_p, long new_count);




/* FOR CLASS PROP_VEC */


    /* allocate a property vector */

extern tml_c_prop_vec prop_vec_class_alloc(tml_c_model to_model);


    /* deallocate a property vector */

extern void prop_vec_free(tml_c_prop_vec this_prop_vec);


    /* return pointer to next member of argument property vector */

extern tml_c_prop_vec prop_vec_get_next(tml_c_prop_vec this_prop_vec);


    /* detach 1st from 2nd */

extern tml_c_prop_vec prop_vec_detach_pvec(tml_c_prop_vec this_prop_vec, tml_c_prop_vec from_vec);


    /* prepend 2nd to 1st */

extern tml_c_prop_vec prop_vec_prepend(tml_c_prop_vec this_prop_vec, tml_c_prop_vec add_element);


    /* append 2nd to 1st */

extern void prop_vec_append(tml_c_prop_vec this_prop_vec, tml_c_prop_vec add_vec);


    /* copy the property vector to a model; if model is zero, use model */
    /* that this property vector is associated with */

extern tml_c_prop_vec prop_vec_copy(tml_c_prop_vec this_prop_vec, tml_c_model to_model);


    /* copy this property vector and all property vectors chained to it */
    /* through next pointers to a model. If model argument is zero, use */
    /* model that this property vector is associated with */

extern tml_c_prop_vec prop_vec_copy_all(tml_c_prop_vec this_prop_vec, tml_c_model to_model);


    /* duplicate this property vector; i.e. copy it to the same model with */
    /* the same member pointer values */

extern tml_c_prop_vec prop_vec_dup(tml_c_prop_vec this_prop_vec);


    /* duplicate this property vector and all property vectors chained to it */
    /* through next pointers */

extern tml_c_prop_vec prop_vec_dup_all(tml_c_prop_vec this_prop_vec);


    /* attach property to prop_vec */

extern void prop_vec_attach_prop(tml_c_prop_vec this_prop_vec, tml_c_property add_prop);


    /* detach a prop_vec's property */

extern void prop_vec_detach_prop(tml_c_prop_vec this_prop_vec);


    /* activate all properties associated with a property vector and the */
    /* vectors chained to it */

extern void prop_vec_activate(tml_c_prop_vec this_prop_vec);


    /* deactivate similarly */

extern void prop_vec_deactivate(tml_c_prop_vec this_prop_vec);


    /* return the reference count associated with a property vector */

extern long prop_vec_get_ref_count(tml_c_prop_vec this_prop_vec);


    /* return a pointer to the property associated with a given property */
    /* vector */

extern tml_c_property prop_vec_get_prop(tml_c_prop_vec this_prop_vec);




/* FOR CLASS PROPERTY */


    /* allocate a property */

extern tml_c_property property_class_alloc(tml_c_model to_model);


    /* deallocate a property */

extern void property_free(tml_c_property this_property);


    /* get the index into the property function table of the function */
    /* associated with instantiation of this property */

extern p_index property_get_index(tml_c_property this_property);


    /* set the index member of the property to the integer argument's value */

extern void property_set_index(tml_c_property this_property, p_index i);


    /* copy a property to a model; if the model is zero, use the model this */
    /* property is associated with */

extern tml_c_property property_copy(tml_c_property this_property, tml_c_model to_model);


    /* duplicate a property in the model with which it is associated */

extern tml_c_property property_dup(tml_c_property this_property);


    /* return origin of entry argument vector for this property */

extern tml_c_argument property_get_entry_av(tml_c_property this_property);


    /* return origin of exit argument vector for this property */

extern tml_c_argument property_get_exit_av(tml_c_property this_property);


    /* attach a vector of entry arguments to a property */

extern void property_attach_entry(tml_c_property this_property, tml_c_argument entry_vec);


    /* attach a vector of exit argument to a property */

extern void property_attach_exit(tml_c_property this_property, tml_c_argument exit_vec);


    /* detach similarly */

extern void property_detach_entry(tml_c_property this_property);


    /* detach similarly */

extern void property_detach_exit(tml_c_property this_property);


    /* activate a property */

extern void property_activate(tml_c_property this_property);


    /* deactivate similarly */

extern void property_deactivate(tml_c_property this_property);


    /* return the reference count associated with the property */

extern long property_get_ref_count(tml_c_property this_property);




/* FOR CLASS ARGUMENT */


    /* allocate an argument */

extern tml_c_argument argument_class_alloc(tml_c_model to_model);


    /* deallocate an argument */

extern void argument_free(tml_c_argument this_argument);


    /* get the model associated with an argument */

extern tml_c_model argument_get_model(tml_c_argument this_argument);


    /* set the value of an integer-valued argument */

extern void argument_set_ival(tml_c_argument this_argument, long value);


    /* set the value of a floating-point argument - true doubles will have */
    /* their fractional part truncated */

extern void argument_set_fval(tml_c_argument this_argument, double value);


    /* set the value of a string argument to the non-null-terminated string */
    /* at char* of val_count bytes length by copying the char* argument to */
    /* space in a string heap */

extern void argument_set_sval(tml_c_argument this_argument, char* value, long val_count);


    /* return a pointer to the "next" argument from this one */

extern tml_c_argument argument_get_next(tml_c_argument this_argument);


    /* return the type of the argument - one of the enum arg_type */

extern long argument_get_vtype(tml_c_argument this_argument);


    /* return the value of an integer argument */

extern long argument_get_ival(tml_c_argument this_argument);


    /* return the value of a float argument */

extern double argument_get_fval(tml_c_argument this_argument);


    /* return the actual pointer to the non-null-terminated value of a */
    /* string argument */

extern char* argument_get_sval(tml_c_argument this_argument);


    /* return the number of bytes in the above non-null-terminated string */

extern long argument_get_scount(tml_c_argument this_argument);


    /* detach 1st from 2nd */

extern tml_c_argument argument_detach(tml_c_argument this_argument, tml_c_argument from_vec);


    /* append 2nd to 1st */

extern void argument_append(tml_c_argument this_argument, tml_c_argument add_arg);


    /* prepend 2nd to 1st */

extern tml_c_argument argument_prepend(tml_c_argument this_argument, tml_c_argument add_arg);


    /* return the reference count associated with an argument */

extern long argument_get_ref_count(tml_c_argument this_argument);


    /* copy an argument to a model; if model is zero, use model this */
    /* argument is in */

extern tml_c_argument argument_copy(tml_c_argument this_argument, tml_c_model to_model);


    /* copy an argument and all arguments chained to it through next */
    /* pointers to a model */
    /* if model is zero, use model this argument is in */

extern tml_c_argument argument_copy_all(tml_c_argument this_argument, tml_c_model to_model);





/* FOR CLASS HISTORY */


    /* no allocator or deallocator included for class history */

    /* set the size of the history array */

extern long history_set_hist_size(tml_c_history this_history, long size);


    /* end the current molecule after the last operation performed */

extern void history_end_molecule(tml_c_history this_history);


    /* undo the last molecule added to history */

extern long history_undo_molecule(tml_c_history this_history);


    /* redo the last molecule added to history */

extern long history_redo_molecule(tml_c_history this_history);



/* FOR CLASS MARK_LIST */


    /* allocate a mark_list */

extern C_MARK_LIST *mark_list_class_alloc(void);


    /* deallocate a mark_list */

extern void         mark_list_free(C_MARK_LIST *this_mark_list);


    /* get the next member of the argument mark_list */

extern C_MARK_LIST *mark_list_get_next(C_MARK_LIST *mark_list_p);


    /* get the prior member of the argument mark_list */

extern C_MARK_LIST *mark_list_get_prior(C_MARK_LIST *mark_list_p);


    /* get the range member the argument mark_list */

extern C_MODEL_RANGE *mark_list_get_range(C_MARK_LIST *mark_list_p);


    /* set the next member of the argument mark_list */

extern void mark_list_set_next(C_MARK_LIST *mark_list_p, C_MARK_LIST *next_ml);


    /* set the prior member of the argument mark_list */

extern void mark_list_set_prior(C_MARK_LIST *mark_list_p, C_MARK_LIST *prior_ml);


    /* set the range member of the argument mark_list */

extern void mark_list_set_range(C_MARK_LIST *mark_list_p, C_MODEL_RANGE *new_range);


    /* extend/shrink the range denominated by this mark_list */

extern long mark_list_extend_mark(C_MARK_LIST *mark_list_p, C_MODEL_POS *new_start, C_MODEL_POS *new_end);

         
    /* highlight the text located by the mark_list */

extern long mark_list_draw(C_MARK_LIST *mark_list_p);


    /* dehighlight the text located by the mark_list */

extern long mark_list_erase(C_MARK_LIST *mark_list_p);



/* FOR CLASS SEARCH_ENV */


    /* allocate a search_env */

extern C_SEARCH_ENV *search_env_class_alloc(C_SEARCH_FUNC use_func, C_COMPILE_FUNC use_compile, C_SEARCH_NAV_FUNC use_for_next, C_SEARCH_NAV_FUNC use_for_prior, C_SEARCH_NAV_FUNC use_for_this);


    /* deallocate a search_env */

extern void search_env_free(C_SEARCH_ENV *this_search_env);


    /* get the next member of the argument search_env */

extern C_SEARCH_ENV *search_env_get_next(C_SEARCH_ENV *search_env_p);


    /* set the next member of the argument search_env */

extern void search_env_set_next(C_SEARCH_ENV *search_env_p, C_SEARCH_ENV *new_next);

   


/* FOR CLASS MODEL */


    /* allocate a model */

extern tml_c_model model_class_alloc(void);


    /* deallocate a model */

extern void model_free(tml_c_model this_model);



    /* INQUIRY FUNCTIONS FOR THE MODEL DATA */


    /* get the root node of the model */

extern tml_c_node model_get_root(tml_c_model this_model);


    /* get the point up to which properties are instantiated for the model */

extern tml_c_node model_get_prop_pos(tml_c_model this_model);



    /* FUNCTIONS TO DIRECTLY MANIPULATE THE MODEL DATA */


    /* make a root for a new model */

extern tml_c_node model_make_root(tml_c_model this_model);


    /* set the root of a model to a given node */

extern tml_c_node model_set_root(tml_c_model this_model, tml_c_node new_root);


    /* inquire the state of the property associated with the p_index argument */

extern void* model_inq_state(tml_c_model this_model, p_index prop_ind);


    /* inquire the state of the property associated with the p_index argument */
    /* at a certain node */

extern void* model_inq_state_at_node(tml_c_model this_model, tml_c_node at_node, p_index prop_ind);


    /* move the current property position to the specified node, setting */
    /* properties as needed */

extern tml_c_node model_move_prop(tml_c_model this_model, tml_c_node to_node);


    /* set the current property position at the specified node */

extern void model_set_prop_pos(tml_c_model this_model, tml_c_node new_prop_pos);



    /* FUNCTIONS WHICH ALLOCATE SPACE IN A MODEL */


    /* allocate a property vector in the model */

extern tml_c_prop_vec model_make_pvec(tml_c_model this_model);


    /* allocate a property in the model */

extern tml_c_property model_make_prop(tml_c_model this_model);


    /* allocate an integer argument in the model */

extern tml_c_argument model_make_int_arg(tml_c_model this_model, long new_value);


    /* allocate a float argument in the model. NOTE - full doubles will */
    /* have their fractional parts truncated. */

extern tml_c_argument model_make_float_arg(tml_c_model this_model, double new_value);


    /* allocate a string argument in the model */

extern tml_c_argument model_make_string_arg(tml_c_model this_model, char* new_value, long new_count);


    /* allocate text characters in a model */

extern char* model_make_goo(tml_c_model this_model, long count, char* contents);


    /* make an object of the specified size in the model */
    /* - for object type extensions */

extern char* model_make_obj(tml_c_model this_model, long obj_size);


    /* write a model to the file indicated by the argument pathname */
    /* and pathname length. Return bytes written or -1 for failure. */

extern long model_d_write_model_path(tml_c_model this_model, char *pathname, long name_len);


    /* write a model to the file indicated by the argument file handle. */
    /* Returns bytes written or -1 for failure. */

extern long model_d_write_model_handle(tml_c_model this_model, void *file_handle);


    /* get a pointer to the history for the model */

extern tml_c_history model_get_past(tml_c_model this_model);


    /* cut the subtree rooted at node from the model */

extern void *model_cut(tml_c_model this_model, tml_c_node cut_node);


    /* paste the subtree rooted at node into the model, attaching */
    /* paste_node to past_to in direction paste_dir */

extern void *model_paste(tml_c_model this_model, tml_c_node paste_node, tml_c_node paste_to, node_dir paste_dir);


    /* get the first mark_list associated with the model */

extern C_MARK_LIST *model_get_first_mark(tml_c_model this_model);


    /* allocate a mark_list in the model; initialize it to makr */
    /* the text indicated by init_range */

extern C_MARK_LIST *model_make_mark(tml_c_model this_model, C_MODEL_RANGE *init_range);


    /* disassociate the argument mark_list from the model */

extern void model_delete_mark(tml_c_model this_model, C_MARK_LIST *to_delete);


    /* highlight the text located by the mark_list in any viewports */
    /* that are attached to the model */

extern long model_mark_draw(tml_c_model this_model, C_MARK_LIST *to_draw);


    /* dehighlight the text located by the mark_list in any viewports */
    /* that are attached to the model */

extern long model_mark_erase(tml_c_model this_model, C_MARK_LIST *to_erase);


    /* push the argument search_env onto the stack associated with the model */

extern void model_push_search_env(tml_c_model *this_model, C_SEARCH_ENV *new_envir);


    /* pop the top search_env from the stack associated with the model */

extern void model_pop_search_env(tml_c_model *this_model);


    /* search for the regular expression indicated by pattern */
    /* within the range indicated by the model_range argument */

extern C_MODEL_RANGE *model_search_exp(tml_c_model this_model, void *pattern, C_MODEL_RANGE *search_range, long forward, C_MODEL_RANGE *model_range_p);


    /* compile search_exp of length search_exp_len into a compiled regular */
    /* expression of maximum lenght max_comp_len. Case sensitivity is a */
    /* switch: 1 for case sensitive, 0 for not case sensitive */

extern long model_compile_exp(tml_c_model this_model, char *search_exp, long search_exp_len, void *compiled_exp, long max_comp_len, long case_sens);


    /* search for the string of length search_exp_len within the */
    /* model_range specified */

extern C_MODEL_RANGE *model_search(tml_c_model this_model, char *search_exp, long search_exp_len, C_MODEL_RANGE *search_range, long forward, long case_sens, C_MODEL_RANGE *model_range_p);
                                                                                                     

    /* updates, in a crude fashion, the display of the model when that model */
    /* has been changed*/

extern void model_update_display(tml_c_model this_model);




/* FOR CLASS DRAW_MARK */


    /* allocate a draw_mark */

extern tml_c_draw_mark draw_mark_alloc(void);


    /* deallocate a draw_mark */

extern void draw_mark_free(tml_c_draw_mark this_draw_mark);


    /* return the next member of a draw_mark */

extern tml_c_draw_mark draw_mark_get_next(tml_c_draw_mark this_draw_mark);


    /* return the prior member of a draw_port */

extern tml_c_draw_mark draw_mark_get_prior(tml_c_draw_mark this_draw_mark);


    /* return the mark_list associated with this draw_list */

extern tml_c_draw_mark draw_mark_get_model_mark(tml_c_draw_mark this_draw_mark);


    /* set the core drawing function for the draw mark */

extern void draw_mark_set_core_func(tml_c_draw_mark this_draw_mark, C_CORE_M_DRAW new_core);


    /* set the draw function for the draw mark */

extern void draw_mark_set_draw_func(tml_c_draw_mark this_draw_mark, C_DRAW_M_DRAW new_draw);


    /* extend/shrink the area indicated by the draw_mark */

extern void draw_mark_extend(tml_c_draw_mark this_draw_mark, C_MODEL_POS *new_start, C_MODEL_POS *new_end);


    /* highlight the viewport area associated with the draw_mark */

extern void draw_mark_draw(tml_c_draw_mark this_draw_mark);


    /* dehighlight the viewport area associated with the draw_mark */

extern void draw_mark_erase(tml_c_draw_mark this_draw_mark);




/* FOR CLASS PROP_FUNC_ENT */


    /* allocate a property function entry */

extern tml_c_prop_func_ent tml_c_prop_func_ent_class_alloc(void);


    /* deallocate a property function entry */

extern void tml_c_prop_func_ent_free(tml_c_prop_func_ent this_prop_func_ent);


    /* set 1st's next to 2nd */

extern void prop_func_ent_set_next(tml_c_prop_func_ent this_prop_func_ent, tml_c_prop_func_ent next_ent);


    /* set the index member to the argument p_index; this is a */
    /* property index */

extern void prop_func_ent_set_index(tml_c_prop_func_ent this_prop_func_ent, p_index i);


    /* get that index member */

extern p_index prop_func_ent_get_index(tml_c_prop_func_ent this_prop_func_ent);


    /* set the pointer to the function that enters this property */

extern void prop_func_ent_load_entry(tml_c_prop_func_ent this_prop_func_ent, C_PROP_FUNC_PT new_entry);


    /* set the pointer to the function that exits this property */

extern void prop_func_ent_load_exit(tml_c_prop_func_ent this_prop_func_ent, C_PROP_FUNC_PT new_exit);


    /* get that entry function pointer */

extern C_PROP_FUNC_PT prop_func_ent_get_entry(tml_c_prop_func_ent this_prop_func_ent);


    /* get that exit function pointer */

extern C_PROP_FUNC_PT prop_func_ent_get_exit(tml_c_prop_func_ent this_prop_func_ent);


    /* get load status of the prop_func_ent */

extern long prop_func_ent_get_loaded(tml_c_prop_func_ent this_prop_func_ent);


    /* set load status to the argument integer (will be truncated to a short) */

extern void prop_func_ent_set_loaded(tml_c_prop_func_ent this_prop_func_ent, long i);


    /* load the function associated with a property into the property */
    /* function table */

extern void prop_func_ent_load(tml_c_prop_func_ent this_prop_func_ent);





/**********************************************************************************
    Objects used in the formatting component of tml

**********************************************************************************/



/* FOR CLASS CLINE */


    /* no allocator supplied for class cline since it is a container class. */
    /* Use the allocator for the subclass desired. */


    /* deallocate a cline */

extern void cline_free(tml_c_cline this_cline);


    /* get the type of this conceptual line (one of enum cl_type) */

extern long cline_get_type(tml_c_cline this_cline);


    /* get the x position of the upper left corner of bounding box of this */
    /* conceptual line */

extern long cline_get_ulx(tml_c_cline this_cline);


    /* get the y position of the upper left corner of bounding box of this */
    /* conceptual line */

extern long cline_get_uly(tml_c_cline this_cline);


    /* get the x position of the lower left corner of bounding box of this */
    /* conceptual line */

extern long cline_get_lrx(tml_c_cline this_cline);


    /* get the y position of the lower left corner of bounding box of this */
    /* conceptual line */

extern long cline_get_lry(tml_c_cline this_cline);


    /* get the height in pixels of this conceptual line */

extern long cline_get_hi(tml_c_cline this_cline);


    /* get the width in pixels of this conceptual line */

extern long cline_get_wide(tml_c_cline this_cline);


    /* get the page header associated with this conceptual line */

extern tml_c_phead cline_get_phead(tml_c_cline this_cline);


    /* get the format associated with this conceptual line */

extern tml_c_format cline_get_format(tml_c_cline this_cline);


    /* gets the next conceptual line as a tml_c_cline (not virtual) */

extern tml_c_cline cline_inq_next(tml_c_cline this_cline);


    /* gets the prior conceptual line as a tml_c_cline (not virtual) */

extern tml_c_cline cline_inq_prior(tml_c_cline this_cline);


    /* gets the conceptual line below the current conceptual line */
    /* (not virtual) */

extern tml_c_cline cline_get_down(tml_c_cline this_cline);


    /* gets the conceptual line above the current conceptual line */
    /* (not virtual) */

extern tml_c_cline cline_get_up(tml_c_cline this_cline);



    /* TRAITS TO MANIPULATE CONCEPTUAL LINE DATA */


    /* set the page header member of the conceptual line */

extern void cline_set_phead(tml_c_cline this_cline, tml_c_phead on_page);


    /* set the type of the conceptual line to the long argument */
    /* (one of enum cl_type) */

extern void cline_set_type(tml_c_cline this_cline, long new_type);


    /* set the upper left corner coordinates of the bounding box of this */
    /* conceptual line. The first long argument is the x value in pixels; */
    /* the second is the y value. Arguments upper_x and upper_y will be */
    /* truncated to shorts */

extern void cline_set_ul_coord(tml_c_cline this_cline, long upper_x, long upper_y);


    /* set the width member of the conceptual line to the number of pixels */
    /* specified by the long argument, which will be truncated to a short */

extern void cline_set_wide(tml_c_cline this_cline, long set_wide);


    /* set the height member of the conceptual line to the number of pixels */
    /* specified by the long argument, which will be truncated to a short */

extern void cline_set_hi(tml_c_cline this_cline, long new_hi);



    /* TRAITS TO DO THINGS TO A CONCEPTUAL LINE */


    /* splits a free conceptual line in two either horizontally or */
    /* vertically. Vertical splits have a direction of 0; direction values */
    /* greater than zero indicate a horizontal split. If from_top = 0, */
    /* the split will create a new cline of new_size depth (width) */
    /* from the top (left) of the indicated cline. Otherwise the split */
    /* will create a new cline of new_size depth (width) from the */
    /* bottom (right) of the indicated cline. Integer arguments will be */
    /* truncated to shorts. */

extern tml_c_cline cline_split(tml_c_cline this_cline, long new_size, long from_top, long direction);


    /* resizes the height of a conceptual line and all lines below the */
    /* adjusted line to current height plus the long argument (pixels) */
    /* which will be truncated to a short */

extern long cline_resize(tml_c_cline this_cline, long hi_delta);


    /* attach a line before (long before != 0) or after (long before == 0) */
    /* this line-- needs to update the coordinates of the lines below */

extern void cline_attach(tml_c_cline this_cline, tml_c_cline target_line, long before);


    /* detach a line from a conceptual page--needs to update the coordinates */
    /* of the lines below */

extern void cline_detach(tml_c_cline this_cline);



    /* VIRTUAL FUNCTIONS FOR THIS CLASS */


    /* get the next conceptual line (virtual) */

extern tml_c_cline cline_get_next(tml_c_cline this_cline);


    /* set the next pointer of the first argument to the second */

extern void cline_set_next(tml_c_cline this_cline, tml_c_cline next_line);


    /* get the prior conceptual line (virtual) */

extern tml_c_cline cline_get_prior(tml_c_cline this_cline);


    /* set the prior pointer of the first argument to the second */

extern void cline_set_prior(tml_c_cline this_cline, tml_c_cline prior_line);


    /* C_MODEL_POS* argument points to caller-supplied storage; */
    /* the model position at the start of the conceptual line is copied */
    /* into it. If all goes well, the return value equals the second */
    /* argument. */

extern C_MODEL_POS* cline_get_start(tml_c_cline this_cline, C_MODEL_POS* model_pos_p);


    /* C_MODEL_POS* argument points to caller-supplied storage; */
    /* the model position at the end of the conceptual line is copied */
    /* into it. If all goes well, the return value equals the second */
    /* argument. */

extern C_MODEL_POS* cline_get_end(tml_c_cline this_cline, C_MODEL_POS* model_pos_p);


    /* get the starting x offset from first pixel of first character in line */

extern long cline_get_start_x_off(tml_c_cline this_cline);


    /* replace the line represented by the second argument with the line */
    /* represented by by the first argument */

extern void cline_replace(tml_c_cline this_cline, tml_c_cline replace_line);


    /* release the cline argument (delete it) */

extern void cline_release(tml_c_cline this_cline);


    /* allocate some space from this line and link in the line supplied */
    /* (fill_with). If this is not free space, return 0. Otherwise, return */
    /* the free line closest to this left after the allocation. If long */
    /* argument at_bottom = 0, put fill_with at the top (left) of this line, */
    /* else put fill_width at the bottom (left) of the line */

extern tml_c_cline cline_alloc_space(tml_c_cline this_cline, tml_c_cline fill_with, long at_bottom);


    /* C_XY_COORD* argument points to caller-supplied storage */
    /* The xy coordinates of the indicated model position are copied */
    /* into the C_XY_COORD* argument. If all goes well, the return value */
    /* equals the third argument */

extern C_XY_COORD* cline_mpos_2_xy(tml_c_cline this_cline, C_MODEL_POS* find_pos, C_XY_COORD* new_xy_coord);


    /* C_MODEL_POS* argument points to caller-supplied storage */
    /* The model position associated with the C_XY_COORD* argument is copied */
    /* into the C_MODEL_POS* argument. If all goes well, the return value */
    /* equals the third argument. */

extern C_MODEL_POS* cline_xy_2_mpos(tml_c_cline this_cline, C_XY_COORD* where, C_MODEL_POS* new_model_pos);


    /* draw the area for the cline at the xy position indicated by the third */
    /* argument. Return a view_pos if the area is not free, zero otherwise. */

extern tml_c_view_pos cline_draw_line(tml_c_cline this_cline, tml_c_view_port in_viewport, C_SCREEN_XY* where, long start_index);


    /* reformat the line - always called on a derived class object of class */
    /* cline (e.g. tline) */

extern tml_c_cline cline_reformat(tml_c_cline this_cline, C_MODEL_POS *new_start, long *next_ok, long force_start);




/* FOR CLASS TLINE */


    /* allocate a tline; if tml_c_format is nonzero, set the tline's */
    /* format to that value */

extern tml_c_tline tline_class_alloc(tml_c_tline this_tline, tml_c_format this_format);

    /* deallocate a tline */

extern void tline_free(tml_c_tline this_tline);



    /* TRAITS TO ACCESS THE TEXT LINE DATA */


    /* get the tline to the right of this tline */

extern tml_c_cline tline_get_right(tml_c_tline this_tline);


    /* get the tline to the left of this tline */

extern tml_c_cline tline_get_left(tml_c_tline this_tline);


    /* get the font associated with this tline */

extern tml_c_font_tbl tline_get_font(tml_c_tline this_tline);


    /* get the galley depth (in pixels) at which the baseline */
    /* of this tline is located */

extern long tline_get_baseline(tml_c_tline this_tline);


    /* return the number of characters set on this tline */

extern long tline_get_count(tml_c_tline this_tline);



    /* TRAITS TO MANIPULATE THE TEXT LINE DATA */


    /* adjust the baseline for this tline, and all baselines to its right */
    /* (all its right siblings) to the new baseline */

extern long tline_adjust_baseline(tml_c_tline this_tline, long new_baseline_y);


    /* set the galley depth (in pixels) at which the baseline of this tline */
    /* is to be located */

extern void tline_set_baseline(tml_c_tline this_tline, long new_baseline);


    /* set the font to be associated with this tline */

extern void tline_set_font(tml_c_tline this_tline, tml_c_font_tbl new_font);


    /* set the model position at which this tline begins */

extern void tline_set_start(tml_c_tline this_tline, C_MODEL_POS* new_start);


    /* set the number of characters to be formatted on this tline */

extern void tline_set_count(tml_c_tline this_tline, long new_count);


    /* set the width (in pixels) of the formatted text on this tline */

extern void tline_set_text_width(tml_c_tline this_tline, long new_width);


    /* return width of text contained by tml_c_tline argument */

extern long tline_get_text_width(tml_c_tline this_tline);


    /* return width of text contained by tml_c_tline argument and all of */
    /* its right siblings */

extern long tline_get_full_text_width(tml_c_tline this_tline);


    /* set the right member of this tline */

extern void tline_set_right(tml_c_tline this_tline, tml_c_tline new_right);


    /* set the left member of this tline */

extern void tline_set_left(tml_c_tline this_tline, tml_c_tline new_left);




/* FOR CLASS PHEAD */


    /* allocate a page header. If format is nonzero, set the */
    /* phead's format to that value */

extern tml_c_phead phead_class_alloc(tml_c_format to_format);


    /* deallocate a phead */

extern void phead_free(tml_c_phead this_phead);



    /* TRAITS ALLOWING ACCESS TO THE PHEAD DATA FIELDS */


    /* get the page header of the following page */

extern tml_c_phead phead_get_next(tml_c_phead this_phead);


    /* get the page header of the preceding page */

extern tml_c_phead phead_get_prior(tml_c_phead this_phead);


    /* get the page number associated with this phead */

extern long phead_get_page_no(tml_c_phead this_phead);


    /* get the number of the formatted line that begins this page */

extern long phead_get_fline_no(tml_c_phead this_phead);


    /* get the maximum line width */

extern long phead_get_max_line_width(tml_c_phead this_phead);


    /* get the maximum height used */

extern long phead_get_max_used_ht(tml_c_phead this_phead);


    /* get the number of lines formatted on this page */

extern long phead_get_line_cnt(tml_c_phead this_phead);


    /* get the model position associated with the beginning of this page. */
    /* The second argument points to caller-supplied storage. If all goes */
    /* well, the return value equals the second argument. */

extern C_MODEL_POS *phead_get_start(tml_c_phead this_phead, C_MODEL_POS* pos_p);


    /* get the model position associated with the end of this page. The */
    /* second argument points to caller-supplied storage. If all goes */
    /* well, the return value equals the second argument. */

extern C_MODEL_POS *phead_get_end(tml_c_phead this_phead, C_MODEL_POS* pos_p);


    /* get the format associated with this page */

extern tml_c_format phead_get_format(tml_c_phead this_phead);


    /* get value of the format_ok member of this page header */

extern long phead_get_format_ok(tml_c_phead this_phead);


    /* get the height of this page in pixels */

extern long phead_get_hi(tml_c_phead this_phead);


    /* get the width of this page in pixels */

extern long phead_get_wide(tml_c_phead this_phead);


    /* first long argument is height in pixels; second if zero indicates */
    /* search (see below) begins at bottom of page's list of free clines; */
    /* if nonzero indicates search begins at top of that list. Return the */
    /* first free cline in the direction specified in that list that has */
    /* height greater than or equal to the height of this page. */
    /* The long arguments will be truncated to shorts. */

extern tml_c_cline phead_get_freeline(tml_c_phead this_phead, long height, long from_top);


    /* get the formatted line at the top of this phead */

extern tml_c_cline phead_get_tline(tml_c_phead this_phead);


    /* get the formatted line at the bottom of this phead */

extern tml_c_cline phead_get_bline(tml_c_phead this_phead);


    /* inquires whether the page associated with this page header is visible */
    /* on any listed viewport */

extern long phead_is_visible(tml_c_phead this_phead);



    /* TRAITS WHICH ALTER THE PHEAD DATA FIELDS DIRECTLY */


    /* set the format associated with this phead to the value specified */

extern void phead_set_format(tml_c_phead this_phead, tml_c_format new_format);


    /* first long argument is height, second argument is width. Set */
    /* the page's height and width. These long arguments will be */
    /* truncated to shorts. */

extern void phead_set_size(tml_c_phead this_phead, long hi, long wide);


    /* set the model position for the beginning of this page */

extern void phead_set_start_pos(tml_c_phead this_phead, C_MODEL_POS* new_start);


    /* set the model position for the end of this page */

extern void phead_set_end_pos(tml_c_phead this_phead, C_MODEL_POS* new_end);



    /* NAVIGATION AND CREATION FUNCTIONS */


    /* set the "format_ok" member to zero, signalling that page needs to be */
    /* reformatted */

extern void phead_mark_reformat(tml_c_phead this_phead);

    /* if the model doesn't contain any matter to be formatted following */
    /* this page, return zero. Otherwise, if the next page already exists, */
    /* update its start position; if long argument is nonzero, reformat */
    /* the next page and return it. If the next page doesn't yet exist, make */
    /* a new phead, format it if the long argument is nonzero, and */
    /* return it. */

extern tml_c_phead phead_next_page(tml_c_phead this_phead, long reformat);


    /* if the model doesn't contain any matter to be formatted preceding */
    /* this page, return zero. Otherwise, if the prior page already exists, */
    /* update its end position; if the long argument is nonzero, reformat */
    /* the prior page and return it. If the prior page doesn't yet exist, */
    /* make a new phead, format it if the long argument is nonzero, set */
    /* the associated format's first page field to this page, and return */
    /* the new page */

extern tml_c_phead phead_prior_page(tml_c_phead this_phead, long reformat);


    /* Insert a page of material formatted (a) forward from the C_MODEL_POS */
    /* argument if the long argument is nonzero, (b) backward from the */
    /* C_MODEL_POS argument if the long argument is zero. If material is */
    /* formatted forward, insert the new page after this page; if material */
    /* is formatted backward, insert new the page before this page. */

extern tml_c_phead phead_insert_page(C_MODEL_POS* start, long after);


    /* If the page needs reformatting, do so. Then if the argument model */
    /* position isn't formatted on this page, return zero. Otherwise find */
    /* the conceptual line that contains the text associated with this */
    /* model position and return that line. */

extern tml_c_cline phead_mpos_2_cline(tml_c_phead this_phead, C_MODEL_POS* target);



    /* TRAITS THAT MANIPULATE THE CONCEPTUAL LINES ON THE PAGE */


    /* Find the head conceptual line containing the indicated model */
    /* position. If the line is not on the page, return 0. */

extern tml_c_cline phead_mpos_2_hcline(tml_c_phead this_phead, C_MODEL_POS* target);


    /* Return all free lines associated with this page to the free store. */

extern void phead_free_clines(tml_c_phead this_phead);


    /* Integer arguments are x and y coordinates, respectively, in pixels. */
    /* Search for a conceptual line of size greater than or equal to the */
    /* cline argument at x and y coordinates less than or equal to those */
    /* associated with the argument cline. If no such line exists, */
    /* return -1. If such a line exists, split it if necessary to yield a */
    /* line of the same size as the cline argument; set the found line's */
    /* x and y coordinates to the argument values. Replace the argument */
    /* cline with the new one. The long arguments will be truncated */
    /* to shorts. */

extern long phead_line_place(tml_c_phead this_phead, tml_c_cline place_line, long at_x, long at_y);



    /* FORMAT THE PAGE */


    /* new_model_pos (second C_MODEL_POS* argument) points to */
    /* caller-supplied storage. Format the page specified by this_phead, */
    /* starting at the model position specified in argument start. */
    /* If the page is already formatted, and its starting position matches */
    /* the model position specified in start, copy its ending model position */
    /* into new_model_pos. If the page is already formatted, but its */
    /* starting position doesn't match the model position specified in */
    /* start, free the conceptual lines associated with the page. Copy the */
    /* page's ending model position into new_model_pos. If all goes well, */
    /* the return value equals the third argument. */

extern C_MODEL_POS* phead_format_page(tml_c_phead this_phead, C_MODEL_POS* start, C_MODEL_POS* new_model_pos);


    /* like phead_format_page, but formats BACKWARD from start argument */

extern C_MODEL_POS* phead_tamrof_page(tml_c_phead this_phead, C_MODEL_POS* start, C_MODEL_POS* new_model_pos);


    /* like format page, but start indicates the end position of the last */
    /* correctly formatted line on the page before reformatting is to begins. */
    /* The model position returned is the end of the last line reformatted. */

extern C_MODEL_POS* phead_reformat_page(tml_c_phead this_phead, C_MODEL_POS* start, C_MODEL_POS* new_model_pos);




/* FOR CLASS FORMAT */


    /* allocate a format */

extern tml_c_format format_class_alloc(void);


    /* deallocate a format */

extern void format_free(tml_c_format this_format);



    /* TRAITS WHICH ALLOW ACCESS TO THE FORMAT DATA */


    /* get the reference count associated with this format */

extern long format_get_ref_count(tml_c_format this_format);


    /* get the model with which this format is associated */

extern tml_c_model format_get_model(tml_c_format this_format);


    /* get the first page at which this format takes effect */

extern tml_c_phead format_get_first_page(tml_c_format this_format);


    /* get the last page which this format affects */

extern tml_c_phead format_get_last_page(tml_c_format this_format);


    /* If this format is not associated with a model, return zero. */
    /* Otherwise, return the current state in the associated model of the */
    /* property indexed by the second argument */

extern void* format_inq_state(tml_c_format this_format, p_index prop_ind);


    /* If this format is not associated with a model, return zero. */
    /* Otherwise, return the state at the argument node of the property */
    /* indexed by the second argument */

extern void* format_inq_state_at_node(tml_c_format this_format, tml_c_node at_node, p_index prop_ind);


    /* Return page height associated with this format */

extern long format_get_page_hi(tml_c_format this_format);


    /* Return page width associated with this format */

extern long format_get_page_wide(tml_c_format this_format);


    /* Return maximum line width associated with this format */

extern long format_get_max_line_width(tml_c_format this_format);


    /* Return maximum height used on a line */

extern long format_get_max_used_ht(tml_c_format this_format);



    /* TRAITS WHICH ALLOW MANIPULATION OF THE FORMAT DATA */

    /* set the page size associated with this format: first long */
    /* argument is height, second is width. The long arguments are */
    /* truncated to shorts. */

extern void format_set_page_size(tml_c_format this_format, long hi, long wide);


    /* set number of pages to be saved. The long argument will be */
    /* truncated to a short. */

extern void format_set_pages_saved(tml_c_format this_format, long save_num);


    /* set the first page with which this format is associated */

extern void format_set_first_page(tml_c_format this_format, tml_c_phead new_first);


    /* set the last page which this format affects */

extern void format_set_last_page(tml_c_format this_format, tml_c_phead new_last);


    /* set the default for this format of the property indexed by the */
    /* p_index argument to the value indicated by the third argument */

extern long format_set_state(tml_c_format this_format, p_index prop_ind, void* value);


    /* set the value of the property indexed by the p_index argument to zero */

extern long format_unset_state(tml_c_format this_format, p_index prop_ind);


    
    /* TRAITS WHICH MANIPULATE THE FORMAT */


    /* attach a format to a model */

extern void format_attach(tml_c_format this_format, tml_c_model to_model);


    /* detach a format from the model associated with it */

extern void format_detach(tml_c_format this_format);



    /* NAVIGATION TRAITS */


    /* return the page header associated with the model position argument. */
    /* If the position falls on or precedes the first page associated with */
    /* the format, return that page. If the position follows after the last */
    /* page associated with the format, find that page and return it. */
    /* If the position is on the last page associated with the format, */
    /* return that page. Otherwise, start with the first page and reformat */
    /* as necessary to come up with the page on which the argument model */
    /* position is formatted. */

extern tml_c_phead format_mpos_2_phead(tml_c_format this_format, C_MODEL_POS* find_pos);


    /* return the page header associated with the page number indicated by */
    /* the long argument. If no such page exists, return zero. */

extern tml_c_phead format_get_page(tml_c_format this_format, long page_no);




/* FOR CLASS FONT_TABLE */


    /* allocate a font table -- use default graphics system function table  */

extern tml_c_font_tbl font_tbl_class_alloc(void);


    /* allocate a font table -- use specified graphics system function table */

extern tml_c_font_tbl font_tbl_class_alloc_table(void* ftbl);


    /* deallocate a font table */

extern void font_tbl_free(tml_c_font_tbl this_font_tbl);



    /* MEMBER FUNCTIONS TO ACCESS THE DATA */


    /* get the font's name */

extern char* font_tbl_get_name(tml_c_font_tbl this_font_tbl);


    /* get the number of characters in the font's name */

extern long font_tbl_get_name_count(tml_c_font_tbl this_font_tbl);


    /* get the device dependent font id */

extern void* font_tbl_get_handle(tml_c_font_tbl this_font_tbl);


    /* get the height of the font */

extern long font_tbl_get_hi(tml_c_font_tbl this_font_tbl);


    /* get the baseline offset from top of font matrix */

extern long font_tbl_get_baseline(tml_c_font_tbl this_font_tbl);


    /* if the font is fixed-width, return character width. Else return */
    /* the width of the character indicated by the long argument */

extern long font_tbl_get_width(tml_c_font_tbl this_font_tbl, long charac);


    /* get the number of bytes per character */

extern long font_tbl_get_char_size(tml_c_font_tbl this_font_tbl);


    /* get the inter-character spacing associated with the font */

extern long font_tbl_get_icspace(tml_c_font_tbl this_font_tbl);


    /* return the font's current leading in pixels. The default is the font */
    /* height minus the maximum character height. It equals the distance */
    /* from a baseline to the top of the highest possible ascender on the */
    /* line following. Using this default leading value is the equivalent of */
    /* "set-solid" leading in typesetting, but the value is calculated */
    /* differently because we set type (conceptually) down from the upper */
    /* left corner of the bounding box, not up from the baseline. Calls to */
    /* set_leading() with nondefault values will change the font height and */
    /* subsequent calls to get_leading() will reflect this in the leading */
    /* value returned. */


extern long font_tbl_get_leading(tml_c_font_tbl this_font_tbl);



    /* MEMBER FUNCTIONS TO SET THE DATA */


    /* set the font's internal fields to the values associated with the font */
    /* whose name is specified by the char* argument. The long argument */
    /* is the length of that name. */

extern long font_tbl_set_info(tml_c_font_tbl this_font_tbl, char* fname, long flen);


    /* load a font into memory. Zero return value indicates success; */
    /* -1 indicates failure. */

extern long font_tbl_load_font(tml_c_font_tbl this_font_tbl);


    /* change the leading value. See comment under get_leading() above. */

extern long font_tbl_set_leading(tml_c_font_tbl this_font_tbl, long lead_pixels);


    /* set the intercharacter spacing */

extern void font_tbl_set_icspace(tml_c_font_tbl this_font_tbl, long new_icspace);





/**********************************************************************************
    Objects used in the rendering component of tml

**********************************************************************************/


/* FOR CLASS V_CURSOR */


    /* allocate a viewport cursor */

extern tml_c_v_cursor v_cursor_class_alloc(void);

    /* deallocate a viewport cursor */

extern void v_cursor_free(tml_c_v_cursor this_v_cursor);



    /* TRAITS TO ACCESS THE DATA */


    /* get the viewport associated with a v_cursor */

extern tml_c_view_port v_cursor_get_viewport(tml_c_v_cursor this_v_cursor);


    /* C_SCREEN_XY* argument points to caller-supplied storage */
    /* get the uncaged xy coordinates of the v_cursor. If all goes well, */
    /* the return value equals the second argument. */

extern C_SCREEN_XY *v_cursor_get_raw_location(tml_c_v_cursor this_v_cursor, C_SCREEN_XY* xy_p);


    /* C_SCREEN_XY* argument points to caller-supplied storage */
    /* get the caged xy coordinates of the v_cursor. If all goes well, */
    /* the return value equals the second argument. */

extern C_SCREEN_XY *v_cursor_get_caged_location(tml_c_v_cursor this_v_cursor, C_SCREEN_XY* xy_p);


    /* get the viewport position */

extern tml_c_view_pos v_cursor_get_view_pos(tml_c_v_cursor this_v_cursor);


    /* C_SCREEN_XY* argument points to caller-supplied storage */
    /* get the model position at which the v_cursor rests. If all goes well, */
    /* the return value equals the second argument */

extern C_MODEL_POS *v_cursor_get_model_pos(tml_c_v_cursor this_v_cursor, C_MODEL_POS* pos_p);


    /* Returns the visibility of the cursor. If visibility < 1, the cursor */
    /* is not drawn; otherwise the cursor is drawn. */

extern long v_cursor_get_visible(tml_c_v_cursor this_v_cursor);


    /* get function that draws this v_cursor */

extern C_CURSOR_DRAW v_cursor_get_draw_func(tml_c_v_cursor this_v_cursor);


    /* get height of this v_cursor */

extern long v_cursor_get_height(tml_c_v_cursor this_v_cursor);


    /* get width of this v_cursor */

extern long v_cursor_get_width(tml_c_v_cursor this_v_cursor);



    /* TRAITS TO MANIPULATE THE DATA */


    /* attach the argument v_cursor to the argument view_port */

extern void v_cursor_attach(tml_c_v_cursor this_v_cursor, tml_c_view_port to_viewport);


    /* detach the argument v_cursor from the argument view_port */

extern void v_cursor_detach(tml_c_v_cursor this_v_cursor);


    /* Set after_char member to the argument model_pos */

extern void v_cursor_set_model_pos(tml_c_v_cursor this_v_cursor, C_MODEL_POS* new_pos);


    /* If the long argument is zero, increment the visibility of the */
    /* cursor (see v_cursor_get_visible(), above). If the long argument */
    /* is nonzero and the cursor is invisible with visibility < 1, make */
    /* the cursor visible by setting the visibility to 1. Otherwise */
    /* increment the cursor's visiblity. If the cursor's visibility is 0 */
    /* (i.e., just below the threshhold of invisibility) the latter case */
    /* will make the cursor visible. */

extern long v_cursor_enable(tml_c_v_cursor this_v_cursor, long force);


    /* If the long argument is zero, decrement the visibility of the */
    /* cursor (see v_cursor_get_visible(), above). If the long argument */
    /* is nonzero and the cursor visibile with visibility > 1, make the */
    /* cursor invisible by setting the visibility to 0. Otherwise decrement */
    /* the cursor's visibility. If the cursor's visibility is 1 (i.e., just */
    /* above the threshhold of visibility) the latter case will make the */
    /* cursor invisible. */

extern long v_cursor_disable(tml_c_v_cursor this_v_cursor, long force);


    /* sets the visibility flag to the indicated value. 0 means that the */
    /* cursor is not visible, 1 means that the cursor is visible. */
    /* NOTE--this does not draw or erase the cursor */

extern void v_cursor_set_visible(tml_c_v_cursor this_v_cursor, long is_visible);


    /* Draws the cursor at the current caged position. Note that this */
    /* function DOES NOT update any of the other cursor information other */
    /* than cursor visibility. */
    /* Returns 1 if successful, 0 if unsuccessful */

extern long v_cursor_draw(tml_c_v_cursor this_v_cursor);


    /* Erases the cursor at the current caged position. Note that this */
    /* function DOES NOT update any of the other cursor information other */
    /* than cursor visibility. Returns 1 if successful, 0 if unsuccessful*/

extern long v_cursor_erase(tml_c_v_cursor this_v_cursor);


    /* set the function that draws the cursor. */

extern void v_cursor_set_draw_func(tml_c_v_cursor this_v_cursor, C_CURSOR_DRAW new_func);


    /* C_SCREEN_XY* argument points to caller-supplied storage. */
    /* Move the cursor in the direction indicated by the long argument */
    /* (one of enum vp_dir). Returns a pointer to the screen position */
    /* at which the cursor rests after the move. If all goes well, the */
    /* return value equals the third argument. */

extern C_SCREEN_XY *v_cursor_move(tml_c_v_cursor this_v_cursor, vp_dir move_dir, C_SCREEN_XY* new_screen_xy);


    /* move the cursor to the viewport xy indicated by the second argument */

extern long v_cursor_xy_set(tml_c_v_cursor this_v_cursor, C_SCREEN_XY* position);


    /* move the cursor to the model position indicated by the second argument */

extern long v_cursor_mpos_set(tml_c_v_cursor this_v_cursor, C_MODEL_POS* pos_p);




/* FOR CLASS VIEW_PORT */


    /* allocate a viewport */

extern tml_c_view_port view_port_class_alloc(void);


    /* deallocate a viewport */

extern void view_port_free(tml_c_view_port this_view_port);



    /* TRAITS TO ACCESS THE DATA */


    /* return the bitmap handle associated with the viewport */

extern void* view_port_get_handle(tml_c_view_port this_view_port);


    /* get the width of the viewport */

extern long view_port_get_width(tml_c_view_port this_view_port);


    /* get the height of the viewport */

extern long view_port_get_hi(tml_c_view_port this_view_port);


    /* get horizontal offset of viewport on conceptual page */

extern long view_port_get_hoffset(void *view_port_p);


    /* get top line visible in the viewport */

extern tml_c_view_pos view_port_get_top_line(tml_c_view_port this_view_port);


    /* get bottom line visible in the viewport */

extern tml_c_view_pos view_port_get_bottom_line(tml_c_view_port this_view_port);


    /* get the format associated with the viewport */

extern tml_c_format view_port_get_format(tml_c_view_port this_view_port);


    /* get the cursor associated with the viewport */

extern tml_c_v_cursor view_port_get_cursor(tml_c_view_port this_view_port);


    /* get the id of the foreground color associated with the viewport */

extern long view_port_get_foreg_id(tml_c_view_port this_view_port);


    /* get the id of the background color associated with the viewport */

extern long view_port_get_backg_id(tml_c_view_port this_view_port);


    /* get the state of the property indexed by the long argument. */
    /* If there is an entry for this property in the viewport's property */
    /* state table, return it. Otherwise, if a format is associated with */
    /* the viewport return the state of the property in the format's */
    /* property state table (which defaults to the model's property */
    /* state table setting when no additional setting is made in the */
    /* format), but if no format is associated with the viewport, */
    /*  return zero. */

extern void* view_port_inq_state(tml_c_view_port this_view_port, p_index prop_ind);


    /* get the state of the property indexed by the long argument at the */
    /* argument node. */
    /* If the viewport's property state table has an entry for this */
    /* property, return its state. Otherwise return the associated format's */
    /* property state table entry (which defaults to the model's property */
    /* state setting), but if no format is associated with the viewport, */
    /* return 0. */

extern void* view_port_inq_state_at_node(tml_c_view_port this_view_port, tml_c_node at_node, p_index prop_ind);


    /* get the draw marks associaed with the viewport */

extern void *view_port_get_marks(tml_c_view_port view_port_p);



    /* TRAITS TO SET DATA */


    /* assign the pre-allocated bitmap to the viewport, fill in the width */
    /* and height. This function assumes that the viewport fills the bitmap. */


extern long view_port_set_handle(tml_c_view_port this_view_port, tml_c_bitmap_id bitmap_id);


    /* attach a format to the viewport. If the format argument is zero, */
    /* create a new one; width and/or depth of zero means that the */
    /* viewport's dimension(s) (is/are) used to initialize the format's */
    /* corresponding dimension(s). Returns -1 if allocation is necessary */
    /* and it fails; returns 0 otherwise */

extern long view_port_attach_format(tml_c_view_port this_view_port, tml_c_format to_format, long fx_size, long fy_size);


    /* detach a format from the viewport. If the format's reference count */
    /* becomes zero in doing so, the format is deleted. */

extern void view_port_detach_format(tml_c_view_port this_view_port);


    /* attach a viewport cursor to the viewport. If the argument cursor */
    /* is attached to another viewport, an error is returned. If the */
    /* argument C_SCREEN_XY* is NULL, the cursor is set to the upper */
    /* left hand corner of the screen (coordinates {0,0}); otherwise the */
    /* coordinates supplied by the C_SCREEN_XY* argument are used. */

extern long view_port_attach_cursor(tml_c_view_port this_view_port, tml_c_v_cursor to_cursor, C_SCREEN_XY* set_to);


    /* detach a cursor from the viewport */

extern void view_port_detach_cursor(tml_c_view_port this_view_port);


    /* set the top line of the viewport to be the argument view position */

extern void view_port_set_top_line(tml_c_view_port this_view_port, tml_c_view_pos new_top);


    /* set the bottom line of the viewport to be the argument view position */

extern void view_port_set_bottom_line(tml_c_view_port this_view_port, tml_c_view_pos new_bottom);


    /* set the foreground color of the viewport to the color identified */
    /* by the long argument */

extern void view_port_set_foreg_id(tml_c_view_port this_view_port, long new_id);


    /* set the background color of the viewport to the color identified */
    /* by the long argument */

extern void view_port_set_backg_id(tml_c_view_port this_view_port, long new_id);


    /* set the default state of the indexed property in the viewport's */
    /* property state table to the value pointed to by the void* argument. */

extern long view_port_set_state(tml_c_view_port this_view_port, p_index prop_ind, void* value);


    /* cancel property override at the viewport level for the property */
    /* indexed by the p_index argument */

extern long view_port_unset_state(tml_c_view_port this_view_port, p_index prop_ind);


    /* add the argument draw_list to the viewport */

extern void view_port_add_mark(tml_c_view_port view_port_p, tml_c_draw_mark new_mark);


    /* detach the argument draw_list from the viewport */

extern void view_port_detach_mark(tml_c_view_port view_port_p, tml_c_draw_mark del_mark);



/* ROUTINES WHICH ALLOCATE AND FREE VIEW POSITIONS IN THE VIEWPORT */


    /* allocate a view position from the view position store for this */
    /* viewport. If none are available, add some and return one. */

extern tml_c_view_pos view_port_get_pos(tml_c_view_port this_view_port);


    /* return a view_pos to the free heap for the viewport, initializing */
    /* the contents. When a view_pos is freed, all view_pos to the right */
    /* of the free view_pos are also freed. */

extern void view_port_free_pos(tml_c_view_port this_view_port, tml_c_view_pos to_free);


    /* free all the view_pos associated with a viewport. */

extern void view_port_free_all_pos(tml_c_view_port this_view_port);



    /* ROUTINES THAT ACTUALLY DRAW THINGS */


    /* clear (an area of) the viewport. First C_SCREEN_XY* is upper left */
    /* corner of area to be cleared; second is lower right corner. If both */
    /* C_SCREEN_XY* arguments are set to &zero_zero, the whole viewport is */
    /* cleared; if only the second C_SCREEN_XY* is set to &zero_zero, the */
    /* viewport is cleared from the first coordinate to the bottom edge */
    /* of the viewport. */

extern void view_port_clear(tml_c_view_port this_view_port, C_SCREEN_XY* ul, C_SCREEN_XY* lr);


    /* draw the full viewport, starting with the given conceptual line */

extern void view_port_draw(tml_c_view_port this_view_port, tml_c_cline first_line, C_SCREEN_XY* where);


    /* draw the screen, putting the character specified by the */
    /* C_MODEL_POS* argument at (or as close as possible to) the */
    /* coordinates supplied in the C_SCREEN_XY* argument */

extern void view_port_draw_at(tml_c_view_port this_view_port, C_MODEL_POS* draw_char, C_SCREEN_XY* where);


    /* redraw the viewport */

extern void view_port_redraw(tml_c_view_port this_view_port);


    /* redraw the viewport, reformatting as we go, from the supplied */
    /* view_pos until no more is needed. ref_start_mpos indicates the model */
    /* position which begins the conceptual line which is the start of the */
    /* view pos contents, rend_start_mpos indicates the model position */
    /* at which rerendering should start*/

extern void view_port_redraw_at(tml_c_view_port this_view_port, C_MODEL_POS* ref_start_mpos, C_MODEL_POS* rend_start_mpos, tml_c_view_pos start_vpos);



    /* NAVIGATION ROUTINES */


    /* convert from the xy coordinates supplied in the argument */
    /* C_SCREEN_XY* to the view_pos occupying that space in the viewport. */
    /* If no view_pos occupies the space, get the closest one. */

extern tml_c_view_pos view_port_xy_2_vpos(tml_c_view_port this_view_port, C_SCREEN_XY* where);


    /* convert from the xy coordinates supplied in the argument */
    /* C_SCREEN_XY* to the cline occupying that space in the viewport. */
    /* If no cline occupies the space, get the closest one. */

extern tml_c_cline view_port_xy_2_cline(tml_c_view_port this_view_port, C_SCREEN_XY* where);


    /* C_MODEL_POS* argument points to caller-supplied storage */
    /* Returns a pointer to the model position associated with the screen */
    /* position argument. If all goes well, the return value equals the */
    /* third argument. */

extern C_MODEL_POS *view_port_xy_2_mpos(tml_c_view_port this_view_port, C_SCREEN_XY* where, C_MODEL_POS* new_model_pos);


    /* find the first view_pos containing the conceptual line. If the line */
    /* is not on the screen, return 0. */

extern tml_c_view_pos view_port_cline_2_vpos(tml_c_view_port this_view_port, tml_c_cline find_line);


    /* C_SCREEN_XY* argument points to caller-supplied storage */
    /* Returns a pointer to the screen position associated with the cline */
    /* argument. If all goes well, the return value equals the third */
    /* argument. */

extern C_SCREEN_XY *view_port_cline_2_xy(tml_c_view_port this_view_port, tml_c_cline find_line, C_SCREEN_XY* new_screen_xy);


    /* return the view_pos containing the supplied character if that */
    /* character is on the screen. Otherwise return zero. */

extern tml_c_view_pos view_port_mpos_2_vpos(tml_c_view_port this_view_port, C_MODEL_POS* find_char);


    /* C_SCREEN_XY* argument points to caller-supplied storage */
    /* Returns a pointer to the screen position associated with the model */
    /* position argument. If all goes well, the return value equals the */
    /* third argument. If the tml_c_view_pos argument is zero, or if the */
    /* the routine can't find the character indicated by the model position */
    /* argument at a supplied view position, mpos_2_vpos is called to */
    /* find the correct view position. Returns a pointer to */
    /* C_SCREEN_XY {x=0, y=-1} if the character is not on the screen. */

extern C_SCREEN_XY *view_port_mpos_2_xy(tml_c_view_port this_view_port, C_MODEL_POS* find_char, C_SCREEN_XY* new_view_pos, tml_c_view_pos hint);

    /* returns 1 if the page is currently visible in the viewport, */
    /* 0 otherwise */

extern long view_port_is_visible(tml_c_view_port this_view_port, tml_c_phead page);



    /* EDITING ROUTINES */


    /* append string in_string of length count at model position after, */
    /* reformatting and rendering as needed. */

extern long view_port_append_string(tml_c_view_port this_view_port, char* in_string, long count, C_MODEL_POS* after);


    /* prepend string in_string of length count to character at model */
    /* position before, reformatting and rendering as needed. */

extern long view_port_prepend_string(tml_c_view_port this_view_port, char* in_string, long count, C_MODEL_POS* before);


    /* delete the character at the model position, reformatting and */
    /* rendering as needed */

extern long view_port_delete_char(tml_c_view_port this_view_port, C_MODEL_POS* delete_pos);


    /* insert string in_string of length count before the current cursor */
    /* position, reformatting and rerendering as needed. If the cursor */
    /* position is invalid, insert before the first character dominated */
    /* by root. */

extern long view_port_ins_before_cursor(tml_c_view_port this_view_port, char* in_string, long count);


    /* insert string in_string of length count after the current cursor */
    /* position, reformatting and rendering as needed. If the cursor */
    /* position is invalid, insert before the first character dominated */
    /* by root. */

extern long view_port_ins_after_cursor(tml_c_view_port this_view_port, char* in_string, long count);


    /* append a property vector element referencing the argument property */
    /* to the property vector for the argument node. Returns 1 on success, */
    /* zero otherwise */

extern long view_port_insert_prop_at_node(tml_c_view_port this_view_port, tml_c_node* to_node, tml_c_property add_prop);


    /* perform view_port_insert_prop_at_node on a new node created to dominate */
    /* all and only the characters within the given pair of model positions. */
    /* Refresh viewport if necessary. Returns 1 on success, zero otherwise. */

extern long view_port_insert_prop_on_range(tml_c_view_port view_port_p, C_MODEL_POS *pos1, C_MODEL_POS *pos2, tml_c_property add_prop);


    /* delete a property from a node; refresh viewport if anything dominated */
    /* by that node is visible. Return 1 on success, zero otherwise. */

extern long view_port_delete_prop(tml_c_view_port this_view_port, tml_c_property del_prop, tml_c_node from_node);


    /* mark a region in the viewport. Make a range in the model, make a */
    /* model_mark, make a draw_mark, and then add the mark to this viewport */

extern long view_port_mark_region(tml_c_view_port this_view_port, C_MODEL_POS* start_mark, C_MODEL_POS* end_mark);
                                     



/* FOR CLASS VIEW_POS */


    /* allocate a view_pos */

extern tml_c_view_pos view_pos_class_alloc(void);


    /* deallocate a view_pos */

extern void view_pos_free(tml_c_view_pos this_view_pos);



    /* TRAITS TO ACCESS THE DATA */


    /* get the prior view_pos */

extern tml_c_view_pos view_pos_get_prior(tml_c_view_pos this_view_pos);


    /* get the next view_pos */

extern tml_c_view_pos view_pos_get_next(tml_c_view_pos this_view_pos);


    /* get the view_pos above this one */

extern tml_c_view_pos view_pos_get_up(tml_c_view_pos this_view_pos);


    /* get the view_pos below this one */

extern tml_c_view_pos view_pos_get_down(tml_c_view_pos this_view_pos);


    /* get the viewport with which this view_pos is associated */

extern tml_c_view_port view_pos_get_viewport(tml_c_view_pos this_view_pos);


    /* get the x coordinate of the upper left corner of this view_pos */

extern long view_pos_get_x(tml_c_view_pos this_view_pos);


    /* get the y coordinate of the upper left corner of this view_pos */

extern long view_pos_get_y(tml_c_view_pos this_view_pos);


    /* get the conceptual line contained by this view_pos */

extern tml_c_cline view_pos_get_cline(tml_c_view_pos this_view_pos);


    /* C_MODEL_POS* argument points to caller-supplied storage. */
    /* Returns pointer to model position associated with start of this */
    /* view position. If all goes well, the return value equals the second */
    /* argument. */

extern C_MODEL_POS* view_pos_get_start(tml_c_view_pos this_view_pos, C_MODEL_POS* pos_p);


    /* C_MODEL_POS* argument points to caller-supplied storage. */
    /* Returns pointer to model position associated with end of this */
    /* view position. If all goes well, the return value equals the second */
    /* argument. */

extern C_MODEL_POS* view_pos_get_end(tml_c_view_pos this_view_pos, C_MODEL_POS* pos_p);



    /* TRAITS TO MANIPULATE THE DATA */


    /* set the x coordinate of the upper left corner of this view_pos */

extern void view_pos_set_x(tml_c_view_pos this_view_pos, long new_x);


    /* set the y coordinate of the upper left corner of this view_pos */

extern void view_pos_set_y(tml_c_view_pos this_view_pos, long new_y);


    /* cline argument points to storage not to be reused */

extern void view_pos_set_line(tml_c_view_pos this_view_pos, tml_c_cline new_line);



    /* MISCELLANEOUS FUNCTIONS */


    /* disassociate the view_pos from any others it is attached to */

extern void view_pos_detach(tml_c_view_pos this_view_pos);


    /* set the view_pos's viewport member to the argument viewport */

extern void view_pos_attach_port(tml_c_view_pos this_view_pos, tml_c_view_port to_viewport);


    /* attach the first view_pos to the second in the direction indicated */
    /* (one of enum vp_dir). If all goes well, return 0; otherwise */
    /* return -1. */

extern long view_pos_attach_pos(tml_c_view_pos this_view_pos, tml_c_view_pos to_pos, vp_dir dir);


    /* attach a new view_pos to this one in the direction indicated */
    /* (one of enum vp_dir). If all goes well, return the new view position; */
    /* otherwise return 0. */

extern tml_c_view_pos view_pos_add(tml_c_view_pos this_view_pos, vp_dir dir);


    /* adjust the xy positions of a set of view positions. First argument */
    /* is x delta, second is y delta (pixels). A "scrolling" function. */

extern void view_pos_adjust(tml_c_view_pos this_view_pos, long x_delta, long y_delta);


    /* return 1 if the model position argument is found between the */
    /* beginning and end of this view position (inclusive). */
    /* Return 0 otherwise. */

extern long view_pos_contains(tml_c_view_pos this_view_pos, C_MODEL_POS* find_pos);

    


/* UTILITY FUNCTIONS */


    /* get the type identifier of the default graphics system */

extern tml_c_grafsys tml_c_tml_dp_get_dflt_grafsys_type(void);


    /* get the function table for the default graphics system */

extern tml_c_vpftbl tml_c_tml_dp_get_dflt_vpftbl(void);


    /* set the function table for the default graphics system to the value supplied */

extern void tml_c_tml_dp_set_dflt_vpftbl(tml_c_vpftbl ftbl);


    /* get the XWindows viewport function table */

extern tml_c_vpftbl tml_c_tml_dp_get_x_vpftbl(void);


#ifdef apollo
    /* get the gpr viewport function table */

extern tml_c_vpftbl tml_c_tml_dp_get_gpr_vpftbl(void);
#endif


  /* from the graphics system function table given, get the function associated with the index supplied */

extern GEN_FUNCP tml_c_get_graf_func_from_table(tml_c_vpftbl ftbl, long index);


  /* allocate a new property vector in a model */

extern tml_c_prop_vec tml_c_pvec_alloc(tml_c_model to_model);


    /* allocate a new property in a model */

extern tml_c_property tml_c_prop_alloc(tml_c_model to_model);


    /* allocate a new argument in a model */

extern tml_c_argument tml_c_arg_alloc(tml_c_model to_model);


    /* given a start node (first argument) and an end node (second argument), */
    /* isolate a coherent subtree subsuming all text from start to end, and */
    /* create any nodes necessary to preserve properties as far up the tree */
    /* as necessary. Assumes that nodes provided are whole leaf nodes. */
    /* Returns the root of the isolated subtree. */

extern tml_c_node tml_c_make_subtree(tml_c_node start_node, tml_c_node end_node);


    /* copy text from start point (indicated by first C_MODEL_POS* argument) */
    /* to end point (second C_MODEL_POS* argument) to the argument model. */
    /* Startpoint and endpoint, as they are model positions, each consist of */
    /* a node and character index. All subtree structure and properties are */
    /* preserved at the target. The target model may be another text file */
    /* being edited, or it may be a paste buffer. This function does not */
    /* connect the copied subtree into the target model; must call */
    /* tml_c_node_connect_node() to do this. Returns root of copied subtree */
    /* in target model. */

extern tml_c_node tml_c_copy_component(C_MODEL_POS* start, C_MODEL_POS* end, tml_c_model to_model);


    /* create and return a pointer to a word wrap property, with the value */
    /* indicated by the integer argument, in the argument model */

extern tml_c_property tml_c_make_wrap_prop(long wrap_on, tml_c_model in_model);


    /* create and return a pointer to a margin property, with the values */
    /* indicated, in the argument model. First long argument is left */
    /* margin, second is right, third top, fourth bottom. */

extern tml_c_property tml_c_make_margin_prop(long left, long right, long top, long bottom, tml_c_model in_model);


    /* create and return a pointer to an invert property, with the value */
    /* indicated, in the given model */

extern tml_c_property tml_c_make_inv_prop(long inv_on, tml_c_model in_model);


    /* create and return a pointer to a font property, with the values */
    /* indicated, in the given model. Argument char* is font name, integer */
    /* argument is name length. */

extern tml_c_property tml_c_make_font_prop(char* font_name, long name_len, tml_c_model in_model);


    /* create and return a pointer to a font property, with the values */
    /* indicated, in the given model. Argument char* is font name, integer */
    /* argument is name length. The tml_c_grafsys argument specifies */
    /* a graphics system. */

extern tml_c_property tml_c_make_font_prop_grafsys(char* font_name, long name_len, tml_c_model in_model, tml_c_grafsys for_grafsys);


    /* create and return a pointer to a page size property, with the height */
    /* and width indicated, in the given model. */

extern tml_c_property tml_c_make_page_size_prop(long width, long depth, tml_c_model in_model);


    /* create and return a pointer to a tab property, a linked list */
    /* of tab stops, in the given model */

extern tml_c_property tml_c_make_tab_prop(long default_val, long num_stops, long *tab_st_list, tml_c_model in_model);


    /* connect a property to a node at the end of that node's property vector */

extern void tml_c_connect_prop(tml_c_property add_prop, tml_c_node to_node);


    /* linearize the goo in a model - this function replaces the model's */
    /* goo heaps with new goo heaps in which the goo is compressed and */
    /* linearized. This function is called by prune() (see below) so there */
    /* is no need to linearize goo in a separate step if that function */
    /* is called. This function deletes all history associated with the */
    /* model, linearizes goo, then restores the size of the history stack */
    /* to what it was before this function was called. */

extern void tml_c_gc_linearize(tml_c_model old_model);


    /* compress objects and linearize goo - this function calls */
    /* tml_c_gc_linearize() (see above) - it also copies to a new model */
    /* unique copies of all objects still active in the model, compressing */
    /* out unused objects. The compressed goo heaps attached to the old */
    /* model are moved to the new model. */

extern tml_c_model tml_c_prune(tml_c_model old_model);


    /* read a model from the file indicated by the system-dependent */
    /* file handle. Return a pointer to the new model */

extern tml_c_model tml_c_d_read_model_handle(void *file_handle);


    /* read a model from the file indicated by the pathname and pathname */
    /* length arguments. Return a pointer to the new model */

extern tml_c_model tml_c_d_read_model_path(char *pathname, long name_len);


    /* read a model from the file indicated by the pathname and pathname */
    /* length arguments. The oflag and mode arguments are system-dependent */
    /* quantities. When the function returns, the contents of the */
    /* file_handle pointer argument is the system-dependent file_handle */
    /* associated with the file, which remains open. */

extern tml_c_model tml_c_d_read_model_open(char *pathname, long name_len, long oflag, long mode, void* *file_handle_p);


    /* read the ascii file indicated by the argument filename and name */
    /* length; make a model of the file's contents and return a pointer */
    /* to the model. */

extern tml_c_model tml_c_read_uasc_path(char *pathname, long name_len);


    /* read the ascii file indicated by the system-dependent file-handle; */
    /* make a model of the file's contents and return a pointer to the model */

extern tml_c_model tml_c_read_uasc_handle(void *file_handle);


    /* produce a postscript file from a tml model. The output filename is */
    /* outfilename of length outlen. */

extern long tml_c_tml_model_print(tml_c_model this_model, C_PRINT_DB *print_params, char* outfilename, long outlen);


    /* produce a postscript file from a tml file. The input filename is */
    /* infilename of length inlen; the output filename is outfilename of */
    /* length outlen. */

extern long tml_c_tml_file_print(char* infilename, long inlen, C_PRINT_DB *print_params, char* outfilename, long outlen);


    /* update history when a call to prop_vec_detach detaches the only */
    /*   remaining prop_vec in a vector. */
    /* Call this function when a call to prop_vec_detach() returns 0. */
    /* Argument "on_node" is the node whose properties member contained */
    /*   only the one prop_vec "removed". */

extern void tml_c_first_pvec_hist(tml_c_node on_node, tml_c_prop_vec removed);


    /* update history when a call to argument_detach detaches the only */
    /*   remaining argument in a property object's entry argument  vector. */
    /* Call this function when a call to argument_detach() on the entry */
    /*   argument vector of a property returns 0. */
    /* Argument "on_prop" is the property whose entry_arg member contained */
    /*   only the one argument "removed". */

extern void tml_c_first_entry_arg_hist(tml_c_property on_prop, tml_c_argument removed);


    /* update history when a call to argument_detach detaches the only */
    /*   remaining argument in a property object's exit argument vector. */
    /* Call this function when a call to argument_detach() on the exit */
    /*   argument vector of a property returns 0. */
    /* Argument "on_prop" is the property whose exit_arg member contained */
    /*   only the one argument "removed". */

extern void tml_c_first_exit_arg_hist(tml_c_property on_prop, tml_c_argument removed);


#else

/* FROM HERE TO next-to-last "#endif", declarations for programmers using vanilla C       */
/******************************************************************************
   typedefs for C++ classes

******************************************************************************/


typedef char *tml_c_node;

typedef char *tml_c_bnode;

typedef char *tml_c_gnode;

typedef char *tml_c_draw_mark;

typedef char *tml_c_model;

typedef char *tml_c_prop_vec;

typedef char *tml_c_property;

typedef char *tml_c_argument;

typedef char *tml_c_history;

typedef char *tml_c_prop_func_ent;

typedef char *tml_c_format;

typedef char *tml_c_cline;

typedef char *tml_c_tline;

typedef char *tml_c_phead;

typedef char *tml_c_font_tbl;

typedef char *tml_c_view_pos;

typedef char *tml_c_v_cursor;

typedef char *tml_c_view_port;

typedef char *tml_c_bitmap_id;

typedef char *tml_c_vpftbl;

typedef char *tml_c_grafsys;

/******************************************************************************
   typedefs and instances for extensibility

******************************************************************************/


typedef char *node_type;

typedef char *p_index;

typedef char *arg_type;

extern node_type undefined_node, bobj, branch, gobj, pgobj, leaf, is_free;

extern p_index undefined_prop, tml_wrap_prop, tml_margin_prop,
               tml_font_prop, tml_inv_prop, tml_pgsize_prop,
               tml_header_prop, tml_footer_prop, tml_form_feed_prop,
               tml_char_set_prop, tml_union_char_set_prop, tml_tab_prop;

extern arg_type int_val_arg, float_val_arg, string_val_arg, no_val_arg,
                subtree_val_arg, char_set_val_arg, union_char_set_val_arg;

/******************************************************************************
    constants used in the classes

******************************************************************************/


/* BOOLEAN REFERENCES FOR PROPERTY VALUES */


#define tml_true_prop 1

#define tml_false_prop 0
    


/******************************************************************************
    Enumerated types

******************************************************************************/

/* ENUMERATED CLASS OF DIRECTIONS FROM ONE NODE TO ANOTHER */
/* enum node_dir {n_up, n_right, n_left, n_down}; */
#define n_up    0
#define n_right 1
#define n_left  2
#define n_down  3
typedef long node_dir;



/* ENUMERATED CLASS OF TYPES OF VALUES FOR TML VERSION 1 CLASS ARGUMENT */
/* enum arg_type_v1 {int_val, float_val, string_val, no_val}; */
#define int_val    0
#define float_val  1
#define string_val 2
#define no_val     3
typedef long arg_type_v1;



/* ENUMERATED CLASS OF TYPES OF CONCEPTUAL LINES */
/* enum  cl_type {cl_free, cl_text, cl_graphic, cl_blank, cl_gutter, cl_other}; */
#define cl_free    0
#define cl_text    1
#define cl_graphic 2
#define cl_blank   3
#define cl_gutter  4
#define cl_other   5
typedef long cl_type;



/* ENUMERATED CLASS OF THE DIRECTIONS THE CURSOR CAN MOVE */
/* enum vp_dir {vp_right, vp_left, vp_up, vp_down,
                vp_begin_line, vp_end_line, vp_top, vp_bottom}; */
#define vp_right      0
#define vp_left       1
#define vp_up         2
#define vp_down       3
#define vp_begin_line 4
#define vp_end_line   5
#define vp_top        6
#define vp_bottom     7
typedef long vp_dir;



/* ENUMERATED CLASS OF PRINTING ORIENTATIONS */
/* enum orientation {portrait, landscape} */
#define portrait      0
#define landscape     1
typedef long orientation;



/* ENUMERATED CLASS OF MARGIN LOCATIONS */
/* enum marg_loc {top_marg, bottom_marg, left_marg, right_marg}; */
#define top_marg      0
#define bottom_marg   1
#define left_marg     2
#define right_marg    3
typedef long marg_loc;



/* ENUMERATED CLASS OF WRAP PROPERTY TYPES */
/* enum wrap_type {no_wrap, char_wrap, word_wrap}; */
#define no_wrap       0
#define char_wrap     1
#define word_wrap     2
typedef long wrap_type;



/******************************************************************************
    Special initializer for users of TML with other languages
******************************************************************************/
    /* ARGS:  */

extern  tml_init();



/******************************************************************************
    Generic function pointer
******************************************************************************/

typedef char *(*GEN_FUNCP)();



/******************************************************************************
    Structures used for general purposes with allocator and
    deallocator for each where necessary
******************************************************************************/


/* STRUCTURE USED TO CAPTURE CONTENTS OF A MEMBER OF CLASS MODEL_POS */
/* DEFINED IN tml_c_structs.h */

    /* allocate a C_MODEL_POS */
    /* ARGS:  */

extern C_MODEL_POS *c_model_pos_alloc();

    /* deallocate a C_MODEL_POS */
    /* ARGS: C_MODEL_POS *this_c_model_pos */

extern          c_model_pos_free();



/* STRUCTURE USED TO CAPTURE CONTENTS OF A MEMBER OF CLASS MODEL_RANGE */
/* DEFINED IN tml_c_structs.h */

    /* allocate a C_MODEL_RANGE */
    /* ARGS:  */

extern C_MODEL_RANGE *c_model_range_alloc();

    /* deallocate a C_MODEL_RANGE */
    /* ARGS: C_MODEL_RANGE *this_c_model_range */

extern            c_model_range_free();



/* STRUCTURE USED TO CAPTURE CONTENTS OF A MEMBER OF CLASS MARK_LIST */
/* DEFINED IN tml_c_structs.h */

    /* allocate a C_MARK_LIST */
    /* ARGS:  */

extern C_MARK_LIST *c_mark_list_alloc();

    /* deallocate a C_MARK_LIST */
    /* ARGS: C_MARK_LIST *this_c_mark_list */

extern          c_mark_list_free();



/* STRUCTURE USED TO CAPTURE CONTENTS OF A MEMBER OF CLASS XY_COORD */
/* DEFINED IN tml_c_structs.h */
 
    /* allocate a C_XY_COORD */
    /* ARGS:  */

extern C_XY_COORD *c_xy_coord_alloc();

    /* deallocate a C_XY_COORD */
    /* ARGS: C_XY_COORD *this_c_xy_coord */

extern         c_xy_coord_free();



/* STRUCTURE USED TO CAPTURE CONTENTS OF A MEMBER OF CLASS SCREEN_XY */
/* DEFINED IN tml_c_structs.h */

    /* UPPER LEFT CORNER OF A VIEWPORT */

    extern C_SCREEN_XY zero_zero;

    /* allocate a C_SCREEN_XY */
    /* ARGS:  */

extern C_SCREEN_XY *c_screen_xy_alloc();

    /* deallocate a C_SCREEN_XY */
    /* ARGS: C_SCREEN_XY *this_c_screen_xy */

extern          c_screen_xy_free();



/* STRUCTURE USED TO SPECIFY PRINTING PARAMETERS */
/* DEFINED IN tml_c_structs.h */

    /* allocate a C_PRINT_DB */
    /* ARGS:  */

extern C_PRINT_DB *c_print_db_alloc();

    /* deallocate a C_PRINT_DB */
    /* ARGS: C_PRINT_DB *this_c_print_db */

extern         c_print_db_free();



/* STRUCTURE FOR CAPTURING THE CONTENTS OF A MEMBER OF CLASS SEARCH_ENV */
/* DEFINED IN tml_c_structs.h */

    /* allocate a C_SEARCH_ENV */
    /* ARGS:  */

extern C_SEARCH_ENV *c_search_env_alloc();

    /* deallocate a C_SEARCH_ENV */
    /* ARGS: C_SEARCH_ENV *this_search_env */

extern           c_search_env_free();




/******************************************************************************
    Objects used in the modelling component of tml

******************************************************************************/


/* CLASS MODEL_POS */

    /* allocate a model_pos from free store */
    /* ARGS:  */

extern C_MODEL_POS *model_pos_class_alloc();


    /* deallocate a model_pos */
    /* ARGS: C_MODEL_POS *this_model_pos */

extern  model_pos_free();


    /* return 1 if first position is same as second, 0 otherwise */
    /* ARGS: C_MODEL_POS* this_model_pos, C_MODEL_POS* comp_pos */

extern long model_pos_eq();


    /* return 1 if first position is not the same as the second, 0 otherwise */
    /* ARGS: C_MODEL_POS* this_model_pos, C_MODEL_POS* comp_pos */

extern long model_pos_neq();


    /* return 1 if first position is less than second, 0 otherwise */
    /* ARGS: C_MODEL_POS* this_model_pos, C_MODEL_POS* comp_pos */

extern long model_pos_lt();


    /* return 1 if first position is less than or equal to second, */
    /* 0 otherwise */
    /* ARGS: C_MODEL_POS* this_model_pos, C_MODEL_POS* comp_pos */

extern long model_pos_le();


    /* return 1 if first position is greater than second, 0 otherwise */
    /* ARGS: C_MODEL_POS* this_model_pos, C_MODEL_POS* comp_pos */

extern long model_pos_gt();


    /* return 1 if first position is greater than or equal to second, */
    /* 0 otherwise */
    /* ARGS: C_MODEL_POS* this_model_pos, C_MODEL_POS* comp_pos */

extern long model_pos_ge();


    /* second C_MODEL_POS* argument points to caller-supplied storage */
    /* a pointer to the next formattable model position is written into the */
    /* second argument. If all goes well, the return value equals the second */
    /* argument. */
    /* ARGS: C_MODEL_POS* this_model_pos, C_MODEL_POS* next_pos */

extern C_MODEL_POS *model_pos_get_next();


    /* second C_MODEL_POS* argument points to caller-supplied storage */
    /* a pointer to the prior formattable model position is written into the */
    /* second argument. If all goes well, the return value equals the second */
    /* argument. */
    /* ARGS: C_MODEL_POS* this_model_pos, C_MODEL_POS* prior_pos */

extern C_MODEL_POS *model_pos_get_prior();


    /* mark the page header associated with the argument model position */
    /* as needing to be reformatted */
    /* ARGS: C_MODEL_POS *this_model_pos */

extern  model_pos_mark_changed();


    /* returns zero if the node at which the model position is set is not */
    /* formattable or if the model position's node member is NULL. */
    /* Otherwise, the nonzero return value depends on the node's type. */
    /* ARGS: C_MODEL_POS* this_model_pos */

extern long model_pos_can_format();


    /*redraws every line on which the model pos occurs (if visible). Assumes */
    /* that the change does not alter the format of the conceptual page*/
    /* ARGS: C_MODEL_POS *this_model_pos */

extern  model_pos_in_place_redraw();




/* FOR CLASS MODEL_RANGE */


    /* allocate a model_range from free store */
    /* ARGS:  */

extern C_MODEL_RANGE *model_range_class_alloc();


    /* deallocate a model_range */
    /* ARGS: C_MODEL_RANGE *model_range_p */

extern  model_range_free();


    /* set the start member of the argument model_range to the model */
    /* position specified by new_start */
    /* ARGS: C_MODEL_RANGE *model_range_p, C_MODEL_POS *new_start */

extern  model_range_set_start();


    /* set the end member of the argument model_range to the model */
    /* position specified by new_end */
    /* ARGS: C_MODEL_RANGE *model_range_p, C_MODEL_POS *new_end */

extern  model_range_set_end();


    /* get the start member of a model range */
    /* Second argument points to user-supplied storage. If all goes well, */
    /* the return value equals the second argument. */
    /* ARGS: C_MODEL_RANGE *model_range_p, C_MODEL_POS *model_pos_p */

extern C_MODEL_POS *model_range_get_start();


    /* get the end member of a model range */
    /* Second argument points to user-supplied storage. If all goes well, */
    /* the return value equals the second argument. */
    /* ARGS: C_MODEL_RANGE *model_range_p, C_MODEL_POS *model_pos_p */

extern C_MODEL_POS *model_range_get_end();


    /* isolate the argument model_range into a subtree */
    /* Return a pointer to the root of the subtree or NULL if the */
    /* model_range is bogus */
    /* ARGS: C_MODEL_RANGE *model_range_p */

extern char *model_range_isolate();


    /* return 1 if the argument model_range contains target_pos, */
    /* 0 otherwise. */
    /* ARGS: C_MODEL_RANGE *model_range_p, C_MODEL_POS *target_pos */

extern long model_range_contains();




/* FOR CLASS NODE */


    /* no allocator for class node is provided since class node is a */
    /* container class. Use the allocator appropriate for the subclass */
    /* desired. */

    /* deallocate a node: dissociate it from its properties. If it is */
    /* associated with a model, remove it from the model and make it the */
    /* head of the free list for its subclass of node. */
    /* ARGS: tml_c_node this_node */

extern  node_free();



    /* FUNCTIONS ALLOWING INQUIRY AND SETTING OF THE DATA CONTAINED */
    /* IN THE CLASS */


    /* get model associated with tml_c_node node */
    /* ARGS: tml_c_node this_node */

extern tml_c_model node_get_model();


    /* set model member of tml_c_node to tml_c_model */
    /* ARGS: tml_c_node this_node, tml_c_model new_model */

extern  node_set_model();


    /* get node type member of tml_c_node - one of enum node_type */
    /* ARGS: tml_c_node this_node */

extern node_type node_get_type();


    /* set node type member of tml_c_node to node_type specified by node_type */
    /* ARGS: tml_c_node this_node, node_type this_type */

extern  node_set_type();


    /* find the level of tml_c_node in the tree by counting */
    /* ARGS: tml_c_node this_node */

extern long node_get_level();


    /* get the one-based sibling order (leftmost is 1) of tml_c_node */
    /* ARGS: tml_c_node this_node */

extern long node_get_sib_order();


    /* get the property vector associated with tml_c_node */
    /* ARGS: tml_c_node this_node */

extern tml_c_prop_vec node_get_props();


    /* get the model_range spanned by the subtree rooted at node */
    /* ARGS: tml_c_node this_node, C_MODEL_RANGE *model_range_p */

extern C_MODEL_RANGE *node_get_span();



    /* FUNCTIONS FOR PLACING AND MOVING NODES IN A TREE */


    /* connect 2nd node to 1st in direction specified - one of enum node_dir */
    /* ARGS: tml_c_node this_node, tml_c_node to_connect, node_dir dir */

extern tml_c_node node_connect_node();


    /* take a node out of a tree */
    /* ARGS: tml_c_node this_node */

extern  node_detach();


    /* move 1st node to place in direction specified from 2nd node */
    /* ARGS: tml_c_node this_node, tml_c_node move_to, node_dir dir */

extern tml_c_node node_move();



    /* FUNCTIONS DEALING WITH PROPERTIES OF NODES */


    /* attach a property vector to a node */
    /* ARGS: tml_c_node this_node, tml_c_prop_vec add_pvec */

extern  node_prop_attach();


    /* detach a node's property vector from it */
    /* ARGS: tml_c_node this_node */

extern  node_prop_detach();


    /* deactivate all the properties of a node */
    /* ARGS: tml_c_node this_node */

extern  node_exit_props();


    /* activate all the properties of a node */
    /* ARGS: tml_c_node this_node */

extern  node_enter_props();



    /* VIRTUAL FUNCTIONS--ACTUAL IMPLEMENTATION MAY DIFFER */
    /* FOR VARIOUS SUB-CLASSES */


    /* does this node contain formattable information? 1==yes, 0==no */
    /* ARGS: tml_c_node this_node */

extern long node_can_format();


    /* for text nodes (gnodes) give number of characters dominated */
    /* ARGS: tml_c_node this_node */

extern long node_get_max_index();



    /* THE FOLLOWING VIRTUAL FUNCTIONS ARE ILL-DEFINED */
    /* FOR ALL BUT THE DERIVED CLASSES */


    /* return pointer to formattable contents dominated by this node */
    /* ARGS: tml_c_node this_node */

extern char *node_get_contents();


    /* get new copy of node in model specified; if model is zero, */
    /* use same model as node is associated with */
    /* ARGS: tml_c_node this_node, tml_c_model to_model */

extern tml_c_node node_copy();


    /* copy the subtree below node to model specified; if model is zero, */
    /* use same model as node is associated with */
    /* ARGS: tml_c_node this_node, tml_c_model to_model */

extern tml_c_node node_copy_all();


    /* duplicate this node in its model; use same property vector as */
    /* original node */
    /* ARGS: tml_c_node this_node */

extern tml_c_node node_dup();


    /* duplicate, as above, the subtree at node in its model */
    /* ARGS: tml_c_node this_node */

extern tml_c_node node_dup_all();


    /* split nodes for insertion; implementation varies with node type */
    /* for gnode: split at text index specified by long; put portion to */
    /* left of that index into one node; put portion starting at that */
    /* index into another node; create an empty node; make these three */
    /* nodes siblings in order left-empty-right. */
    /* ARGS: tml_c_node this_node, long split_point */

extern tml_c_node node_split();



    /* NAVIGATION FUNCTIONS */


    /* get next node in traversal order; direction traveled from returned */
    /* in node_dir* argument if returning from lower level */
    /* ARGS: tml_c_node this_node, node_dir* dir */

extern tml_c_node node_get_next();


    /* get previous node in traversal order; direction traveled from returned */
    /* in node_dir* argment if returning from lower level */
    /* ARGS: tml_c_node this_node, node_dir* dir */

extern tml_c_node node_get_prev();


    /* get the node in the specified direction (one of node_dir) from */
    /* node argument */
    /* ARGS: tml_c_node this_node, node_dir dir */

extern tml_c_node node_get_node();


    /* make the first node's pointer in direction specified by node_dir*/
    /* (one of node_dir) point to second node */
    /* ARGS: tml_c_node this_node, tml_c_node to_node, node_dir dir */

extern  node_set_node();


    /* get deepest rightmost node in subtree which has this node as root */
    /* or rightmost sibling of this node if this node is childless */
    /* ARGS: tml_c_node this_node */

extern tml_c_node node_get_last();



    /* FUNCTIONS MEANINGFUL ONLY FOR CERTAIN SUBCLASSES */


    /* MEANINGFUL ONLY FOR GNODES AND OTHER TYPES HANDLING CHARACTER DATA */


    /* associate data_count bytes of data at data_start with this_node. */
    /* Returns 0 if applied to a nonterminal type of node, otherwise */
    /* dissociates node from any data it currently dominates, and copies */
    /* data at data_start into new data space. New data space is allocated */
    /* from node's associated model if any, otherwise from free space. */
    /* ARGS: tml_c_node this_node, char *data_start, long data_count */

extern long node_attach_data();

    /* insert add_count characters of add_data after character at */
    /* after_index in goo associated with this_node. Returns model position */
    /* of last character inserted. C_MODEL_POS argument points to */
    /* user-supplied storage. */
    /* ARGS: tml_c_node this_node, long after_index, char *add_data, long add_count, C_MODEL_POS *model_pos_p */

extern C_MODEL_POS *node_insert();

    /* delete delete_num characters at delete_index from goo associated with */
    /* this_node. Return model position of node == 0 and char_index == -1. */
    /* (Implementations of this function for appropriate subclasses can */
    /* return a good model position.) */
    /* C_MODEL_POS argument points to user-supplied storage. */
    /* ARGS: tml_c_node this_node, long delete_index, long delete_num, C_MODEL_POS *model_pos_p */

extern C_MODEL_POS *node_del_cont();


    /* overstrike, with rep_with, over_num characters of goo associated with */
    /* this_node, starting at begin_index. Returns model position of */
    /* node == 0 and char_index == -1. */
    /* (Implementations of this function for appropriate subclasses can */
    /* return a good model position.) */
    /* C_MODEL_POS argument points to user-supplied storage. */
    /* ARGS: tml_c_node this_node, long begin_index, char *rep_with, long over_num, C_MODEL_POS *model_pos_p */

extern C_MODEL_POS *node_overstrike();



    /* FORMATTING FUNCTIONS */


    /* get the height of the bounding box for information to be formatted */
    /* out of this node */
    /* ARGS: tml_c_node this_node, tml_c_format in_format */

extern long node_get_format_hi();


    /* return conceptual line associated with the given node under control */
    /* of the given format. Formatting is to start at a point (e.g. */
    /* character index) in the formattable information (e.g. text string) */
    /* dominated by the node. This point is specified by the first long */
    /* argument. The second and third long arguments give the height and */
    /* width, respectively, of the space remaining on the current conceptual */
    /* page. The final long argument indicates whether this is the first time */
    /* this node has been formatted (i.e. whether formatting or reformatting */
    /* is being performed.) */
    /* ARGS: tml_c_node this_node, tml_c_format in_format, long start_index, long space_hi, long space_wide, long first_concep_pixel */

extern tml_c_cline node_format_line();


    /* like node_format_line, but it formats BACKWARD from the point specified. */
    /* ARGS: tml_c_node this_node, tml_c_format in_format, long start_index, long space_hi, long space_wide, long first_concep_pixel */

extern tml_c_cline node_tamrof_line();




/* FOR CLASS BNODE */

    /* -- DERIVED CLASS FOR NON-TERMINAL NODES IN THE MODEL STRUCTURE TREE */


    /* allocate a bnode in the given model. If the argument is zero, */
    /*   allocate from free space. */
    /* ARGS: tml_c_model to_model */

extern tml_c_bnode bnode_class_alloc();


    /* deallocate a bnode */
    /* ARGS: tml_c_bnode this_bnode */

extern  bnode_free();




/* FOR CLASS GNODE */

    /* -- DERIVED CLASS FOR A TERMINAL OBJECT WHICH POINTS TO */
    /* CHARACTERS IN THE GOO HEAP */


    /* allocate a gnode in the given model. If the argument is zero, */
    /*   allocate from free space. */
    /* ARGS: tml_c_model to_model */

extern tml_c_gnode gnode_class_alloc();


    /* deallocate a gnode */
    /* ARGS: tml_c_gnode this_gnode */

extern  gnode_free();


    /* after this call, the argument gnode dominates text equivalent to the */
    /* null-terminated string specified by the argument char*. The gnode's */
    /* copy doesn't include the null terminator */
    /* ARGS: tml_c_gnode this_gnode, char *s */

extern  gnode_gets();


    /* get the non-null-terminated string dominated by the node */
    /* ARGS: tml_c_gnode this_gnode */

extern char *gnode_get_contents();


    /* get the number of characters in the non-null-terminated string */
    /* dominated by the node */
    /* ARGS: tml_c_gnode this_gnode */

extern long gnode_get_char_count();


    /* get a null-terminated copy of the string dominated by the node */
    /* ARGS: tml_c_gnode this_gnode */

extern char* gnode_get_0t_contents();


    /* set the character count of a gnode */
    /* ARGS: tml_c_gnode gnode_p, long new_count */

extern  gnode_set_char_count();




/* FOR CLASS PROP_VEC */


    /* allocate a property vector */
    /* ARGS: tml_c_model to_model */

extern tml_c_prop_vec prop_vec_class_alloc();


    /* deallocate a property vector */
    /* ARGS: tml_c_prop_vec this_prop_vec */

extern  prop_vec_free();


    /* return pointer to next member of argument property vector */
    /* ARGS: tml_c_prop_vec this_prop_vec */

extern tml_c_prop_vec prop_vec_get_next();


    /* detach 1st from 2nd */
    /* ARGS: tml_c_prop_vec this_prop_vec, tml_c_prop_vec from_vec */

extern tml_c_prop_vec prop_vec_detach_pvec();


    /* prepend 2nd to 1st */
    /* ARGS: tml_c_prop_vec this_prop_vec, tml_c_prop_vec add_element */

extern tml_c_prop_vec prop_vec_prepend();


    /* append 2nd to 1st */
    /* ARGS: tml_c_prop_vec this_prop_vec, tml_c_prop_vec add_vec */

extern  prop_vec_append();


    /* copy the property vector to a model; if model is zero, use model */
    /* that this property vector is associated with */
    /* ARGS: tml_c_prop_vec this_prop_vec, tml_c_model to_model */

extern tml_c_prop_vec prop_vec_copy();


    /* copy this property vector and all property vectors chained to it */
    /* through next pointers to a model. If model argument is zero, use */
    /* model that this property vector is associated with */
    /* ARGS: tml_c_prop_vec this_prop_vec, tml_c_model to_model */

extern tml_c_prop_vec prop_vec_copy_all();


    /* duplicate this property vector; i.e. copy it to the same model with */
    /* the same member pointer values */
    /* ARGS: tml_c_prop_vec this_prop_vec */

extern tml_c_prop_vec prop_vec_dup();


    /* duplicate this property vector and all property vectors chained to it */
    /* through next pointers */
    /* ARGS: tml_c_prop_vec this_prop_vec */

extern tml_c_prop_vec prop_vec_dup_all();


    /* attach property to prop_vec */
    /* ARGS: tml_c_prop_vec this_prop_vec, tml_c_property add_prop */

extern  prop_vec_attach_prop();


    /* detach a prop_vec's property */
    /* ARGS: tml_c_prop_vec this_prop_vec */

extern  prop_vec_detach_prop();


    /* activate all properties associated with a property vector and the */
    /* vectors chained to it */
    /* ARGS: tml_c_prop_vec this_prop_vec */

extern  prop_vec_activate();


    /* deactivate similarly */
    /* ARGS: tml_c_prop_vec this_prop_vec */

extern  prop_vec_deactivate();


    /* return the reference count associated with a property vector */
    /* ARGS: tml_c_prop_vec this_prop_vec */

extern long prop_vec_get_ref_count();


    /* return a pointer to the property associated with a given property */
    /* vector */
    /* ARGS: tml_c_prop_vec this_prop_vec */

extern tml_c_property prop_vec_get_prop();




/* FOR CLASS PROPERTY */


    /* allocate a property */
    /* ARGS: tml_c_model to_model */

extern tml_c_property property_class_alloc();


    /* deallocate a property */
    /* ARGS: tml_c_property this_property */

extern  property_free();


    /* get the index into the property function table of the function */
    /* associated with instantiation of this property */
    /* ARGS: tml_c_property this_property */

extern p_index property_get_index();


    /* set the index member of the property to the integer argument's value */
    /* ARGS: tml_c_property this_property, p_index i */

extern  property_set_index();


    /* copy a property to a model; if the model is zero, use the model this */
    /* property is associated with */
    /* ARGS: tml_c_property this_property, tml_c_model to_model */

extern tml_c_property property_copy();


    /* duplicate a property in the model with which it is associated */
    /* ARGS: tml_c_property this_property */

extern tml_c_property property_dup();


    /* return origin of entry argument vector for this property */
    /* ARGS: tml_c_property this_property */

extern tml_c_argument property_get_entry_av();


    /* return origin of exit argument vector for this property */
    /* ARGS: tml_c_property this_property */

extern tml_c_argument property_get_exit_av();


    /* attach a vector of entry arguments to a property */
    /* ARGS: tml_c_property this_property, tml_c_argument entry_vec */

extern  property_attach_entry();


    /* attach a vector of exit argument to a property */
    /* ARGS: tml_c_property this_property, tml_c_argument exit_vec */

extern  property_attach_exit();


    /* detach similarly */
    /* ARGS: tml_c_property this_property */

extern  property_detach_entry();


    /* detach similarly */
    /* ARGS: tml_c_property this_property */

extern  property_detach_exit();


    /* activate a property */
    /* ARGS: tml_c_property this_property */

extern  property_activate();


    /* deactivate similarly */
    /* ARGS: tml_c_property this_property */

extern  property_deactivate();


    /* return the reference count associated with the property */
    /* ARGS: tml_c_property this_property */

extern long property_get_ref_count();




/* FOR CLASS ARGUMENT */


    /* allocate an argument */
    /* ARGS: tml_c_model to_model */

extern tml_c_argument argument_class_alloc();


    /* deallocate an argument */
    /* ARGS: tml_c_argument this_argument */

extern  argument_free();


    /* get the model associated with an argument */
    /* ARGS: tml_c_argument this_argument */

extern tml_c_model argument_get_model();


    /* set the value of an integer-valued argument */
    /* ARGS: tml_c_argument this_argument, long value */

extern  argument_set_ival();


    /* set the value of a floating-point argument - true doubles will have */
    /* their fractional part truncated */
    /* ARGS: tml_c_argument this_argument, double value */

extern  argument_set_fval();


    /* set the value of a string argument to the non-null-terminated string */
    /* at char* of val_count bytes length by copying the char* argument to */
    /* space in a string heap */
    /* ARGS: tml_c_argument this_argument, char* value, long val_count */

extern  argument_set_sval();


    /* return a pointer to the "next" argument from this one */
    /* ARGS: tml_c_argument this_argument */

extern tml_c_argument argument_get_next();


    /* return the type of the argument - one of the enum arg_type */
    /* ARGS: tml_c_argument this_argument */

extern long argument_get_vtype();


    /* return the value of an integer argument */
    /* ARGS: tml_c_argument this_argument */

extern long argument_get_ival();


    /* return the value of a float argument */
    /* ARGS: tml_c_argument this_argument */

extern double argument_get_fval();


    /* return the actual pointer to the non-null-terminated value of a */
    /* string argument */
    /* ARGS: tml_c_argument this_argument */

extern char* argument_get_sval();


    /* return the number of bytes in the above non-null-terminated string */
    /* ARGS: tml_c_argument this_argument */

extern long argument_get_scount();


    /* detach 1st from 2nd */
    /* ARGS: tml_c_argument this_argument, tml_c_argument from_vec */

extern tml_c_argument argument_detach();


    /* append 2nd to 1st */
    /* ARGS: tml_c_argument this_argument, tml_c_argument add_arg */

extern  argument_append();


    /* prepend 2nd to 1st */
    /* ARGS: tml_c_argument this_argument, tml_c_argument add_arg */

extern tml_c_argument argument_prepend();


    /* return the reference count associated with an argument */
    /* ARGS: tml_c_argument this_argument */

extern long argument_get_ref_count();


    /* copy an argument to a model; if model is zero, use model this */
    /* argument is in */
    /* ARGS: tml_c_argument this_argument, tml_c_model to_model */

extern tml_c_argument argument_copy();


    /* copy an argument and all arguments chained to it through next */
    /* pointers to a model */
    /* if model is zero, use model this argument is in */
    /* ARGS: tml_c_argument this_argument, tml_c_model to_model */

extern tml_c_argument argument_copy_all();





/* FOR CLASS HISTORY */


    /* no allocator or deallocator included for class history */

    /* set the size of the history array */
    /* ARGS: tml_c_history this_history, long size */

extern long history_set_hist_size();


    /* end the current molecule after the last operation performed */
    /* ARGS: tml_c_history this_history */

extern  history_end_molecule();


    /* undo the last molecule added to history */
    /* ARGS: tml_c_history this_history */

extern long history_undo_molecule();


    /* redo the last molecule added to history */
    /* ARGS: tml_c_history this_history */

extern long history_redo_molecule();



/* FOR CLASS MARK_LIST */


    /* allocate a mark_list */
    /* ARGS:  */

extern C_MARK_LIST *mark_list_class_alloc();


    /* deallocate a mark_list */
    /* ARGS: C_MARK_LIST *this_mark_list */

extern          mark_list_free();


    /* get the next member of the argument mark_list */
    /* ARGS: C_MARK_LIST *mark_list_p */

extern C_MARK_LIST *mark_list_get_next();


    /* get the prior member of the argument mark_list */
    /* ARGS: C_MARK_LIST *mark_list_p */

extern C_MARK_LIST *mark_list_get_prior();


    /* get the range member the argument mark_list */
    /* ARGS: C_MARK_LIST *mark_list_p */

extern C_MODEL_RANGE *mark_list_get_range();


    /* set the next member of the argument mark_list */
    /* ARGS: C_MARK_LIST *mark_list_p, C_MARK_LIST *next_ml */

extern  mark_list_set_next();


    /* set the prior member of the argument mark_list */
    /* ARGS: C_MARK_LIST *mark_list_p, C_MARK_LIST *prior_ml */

extern  mark_list_set_prior();


    /* set the range member of the argument mark_list */
    /* ARGS: C_MARK_LIST *mark_list_p, C_MODEL_RANGE *new_range */

extern  mark_list_set_range();


    /* extend/shrink the range denominated by this mark_list */
    /* ARGS: C_MARK_LIST *mark_list_p, C_MODEL_POS *new_start, C_MODEL_POS *new_end */

extern long mark_list_extend_mark();

         
    /* highlight the text located by the mark_list */
    /* ARGS: C_MARK_LIST *mark_list_p */

extern long mark_list_draw();


    /* dehighlight the text located by the mark_list */
    /* ARGS: C_MARK_LIST *mark_list_p */

extern long mark_list_erase();



/* FOR CLASS SEARCH_ENV */


    /* allocate a search_env */
    /* ARGS: C_SEARCH_FUNC use_func, C_COMPILE_FUNC use_compile, C_SEARCH_NAV_FUNC use_for_next, C_SEARCH_NAV_FUNC use_for_prior, C_SEARCH_NAV_FUNC use_for_this */

extern C_SEARCH_ENV *search_env_class_alloc();


    /* deallocate a search_env */
    /* ARGS: C_SEARCH_ENV *this_search_env */

extern  search_env_free();


    /* get the next member of the argument search_env */
    /* ARGS: C_SEARCH_ENV *search_env_p */

extern C_SEARCH_ENV *search_env_get_next();


    /* set the next member of the argument search_env */
    /* ARGS: C_SEARCH_ENV *search_env_p, C_SEARCH_ENV *new_next */

extern  search_env_set_next();

   


/* FOR CLASS MODEL */


    /* allocate a model */
    /* ARGS:  */

extern tml_c_model model_class_alloc();


    /* deallocate a model */
    /* ARGS: tml_c_model this_model */

extern  model_free();



    /* INQUIRY FUNCTIONS FOR THE MODEL DATA */


    /* get the root node of the model */
    /* ARGS: tml_c_model this_model */

extern tml_c_node model_get_root();


    /* get the point up to which properties are instantiated for the model */
    /* ARGS: tml_c_model this_model */

extern tml_c_node model_get_prop_pos();



    /* FUNCTIONS TO DIRECTLY MANIPULATE THE MODEL DATA */


    /* make a root for a new model */
    /* ARGS: tml_c_model this_model */

extern tml_c_node model_make_root();


    /* set the root of a model to a given node */
    /* ARGS: tml_c_model this_model, tml_c_node new_root */

extern tml_c_node model_set_root();


    /* inquire the state of the property associated with the p_index argument */
    /* ARGS: tml_c_model this_model, p_index prop_ind */

extern char *model_inq_state();


    /* inquire the state of the property associated with the p_index argument */
    /* at a certain node */
    /* ARGS: tml_c_model this_model, tml_c_node at_node, p_index prop_ind */

extern char *model_inq_state_at_node();


    /* move the current property position to the specified node, setting */
    /* properties as needed */
    /* ARGS: tml_c_model this_model, tml_c_node to_node */

extern tml_c_node model_move_prop();


    /* set the current property position at the specified node */
    /* ARGS: tml_c_model this_model, tml_c_node new_prop_pos */

extern  model_set_prop_pos();



    /* FUNCTIONS WHICH ALLOCATE SPACE IN A MODEL */


    /* allocate a property vector in the model */
    /* ARGS: tml_c_model this_model */

extern tml_c_prop_vec model_make_pvec();


    /* allocate a property in the model */
    /* ARGS: tml_c_model this_model */

extern tml_c_property model_make_prop();


    /* allocate an integer argument in the model */
    /* ARGS: tml_c_model this_model, long new_value */

extern tml_c_argument model_make_int_arg();


    /* allocate a float argument in the model. NOTE - full doubles will */
    /* have their fractional parts truncated. */
    /* ARGS: tml_c_model this_model, double new_value */

extern tml_c_argument model_make_float_arg();


    /* allocate a string argument in the model */
    /* ARGS: tml_c_model this_model, char* new_value, long new_count */

extern tml_c_argument model_make_string_arg();


    /* allocate text characters in a model */
    /* ARGS: tml_c_model this_model, long count, char* contents */

extern char* model_make_goo();


    /* make an object of the specified size in the model */
    /* - for object type extensions */
    /* ARGS: tml_c_model this_model, long obj_size */

extern char* model_make_obj();


    /* write a model to the file indicated by the argument pathname */
    /* and pathname length. Return bytes written or -1 for failure. */
    /* ARGS: tml_c_model this_model, char *pathname, long name_len */

extern long model_d_write_model_path();


    /* write a model to the file indicated by the argument file handle. */
    /* Returns bytes written or -1 for failure. */
    /* ARGS: tml_c_model this_model, char *file_handle */

extern long model_d_write_model_handle();


    /* get a pointer to the history for the model */
    /* ARGS: tml_c_model this_model */

extern tml_c_history model_get_past();


    /* cut the subtree rooted at node from the model */
    /* ARGS: tml_c_model this_model, tml_c_node cut_node */

extern char *model_cut();


    /* paste the subtree rooted at node into the model, attaching */
    /* paste_node to past_to in direction paste_dir */
    /* ARGS: tml_c_model this_model, tml_c_node paste_node, tml_c_node paste_to, node_dir paste_dir */

extern char *model_paste();


    /* get the first mark_list associated with the model */
    /* ARGS: tml_c_model this_model */

extern C_MARK_LIST *model_get_first_mark();


    /* allocate a mark_list in the model; initialize it to makr */
    /* the text indicated by init_range */
    /* ARGS: tml_c_model this_model, C_MODEL_RANGE *init_range */

extern C_MARK_LIST *model_make_mark();


    /* disassociate the argument mark_list from the model */
    /* ARGS: tml_c_model this_model, C_MARK_LIST *to_delete */

extern  model_delete_mark();


    /* highlight the text located by the mark_list in any viewports */
    /* that are attached to the model */
    /* ARGS: tml_c_model this_model, C_MARK_LIST *to_draw */

extern long model_mark_draw();


    /* dehighlight the text located by the mark_list in any viewports */
    /* that are attached to the model */
    /* ARGS: tml_c_model this_model, C_MARK_LIST *to_erase */

extern long model_mark_erase();


    /* push the argument search_env onto the stack associated with the model */
    /* ARGS: tml_c_model *this_model, C_SEARCH_ENV *new_envir */

extern  model_push_search_env();


    /* pop the top search_env from the stack associated with the model */
    /* ARGS: tml_c_model *this_model */

extern  model_pop_search_env();


    /* search for the regular expression indicated by pattern */
    /* within the range indicated by the model_range argument */
    /* ARGS: tml_c_model this_model, char *pattern, C_MODEL_RANGE *search_range, long forward, C_MODEL_RANGE *model_range_p */

extern C_MODEL_RANGE *model_search_exp();


    /* compile search_exp of length search_exp_len into a compiled regular */
    /* expression of maximum lenght max_comp_len. Case sensitivity is a */
    /* switch: 1 for case sensitive, 0 for not case sensitive */
    /* ARGS: tml_c_model this_model, char *search_exp, long search_exp_len, char *compiled_exp, long max_comp_len, long case_sens */

extern long model_compile_exp();


    /* search for the string of length search_exp_len within the */
    /* model_range specified */
    /* ARGS: tml_c_model this_model, char *search_exp, long search_exp_len, C_MODEL_RANGE *search_range, long forward, long case_sens, C_MODEL_RANGE *model_range_p */

extern C_MODEL_RANGE *model_search();
                                                                                                     

    /* updates, in a crude fashion, the display of the model when that model */
    /* has been changed*/
    /* ARGS: tml_c_model this_model */

extern  model_update_display();




/* FOR CLASS DRAW_MARK */


    /* allocate a draw_mark */
    /* ARGS:  */

extern tml_c_draw_mark draw_mark_alloc();


    /* deallocate a draw_mark */
    /* ARGS: tml_c_draw_mark this_draw_mark */

extern  draw_mark_free();


    /* return the next member of a draw_mark */
    /* ARGS: tml_c_draw_mark this_draw_mark */

extern tml_c_draw_mark draw_mark_get_next();


    /* return the prior member of a draw_port */
    /* ARGS: tml_c_draw_mark this_draw_mark */

extern tml_c_draw_mark draw_mark_get_prior();


    /* return the mark_list associated with this draw_list */
    /* ARGS: tml_c_draw_mark this_draw_mark */

extern tml_c_draw_mark draw_mark_get_model_mark();


    /* set the core drawing function for the draw mark */
    /* ARGS: tml_c_draw_mark this_draw_mark, C_CORE_M_DRAW new_core */

extern  draw_mark_set_core_func();


    /* set the draw function for the draw mark */
    /* ARGS: tml_c_draw_mark this_draw_mark, C_DRAW_M_DRAW new_draw */

extern  draw_mark_set_draw_func();


    /* extend/shrink the area indicated by the draw_mark */
    /* ARGS: tml_c_draw_mark this_draw_mark, C_MODEL_POS *new_start, C_MODEL_POS *new_end */

extern  draw_mark_extend();


    /* highlight the viewport area associated with the draw_mark */
    /* ARGS: tml_c_draw_mark this_draw_mark */

extern  draw_mark_draw();


    /* dehighlight the viewport area associated with the draw_mark */
    /* ARGS: tml_c_draw_mark this_draw_mark */

extern  draw_mark_erase();




/* FOR CLASS PROP_FUNC_ENT */


    /* allocate a property function entry */
    /* ARGS:  */

extern tml_c_prop_func_ent tml_c_prop_func_ent_class_alloc();


    /* deallocate a property function entry */
    /* ARGS: tml_c_prop_func_ent this_prop_func_ent */

extern  tml_c_prop_func_ent_free();


    /* set 1st's next to 2nd */
    /* ARGS: tml_c_prop_func_ent this_prop_func_ent, tml_c_prop_func_ent next_ent */

extern  prop_func_ent_set_next();


    /* set the index member to the argument p_index; this is a */
    /* property index */
    /* ARGS: tml_c_prop_func_ent this_prop_func_ent, p_index i */

extern  prop_func_ent_set_index();


    /* get that index member */
    /* ARGS: tml_c_prop_func_ent this_prop_func_ent */

extern p_index prop_func_ent_get_index();


    /* set the pointer to the function that enters this property */
    /* ARGS: tml_c_prop_func_ent this_prop_func_ent, C_PROP_FUNC_PT new_entry */

extern  prop_func_ent_load_entry();


    /* set the pointer to the function that exits this property */
    /* ARGS: tml_c_prop_func_ent this_prop_func_ent, C_PROP_FUNC_PT new_exit */

extern  prop_func_ent_load_exit();


    /* get that entry function pointer */
    /* ARGS: tml_c_prop_func_ent this_prop_func_ent */

extern C_PROP_FUNC_PT prop_func_ent_get_entry();


    /* get that exit function pointer */
    /* ARGS: tml_c_prop_func_ent this_prop_func_ent */

extern C_PROP_FUNC_PT prop_func_ent_get_exit();


    /* get load status of the prop_func_ent */
    /* ARGS: tml_c_prop_func_ent this_prop_func_ent */

extern long prop_func_ent_get_loaded();


    /* set load status to the argument integer (will be truncated to a short) */
    /* ARGS: tml_c_prop_func_ent this_prop_func_ent, long i */

extern  prop_func_ent_set_loaded();


    /* load the function associated with a property into the property */
    /* function table */
    /* ARGS: tml_c_prop_func_ent this_prop_func_ent */

extern  prop_func_ent_load();





/**********************************************************************************
    Objects used in the formatting component of tml

**********************************************************************************/



/* FOR CLASS CLINE */


    /* no allocator supplied for class cline since it is a container class. */
    /* Use the allocator for the subclass desired. */


    /* deallocate a cline */
    /* ARGS: tml_c_cline this_cline */

extern  cline_free();


    /* get the type of this conceptual line (one of enum cl_type) */
    /* ARGS: tml_c_cline this_cline */

extern long cline_get_type();


    /* get the x position of the upper left corner of bounding box of this */
    /* conceptual line */
    /* ARGS: tml_c_cline this_cline */

extern long cline_get_ulx();


    /* get the y position of the upper left corner of bounding box of this */
    /* conceptual line */
    /* ARGS: tml_c_cline this_cline */

extern long cline_get_uly();


    /* get the x position of the lower left corner of bounding box of this */
    /* conceptual line */
    /* ARGS: tml_c_cline this_cline */

extern long cline_get_lrx();


    /* get the y position of the lower left corner of bounding box of this */
    /* conceptual line */
    /* ARGS: tml_c_cline this_cline */

extern long cline_get_lry();


    /* get the height in pixels of this conceptual line */
    /* ARGS: tml_c_cline this_cline */

extern long cline_get_hi();


    /* get the width in pixels of this conceptual line */
    /* ARGS: tml_c_cline this_cline */

extern long cline_get_wide();


    /* get the page header associated with this conceptual line */
    /* ARGS: tml_c_cline this_cline */

extern tml_c_phead cline_get_phead();


    /* get the format associated with this conceptual line */
    /* ARGS: tml_c_cline this_cline */

extern tml_c_format cline_get_format();


    /* gets the next conceptual line as a tml_c_cline (not virtual) */
    /* ARGS: tml_c_cline this_cline */

extern tml_c_cline cline_inq_next();


    /* gets the prior conceptual line as a tml_c_cline (not virtual) */
    /* ARGS: tml_c_cline this_cline */

extern tml_c_cline cline_inq_prior();


    /* gets the conceptual line below the current conceptual line */
    /* (not virtual) */
    /* ARGS: tml_c_cline this_cline */

extern tml_c_cline cline_get_down();


    /* gets the conceptual line above the current conceptual line */
    /* (not virtual) */
    /* ARGS: tml_c_cline this_cline */

extern tml_c_cline cline_get_up();



    /* TRAITS TO MANIPULATE CONCEPTUAL LINE DATA */


    /* set the page header member of the conceptual line */
    /* ARGS: tml_c_cline this_cline, tml_c_phead on_page */

extern  cline_set_phead();


    /* set the type of the conceptual line to the long argument */
    /* (one of enum cl_type) */
    /* ARGS: tml_c_cline this_cline, long new_type */

extern  cline_set_type();


    /* set the upper left corner coordinates of the bounding box of this */
    /* conceptual line. The first long argument is the x value in pixels; */
    /* the second is the y value. Arguments upper_x and upper_y will be */
    /* truncated to shorts */
    /* ARGS: tml_c_cline this_cline, long upper_x, long upper_y */

extern  cline_set_ul_coord();


    /* set the width member of the conceptual line to the number of pixels */
    /* specified by the long argument, which will be truncated to a short */
    /* ARGS: tml_c_cline this_cline, long set_wide */

extern  cline_set_wide();


    /* set the height member of the conceptual line to the number of pixels */
    /* specified by the long argument, which will be truncated to a short */
    /* ARGS: tml_c_cline this_cline, long new_hi */

extern  cline_set_hi();



    /* TRAITS TO DO THINGS TO A CONCEPTUAL LINE */


    /* splits a free conceptual line in two either horizontally or */
    /* vertically. Vertical splits have a direction of 0; direction values */
    /* greater than zero indicate a horizontal split. If from_top = 0, */
    /* the split will create a new cline of new_size depth (width) */
    /* from the top (left) of the indicated cline. Otherwise the split */
    /* will create a new cline of new_size depth (width) from the */
    /* bottom (right) of the indicated cline. Integer arguments will be */
    /* truncated to shorts. */
    /* ARGS: tml_c_cline this_cline, long new_size, long from_top, long direction */

extern tml_c_cline cline_split();


    /* resizes the height of a conceptual line and all lines below the */
    /* adjusted line to current height plus the long argument (pixels) */
    /* which will be truncated to a short */
    /* ARGS: tml_c_cline this_cline, long hi_delta */

extern long cline_resize();


    /* attach a line before (long before != 0) or after (long before == 0) */
    /* this line-- needs to update the coordinates of the lines below */
    /* ARGS: tml_c_cline this_cline, tml_c_cline target_line, long before */

extern  cline_attach();


    /* detach a line from a conceptual page--needs to update the coordinates */
    /* of the lines below */
    /* ARGS: tml_c_cline this_cline */

extern  cline_detach();



    /* VIRTUAL FUNCTIONS FOR THIS CLASS */


    /* get the next conceptual line (virtual) */
    /* ARGS: tml_c_cline this_cline */

extern tml_c_cline cline_get_next();


    /* set the next pointer of the first argument to the second */
    /* ARGS: tml_c_cline this_cline, tml_c_cline next_line */

extern  cline_set_next();


    /* get the prior conceptual line (virtual) */
    /* ARGS: tml_c_cline this_cline */

extern tml_c_cline cline_get_prior();


    /* set the prior pointer of the first argument to the second */
    /* ARGS: tml_c_cline this_cline, tml_c_cline prior_line */

extern  cline_set_prior();


    /* C_MODEL_POS* argument points to caller-supplied storage; */
    /* the model position at the start of the conceptual line is copied */
    /* into it. If all goes well, the return value equals the second */
    /* argument. */
    /* ARGS: tml_c_cline this_cline, C_MODEL_POS* model_pos_p */

extern C_MODEL_POS* cline_get_start();


    /* C_MODEL_POS* argument points to caller-supplied storage; */
    /* the model position at the end of the conceptual line is copied */
    /* into it. If all goes well, the return value equals the second */
    /* argument. */
    /* ARGS: tml_c_cline this_cline, C_MODEL_POS* model_pos_p */

extern C_MODEL_POS* cline_get_end();


    /* get the starting x offset from first pixel of first character in line */
    /* ARGS: tml_c_cline this_cline */

extern long cline_get_start_x_off();


    /* replace the line represented by the second argument with the line */
    /* represented by by the first argument */
    /* ARGS: tml_c_cline this_cline, tml_c_cline replace_line */

extern  cline_replace();


    /* release the cline argument (delete it) */
    /* ARGS: tml_c_cline this_cline */

extern  cline_release();


    /* allocate some space from this line and link in the line supplied */
    /* (fill_with). If this is not free space, return 0. Otherwise, return */
    /* the free line closest to this left after the allocation. If long */
    /* argument at_bottom = 0, put fill_with at the top (left) of this line, */
    /* else put fill_width at the bottom (left) of the line */
    /* ARGS: tml_c_cline this_cline, tml_c_cline fill_with, long at_bottom */

extern tml_c_cline cline_alloc_space();


    /* C_XY_COORD* argument points to caller-supplied storage */
    /* The xy coordinates of the indicated model position are copied */
    /* into the C_XY_COORD* argument. If all goes well, the return value */
    /* equals the third argument */
    /* ARGS: tml_c_cline this_cline, C_MODEL_POS* find_pos, C_XY_COORD* new_xy_coord */

extern C_XY_COORD* cline_mpos_2_xy();


    /* C_MODEL_POS* argument points to caller-supplied storage */
    /* The model position associated with the C_XY_COORD* argument is copied */
    /* into the C_MODEL_POS* argument. If all goes well, the return value */
    /* equals the third argument. */
    /* ARGS: tml_c_cline this_cline, C_XY_COORD* where, C_MODEL_POS* new_model_pos */

extern C_MODEL_POS* cline_xy_2_mpos();


    /* draw the area for the cline at the xy position indicated by the third */
    /* argument. Return a view_pos if the area is not free, zero otherwise. */
    /* ARGS: tml_c_cline this_cline, tml_c_view_port in_viewport, C_SCREEN_XY* where, long start_index */

extern tml_c_view_pos cline_draw_line();


    /* reformat the line - always called on a derived class object of class */
    /* cline (e.g. tline) */
    /* ARGS: tml_c_cline this_cline, C_MODEL_POS *new_start, long *next_ok, long force_start */

extern tml_c_cline cline_reformat();




/* FOR CLASS TLINE */


    /* allocate a tline; if tml_c_format is nonzero, set the tline's */
    /* format to that value */
    /* ARGS: tml_c_tline this_tline, tml_c_format this_format */

extern tml_c_tline tline_class_alloc();

    /* deallocate a tline */
    /* ARGS: tml_c_tline this_tline */

extern  tline_free();



    /* TRAITS TO ACCESS THE TEXT LINE DATA */


    /* get the tline to the right of this tline */
    /* ARGS: tml_c_tline this_tline */

extern tml_c_cline tline_get_right();


    /* get the tline to the left of this tline */
    /* ARGS: tml_c_tline this_tline */

extern tml_c_cline tline_get_left();


    /* get the font associated with this tline */
    /* ARGS: tml_c_tline this_tline */

extern tml_c_font_tbl tline_get_font();


    /* get the galley depth (in pixels) at which the baseline */
    /* of this tline is located */
    /* ARGS: tml_c_tline this_tline */

extern long tline_get_baseline();


    /* return the number of characters set on this tline */
    /* ARGS: tml_c_tline this_tline */

extern long tline_get_count();



    /* TRAITS TO MANIPULATE THE TEXT LINE DATA */


    /* adjust the baseline for this tline, and all baselines to its right */
    /* (all its right siblings) to the new baseline */
    /* ARGS: tml_c_tline this_tline, long new_baseline_y */

extern long tline_adjust_baseline();


    /* set the galley depth (in pixels) at which the baseline of this tline */
    /* is to be located */
    /* ARGS: tml_c_tline this_tline, long new_baseline */

extern  tline_set_baseline();


    /* set the font to be associated with this tline */
    /* ARGS: tml_c_tline this_tline, tml_c_font_tbl new_font */

extern  tline_set_font();


    /* set the model position at which this tline begins */
    /* ARGS: tml_c_tline this_tline, C_MODEL_POS* new_start */

extern  tline_set_start();


    /* set the number of characters to be formatted on this tline */
    /* ARGS: tml_c_tline this_tline, long new_count */

extern  tline_set_count();


    /* set the width (in pixels) of the formatted text on this tline */
    /* ARGS: tml_c_tline this_tline, long new_width */

extern  tline_set_text_width();


    /* return width of text contained by tml_c_tline argument */
    /* ARGS: tml_c_tline this_tline */

extern long tline_get_text_width();


    /* return width of text contained by tml_c_tline argument and all of */
    /* its right siblings */
    /* ARGS: tml_c_tline this_tline */

extern long tline_get_full_text_width();


    /* set the right member of this tline */
    /* ARGS: tml_c_tline this_tline, tml_c_tline new_right */

extern  tline_set_right();


    /* set the left member of this tline */
    /* ARGS: tml_c_tline this_tline, tml_c_tline new_left */

extern  tline_set_left();




/* FOR CLASS PHEAD */


    /* allocate a page header. If format is nonzero, set the */
    /* phead's format to that value */
    /* ARGS: tml_c_format to_format */

extern tml_c_phead phead_class_alloc();


    /* deallocate a phead */
    /* ARGS: tml_c_phead this_phead */

extern  phead_free();



    /* TRAITS ALLOWING ACCESS TO THE PHEAD DATA FIELDS */


    /* get the page header of the following page */
    /* ARGS: tml_c_phead this_phead */

extern tml_c_phead phead_get_next();


    /* get the page header of the preceding page */
    /* ARGS: tml_c_phead this_phead */

extern tml_c_phead phead_get_prior();


    /* get the page number associated with this phead */
    /* ARGS: tml_c_phead this_phead */

extern long phead_get_page_no();


    /* get the number of the formatted line that begins this page */
    /* ARGS: tml_c_phead this_phead */

extern long phead_get_fline_no();


    /* get the maximum line width */
    /* ARGS: tml_c_phead this_phead */

extern long phead_get_max_line_width();


    /* get the maximum height used */
    /* ARGS: tml_c_phead this_phead */

extern long phead_get_max_used_ht();


    /* get the number of lines formatted on this page */
    /* ARGS: tml_c_phead this_phead */

extern long phead_get_line_cnt();


    /* get the model position associated with the beginning of this page. */
    /* The second argument points to caller-supplied storage. If all goes */
    /* well, the return value equals the second argument. */
    /* ARGS: tml_c_phead this_phead, C_MODEL_POS* pos_p */

extern C_MODEL_POS *phead_get_start();


    /* get the model position associated with the end of this page. The */
    /* second argument points to caller-supplied storage. If all goes */
    /* well, the return value equals the second argument. */
    /* ARGS: tml_c_phead this_phead, C_MODEL_POS* pos_p */

extern C_MODEL_POS *phead_get_end();


    /* get the format associated with this page */
    /* ARGS: tml_c_phead this_phead */

extern tml_c_format phead_get_format();


    /* get value of the format_ok member of this page header */
    /* ARGS: tml_c_phead this_phead */

extern long phead_get_format_ok();


    /* get the height of this page in pixels */
    /* ARGS: tml_c_phead this_phead */

extern long phead_get_hi();


    /* get the width of this page in pixels */
    /* ARGS: tml_c_phead this_phead */

extern long phead_get_wide();


    /* first long argument is height in pixels; second if zero indicates */
    /* search (see below) begins at bottom of page's list of free clines; */
    /* if nonzero indicates search begins at top of that list. Return the */
    /* first free cline in the direction specified in that list that has */
    /* height greater than or equal to the height of this page. */
    /* The long arguments will be truncated to shorts. */
    /* ARGS: tml_c_phead this_phead, long height, long from_top */

extern tml_c_cline phead_get_freeline();


    /* get the formatted line at the top of this phead */
    /* ARGS: tml_c_phead this_phead */

extern tml_c_cline phead_get_tline();


    /* get the formatted line at the bottom of this phead */
    /* ARGS: tml_c_phead this_phead */

extern tml_c_cline phead_get_bline();


    /* inquires whether the page associated with this page header is visible */
    /* on any listed viewport */
    /* ARGS: tml_c_phead this_phead */

extern long phead_is_visible();



    /* TRAITS WHICH ALTER THE PHEAD DATA FIELDS DIRECTLY */


    /* set the format associated with this phead to the value specified */
    /* ARGS: tml_c_phead this_phead, tml_c_format new_format */

extern  phead_set_format();


    /* first long argument is height, second argument is width. Set */
    /* the page's height and width. These long arguments will be */
    /* truncated to shorts. */
    /* ARGS: tml_c_phead this_phead, long hi, long wide */

extern  phead_set_size();


    /* set the model position for the beginning of this page */
    /* ARGS: tml_c_phead this_phead, C_MODEL_POS* new_start */

extern  phead_set_start_pos();


    /* set the model position for the end of this page */
    /* ARGS: tml_c_phead this_phead, C_MODEL_POS* new_end */

extern  phead_set_end_pos();



    /* NAVIGATION AND CREATION FUNCTIONS */


    /* set the "format_ok" member to zero, signalling that page needs to be */
    /* reformatted */
    /* ARGS: tml_c_phead this_phead */

extern  phead_mark_reformat();

    /* if the model doesn't contain any matter to be formatted following */
    /* this page, return zero. Otherwise, if the next page already exists, */
    /* update its start position; if long argument is nonzero, reformat */
    /* the next page and return it. If the next page doesn't yet exist, make */
    /* a new phead, format it if the long argument is nonzero, and */
    /* return it. */
    /* ARGS: tml_c_phead this_phead, long reformat */

extern tml_c_phead phead_next_page();


    /* if the model doesn't contain any matter to be formatted preceding */
    /* this page, return zero. Otherwise, if the prior page already exists, */
    /* update its end position; if the long argument is nonzero, reformat */
    /* the prior page and return it. If the prior page doesn't yet exist, */
    /* make a new phead, format it if the long argument is nonzero, set */
    /* the associated format's first page field to this page, and return */
    /* the new page */
    /* ARGS: tml_c_phead this_phead, long reformat */

extern tml_c_phead phead_prior_page();


    /* Insert a page of material formatted (a) forward from the C_MODEL_POS */
    /* argument if the long argument is nonzero, (b) backward from the */
    /* C_MODEL_POS argument if the long argument is zero. If material is */
    /* formatted forward, insert the new page after this page; if material */
    /* is formatted backward, insert new the page before this page. */
    /* ARGS: C_MODEL_POS* start, long after */

extern tml_c_phead phead_insert_page();


    /* If the page needs reformatting, do so. Then if the argument model */
    /* position isn't formatted on this page, return zero. Otherwise find */
    /* the conceptual line that contains the text associated with this */
    /* model position and return that line. */
    /* ARGS: tml_c_phead this_phead, C_MODEL_POS* target */

extern tml_c_cline phead_mpos_2_cline();



    /* TRAITS THAT MANIPULATE THE CONCEPTUAL LINES ON THE PAGE */


    /* Find the head conceptual line containing the indicated model */
    /* position. If the line is not on the page, return 0. */
    /* ARGS: tml_c_phead this_phead, C_MODEL_POS* target */

extern tml_c_cline phead_mpos_2_hcline();


    /* Return all free lines associated with this page to the free store. */
    /* ARGS: tml_c_phead this_phead */

extern  phead_free_clines();


    /* Integer arguments are x and y coordinates, respectively, in pixels. */
    /* Search for a conceptual line of size greater than or equal to the */
    /* cline argument at x and y coordinates less than or equal to those */
    /* associated with the argument cline. If no such line exists, */
    /* return -1. If such a line exists, split it if necessary to yield a */
    /* line of the same size as the cline argument; set the found line's */
    /* x and y coordinates to the argument values. Replace the argument */
    /* cline with the new one. The long arguments will be truncated */
    /* to shorts. */
    /* ARGS: tml_c_phead this_phead, tml_c_cline place_line, long at_x, long at_y */

extern long phead_line_place();



    /* FORMAT THE PAGE */


    /* new_model_pos (second C_MODEL_POS* argument) points to */
    /* caller-supplied storage. Format the page specified by this_phead, */
    /* starting at the model position specified in argument start. */
    /* If the page is already formatted, and its starting position matches */
    /* the model position specified in start, copy its ending model position */
    /* into new_model_pos. If the page is already formatted, but its */
    /* starting position doesn't match the model position specified in */
    /* start, free the conceptual lines associated with the page. Copy the */
    /* page's ending model position into new_model_pos. If all goes well, */
    /* the return value equals the third argument. */
    /* ARGS: tml_c_phead this_phead, C_MODEL_POS* start, C_MODEL_POS* new_model_pos */

extern C_MODEL_POS* phead_format_page();


    /* like phead_format_page, but formats BACKWARD from start argument */
    /* ARGS: tml_c_phead this_phead, C_MODEL_POS* start, C_MODEL_POS* new_model_pos */

extern C_MODEL_POS* phead_tamrof_page();


    /* like format page, but start indicates the end position of the last */
    /* correctly formatted line on the page before reformatting is to begins. */
    /* The model position returned is the end of the last line reformatted. */
    /* ARGS: tml_c_phead this_phead, C_MODEL_POS* start, C_MODEL_POS* new_model_pos */

extern C_MODEL_POS* phead_reformat_page();




/* FOR CLASS FORMAT */


    /* allocate a format */
    /* ARGS:  */

extern tml_c_format format_class_alloc();


    /* deallocate a format */
    /* ARGS: tml_c_format this_format */

extern  format_free();



    /* TRAITS WHICH ALLOW ACCESS TO THE FORMAT DATA */


    /* get the reference count associated with this format */
    /* ARGS: tml_c_format this_format */

extern long format_get_ref_count();


    /* get the model with which this format is associated */
    /* ARGS: tml_c_format this_format */

extern tml_c_model format_get_model();


    /* get the first page at which this format takes effect */
    /* ARGS: tml_c_format this_format */

extern tml_c_phead format_get_first_page();


    /* get the last page which this format affects */
    /* ARGS: tml_c_format this_format */

extern tml_c_phead format_get_last_page();


    /* If this format is not associated with a model, return zero. */
    /* Otherwise, return the current state in the associated model of the */
    /* property indexed by the second argument */
    /* ARGS: tml_c_format this_format, p_index prop_ind */

extern char *format_inq_state();


    /* If this format is not associated with a model, return zero. */
    /* Otherwise, return the state at the argument node of the property */
    /* indexed by the second argument */
    /* ARGS: tml_c_format this_format, tml_c_node at_node, p_index prop_ind */

extern char *format_inq_state_at_node();


    /* Return page height associated with this format */
    /* ARGS: tml_c_format this_format */

extern long format_get_page_hi();


    /* Return page width associated with this format */
    /* ARGS: tml_c_format this_format */

extern long format_get_page_wide();


    /* Return maximum line width associated with this format */
    /* ARGS: tml_c_format this_format */

extern long format_get_max_line_width();


    /* Return maximum height used on a line */
    /* ARGS: tml_c_format this_format */

extern long format_get_max_used_ht();



    /* TRAITS WHICH ALLOW MANIPULATION OF THE FORMAT DATA */

    /* set the page size associated with this format: first long */
    /* argument is height, second is width. The long arguments are */
    /* truncated to shorts. */
    /* ARGS: tml_c_format this_format, long hi, long wide */

extern  format_set_page_size();


    /* set number of pages to be saved. The long argument will be */
    /* truncated to a short. */
    /* ARGS: tml_c_format this_format, long save_num */

extern  format_set_pages_saved();


    /* set the first page with which this format is associated */
    /* ARGS: tml_c_format this_format, tml_c_phead new_first */

extern  format_set_first_page();


    /* set the last page which this format affects */
    /* ARGS: tml_c_format this_format, tml_c_phead new_last */

extern  format_set_last_page();


    /* set the default for this format of the property indexed by the */
    /* p_index argument to the value indicated by the third argument */
    /* ARGS: tml_c_format this_format, p_index prop_ind, char *value */

extern long format_set_state();


    /* set the value of the property indexed by the p_index argument to zero */
    /* ARGS: tml_c_format this_format, p_index prop_ind */

extern long format_unset_state();


    
    /* TRAITS WHICH MANIPULATE THE FORMAT */


    /* attach a format to a model */
    /* ARGS: tml_c_format this_format, tml_c_model to_model */

extern  format_attach();


    /* detach a format from the model associated with it */
    /* ARGS: tml_c_format this_format */

extern  format_detach();



    /* NAVIGATION TRAITS */


    /* return the page header associated with the model position argument. */
    /* If the position falls on or precedes the first page associated with */
    /* the format, return that page. If the position follows after the last */
    /* page associated with the format, find that page and return it. */
    /* If the position is on the last page associated with the format, */
    /* return that page. Otherwise, start with the first page and reformat */
    /* as necessary to come up with the page on which the argument model */
    /* position is formatted. */
    /* ARGS: tml_c_format this_format, C_MODEL_POS* find_pos */

extern tml_c_phead format_mpos_2_phead();


    /* return the page header associated with the page number indicated by */
    /* the long argument. If no such page exists, return zero. */
    /* ARGS: tml_c_format this_format, long page_no */

extern tml_c_phead format_get_page();




/* FOR CLASS FONT_TABLE */


    /* allocate a font table -- use default graphics system function table  */
    /* ARGS:  */

extern tml_c_font_tbl font_tbl_class_alloc();


    /* allocate a font table -- use specified graphics system function table */
    /* ARGS: char *ftbl */

extern tml_c_font_tbl font_tbl_class_alloc_table();


    /* deallocate a font table */
    /* ARGS: tml_c_font_tbl this_font_tbl */

extern  font_tbl_free();



    /* MEMBER FUNCTIONS TO ACCESS THE DATA */


    /* get the font's name */
    /* ARGS: tml_c_font_tbl this_font_tbl */

extern char* font_tbl_get_name();


    /* get the number of characters in the font's name */
    /* ARGS: tml_c_font_tbl this_font_tbl */

extern long font_tbl_get_name_count();


    /* get the device dependent font id */
    /* ARGS: tml_c_font_tbl this_font_tbl */

extern char *font_tbl_get_handle();


    /* get the height of the font */
    /* ARGS: tml_c_font_tbl this_font_tbl */

extern long font_tbl_get_hi();


    /* get the baseline offset from top of font matrix */
    /* ARGS: tml_c_font_tbl this_font_tbl */

extern long font_tbl_get_baseline();


    /* if the font is fixed-width, return character width. Else return */
    /* the width of the character indicated by the long argument */
    /* ARGS: tml_c_font_tbl this_font_tbl, long charac */

extern long font_tbl_get_width();


    /* get the number of bytes per character */
    /* ARGS: tml_c_font_tbl this_font_tbl */

extern long font_tbl_get_char_size();


    /* get the inter-character spacing associated with the font */
    /* ARGS: tml_c_font_tbl this_font_tbl */

extern long font_tbl_get_icspace();


    /* return the font's current leading in pixels. The default is the font */
    /* height minus the maximum character height. It equals the distance */
    /* from a baseline to the top of the highest possible ascender on the */
    /* line following. Using this default leading value is the equivalent of */
    /* "set-solid" leading in typesetting, but the value is calculated */
    /* differently because we set type (conceptually) down from the upper */
    /* left corner of the bounding box, not up from the baseline. Calls to */
    /* set_leading() with nondefault values will change the font height and */
    /* subsequent calls to get_leading() will reflect this in the leading */
    /* value returned. */

    /* ARGS: tml_c_font_tbl this_font_tbl */

extern long font_tbl_get_leading();



    /* MEMBER FUNCTIONS TO SET THE DATA */


    /* set the font's internal fields to the values associated with the font */
    /* whose name is specified by the char* argument. The long argument */
    /* is the length of that name. */
    /* ARGS: tml_c_font_tbl this_font_tbl, char* fname, long flen */

extern long font_tbl_set_info();


    /* load a font into memory. Zero return value indicates success; */
    /* -1 indicates failure. */
    /* ARGS: tml_c_font_tbl this_font_tbl */

extern long font_tbl_load_font();


    /* change the leading value. See comment under get_leading() above. */
    /* ARGS: tml_c_font_tbl this_font_tbl, long lead_pixels */

extern long font_tbl_set_leading();


    /* set the intercharacter spacing */
    /* ARGS: tml_c_font_tbl this_font_tbl, long new_icspace */

extern  font_tbl_set_icspace();





/**********************************************************************************
    Objects used in the rendering component of tml

**********************************************************************************/


/* FOR CLASS V_CURSOR */


    /* allocate a viewport cursor */
    /* ARGS:  */

extern tml_c_v_cursor v_cursor_class_alloc();

    /* deallocate a viewport cursor */
    /* ARGS: tml_c_v_cursor this_v_cursor */

extern  v_cursor_free();



    /* TRAITS TO ACCESS THE DATA */


    /* get the viewport associated with a v_cursor */
    /* ARGS: tml_c_v_cursor this_v_cursor */

extern tml_c_view_port v_cursor_get_viewport();


    /* C_SCREEN_XY* argument points to caller-supplied storage */
    /* get the uncaged xy coordinates of the v_cursor. If all goes well, */
    /* the return value equals the second argument. */
    /* ARGS: tml_c_v_cursor this_v_cursor, C_SCREEN_XY* xy_p */

extern C_SCREEN_XY *v_cursor_get_raw_location();


    /* C_SCREEN_XY* argument points to caller-supplied storage */
    /* get the caged xy coordinates of the v_cursor. If all goes well, */
    /* the return value equals the second argument. */
    /* ARGS: tml_c_v_cursor this_v_cursor, C_SCREEN_XY* xy_p */

extern C_SCREEN_XY *v_cursor_get_caged_location();


    /* get the viewport position */
    /* ARGS: tml_c_v_cursor this_v_cursor */

extern tml_c_view_pos v_cursor_get_view_pos();


    /* C_SCREEN_XY* argument points to caller-supplied storage */
    /* get the model position at which the v_cursor rests. If all goes well, */
    /* the return value equals the second argument */
    /* ARGS: tml_c_v_cursor this_v_cursor, C_MODEL_POS* pos_p */

extern C_MODEL_POS *v_cursor_get_model_pos();


    /* Returns the visibility of the cursor. If visibility < 1, the cursor */
    /* is not drawn; otherwise the cursor is drawn. */
    /* ARGS: tml_c_v_cursor this_v_cursor */

extern long v_cursor_get_visible();


    /* get function that draws this v_cursor */
    /* ARGS: tml_c_v_cursor this_v_cursor */

extern C_CURSOR_DRAW v_cursor_get_draw_func();


    /* get height of this v_cursor */
    /* ARGS: tml_c_v_cursor this_v_cursor */

extern long v_cursor_get_height();


    /* get width of this v_cursor */
    /* ARGS: tml_c_v_cursor this_v_cursor */

extern long v_cursor_get_width();



    /* TRAITS TO MANIPULATE THE DATA */


    /* attach the argument v_cursor to the argument view_port */
    /* ARGS: tml_c_v_cursor this_v_cursor, tml_c_view_port to_viewport */

extern  v_cursor_attach();


    /* detach the argument v_cursor from the argument view_port */
    /* ARGS: tml_c_v_cursor this_v_cursor */

extern  v_cursor_detach();


    /* Set after_char member to the argument model_pos */
    /* ARGS: tml_c_v_cursor this_v_cursor, C_MODEL_POS* new_pos */

extern  v_cursor_set_model_pos();


    /* If the long argument is zero, increment the visibility of the */
    /* cursor (see v_cursor_get_visible(), above). If the long argument */
    /* is nonzero and the cursor is invisible with visibility < 1, make */
    /* the cursor visible by setting the visibility to 1. Otherwise */
    /* increment the cursor's visiblity. If the cursor's visibility is 0 */
    /* (i.e., just below the threshhold of invisibility) the latter case */
    /* will make the cursor visible. */
    /* ARGS: tml_c_v_cursor this_v_cursor, long force */

extern long v_cursor_enable();


    /* If the long argument is zero, decrement the visibility of the */
    /* cursor (see v_cursor_get_visible(), above). If the long argument */
    /* is nonzero and the cursor visibile with visibility > 1, make the */
    /* cursor invisible by setting the visibility to 0. Otherwise decrement */
    /* the cursor's visibility. If the cursor's visibility is 1 (i.e., just */
    /* above the threshhold of visibility) the latter case will make the */
    /* cursor invisible. */
    /* ARGS: tml_c_v_cursor this_v_cursor, long force */

extern long v_cursor_disable();


    /* sets the visibility flag to the indicated value. 0 means that the */
    /* cursor is not visible, 1 means that the cursor is visible. */
    /* NOTE--this does not draw or erase the cursor */
    /* ARGS: tml_c_v_cursor this_v_cursor, long is_visible */

extern  v_cursor_set_visible();


    /* Draws the cursor at the current caged position. Note that this */
    /* function DOES NOT update any of the other cursor information other */
    /* than cursor visibility. */
    /* Returns 1 if successful, 0 if unsuccessful */
    /* ARGS: tml_c_v_cursor this_v_cursor */

extern long v_cursor_draw();


    /* Erases the cursor at the current caged position. Note that this */
    /* function DOES NOT update any of the other cursor information other */
    /* than cursor visibility. Returns 1 if successful, 0 if unsuccessful*/
    /* ARGS: tml_c_v_cursor this_v_cursor */

extern long v_cursor_erase();


    /* set the function that draws the cursor. */
    /* ARGS: tml_c_v_cursor this_v_cursor, C_CURSOR_DRAW new_func */

extern  v_cursor_set_draw_func();


    /* C_SCREEN_XY* argument points to caller-supplied storage. */
    /* Move the cursor in the direction indicated by the long argument */
    /* (one of enum vp_dir). Returns a pointer to the screen position */
    /* at which the cursor rests after the move. If all goes well, the */
    /* return value equals the third argument. */
    /* ARGS: tml_c_v_cursor this_v_cursor, vp_dir move_dir, C_SCREEN_XY* new_screen_xy */

extern C_SCREEN_XY *v_cursor_move();


    /* move the cursor to the viewport xy indicated by the second argument */
    /* ARGS: tml_c_v_cursor this_v_cursor, C_SCREEN_XY* position */

extern long v_cursor_xy_set();


    /* move the cursor to the model position indicated by the second argument */
    /* ARGS: tml_c_v_cursor this_v_cursor, C_MODEL_POS* pos_p */

extern long v_cursor_mpos_set();




/* FOR CLASS VIEW_PORT */


    /* allocate a viewport */
    /* ARGS:  */

extern tml_c_view_port view_port_class_alloc();


    /* deallocate a viewport */
    /* ARGS: tml_c_view_port this_view_port */

extern  view_port_free();



    /* TRAITS TO ACCESS THE DATA */


    /* return the bitmap handle associated with the viewport */
    /* ARGS: tml_c_view_port this_view_port */

extern char *view_port_get_handle();


    /* get the width of the viewport */
    /* ARGS: tml_c_view_port this_view_port */

extern long view_port_get_width();


    /* get the height of the viewport */
    /* ARGS: tml_c_view_port this_view_port */

extern long view_port_get_hi();


    /* get horizontal offset of viewport on conceptual page */
    /* ARGS: char *view_port_p */

extern long view_port_get_hoffset();


    /* get top line visible in the viewport */
    /* ARGS: tml_c_view_port this_view_port */

extern tml_c_view_pos view_port_get_top_line();


    /* get bottom line visible in the viewport */
    /* ARGS: tml_c_view_port this_view_port */

extern tml_c_view_pos view_port_get_bottom_line();


    /* get the format associated with the viewport */
    /* ARGS: tml_c_view_port this_view_port */

extern tml_c_format view_port_get_format();


    /* get the cursor associated with the viewport */
    /* ARGS: tml_c_view_port this_view_port */

extern tml_c_v_cursor view_port_get_cursor();


    /* get the id of the foreground color associated with the viewport */
    /* ARGS: tml_c_view_port this_view_port */

extern long view_port_get_foreg_id();


    /* get the id of the background color associated with the viewport */
    /* ARGS: tml_c_view_port this_view_port */

extern long view_port_get_backg_id();


    /* get the state of the property indexed by the long argument. */
    /* If there is an entry for this property in the viewport's property */
    /* state table, return it. Otherwise, if a format is associated with */
    /* the viewport return the state of the property in the format's */
    /* property state table (which defaults to the model's property */
    /* state table setting when no additional setting is made in the */
    /* format), but if no format is associated with the viewport, */
    /*  return zero. */
    /* ARGS: tml_c_view_port this_view_port, p_index prop_ind */

extern char *view_port_inq_state();


    /* get the state of the property indexed by the long argument at the */
    /* argument node. */
    /* If the viewport's property state table has an entry for this */
    /* property, return its state. Otherwise return the associated format's */
    /* property state table entry (which defaults to the model's property */
    /* state setting), but if no format is associated with the viewport, */
    /* return 0. */
    /* ARGS: tml_c_view_port this_view_port, tml_c_node at_node, p_index prop_ind */

extern char *view_port_inq_state_at_node();


    /* get the draw marks associaed with the viewport */
    /* ARGS: tml_c_view_port view_port_p */

extern char *view_port_get_marks();



    /* TRAITS TO SET DATA */


    /* assign the pre-allocated bitmap to the viewport, fill in the width */
    /* and height. This function assumes that the viewport fills the bitmap. */

    /* ARGS: tml_c_view_port this_view_port, tml_c_bitmap_id bitmap_id */

extern long view_port_set_handle();


    /* attach a format to the viewport. If the format argument is zero, */
    /* create a new one; width and/or depth of zero means that the */
    /* viewport's dimension(s) (is/are) used to initialize the format's */
    /* corresponding dimension(s). Returns -1 if allocation is necessary */
    /* and it fails; returns 0 otherwise */
    /* ARGS: tml_c_view_port this_view_port, tml_c_format to_format, long fx_size, long fy_size */

extern long view_port_attach_format();


    /* detach a format from the viewport. If the format's reference count */
    /* becomes zero in doing so, the format is deleted. */
    /* ARGS: tml_c_view_port this_view_port */

extern  view_port_detach_format();


    /* attach a viewport cursor to the viewport. If the argument cursor */
    /* is attached to another viewport, an error is returned. If the */
    /* argument C_SCREEN_XY* is NULL, the cursor is set to the upper */
    /* left hand corner of the screen (coordinates {0,0}); otherwise the */
    /* coordinates supplied by the C_SCREEN_XY* argument are used. */
    /* ARGS: tml_c_view_port this_view_port, tml_c_v_cursor to_cursor, C_SCREEN_XY* set_to */

extern long view_port_attach_cursor();


    /* detach a cursor from the viewport */
    /* ARGS: tml_c_view_port this_view_port */

extern  view_port_detach_cursor();


    /* set the top line of the viewport to be the argument view position */
    /* ARGS: tml_c_view_port this_view_port, tml_c_view_pos new_top */

extern  view_port_set_top_line();


    /* set the bottom line of the viewport to be the argument view position */
    /* ARGS: tml_c_view_port this_view_port, tml_c_view_pos new_bottom */

extern  view_port_set_bottom_line();


    /* set the foreground color of the viewport to the color identified */
    /* by the long argument */
    /* ARGS: tml_c_view_port this_view_port, long new_id */

extern  view_port_set_foreg_id();


    /* set the background color of the viewport to the color identified */
    /* by the long argument */
    /* ARGS: tml_c_view_port this_view_port, long new_id */

extern  view_port_set_backg_id();


    /* set the default state of the indexed property in the viewport's */
    /* property state table to the value pointed to by the char *argument. */
    /* ARGS: tml_c_view_port this_view_port, p_index prop_ind, char *value */

extern long view_port_set_state();


    /* cancel property override at the viewport level for the property */
    /* indexed by the p_index argument */
    /* ARGS: tml_c_view_port this_view_port, p_index prop_ind */

extern long view_port_unset_state();


    /* add the argument draw_list to the viewport */
    /* ARGS: tml_c_view_port view_port_p, tml_c_draw_mark new_mark */

extern  view_port_add_mark();


    /* detach the argument draw_list from the viewport */
    /* ARGS: tml_c_view_port view_port_p, tml_c_draw_mark del_mark */

extern  view_port_detach_mark();



/* ROUTINES WHICH ALLOCATE AND FREE VIEW POSITIONS IN THE VIEWPORT */


    /* allocate a view position from the view position store for this */
    /* viewport. If none are available, add some and return one. */
    /* ARGS: tml_c_view_port this_view_port */

extern tml_c_view_pos view_port_get_pos();


    /* return a view_pos to the free heap for the viewport, initializing */
    /* the contents. When a view_pos is freed, all view_pos to the right */
    /* of the free view_pos are also freed. */
    /* ARGS: tml_c_view_port this_view_port, tml_c_view_pos to_free */

extern  view_port_free_pos();


    /* free all the view_pos associated with a viewport. */
    /* ARGS: tml_c_view_port this_view_port */

extern  view_port_free_all_pos();



    /* ROUTINES THAT ACTUALLY DRAW THINGS */


    /* clear (an area of) the viewport. First C_SCREEN_XY* is upper left */
    /* corner of area to be cleared; second is lower right corner. If both */
    /* C_SCREEN_XY* arguments are set to &zero_zero, the whole viewport is */
    /* cleared; if only the second C_SCREEN_XY* is set to &zero_zero, the */
    /* viewport is cleared from the first coordinate to the bottom edge */
    /* of the viewport. */
    /* ARGS: tml_c_view_port this_view_port, C_SCREEN_XY* ul, C_SCREEN_XY* lr */

extern  view_port_clear();


    /* draw the full viewport, starting with the given conceptual line */
    /* ARGS: tml_c_view_port this_view_port, tml_c_cline first_line, C_SCREEN_XY* where */

extern  view_port_draw();


    /* draw the screen, putting the character specified by the */
    /* C_MODEL_POS* argument at (or as close as possible to) the */
    /* coordinates supplied in the C_SCREEN_XY* argument */
    /* ARGS: tml_c_view_port this_view_port, C_MODEL_POS* draw_char, C_SCREEN_XY* where */

extern  view_port_draw_at();


    /* redraw the viewport */
    /* ARGS: tml_c_view_port this_view_port */

extern  view_port_redraw();


    /* redraw the viewport, reformatting as we go, from the supplied */
    /* view_pos until no more is needed. ref_start_mpos indicates the model */
    /* position which begins the conceptual line which is the start of the */
    /* view pos contents, rend_start_mpos indicates the model position */
    /* at which rerendering should start*/
    /* ARGS: tml_c_view_port this_view_port, C_MODEL_POS* ref_start_mpos, C_MODEL_POS* rend_start_mpos, tml_c_view_pos start_vpos */

extern  view_port_redraw_at();



    /* NAVIGATION ROUTINES */


    /* convert from the xy coordinates supplied in the argument */
    /* C_SCREEN_XY* to the view_pos occupying that space in the viewport. */
    /* If no view_pos occupies the space, get the closest one. */
    /* ARGS: tml_c_view_port this_view_port, C_SCREEN_XY* where */

extern tml_c_view_pos view_port_xy_2_vpos();


    /* convert from the xy coordinates supplied in the argument */
    /* C_SCREEN_XY* to the cline occupying that space in the viewport. */
    /* If no cline occupies the space, get the closest one. */
    /* ARGS: tml_c_view_port this_view_port, C_SCREEN_XY* where */

extern tml_c_cline view_port_xy_2_cline();


    /* C_MODEL_POS* argument points to caller-supplied storage */
    /* Returns a pointer to the model position associated with the screen */
    /* position argument. If all goes well, the return value equals the */
    /* third argument. */
    /* ARGS: tml_c_view_port this_view_port, C_SCREEN_XY* where, C_MODEL_POS* new_model_pos */

extern C_MODEL_POS *view_port_xy_2_mpos();


    /* find the first view_pos containing the conceptual line. If the line */
    /* is not on the screen, return 0. */
    /* ARGS: tml_c_view_port this_view_port, tml_c_cline find_line */

extern tml_c_view_pos view_port_cline_2_vpos();


    /* C_SCREEN_XY* argument points to caller-supplied storage */
    /* Returns a pointer to the screen position associated with the cline */
    /* argument. If all goes well, the return value equals the third */
    /* argument. */
    /* ARGS: tml_c_view_port this_view_port, tml_c_cline find_line, C_SCREEN_XY* new_screen_xy */

extern C_SCREEN_XY *view_port_cline_2_xy();


    /* return the view_pos containing the supplied character if that */
    /* character is on the screen. Otherwise return zero. */
    /* ARGS: tml_c_view_port this_view_port, C_MODEL_POS* find_char */

extern tml_c_view_pos view_port_mpos_2_vpos();


    /* C_SCREEN_XY* argument points to caller-supplied storage */
    /* Returns a pointer to the screen position associated with the model */
    /* position argument. If all goes well, the return value equals the */
    /* third argument. If the tml_c_view_pos argument is zero, or if the */
    /* the routine can't find the character indicated by the model position */
    /* argument at a supplied view position, mpos_2_vpos is called to */
    /* find the correct view position. Returns a pointer to */
    /* C_SCREEN_XY {x=0, y=-1} if the character is not on the screen. */
    /* ARGS: tml_c_view_port this_view_port, C_MODEL_POS* find_char, C_SCREEN_XY* new_view_pos, tml_c_view_pos hint */

extern C_SCREEN_XY *view_port_mpos_2_xy();

    /* returns 1 if the page is currently visible in the viewport, */
    /* 0 otherwise */
    /* ARGS: tml_c_view_port this_view_port, tml_c_phead page */

extern long view_port_is_visible();



    /* EDITING ROUTINES */


    /* append string in_string of length count at model position after, */
    /* reformatting and rendering as needed. */
    /* ARGS: tml_c_view_port this_view_port, char* in_string, long count, C_MODEL_POS* after */

extern long view_port_append_string();


    /* prepend string in_string of length count to character at model */
    /* position before, reformatting and rendering as needed. */
    /* ARGS: tml_c_view_port this_view_port, char* in_string, long count, C_MODEL_POS* before */

extern long view_port_prepend_string();


    /* delete the character at the model position, reformatting and */
    /* rendering as needed */
    /* ARGS: tml_c_view_port this_view_port, C_MODEL_POS* delete_pos */

extern long view_port_delete_char();


    /* insert string in_string of length count before the current cursor */
    /* position, reformatting and rerendering as needed. If the cursor */
    /* position is invalid, insert before the first character dominated */
    /* by root. */
    /* ARGS: tml_c_view_port this_view_port, char* in_string, long count */

extern long view_port_ins_before_cursor();


    /* insert string in_string of length count after the current cursor */
    /* position, reformatting and rendering as needed. If the cursor */
    /* position is invalid, insert before the first character dominated */
    /* by root. */
    /* ARGS: tml_c_view_port this_view_port, char* in_string, long count */

extern long view_port_ins_after_cursor();


    /* append a property vector element referencing the argument property */
    /* to the property vector for the argument node. Returns 1 on success, */
    /* zero otherwise */
    /* ARGS: tml_c_view_port this_view_port, tml_c_node* to_node, tml_c_property add_prop */

extern long view_port_insert_prop_at_node();


    /* perform view_port_insert_prop_at_node on a new node created to dominate */
    /* all and only the characters within the given pair of model positions. */
    /* Refresh viewport if necessary. Returns 1 on success, zero otherwise. */
    /* ARGS: tml_c_view_port view_port_p, C_MODEL_POS *pos1, C_MODEL_POS *pos2, tml_c_property add_prop */

extern long view_port_insert_prop_on_range();


    /* delete a property from a node; refresh viewport if anything dominated */
    /* by that node is visible. Return 1 on success, zero otherwise. */
    /* ARGS: tml_c_view_port this_view_port, tml_c_property del_prop, tml_c_node from_node */

extern long view_port_delete_prop();


    /* mark a region in the viewport. Make a range in the model, make a */
    /* model_mark, make a draw_mark, and then add the mark to this viewport */
    /* ARGS: tml_c_view_port this_view_port, C_MODEL_POS* start_mark, C_MODEL_POS* end_mark */

extern long view_port_mark_region();
                                     



/* FOR CLASS VIEW_POS */


    /* allocate a view_pos */
    /* ARGS:  */

extern tml_c_view_pos view_pos_class_alloc();


    /* deallocate a view_pos */
    /* ARGS: tml_c_view_pos this_view_pos */

extern  view_pos_free();



    /* TRAITS TO ACCESS THE DATA */


    /* get the prior view_pos */
    /* ARGS: tml_c_view_pos this_view_pos */

extern tml_c_view_pos view_pos_get_prior();


    /* get the next view_pos */
    /* ARGS: tml_c_view_pos this_view_pos */

extern tml_c_view_pos view_pos_get_next();


    /* get the view_pos above this one */
    /* ARGS: tml_c_view_pos this_view_pos */

extern tml_c_view_pos view_pos_get_up();


    /* get the view_pos below this one */
    /* ARGS: tml_c_view_pos this_view_pos */

extern tml_c_view_pos view_pos_get_down();


    /* get the viewport with which this view_pos is associated */
    /* ARGS: tml_c_view_pos this_view_pos */

extern tml_c_view_port view_pos_get_viewport();


    /* get the x coordinate of the upper left corner of this view_pos */
    /* ARGS: tml_c_view_pos this_view_pos */

extern long view_pos_get_x();


    /* get the y coordinate of the upper left corner of this view_pos */
    /* ARGS: tml_c_view_pos this_view_pos */

extern long view_pos_get_y();


    /* get the conceptual line contained by this view_pos */
    /* ARGS: tml_c_view_pos this_view_pos */

extern tml_c_cline view_pos_get_cline();


    /* C_MODEL_POS* argument points to caller-supplied storage. */
    /* Returns pointer to model position associated with start of this */
    /* view position. If all goes well, the return value equals the second */
    /* argument. */
    /* ARGS: tml_c_view_pos this_view_pos, C_MODEL_POS* pos_p */

extern C_MODEL_POS* view_pos_get_start();


    /* C_MODEL_POS* argument points to caller-supplied storage. */
    /* Returns pointer to model position associated with end of this */
    /* view position. If all goes well, the return value equals the second */
    /* argument. */
    /* ARGS: tml_c_view_pos this_view_pos, C_MODEL_POS* pos_p */

extern C_MODEL_POS* view_pos_get_end();



    /* TRAITS TO MANIPULATE THE DATA */


    /* set the x coordinate of the upper left corner of this view_pos */
    /* ARGS: tml_c_view_pos this_view_pos, long new_x */

extern  view_pos_set_x();


    /* set the y coordinate of the upper left corner of this view_pos */
    /* ARGS: tml_c_view_pos this_view_pos, long new_y */

extern  view_pos_set_y();


    /* cline argument points to storage not to be reused */
    /* ARGS: tml_c_view_pos this_view_pos, tml_c_cline new_line */

extern  view_pos_set_line();



    /* MISCELLANEOUS FUNCTIONS */


    /* disassociate the view_pos from any others it is attached to */
    /* ARGS: tml_c_view_pos this_view_pos */

extern  view_pos_detach();


    /* set the view_pos's viewport member to the argument viewport */
    /* ARGS: tml_c_view_pos this_view_pos, tml_c_view_port to_viewport */

extern  view_pos_attach_port();


    /* attach the first view_pos to the second in the direction indicated */
    /* (one of enum vp_dir). If all goes well, return 0; otherwise */
    /* return -1. */
    /* ARGS: tml_c_view_pos this_view_pos, tml_c_view_pos to_pos, vp_dir dir */

extern long view_pos_attach_pos();


    /* attach a new view_pos to this one in the direction indicated */
    /* (one of enum vp_dir). If all goes well, return the new view position; */
    /* otherwise return 0. */
    /* ARGS: tml_c_view_pos this_view_pos, vp_dir dir */

extern tml_c_view_pos view_pos_add();


    /* adjust the xy positions of a set of view positions. First argument */
    /* is x delta, second is y delta (pixels). A "scrolling" function. */
    /* ARGS: tml_c_view_pos this_view_pos, long x_delta, long y_delta */

extern  view_pos_adjust();


    /* return 1 if the model position argument is found between the */
    /* beginning and end of this view position (inclusive). */
    /* Return 0 otherwise. */
    /* ARGS: tml_c_view_pos this_view_pos, C_MODEL_POS* find_pos */

extern long view_pos_contains();

    


/* UTILITY FUNCTIONS */


    /* get the type identifier of the default graphics system */
    /* ARGS:  */

extern tml_c_grafsys tml_c_tml_dp_get_dflt_grafsys_type();


    /* get the function table for the default graphics system */
    /* ARGS:  */

extern tml_c_vpftbl tml_c_tml_dp_get_dflt_vpftbl();


    /* set the function table for the default graphics system to the value supplied */
    /* ARGS: tml_c_vpftbl ftbl */

extern  tml_c_tml_dp_set_dflt_vpftbl();


    /* get the XWindows viewport function table */
    /* ARGS:  */

extern tml_c_vpftbl tml_c_tml_dp_get_x_vpftbl();


#ifdef apollo
    /* get the gpr viewport function table */
    /* ARGS:  */

extern tml_c_vpftbl tml_c_tml_dp_get_gpr_vpftbl();
#endif


  /* from the graphics system function table given, get the function associated with the index supplied */
    /* ARGS: tml_c_vpftbl ftbl, long index */

extern GEN_FUNCP tml_c_get_graf_func_from_table();


  /* allocate a new property vector in a model */
    /* ARGS: tml_c_model to_model */

extern tml_c_prop_vec tml_c_pvec_alloc();


    /* allocate a new property in a model */
    /* ARGS: tml_c_model to_model */

extern tml_c_property tml_c_prop_alloc();


    /* allocate a new argument in a model */
    /* ARGS: tml_c_model to_model */

extern tml_c_argument tml_c_arg_alloc();


    /* given a start node (first argument) and an end node (second argument), */
    /* isolate a coherent subtree subsuming all text from start to end, and */
    /* create any nodes necessary to preserve properties as far up the tree */
    /* as necessary. Assumes that nodes provided are whole leaf nodes. */
    /* Returns the root of the isolated subtree. */
    /* ARGS: tml_c_node start_node, tml_c_node end_node */

extern tml_c_node tml_c_make_subtree();


    /* copy text from start point (indicated by first C_MODEL_POS* argument) */
    /* to end point (second C_MODEL_POS* argument) to the argument model. */
    /* Startpoint and endpoint, as they are model positions, each consist of */
    /* a node and character index. All subtree structure and properties are */
    /* preserved at the target. The target model may be another text file */
    /* being edited, or it may be a paste buffer. This function does not */
    /* connect the copied subtree into the target model; must call */
    /* tml_c_node_connect_node() to do this. Returns root of copied subtree */
    /* in target model. */
    /* ARGS: C_MODEL_POS* start, C_MODEL_POS* end, tml_c_model to_model */

extern tml_c_node tml_c_copy_component();


    /* create and return a pointer to a word wrap property, with the value */
    /* indicated by the integer argument, in the argument model */
    /* ARGS: long wrap_on, tml_c_model in_model */

extern tml_c_property tml_c_make_wrap_prop();


    /* create and return a pointer to a margin property, with the values */
    /* indicated, in the argument model. First long argument is left */
    /* margin, second is right, third top, fourth bottom. */
    /* ARGS: long left, long right, long top, long bottom, tml_c_model in_model */

extern tml_c_property tml_c_make_margin_prop();


    /* create and return a pointer to an invert property, with the value */
    /* indicated, in the given model */
    /* ARGS: long inv_on, tml_c_model in_model */

extern tml_c_property tml_c_make_inv_prop();


    /* create and return a pointer to a font property, with the values */
    /* indicated, in the given model. Argument char* is font name, integer */
    /* argument is name length. */
    /* ARGS: char* font_name, long name_len, tml_c_model in_model */

extern tml_c_property tml_c_make_font_prop();


    /* create and return a pointer to a font property, with the values */
    /* indicated, in the given model. Argument char* is font name, integer */
    /* argument is name length. The tml_c_grafsys argument specifies */
    /* a graphics system. */
    /* ARGS: char* font_name, long name_len, tml_c_model in_model, tml_c_grafsys for_grafsys */

extern tml_c_property tml_c_make_font_prop_grafsys();


    /* create and return a pointer to a page size property, with the height */
    /* and width indicated, in the given model. */
    /* ARGS: long width, long depth, tml_c_model in_model */

extern tml_c_property tml_c_make_page_size_prop();


    /* create and return a pointer to a tab property, a linked list */
    /* of tab stops, in the given model */
    /* ARGS: long default_val, long num_stops, long *tab_st_list, tml_c_model in_model */

extern tml_c_property tml_c_make_tab_prop();


    /* connect a property to a node at the end of that node's property vector */
    /* ARGS: tml_c_property add_prop, tml_c_node to_node */

extern  tml_c_connect_prop();


    /* linearize the goo in a model - this function replaces the model's */
    /* goo heaps with new goo heaps in which the goo is compressed and */
    /* linearized. This function is called by prune() (see below) so there */
    /* is no need to linearize goo in a separate step if that function */
    /* is called. This function deletes all history associated with the */
    /* model, linearizes goo, then restores the size of the history stack */
    /* to what it was before this function was called. */
    /* ARGS: tml_c_model old_model */

extern  tml_c_gc_linearize();


    /* compress objects and linearize goo - this function calls */
    /* tml_c_gc_linearize() (see above) - it also copies to a new model */
    /* unique copies of all objects still active in the model, compressing */
    /* out unused objects. The compressed goo heaps attached to the old */
    /* model are moved to the new model. */
    /* ARGS: tml_c_model old_model */

extern tml_c_model tml_c_prune();


    /* read a model from the file indicated by the system-dependent */
    /* file handle. Return a pointer to the new model */
    /* ARGS: char *file_handle */

extern tml_c_model tml_c_d_read_model_handle();


    /* read a model from the file indicated by the pathname and pathname */
    /* length arguments. Return a pointer to the new model */
    /* ARGS: char *pathname, long name_len */

extern tml_c_model tml_c_d_read_model_path();


    /* read a model from the file indicated by the pathname and pathname */
    /* length arguments. The oflag and mode arguments are system-dependent */
    /* quantities. When the function returns, the contents of the */
    /* file_handle pointer argument is the system-dependent file_handle */
    /* associated with the file, which remains open. */
    /* ARGS: char *pathname, long name_len, long oflag, long mode, char **file_handle_p */

extern tml_c_model tml_c_d_read_model_open();


    /* read the ascii file indicated by the argument filename and name */
    /* length; make a model of the file's contents and return a pointer */
    /* to the model. */
    /* ARGS: char *pathname, long name_len */

extern tml_c_model tml_c_read_uasc_path();


    /* read the ascii file indicated by the system-dependent file-handle; */
    /* make a model of the file's contents and return a pointer to the model */
    /* ARGS: char *file_handle */

extern tml_c_model tml_c_read_uasc_handle();


    /* produce a postscript file from a tml model. The output filename is */
    /* outfilename of length outlen. */
    /* ARGS: tml_c_model this_model, C_PRINT_DB *print_params, char* outfilename, long outlen */

extern long tml_c_tml_model_print();


    /* produce a postscript file from a tml file. The input filename is */
    /* infilename of length inlen; the output filename is outfilename of */
    /* length outlen. */
    /* ARGS: char* infilename, long inlen, C_PRINT_DB *print_params, char* outfilename, long outlen */

extern long tml_c_tml_file_print();


    /* update history when a call to prop_vec_detach detaches the only */
    /*   remaining prop_vec in a vector. */
    /* Call this function when a call to prop_vec_detach() returns 0. */
    /* Argument "on_node" is the node whose properties member contained */
    /*   only the one prop_vec "removed". */
    /* ARGS: tml_c_node on_node, tml_c_prop_vec removed */

extern  tml_c_first_pvec_hist();


    /* update history when a call to argument_detach detaches the only */
    /*   remaining argument in a property object's entry argument  vector. */
    /* Call this function when a call to argument_detach() on the entry */
    /*   argument vector of a property returns 0. */
    /* Argument "on_prop" is the property whose entry_arg member contained */
    /*   only the one argument "removed". */
    /* ARGS: tml_c_property on_prop, tml_c_argument removed */

extern  tml_c_first_entry_arg_hist();


    /* update history when a call to argument_detach detaches the only */
    /*   remaining argument in a property object's exit argument vector. */
    /* Call this function when a call to argument_detach() on the exit */
    /*   argument vector of a property returns 0. */
    /* Argument "on_prop" is the property whose exit_arg member contained */
    /*   only the one argument "removed". */
    /* ARGS: tml_c_property on_prop, tml_c_argument removed */

extern  tml_c_first_exit_arg_hist();

#endif

#endif
/* tml_c_types_h */
