/**************************************************************************************
    File tml_extend.cxx
    Declarations of persistent types used by TMLib

    (c) Copyright Apollo Computer Inc., 1990 (unpublished)
     --------------------------------------------------------------------
    |THE FOLLOWING PROGRAMS ARE THE SOLE PROPERTY OF APOLLO COMPUTER INC.|
    |      AND CONTAIN ITS PROPRIETARY AND CONFIDENTIAL INFORMATION.     |
     --------------------------------------------------------------------
**************************************************************************************/

/**********************************************************************************
    SEE EXPLANATORY NOTE AT HEAD OF FILE EXTEND.CXX
**********************************************************************************/

#define tml_extend_cxx

/* File csm_extend.cxx defines types for the CSM */

#include <tml/csm_extend.cxx>
#include <tml/tml_objects.hxx>
#include <tml/tml_functions.hxx>
#include <tml/tml_dp_graph.hxx>

/**********************************************************************************
    sample objects for preloaded types
**********************************************************************************/

    extern vpftbl       dflt_view_port_function_table;
#ifdef apollo
    extern              GEN_FUNCP gpr_dflt_view_port_function_table[];
#endif

#ifdef apollo_global
    vpftbl*             dflt_vpftbl;
    vpftbl*             tml_sample_xwindows_grafsys;
    vpftbl*             tml_sample_gpr_grafsys;
#else
    vpftbl*             tml_sample_xwindows_grafsys = (vpftbl*) &dflt_view_port_function_table;
#ifdef apollo
    vpftbl*             tml_sample_gpr_grafsys = (vpftbl*) gpr_dflt_view_port_function_table;
#endif
    union_char_set_arg  tml_sample_union_char_set_arg;
    char_set_arg        tml_sample_char_set_arg;
    subtree_arg         tml_sample_subtree_arg;
    string_arg          tml_sample_string_arg;
    float_arg           tml_sample_float_arg;
    int_arg             tml_sample_int_arg;
    argument            tml_sample_argument;
    font_property       tml_sample_font_prop;
    property            tml_sample_prop;
    pgnode              tml_sample_pgnode;
    gnode               tml_sample_gnode;
    bnode               tml_sample_bnode;
#endif

/**********************************************************************************
    preloaded graphics system types
**********************************************************************************/

#ifdef apollo_global
tml_ext_type tml_xwindows_grafsys_ext_t;
tml_ext_type tml_gpr_grafsys_ext_t;

tml_type tml_xwindows_grafsys_t;
tml_type tml_gpr_grafsys_t;

tml_type *tml_xwindows_grafsys;
tml_type *tml_gpr_grafsys;

#else
tml_ext_type tml_xwindows_grafsys_ext_t
    = { 0x48fd7f15,
        0x5000,
        0x0000,
        0x0d,
        {0x00, 0x02, 0x2e, 0xcb, 0x00, 0x00, 0x00} };

#ifdef apollo
tml_ext_type tml_gpr_grafsys_ext_t
    = { 0x48fd7f20,
        0x7000,
        0x0000,
        0x0d,
        {0x00, 0x02, 0x2e, 0xcb, 0x00, 0x00, 0x00} };
#endif

/* Types declared for CSM in csm_extend.cxx appear at tail of queue */

tml_type tml_xwindows_grafsys_t         = {&tml_xwindows_grafsys_ext_t, &tml_sample_xwindows_grafsys, &tml_char_set_t};

#ifdef apollo
tml_type tml_gpr_grafsys_t              = {&tml_gpr_grafsys_ext_t, &tml_sample_gpr_grafsys, &tml_xwindows_grafsys_t};
#endif

tml_type* const tml_xwindows_grafsys    = &tml_xwindows_grafsys_t;

#ifdef apollo
tml_type* const tml_gpr_grafsys         = &tml_gpr_grafsys_t;
#endif

#endif

/**********************************************************************************
    preloaded argument types
**********************************************************************************/

#ifdef apollo_global
tml_ext_type tml_union_char_set_val_ext_t;
tml_ext_type tml_char_set_val_ext_t;
tml_ext_type tml_subtree_val_ext_t;
tml_ext_type tml_string_val_ext_t;
tml_ext_type tml_float_val_ext_t;
tml_ext_type tml_int_val_ext_t;
tml_ext_type tml_no_val_ext_t;

tml_type tml_union_char_set_val_t;
tml_type tml_char_set_val_t;
tml_type tml_subtree_val_t;
tml_type tml_string_val_t;
tml_type tml_float_val_t;
tml_type tml_int_val_t;
tml_type tml_no_val_t;

tml_type *union_char_set_val_arg;
tml_type *char_set_val_arg;
tml_type *subtree_val_arg;
tml_type *string_val_arg;
tml_type *float_val_arg;
tml_type *int_val_arg;
tml_type *no_val_arg;

#else
tml_ext_type tml_union_char_set_val_ext_t
    = { 0x41442d1c,
        0xa000,
        0x0000,
        0x0d,
        {0x00, 0x01, 0x9f, 0xab, 0x00, 0x00, 0x00} };

tml_ext_type tml_char_set_val_ext_t
    = { 0x41442c21,
        0x5000,
        0x0000,
        0x0d,
        {0x00, 0x01, 0x9f, 0xab, 0x00, 0x00, 0x00} };

tml_ext_type tml_subtree_val_ext_t
    = { 0x405b4f33,
        0x4000,
        0x0000,
        0x0d,
        {0x00, 0x01, 0x9f, 0xab, 0x00, 0x00, 0x00} };

tml_ext_type tml_string_val_ext_t
    = { 0x405b4f0a,
        0xf000,
        0x0000,
        0x0d,
        {0x00, 0x01, 0x9f, 0xab, 0x00, 0x00, 0x00} };

tml_ext_type tml_float_val_ext_t
    = { 0x405b4ec2,
        0x9000,
        0x0000,
        0x0d,
        {0x00, 0x01, 0x9f, 0xab, 0x00, 0x00, 0x00} };

tml_ext_type tml_int_val_ext_t
    = { 0x405b4e8f,
        0x5000,
        0x0000,
        0x0d,
        {0x00, 0x01, 0x9f, 0xab, 0x00, 0x00, 0x00} };

tml_ext_type tml_no_val_ext_t
    = { 0x405b4e60,
        0x2000,
        0x0000,
        0x0d,
        {0x00, 0x01, 0x9f, 0xab, 0x00, 0x00, 0x00} };

#ifdef apollo
tml_type tml_union_char_set_val_t      = {&tml_union_char_set_val_ext_t, &tml_sample_union_char_set_arg, &tml_gpr_grafsys_t};
#else
tml_type tml_union_char_set_val_t      = {&tml_union_char_set_val_ext_t, &tml_sample_union_char_set_arg, &tml_xwindows_grafsys_t};
#endif
tml_type tml_char_set_val_t            = {&tml_char_set_val_ext_t, &tml_sample_char_set_arg, &tml_union_char_set_val_t};
tml_type tml_subtree_val_t             = {&tml_subtree_val_ext_t, &tml_sample_subtree_arg, &tml_char_set_val_t};
tml_type tml_string_val_t              = {&tml_string_val_ext_t, &tml_sample_string_arg, &tml_subtree_val_t};
tml_type tml_float_val_t               = {&tml_float_val_ext_t, &tml_sample_float_arg, &tml_string_val_t};
tml_type tml_int_val_t                 = {&tml_int_val_ext_t, &tml_sample_int_arg, &tml_float_val_t};
tml_type tml_no_val_t                  = {&tml_no_val_ext_t, &tml_sample_argument, &tml_int_val_t};

tml_type* const union_char_set_val_arg = &tml_union_char_set_val_t;
tml_type* const char_set_val_arg       = &tml_char_set_val_t;
tml_type* const subtree_val_arg        = &tml_subtree_val_t;
tml_type* const string_val_arg         = &tml_string_val_t;
tml_type* const float_val_arg          = &tml_float_val_t;
tml_type* const int_val_arg            = &tml_int_val_t;
tml_type* const no_val_arg             = &tml_no_val_t;
#endif

/**********************************************************************************
    preloaded property types
**********************************************************************************/

#ifdef apollo_global
tml_ext_type tml_tab_prop_ext_t;
tml_ext_type tml_union_char_set_prop_ext_t;
tml_ext_type tml_char_set_prop_ext_t;
tml_ext_type tml_form_feed_prop_ext_t;
tml_ext_type tml_footer_prop_ext_t;
tml_ext_type tml_header_prop_ext_t;
tml_ext_type tml_pgsize_prop_ext_t;
tml_ext_type tml_inv_prop_ext_t;
tml_ext_type tml_font_prop_ext_t;
tml_ext_type tml_margin_prop_ext_t;
tml_ext_type tml_wrap_prop_ext_t;
tml_ext_type tml_undefined_prop_ext_t;

tml_type tml_tab_prop_t;
tml_type tml_union_char_set_prop_t;
tml_type tml_char_set_prop_t;
tml_type tml_form_feed_prop_t;
tml_type tml_footer_prop_t;
tml_type tml_header_prop_t;
tml_type tml_pgsize_prop_t;
tml_type tml_inv_prop_t;
tml_type tml_font_prop_t;
tml_type tml_margin_prop_t;
tml_type tml_wrap_prop_t;
tml_type tml_undefined_prop_t;

tml_type *tml_tab_prop;
tml_type *tml_union_char_set_prop;
tml_type *tml_char_set_prop;
tml_type *tml_form_feed_prop;
tml_type *tml_footer_prop;
tml_type *tml_header_prop;
tml_type *tml_pgsize_prop;
tml_type *tml_inv_prop;
tml_type *tml_font_prop;
tml_type *tml_margin_prop;
tml_type *tml_wrap_prop;
tml_type *undefined_prop;

#else
tml_ext_type tml_tab_prop_ext_t
    = { 0x42fbf155,
        0x5000,
        0x0000,
        0x0d,
        {0x00, 0x01, 0x19, 0x9f, 0x00, 0x00, 0x00} };

tml_ext_type tml_union_char_set_prop_ext_t
    = { 0x4115f074,
        0x1000,
        0x0000,
        0x0d,
        {0x00, 0x01, 0x9f, 0xab, 0x00, 0x00, 0x00} };

tml_ext_type tml_char_set_prop_ext_t
    = { 0x40f60484,
        0xa000,
        0x0000,
        0x0d,
        {0x00, 0x01, 0x9f, 0xab, 0x00, 0x00, 0x00} };
    
tml_ext_type tml_form_feed_prop_ext_t
    = { 0x405b4416,
        0xe000,
        0x0000,
        0x0d,
        {0x00, 0x01, 0x9f, 0xab, 0x00, 0x00, 0x00} };

tml_ext_type tml_footer_prop_ext_t
    = { 0x405b43d9,
        0xa000,
        0x0000,
        0x0d,
        {0x00, 0x01, 0x9f, 0xab, 0x00, 0x00, 0x00} };

tml_ext_type tml_header_prop_ext_t
    = { 0x405b439d,
        0x7000,
        0x0000,
        0x0d,
        {0x00, 0x01, 0x9f, 0xab, 0x00, 0x00, 0x00} };

tml_ext_type tml_pgsize_prop_ext_t
    = { 0x405b437b,
        0x4000,
        0x0000,
        0x0d,
        {0x00, 0x01, 0x9f, 0xab, 0x00, 0x00, 0x00} };

tml_ext_type tml_inv_prop_ext_t
    = { 0x405b4345,
        0x1000,
        0x0000,
        0x0d,
        {0x00, 0x01, 0x9f, 0xab, 0x00, 0x00, 0x00} };

tml_ext_type tml_font_prop_ext_t
    = { 0x405b430e,
        0xd000,
        0x0000,
        0x0d,
        {0x00, 0x01, 0x9f, 0xab, 0x00, 0x00, 0x00} };

tml_ext_type tml_margin_prop_ext_t
    = { 0x405b42b2,
        0xa000,
        0x0000,
        0x0d,
        {0x00, 0x01, 0x9f, 0xab, 0x00, 0x00, 0x00} };

tml_ext_type tml_wrap_prop_ext_t
    = { 0x405b4287,
        0x7000,
        0x0000,
        0x0d,
        {0x00, 0x01, 0x9f, 0xab, 0x00, 0x00, 0x00} };

tml_ext_type tml_undefined_prop_ext_t
    = { 0x405b41fe,
        0x3000,
        0x0000,
        0x0d,
        {0x00, 0x01, 0x9f, 0xab, 0x00, 0x00, 0x00} };

tml_type tml_tab_prop_t            = {&tml_tab_prop_ext_t, &tml_sample_prop, &tml_no_val_t};
tml_type tml_union_char_set_prop_t = {&tml_union_char_set_prop_ext_t, &tml_sample_prop, &tml_tab_prop_t};
tml_type tml_char_set_prop_t       = {&tml_char_set_prop_ext_t, &tml_sample_prop, &tml_union_char_set_prop_t};
tml_type tml_form_feed_prop_t      = {&tml_form_feed_prop_ext_t, &tml_sample_prop, &tml_char_set_prop_t};
tml_type tml_footer_prop_t         = {&tml_footer_prop_ext_t, &tml_sample_prop, &tml_form_feed_prop_t};
tml_type tml_header_prop_t         = {&tml_header_prop_ext_t, &tml_sample_prop, &tml_footer_prop_t};
tml_type tml_pgsize_prop_t         = {&tml_pgsize_prop_ext_t, &tml_sample_prop, &tml_header_prop_t};
tml_type tml_inv_prop_t            = {&tml_inv_prop_ext_t, &tml_sample_prop, &tml_pgsize_prop_t};
tml_type tml_font_prop_t           = {&tml_font_prop_ext_t, &tml_sample_font_prop, &tml_inv_prop_t};
tml_type tml_margin_prop_t         = {&tml_margin_prop_ext_t, &tml_sample_prop, &tml_font_prop_t};
tml_type tml_wrap_prop_t           = {&tml_wrap_prop_ext_t, &tml_sample_prop, &tml_margin_prop_t};
tml_type tml_undefined_prop_t      = {&tml_undefined_prop_ext_t, 0, 0};

tml_type* const tml_tab_prop            = &tml_tab_prop_t;
tml_type* const tml_union_char_set_prop = &tml_union_char_set_prop_t;
tml_type* const tml_char_set_prop       = &tml_char_set_prop_t;
tml_type* const tml_form_feed_prop      = &tml_form_feed_prop_t;
tml_type* const tml_footer_prop         = &tml_footer_prop_t;
tml_type* const tml_header_prop         = &tml_header_prop_t;
tml_type* const tml_pgsize_prop         = &tml_pgsize_prop_t;
tml_type* const tml_inv_prop            = &tml_inv_prop_t;
tml_type* const tml_font_prop           = &tml_font_prop_t;
tml_type* const tml_margin_prop         = &tml_margin_prop_t;
tml_type* const tml_wrap_prop           = &tml_wrap_prop_t;
tml_type* const undefined_prop          = &tml_undefined_prop_t;

#endif

/**********************************************************************************
    preloaded node types
**********************************************************************************/

#ifdef apollo_global
tml_ext_type tml_free_ext_t;
tml_ext_type tml_leaf_ext_t;
tml_ext_type tml_pgobj_ext_t;
tml_ext_type tml_gobj_ext_t;
tml_ext_type tml_branch_ext_t;
tml_ext_type tml_bobj_ext_t;
tml_ext_type tml_undefined_node_ext_t;

tml_type tml_free_t;
tml_type tml_leaf_t;
tml_type tml_pgobj_t;
tml_type tml_gobj_t;
tml_type tml_branch_t;
tml_type tml_bobj_t;
tml_type tml_undefined_node_t;

tml_type *is_free;
tml_type *leaf;
tml_type *pgobj;
tml_type *gobj;
tml_type *branch;
tml_type *bobj;
tml_type *undefined_node;

#else
tml_ext_type tml_free_ext_t
    = { 0x405b3cda,
        0x8000,
        0x0000,
        0x0d,
        {0x00, 0x01, 0x9f, 0xab, 0x00, 0x00, 0x00} };

tml_ext_type tml_leaf_ext_t
    = { 0x405b3c4c,
        0x2000,
        0x0000,
        0x0d,
        {0x00, 0x01, 0x9f, 0xab, 0x00, 0x00, 0x00} };

tml_ext_type tml_pgobj_ext_t
    = { 0x405b3c99,
        0x5000,
        0x0000,
        0x0d,
        {0x00, 0x01, 0x9f, 0xab, 0x00, 0x00, 0x00} };

tml_ext_type tml_gobj_ext_t
    = { 0x405b3c0b,
        0xf000,
        0x0000,
        0x0d,
        {0x00, 0x01, 0x9f, 0xab, 0x00, 0x00, 0x00} };

tml_ext_type tml_branch_ext_t
    = { 0x405b3b66,
        0xb000,
        0x0000,
        0x0d,
        {0x00, 0x01, 0x9f, 0xab, 0x00, 0x00, 0x00} };

tml_ext_type tml_bobj_ext_t
    = { 0x405b3b06,
        0x7000,
        0x0000,
        0x0d,
        {0x00, 0x01, 0x9f, 0xab, 0x00, 0x00, 0x00} };

tml_ext_type tml_undefined_node_ext_t
    = { 0x405b39e7,
        0x3000,
        0x0000,
        0x0d,
        {0x00, 0x01, 0x9f, 0xab, 0x00, 0x00, 0x00} };

tml_type tml_free_t           = {&tml_free_ext_t, 0, 0};
tml_type tml_leaf_t           = {&tml_leaf_ext_t, 0, 0};
tml_type tml_pgobj_t          = {&tml_pgobj_ext_t, &tml_sample_pgnode, &tml_wrap_prop_t};
tml_type tml_gobj_t           = {&tml_gobj_ext_t, &tml_sample_gnode, &tml_pgobj_t};
tml_type tml_branch_t         = {&tml_branch_ext_t, 0, 0};
tml_type tml_bobj_t           = {&tml_bobj_ext_t, &tml_sample_bnode, &tml_gobj_t};
tml_type tml_undefined_node_t = {&tml_undefined_node_ext_t, 0, 0};

tml_type* const is_free        = &tml_free_t;
tml_type* const leaf           = &tml_leaf_t;
tml_type* const pgobj          = &tml_pgobj_t;
tml_type* const gobj           = &tml_gobj_t;
tml_type* const branch         = &tml_branch_t;
tml_type* const bobj           = &tml_bobj_t;
tml_type* const undefined_node = &tml_undefined_node_t;

#endif

/**********************************************************************************
    objects for restoration of TML Version 1 types
**********************************************************************************/

bnode* extend_cxx_make_bnode(model* in_model)
{
    return(new bnode(in_model));
}

gnode* extend_cxx_make_gnode(model* in_model)
{
    return(new gnode(in_model));
}

property* extend_cxx_make_property(model* in_model)
{
    return(in_model->make_prop());
}

font_property* extend_cxx_make_font_property(model* in_model)
{
    return((font_property*) in_model->make_prop((long) sizeof(font_property)));
}

#ifdef apollo_global
tml_ext_type_v1 tml_pgsize_prop_ext_t_v1;
tml_ext_type_v1 tml_inv_prop_ext_t_v1;
tml_ext_type_v1 tml_font_prop_ext_t_v1;
tml_ext_type_v1 tml_margin_prop_ext_t_v1;
tml_ext_type_v1 tml_wrap_prop_ext_t_v1;
tml_ext_type_v1 gobj_ext_t_v1;
tml_ext_type_v1 bobj_ext_t_v1;

type_trans_v1 tml_pgsize_trans;
type_trans_v1 tml_inv_trans;
type_trans_v1 tml_font_trans;
type_trans_v1 tml_margin_trans;
type_trans_v1 tml_wrap_trans;
type_trans_v1 tml_gobj_trans;
type_trans_v1 tml_bobj_trans;

type_trans_queue_v1 type_trans_q_v1;

#else
tml_ext_type_v1 tml_pgsize_prop_ext_t_v1 = { 0, 11 };
tml_ext_type_v1 tml_inv_prop_ext_t_v1    = { 0, 10 };
tml_ext_type_v1 tml_font_prop_ext_t_v1   = { 0, 9 };
tml_ext_type_v1 tml_margin_prop_ext_t_v1 = { 0, 8 };
tml_ext_type_v1 tml_wrap_prop_ext_t_v1   = { 0, 7 };
tml_ext_type_v1 gobj_ext_t_v1            = { 0, 3 };
tml_ext_type_v1 bobj_ext_t_v1            = { 0, 1 };

type_trans_v1 tml_pgsize_trans = { &tml_pgsize_prop_ext_t_v1, &tml_pgsize_prop_t, (GEN_FUNCP) &extend_cxx_make_property, 0 };
type_trans_v1 tml_inv_trans    = { &tml_inv_prop_ext_t_v1, &tml_inv_prop_t, (GEN_FUNCP) &extend_cxx_make_property, &tml_pgsize_trans };
type_trans_v1 tml_font_trans   = { &tml_font_prop_ext_t_v1, &tml_font_prop_t, (GEN_FUNCP) &extend_cxx_make_property, &tml_inv_trans };
type_trans_v1 tml_margin_trans = { &tml_margin_prop_ext_t_v1, &tml_margin_prop_t, (GEN_FUNCP) &extend_cxx_make_property, &tml_font_trans };
type_trans_v1 tml_wrap_trans   = { &tml_wrap_prop_ext_t_v1, &tml_wrap_prop_t, (GEN_FUNCP) &extend_cxx_make_property, &tml_margin_trans };
type_trans_v1 tml_gobj_trans   = { &gobj_ext_t_v1, &tml_gobj_t, (GEN_FUNCP) &extend_cxx_make_gnode, &tml_wrap_trans };
type_trans_v1 tml_bobj_trans   = { &bobj_ext_t_v1, &tml_bobj_t, (GEN_FUNCP) &extend_cxx_make_bnode, &tml_gobj_trans };

type_trans_queue_v1 type_trans_q_v1 = {&tml_bobj_trans, &tml_pgsize_trans};

#endif

void type_trans_queue_v1::queue_type_trans(type_trans_v1* new_trans)
{
    tail->next = new_trans;
    tail = new_trans;
}

/********************************************************************************************
    Utility functions for type queue used in restoring TML Version 1 models
********************************************************************************************/
type_trans_v1* tml_type_trans_q_v1_get_head()
{
    return type_trans_q_v1.head;
}

void tml_type_trans_q_v1_queue_type(type_trans_v1* new_trans)
{
    type_trans_q_v1.queue_type_trans(new_trans);
}

/********************************************************************************************
    Utility function to return type identifier for default graphics system
********************************************************************************************/
grafsys tml_dp_get_dflt_grafsys_type()
{
    return tml_dp_get_dflt_vpftbl()->grafsys_type;
}

#include <tml/extend.cxx>
