/**************************************************************************************
    tml_os_io_aegis.c
    Operating system depending input/output functions for aegis

    (c) Copyright Apollo Computer Inc., 1987 (unpublished)
     --------------------------------------------------------------------
    |THE FOLLOWING PROGRAMS ARE THE SOLE PROPERTY OF APOLLO COMPUTER INC.|
    |      AND CONTAIN ITS PROPRIETARY AND CONFIDENTIAL INFORMATION.     |
     --------------------------------------------------------------------
**************************************************************************************/

#include <stdio.h>
#include <sys/types.h>

#include <apollo/base.h>
#include <apollo/sys/ubase.h>
#include <apollo/ms.h>
#include <apollo/ios.h>
#include <apollo/sys/uid_list.h>
#include <apollo/sys/name.h>
#include <apollo/sys/file.h>
#include <tml/tml_c_types.h>
#include <tml/tml_os_depends.h>

extern uid_$t tml_$uid;

#define MAPSIZE 16000

typedef long (*PFI)();

typedef struct {
        char           *fp;            /* address where file is originally mapped */
        ios_$id_t      stream_handle;  /* used when reading via ios calls*/
        long           cur_pos;        /* current offset in file from fp */
        long           file_size;      /* actual number of bytes in file */
        unsigned long  len_mapped;     /* we map the file in 16K blocks */
        long           fp_type;        /* 1 = tml object, 2 = usac object, 0 = don't know, use ios calls*/
        ms_$acc_mode_t curr_access;    /* current access of object      */
        char           *pathname;      /* filename of object            */
        long           pathname_len;   /* filename length of object     */
    } file_descr_t;


/*make a file_desc_t, return a pointer to it*/
char *tml_make_fhandle(file_ident, file_type)
    long *file_ident;
    long file_type;
{
    file_descr_t   *return_handle;
    return_handle = (file_descr_t *)tml_c_tml_os_alloc(1, (long) sizeof(file_descr_t));
    return_handle->fp = 0;
    return_handle->stream_handle = (ios_$id_t)*file_ident;
    return_handle->cur_pos = return_handle->file_size = 0;
    return_handle->len_mapped = 0;
    return_handle->pathname = 0;
    return_handle->pathname_len = 0;
    return_handle->fp_type = file_type;
    return ((char *)(return_handle));
}

/*get rid of a file_descr_t*/
void tml_free_fhandle(fhandle)
    file_descr_t *fhandle;
{
    if (fhandle->pathname)
        free(fhandle->pathname);

    free(fhandle);
}
    
/* returns file pointer as void * */
/* returns NULL on failure of open() */
/* This function is called by ios_$open or ios_$create for a file of type TML */
/* no concurrency checking needed since we only allow 1 write and n readers */
void *tml_os_open_obj(xoid, open_opts, sts)
 xoid_$t xoid;
 ios_$open_options_t *open_opts;
 status_$t *sts;
{
    file_descr_t        *fdp;
    ios_$open_options_t  oopts;
    ios_$create_mode_t   cmode;
    ios_$id_t            sid;
    ms_$acc_mode_t       access;
    unsigned long        size;
    short                attr_size;
    ms_$attrib_t         attr;
    status_$t            status;

    fdp = (file_descr_t *) tml_c_tml_os_alloc(1, (long) sizeof(file_descr_t));
    size = MAPSIZE;

    if ( !(*open_opts & ios_$write_opt) || (*open_opts & ios_$inquire_only_opt))
        if (*open_opts & ios_$read_intend_write_opt)  access = ms_$riw;
        else access = ms_$r;
    else access = ms_$wr;

    fdp->fp = ms_$mapl_stream( xoid, (long)0, size, ms_$nr_xor_1w, access,
                               true, &fdp->len_mapped, &status );

    if (status.all != status_$ok) {
        printf("tml_os_obj_open failed in mapl\n");
        tml_c_tml_os_free((void*) fdp );
        *sts = status;
        return((void *) 0);
    }
    else {
        if (fdp->len_mapped == 0)
            fdp->file_size = 0;
        else {
            ms_$attributes( fdp->fp, &attr, &attr_size, (short) sizeof(attr), &status );
            if (status.all == status_$ok)
                fdp->file_size = attr.cur_len;
            else
                fdp->file_size = 0;
        }
        fdp->cur_pos = 0;
        return((void *) fdp);
    }
}

void tml_os_relock(fdp, open_opts, sts)
 file_descr_t *fdp;
 ios_$open_options_t *open_opts;
 status_$t *sts;
{
    ms_$acc_mode_t access;
    status_$t      status;

    status.all = status_$ok;
    if ( !(*open_opts & ios_$write_opt) )
        if (*open_opts & ios_$read_intend_write_opt)  access = ms_$riw;
        else access = ms_$r;
    else access = ms_$wr;

    ms_$relock( fdp->fp, access, &status);
    *sts = status;
}
                
/*return indicates type of file we are dealing with */
/* 0: type to be accessed through stream calls */
/* 1: tml object */
/* 2: uasc object */
/* 3: unstruct object */
long tml_file_type(filename, name_len )
 char *filename;
 short name_len;
{
    uid_$t             type_uid;
    uid_$t             obj_uid;
    status_$t          status;
    file_$get_mode_t   obj_mode;
    unsigned short     finfo_size;
    obj_info_$t        obj_info;
    file_$attributes_t obj_finfo;

    /* all this just to get the obj_uid in order to get the obj type */
    name_$resolve(filename, name_len, &obj_uid, &status);
    if (status.all != status_$ok) {
        return -1;
    }

    obj_mode = file_$get_mgr_data;
    finfo_size = (unsigned short)(sizeof(file_$attributes_t));
    file_$get_attr_info(obj_uid, obj_mode, finfo_size, &obj_info, &obj_finfo, &status);
    if (status.all != status_$ok)
        return -1;

    if ((obj_finfo.type_uid.high == tml_$uid.high)
      && (obj_finfo.type_uid.low == tml_$uid.low))
        return 1;
    else if ((obj_finfo.type_uid.high == uasc_$uid.high)
      && (obj_finfo.type_uid.low == uasc_$uid.low))
        return 2;
    else if ((obj_finfo.type_uid.high == unstruct_$uid.high)
      && (obj_finfo.type_uid.low == unstruct_$uid.low))
        return 3;
    else
        return 0;
}

/* returns pointer to file cast as a void * */
/* returns NULL on failure of open(). Mode is ignored in Aegis. */
void *tml_os_open(filename, name_len, oflag, mode)
 char *filename;
 long name_len;
 long oflag;
 long mode;
{
    file_descr_t        *fdp;
    ios_$open_options_t  oopts;
    ios_$create_mode_t   cmode;
    ios_$id_t            sid;
    unsigned long int    len;
    ms_$attrib_t         attr;
    unsigned long int    size;
    status_$t            status;
    uid_$t               obj_uid;
    boolean              fmc;
    boolean              allow_extend;
    short                nlen = name_len;

    fdp = (file_descr_t *) tml_c_tml_os_alloc(1, (long) sizeof(file_descr_t));
    fdp->pathname = (char *) tml_c_tml_os_alloc(1, (long) name_len);
    fdp->pathname_len = name_len;
    tml_c_tml_os_byte_copy(filename, fdp->pathname, name_len);
    len = MAPSIZE;

    /*if we are creating a new file, just do it and get out*/
    if (oflag & O_CREAT) {
        /* try to create the file*/
        fdp->fp = ms_$crmapl(filename, nlen, (long)0, len, ms_$nr_xor_1w, &status);
        /*if it already exists, just fall through to the case of opening an existing file*/
        if ((status.all & 0x0fffffff) != name_$already_exists){
            /*if the create failed, return 0 and get out*/
            if (status.all != status_$ok){
                printf("create failed : status = %h\n", status.all);
                return((void *) 0);
                }
            /* get the obj_uid so we can set the type uid */
            name_$resolve( filename, nlen, &obj_uid, &status );
            if (status.all != status_$ok) {
                ms_$unmap( fdp->fp, len, &status );
                printf("open failed getting uid\n");
                return((void *) 0);
                }

            file_$set_type( obj_uid, tml_$uid, &status );
            if (status.all != status_$ok) {
                ms_$unmap( fdp->fp, len, &status );
                printf("open failed setting object type\n");
                return((void *) 0);
                }
            fdp->len_mapped = len;
            fdp->fp_type = 1;
            return((void *) fdp);
            }
        else if (oflag & O_EXCL){
            tml_c_tml_os_free(fdp->pathname);
            tml_c_tml_os_free((void*) fdp);
            return ((void *) 0);
            }
        }
    
    /*if we got here, then the file should already exist*/
    /*first, find out what the type of the file is*/
    fdp->fp_type = tml_file_type(filename, nlen);

    /*set up the options for open*/
    allow_extend = true;
    if (((oflag & O_RDWR) != 0) || ((oflag & O_WRONLY) != 0) ||
         ((oflag & O_TRUNC) != 0) || ((oflag & O_CREAT) != 0))
        fdp->curr_access = ms_$riw;
    else if ((oflag & O_APPEND) != 0) {
        printf("tml_os_open failed; APPEND not supported\n");
        return((void *) 0);
        }
    else {
        allow_extend = false;
        fdp->curr_access = ms_$r;
        }

                      
    switch (fdp->fp_type){
        /*case 0--file type is unknown--use ios calls*/
      case 0:
        /*for unknown file types, only open for read*/
        if (oflag != O_RDONLY){
            printf("read only required for unknown file types\n");
            tml_c_tml_os_free(fdp->pathname);
            tml_c_tml_os_free((void*) fdp);
            return ((void *) 0);
            }
        fdp->stream_handle = ios_$open(filename, nlen, ios_$unregulated_opt, &status);
        if (status.all != status_$ok){
            printf("error on ios_open, status = %h\n", status.all);
            return ((void *) 0);
            }
        return ((void *) fdp);
        break;

      case 1:
        /*it's a tml object--just map it*/
        fdp->fp = ms_$mapl(filename, nlen, (long)0, len, ms_$nr_xor_1w, fdp->curr_access,
                allow_extend, &fdp->len_mapped, &status);
        if (status.all != status_$ok) {
            tml_c_tml_os_free(fdp->pathname);
            tml_c_tml_os_free((void*) fdp);
            return((void *) 0);
            }
        fdp->cur_pos = 0;
        break;

      case 2:
        /*it's a UASC object--map it, and skip the first 32 bytes of trash*/
        fdp->fp = ms_$mapl(filename, nlen, (long)0, len, ms_$nr_xor_1w, fdp->curr_access,
                allow_extend, &fdp->len_mapped, &status);
        if (status.all != status_$ok) {
            tml_c_tml_os_free(fdp->pathname);
            tml_c_tml_os_free((void*) fdp);
            return((void *) 0);
            }
        fdp->cur_pos = 32;
        break;

      case 3:
        /*its an unstruct object--map it, and start at byte 0*/
        fdp->fp = ms_$mapl(filename, nlen, (long)0, len, ms_$nr_xor_1w, fdp->curr_access,
                allow_extend, &fdp->len_mapped, &status);
        if (status.all != status_$ok) {
            tml_c_tml_os_free(fdp->pathname);
            tml_c_tml_os_free((void*) fdp);
            return((void *) 0);
            }
        fdp->cur_pos = 0;
        break;

      default:
        printf("attempt to open file with no known filetype\n");
        tml_c_tml_os_free(fdp->pathname);
        tml_c_tml_os_free((void*) fdp);
        return ((void *) 0);
        break;
    }

    /*now, set the file size*/
    if ( (oflag & O_TRUNC) != 0)  {
        ms_$relock( fdp->fp, ms_$wr, &status);
        ms_$truncate(fdp->fp, (fdp->fp_type == 2) ? (long)32 : (long)0, &status);
        if (status.all != status_$ok) {
            ms_$unmap( fdp->fp, fdp->len_mapped, &status );
            printf("open failed in truncate\n");
            tml_c_tml_os_free(fdp->pathname);
            tml_c_tml_os_free((void*) fdp);
            return((void *) 0);
            }
        fdp->len_mapped = 0;
        fdp->file_size = 0;
        }
    else {
        ms_$attributes( fdp->fp, &attr, &nlen, (short) sizeof(attr), &status );
        if (status.all == status_$ok)
            fdp->file_size = attr.cur_len;
        else {
            ms_$unmap( fdp->fp, fdp->len_mapped, &status );
            printf("open failed getting attributes\n");
            tml_c_tml_os_free(fdp->pathname);
            tml_c_tml_os_free((void*) fdp);
            return((void *) 0);
            }
        }
    return((void *) fdp);
}


/* return: */
/*   number of bytes_read if successful */
/*   0 on eof */
/*   -1 on failure of read() */
long tml_os_read(fdp_in, buf, nbyte)
 void *fdp_in;
 char *buf;
 unsigned long nbyte;
{
    register long nbytes_to_copy = 0;
    register long bytes_read = 0;

    file_descr_t* fdp = (file_descr_t*) fdp_in;

    char      *file_ptr;
    long       i;
    long       remap_size;
    status_$t  status;

    status.all = status_$ok;
    switch (fdp->fp_type){
     case 0:
            while (nbyte){
                bytes_read = ios_$get(fdp->stream_handle, ios_$no_rec_bndry_opt, buf, (long) nbyte, &status);
                if (status.all != status_$ok){
                    if (status.all == ios_$buffer_too_small)
                        return nbytes_to_copy;
                    else if (status.all = ios_$end_of_file)
                        return nbytes_to_copy;
                    else {
                        printf("tml_os_read failed : ios_$put failed, status = %X\n,", status.all);
                        return -1;
                        }
                    }
               nbytes_to_copy += bytes_read;
               buf += bytes_read;
               nbyte -= bytes_read;
               }
             break;

      case 1:
      case 2:
      case 3:
        /* check if enough of the file has been mapped */
        if ((fdp->cur_pos + nbyte) > fdp->len_mapped)
            if (fdp->file_size > fdp->len_mapped) {
                /* there is more file to be mapped, so map it */
                remap_size = (MAPSIZE > nbyte) ? MAPSIZE : nbyte;
                if ((remap_size + fdp->cur_pos) > fdp->file_size)
                    remap_size = fdp->file_size - fdp->cur_pos;
                fdp->fp = ms_$remap( fdp->fp, (long)0, fdp->len_mapped + remap_size, &(fdp->len_mapped), &status );

                   /* Amount mapped may be rounded up to next segment
                      boundary; so use file size to find out how many
                      bytes can be read
                   */  

                if (status.all == status_$ok)
                    nbytes_to_copy = (fdp->file_size - fdp->cur_pos) < nbyte ? (fdp->file_size - fdp->cur_pos) : nbyte;
            }
            else
            /* the entire file has already been mapped; copy to eof */
            nbytes_to_copy = fdp->file_size - fdp->cur_pos;
        else
            /*see if the requested number goes beyond the size of the file*/
            if (fdp->cur_pos+nbyte > fdp->file_size)
                nbytes_to_copy = fdp->file_size - fdp->cur_pos;
            else
                /* there are enough bytes left to copy what the user requested */
                nbytes_to_copy = nbyte;
    
        if (status.all == status_$ok) {
            if (buf != tml_c_tml_os_byte_copy((fdp->fp + fdp->cur_pos), buf, nbytes_to_copy))
                return -1;
            fdp->cur_pos += nbytes_to_copy;
        }
        else
            nbytes_to_copy = -1;
        break;

     default:   /* Unix file descriptor, e.g. standard input */
        while (nbyte) {
            bytes_read = read(fdp->stream_handle, buf, nbyte);
            if (0 == bytes_read)                   /* EOF*/
                break;
            else if (-1 == bytes_read) {           /* IO ERROR*/
                nbytes_to_copy = -1;
                break;
                }
            buf += bytes_read;
            nbyte -= bytes_read;
            nbytes_to_copy += bytes_read;
            }
        break;
    }
    return nbytes_to_copy;
}

long tml_os_write(fdp_in, buf, nbytes)
 void *fdp_in;
 char *buf;
 unsigned long nbytes;
{
    file_descr_t* fdp = (file_descr_t*) fdp_in;

    char          *file_ptr,
                  *buf_check;
    unsigned long  size,
                   remap_size;
    long           i;
    long           bytes_written = 0;
    long           total_bytes_written = 0;
    status_$t      status;

    status.all = status_$ok;
    switch(fdp->fp_type){
      case 0:
        /*not a mapped file that we know about--use ios calls*/
        ios_$put(fdp->stream_handle, ios_$no_put_get_opts, buf, (long) nbytes, &status);
        if (status.all != status_$ok){
            printf("tml_os_write failed : ios_$put failed, status = %X\n,", status.all);
            return 0;
            }
        else
            return nbytes;

      case 1:
      case 2:
      case 3:
        /*mapped tml, unstruct or UASC file*/
        /* check if enough of the file has been mapped */
        if (fdp->cur_pos+nbytes > fdp->len_mapped) {
            /* remap the file if it is growing beyond the currently mapped point */
            /* map a large amount so subsequent small writes will just work */
            remap_size = (nbytes > MAPSIZE) ? nbytes : MAPSIZE;
            size = fdp->len_mapped + remap_size;
            fdp->fp = ms_$remap( fdp->fp, (long)0, size, &(fdp->len_mapped), &status );
            if (status.all != status_$ok){
                printf("tml_os_write fail : unable to extend mapped file size\n");
                return 0;
                }
            }
                         
        buf_check = tml_c_tml_os_byte_copy(buf, fdp->fp + fdp->cur_pos, nbytes);
        if (buf_check != (fdp->fp + fdp->cur_pos))
            for(i=0, file_ptr = fdp->fp + fdp->cur_pos; i < nbytes; i++)
                *file_ptr++ = *buf++;
        fdp->cur_pos += nbytes;
        if (fdp->cur_pos > fdp->file_size)
            fdp->file_size = fdp->cur_pos;
        return (long) nbytes;

     default:
        /* don't know what it is--just use generic write*/
        while (nbytes) {
            bytes_written = write(fdp->stream_handle, buf, nbytes);

            /* note no special case for return of 0; infinite retry*/

            if (-1 == bytes_written) {        /* IO ERROR*/
                total_bytes_written = -1;
                break;
                }
            buf += bytes_written;
            nbytes -= bytes_written;
            total_bytes_written += bytes_written;
            }
        return(total_bytes_written);
     }
}

/* rewind current file position to BOF */
/* return: */
/*    current position if successful */
/*    -1 on failure of lseek */
long tml_os_rewind(fdp_in)
void *fdp_in;
{
    status_$t status;

    file_descr_t* fdp = (file_descr_t*) fdp_in;

    switch(fdp->fp_type){
        case 0:
            /*not a mapped file that we know about--use ios calls*/
            ios_$seek_to_bof(fdp->stream_handle, &status);
            return status.all;

        case 2:
            /* Allow for streams header */
            fdp->cur_pos = 32;
            return 0;

        case 1:
        case 3:
            /*mapped tml, unstruct or UASC file*/
            fdp->cur_pos = 0;
            return 0;

        default:
            lseek(fdp->stream_handle, 0L, 0);
            return 0;
     }
}

/* close file associated with fdp_in */
/* deallocate fdp_in */
/* return: */
/*   -1 on failure */
/*    0 on success */
long tml_os_close(fdp_in)
void *fdp_in;
{
    file_descr_t* fdp = (file_descr_t*) fdp_in;

    status_$t status;
    status.all = status_$ok;

    /* if object is on same node, no guarantee it */
    /*   will be written to disk after unmap */
                                                                                                                      
    if (fdp->fp_type) {
        /* truncate object to the current file size */
        if (fdp->file_size != fdp->len_mapped)
            ms_$truncate((void *) fdp->fp, (unsigned long) fdp->file_size, &status);
        ms_$unmap((void *) fdp->fp, fdp->len_mapped, &status );
        }
    else
        ios_$close(fdp->stream_handle, &status);

    tml_c_tml_os_free(fdp->pathname);
    tml_c_tml_os_free((void*) fdp);

    if (status.all != status_$ok)
        return -1;
    else
        return 0;
}

