/**************************************************************************************
    tml_post_print.cxx
    Module to generate postscript for a tml user

    (c) Copyright Apollo Computer Inc., 1987 (unpublished)
     --------------------------------------------------------------------
    |THE FOLLOWING PROGRAMS ARE THE SOLE PROPERTY OF APOLLO COMPUTER INC.|
    |      AND CONTAIN ITS PROPRIETARY AND CONFIDENTIAL INFORMATION.     |
     --------------------------------------------------------------------
**************************************************************************************/

/*
.............................................................................
//
// Member Functions
//   tml_model_print
//   tml_file_print
//
// Private Functions
//   tml_print
//
.............................................................................

Changes:

    11/17/87 hayes created

*/
                   

#include <stdio.h>
#include <string.h>
#include <fcntl.h>

#include <ctype.h>
#include <tml/tml_cxx2.0_linkage_decls.hxx>
#include <tml/tml_objects.hxx>
#include <tml/tml_functions.hxx>
#include <tml/tml_os_depends.hxx>
#include <tml/post_filter.hxx>
 
//
//  make_c_string - Take a string and length and generate a null terminated string
//

char* make_c_string (char* str, long len)
{
   char  *c_file_name = new char [len + 1];

   *c_file_name = '\0';

   strncpy (c_file_name, str, (int) len);
   
   return c_file_name;
}


//
//  tml_print - Generate postscript for a tml model
//
//  Input
//    model - Model id of object
//    id    - Id of tml object
//    parms - Print parameters
//    out_str - Output file name
//
//  Result if success
//            return 0
//         else
//            return -1
//

long tml_print ( model* id, print_db* parms, char* out_str)
{
   post_filter   *filt = new post_filter;
   long           *post_id = new long, st;
                                 
   if ((!id) || ((*post_id = *((long*) tml_os_open (out_str, (long) strlen(out_str), O_WRONLY | O_CREAT, 0666))) == -1))
      return -1;

   st = filt->filter (id, (void*)post_id, parms);

   tml_os_close (post_id);

   delete post_id;
   delete filt;
   delete out_str;

   return st;
}

 
                         
//
//  tml_model_print - Generate postscript output from a tml model
//
//  Input
//    this_model  - Id of tml object
//    print_parms - Print parameters
//    outfilename - Output file name
//    outlen      - Output file name lgt
//
//  Result if success
//            return 0
//         else
//            return -1
//

long tml_model_print ( model* this_model, print_db* print_parms, char* outfilename, long outlen)
{
   return tml_print (this_model, print_parms, make_c_string(outfilename,outlen));
}


//
//  tml_file_print - Generate postscript output from a tml file
//
//  Input
//    infilename  - Tml filename
//    inlen       - Tml filename length
//    print_parms - Print parameters
//    outfilename - Output file name
//    outlen      - Output file name lgt
//
//  Result if success
//            return 0
//         else
//            return -1
//

long tml_file_print ( char* infilename, long inlen, print_db* print_parms, char* outfilename, long outlen)
{
   return tml_print (d_read_model (infilename,inlen), print_parms, make_c_string(outfilename,outlen));
}
