/**************************************************************************************
    tmlsearch.cxx
    Unix-specific TML String Search Routines
    20-Nov-87 LCS

    (c) Copyright Apollo Computer Inc., 1987 (unpublished)
     --------------------------------------------------------------------
    |THE FOLLOWING PROGRAMS ARE THE SOLE PROPERTY OF APOLLO COMPUTER INC.|
    |      AND CONTAIN ITS PROPRIETARY AND CONFIDENTIAL INFORMATION.     |
     --------------------------------------------------------------------
**************************************************************************************/

#include <string.h>
#include <tml/tml_objects.hxx>
#include <tml/tml_os_depends.hxx>
#include <tml/tml_functions.hxx>

#define SUCCESS return 0
#define FAILURE return 1

void model::return_to_default_search_env()
// clear search_env stack and set up defaults
{
    for (; search_stack->get_next();)
        pop_search_env();
  // notice that the first environment pushed onto the
  // stack is preserved.  This procedure reverts you
  // to your default (startup) state, which IS that first
  // enviroment.  We don't want to pop that because if
  // we do so we will cripple searches completely and it
  // would be difficult to restore it even for a know-
  // ledgable user.
}

void model::push_search_env(search_env *new_envir)
// Installs a new search enviroment on the stack
{
    new_envir->set_next(search_stack);
    search_stack = new_envir;
}

void model::pop_search_env()
// Removes the current search environment from the
// search environment stack and reverts to the environment
// that was in force at the time of the last push_search_env
// call.  Notice that it will not remove the first environment
// on the stack, performing a no-op if that is the only env
// defined.  Removing that last env will cripple search, and
// it is non-trivial to replace it in the typical user program.
{
    search_env *holder;

    holder = search_stack;
    if (search_stack->get_next()) {
        search_stack = search_stack->get_next();
        delete holder;
        }
}
     
long                               // YES if pattern was found
     tml_unix_search_exp(              // search using std regexp
       model_range   search_area, // place in model to search in
       model_range&  found_str,   // area where string match was found
       void         *pattern,     // pattern from tml_compile
       long           forward = 1  // whether to search in reverse
       )
{
    register long min_match_size;
             long buffer_size, this_string_size,
                 i,found_off,offset, partial_search,
                 full_searches,start_idx, end_idx;

    char        *match_buffer, *str_loc, *goo_buff;
    model_pos    current_pos,
                 hold_pos,
                 start_pos,
                 end_pos;

    min_match_size = tml_unix_min_match(pattern);
    if (!min_match_size)
         FAILURE;
    buffer_size = min_match_size * 2;

    match_buffer = tml_os_alloc(buffer_size, 1);
    if (!match_buffer)
        FAILURE;
                                                                          
    current_pos = start_pos = search_area.get_start();
    if (!current_pos.in_node->can_format())
        start_pos = current_pos = current_pos.get_next();
    end_pos = search_area.get_end();

    found_off = -1;

    if(forward == 1){
      // forward search
      while (current_pos != end_pos) {
          this_string_size = current_pos.in_node->get_max_index() - current_pos.char_index;
          if (this_string_size > min_match_size){
              found_off = tml_unix_match_re(
                            ((char*)(current_pos.in_node->get_contents())) + current_pos.char_index,
                            this_string_size,
                            pattern);
              if (found_off != -1){
                  current_pos.char_index += (found_off - 1);
                  found_str.set_start(current_pos);
                  found_str.set_end(current_pos);
                  SUCCESS;
                  }
              else
                  current_pos.char_index += this_string_size - 1;
              }
          else{
              hold_pos = current_pos;
              for (i = 0;
                   hold_pos.in_node && (i < buffer_size);
                   i++, hold_pos = hold_pos.get_next())
                   match_buffer[i] = (*(((char *)(hold_pos.in_node->get_contents())) + hold_pos.char_index));
              if (!hold_pos.in_node)
                  FAILURE;
              found_off = tml_unix_match_re(match_buffer, i, pattern);
              if (found_off != -1){
                  current_pos += found_off;
                  found_str.set_start(current_pos);
                  found_str.set_end(current_pos);
                  SUCCESS;
                  }
              else {
                  current_pos += min_match_size;
                  }
              }
          current_pos = current_pos.get_next();
          }
  
      FAILURE;
    }
    else {
      // reverse search
      current_pos = end_pos;                       
      partial_search = 0;
      end_idx = current_pos.char_index;
      start_idx = (current_pos.in_node == start_pos.in_node) ? start_pos.char_index : 0;

      while (current_pos != start_pos) 
      {
          goo_buff = (char*)((gnode*)current_pos.in_node)->get_0t_contents();
          full_searches = (end_idx - start_idx)/min_match_size;
          partial_search = (end_idx - start_idx)%min_match_size;
          offset = 0;

          for (i=0;i<full_searches;++i)
          {
             offset += min_match_size;
             str_loc = &goo_buff[end_idx-offset];
             found_off = tml_unix_match_re(str_loc, min_match_size, pattern);

             if (found_off != -1)
             {
                current_pos.char_index = (end_idx-offset) + found_off;
                found_str.set_start(current_pos);
                found_str.set_end(current_pos);
                SUCCESS;
             }
          }
    
          if (partial_search)  
          {                         
             str_loc = &goo_buff[start_idx];
             strncpy (match_buffer, str_loc, (int) partial_search);
                            
             offset = (min_match_size - partial_search);       
             current_pos = current_pos - ((end_idx - start_idx) + offset);

             if ((!current_pos.in_node) || (current_pos < start_pos))
                FAILURE;

             end_idx = current_pos.in_node->get_max_index();
             goo_buff = (char*)((gnode*)current_pos.in_node)->get_0t_contents();
             str_loc = &goo_buff[end_idx-offset];
             strncat (match_buffer, str_loc, (int) offset);

             found_off = tml_unix_match_re(str_loc, min_match_size, pattern);

             if (found_off != -1)
             {
                current_pos.char_index = (end_idx-offset) + found_off;
                found_str.set_start(current_pos);
                found_str.set_end(current_pos);
                SUCCESS;
             }
          }
          else
             current_pos = current_pos - (end_idx - start_idx);

          if (current_pos.in_node)
          {
             end_idx = current_pos.in_node->get_max_index() - partial_search;
             start_idx = (current_pos.in_node == start_pos.in_node) ? start_pos.char_index : 0;
          }
      }

      FAILURE;

    }
}
                                                                  
long                               // <>0 if exp is OK and pattern fits
     tml_unix_compile_exp(             // compile using std regexp
       char *exp,                 // string holding ASCII pattern
       long   len,                 // length of pattern
       void *pattern,             // buffer for output pattern
       long   pattern_max,         // maximum output pattern size
       long   case_sens=1          // case sensitive?  (default: YUP)
       )
{
    if(tml_unix_compile_re(exp, len, pattern, pattern_max, case_sens))
      SUCCESS;
    else
      FAILURE;
}


// Default char nav funcs.  These do nothing since the default search
// and compile functions go direct to the model or to the character
// string.  Later search environments may make use of char nav routines
// to provide increased functionality (e.g. search for ITALIC /foo/)

char* tml_next_char(model_pos& ptr)
// return curchar and push pointer to next
{
    char *ret_val;
    ret_val = ((char *)((gnode *)ptr.in_node)->get_contents())+ptr.char_index;
    ptr++;
    return ret_val;
}

char* tml_prior_char(model_pos& ptr)
// return curchar and pull pointer to prior
{
    char *ret_val;
    ret_val = ((char *)((gnode *)ptr.in_node)->get_contents())+ptr.char_index;
    ptr--;
    return ret_val;
}
                                                                      
char* tml_this_char(model_pos& ptr)
// return curchar and don't mess with pointer
{
    char *ret_val;
    ret_val = ((char *)((gnode *)ptr.in_node)->get_contents())+ptr.char_index;
    return ret_val;
}

