/**************************************************************************************
    tmlsearch_aegis.cxx
    Aegis-specific TML String Search Routines
    20-Nov-87 LCS

    (c) Copyright Apollo Computer Inc., 1987 (unpublished)
     --------------------------------------------------------------------
    |THE FOLLOWING PROGRAMS ARE THE SOLE PROPERTY OF APOLLO COMPUTER INC.|
    |      AND CONTAIN ITS PROPRIETARY AND CONFIDENTIAL INFORMATION.     |
     --------------------------------------------------------------------
**************************************************************************************/

#include <tml/tml_cxx2.0_linkage_decls.hxx>
#include <tml/tml_objects.hxx>
#include <tml/tml_os_depends.hxx>
#include <tml/tml_functions.hxx>

#define SUCCESS return 0
#define FAILURE return 1

long                               // YES if pattern was found
     tml_aegis_search_exp(              // search using std regexp
       model_range   search_area, // place in model to search in
       model_range&  found_str,   // area where string match was found
       void         *pattern,     // pattern from tml_compile
       long           forward = 1  // whether to search in reverse
       )
{
    register long min_match_size,
                 buffer_size,
                 this_string_size,
                 i,j,
                 found_off,
                 tmp_found_off;
    char        *match_buffer;
    model_pos    current_pos,
                 hold_pos,
                 start_pos,
                 end_pos;

    min_match_size = tml_aegis_min_match(pattern);
    if (!min_match_size)
         FAILURE;
    buffer_size = min_match_size * 2;

    match_buffer = tml_os_alloc(buffer_size, 1);
    if (!match_buffer)
        FAILURE;

    current_pos = start_pos = search_area.get_start();
    end_pos = search_area.get_end();

    found_off = -1;

    if(forward == 1){
      // forward search
      while (current_pos.in_node != end_pos.in_node) {
          this_string_size = current_pos.in_node->get_max_index() - current_pos.char_index;
          if (this_string_size > min_match_size){
              found_off = tml_aegis_match_re(
                            (char*)(current_pos.in_node->get_contents()),
                            this_string_size,
                            pattern);
              if (found_off != -1){
                  current_pos.char_index += (found_off - 1);
                  found_str.set_start(current_pos);
                  found_str.set_end(current_pos);
                  SUCCESS;
                  }
              else
                  current_pos.char_index += this_string_size - 1;
              }
          else{
              hold_pos = current_pos;
              for (i = 0;
                   hold_pos.in_node && (i < buffer_size);
                   i++, hold_pos = hold_pos.get_next())
                  *(match_buffer + i) = (*(char *)(hold_pos.in_node->get_contents()));
              if (!hold_pos.in_node)
                  FAILURE;
              found_off = tml_aegis_match_re(match_buffer, i, pattern);
              if (found_off != -1){
                  for(i = 0; i < found_off; i++)
                      current_pos = current_pos.get_next();
                  found_str.set_start(current_pos);
                  found_str.set_end(current_pos);
                  SUCCESS;
                  }
              else {
                  for (i = 0; i < min_match_size; i++)
                      current_pos = current_pos.get_next();
                  }
              }
          current_pos = current_pos.get_next();
          }
  
      if (end_pos.char_index < min_match_size)
          FAILURE;
       
      found_off = tml_aegis_match_re(
                    (char*)(end_pos.in_node->get_contents()),
                    end_pos.char_index,
                    pattern);
      if (found_off == -1)
          FAILURE;

      end_pos.char_index = found_off;
      found_str.set_start(end_pos);
      found_str.set_end(end_pos);
      SUCCESS;
    }
    else {
      // reverse search
      current_pos = end_pos;
      while (current_pos.in_node != start_pos.in_node) {
          this_string_size = current_pos.in_node->get_max_index() - current_pos.char_index;
//printf("this_string_size=%d\n",this_string_size);
          if (this_string_size > min_match_size){
//printf("a - searching string \"");
//for(ii=0;ii<this_string_size;ii++)printf("%c",*((char*)(current_pos.in_node->get_contents())+ii));
//printf("\"\n");
              found_off = tml_aegis_match_re(
                            (char*)(current_pos.in_node->get_contents()),
                            this_string_size,
                            pattern);
              if (found_off != -1){
              i = found_off;
              j = start_pos.in_node->get_max_index()-found_off+1;
              while( (j>0) &&
                   ( (tmp_found_off = tml_aegis_match_re(
                      ((char*)(start_pos.in_node->get_contents())+i),
                       j,
                       pattern)) != -1)
                     ) {
                    i += tmp_found_off;
                    found_off = i;
                    i++;
                    j = start_pos.in_node->get_max_index()-found_off+1;
                  }
                  current_pos.char_index += (found_off - 1);
                  found_str.set_start(current_pos);
                  found_str.set_end(current_pos);
                  SUCCESS;
                  }
              else
                  current_pos.char_index -= this_string_size;
              }
          else{
              hold_pos = current_pos;
              for (i = 0;
                   hold_pos.in_node && (i < buffer_size);
                   i++, hold_pos = hold_pos.get_prior()) {
                     for(j = i; j>0; j--) match_buffer[j] = match_buffer[j-1];
                     match_buffer[0] = (*(char *)(hold_pos.in_node->get_contents()));
                  }
//if(!hold_pos.in_node) printf("Fell off beginning of span\n");
              if (!hold_pos.in_node)
                  FAILURE;
//printf("b - searching string \"");
//for(ii=0;ii<this_string_size;ii++)printf("%c",match_buffer[i]);
//printf("\"\n");
              found_off = tml_aegis_match_re(match_buffer, i, pattern);
              if (found_off != -1){
                  for(i = 0; i < found_off; i++)
                      current_pos = current_pos.get_next();
                  found_str.set_start(current_pos);
                  found_str.set_end(current_pos);
                  SUCCESS;
                  }
              else {
                  for (i = 0; i < min_match_size; i++)
                      current_pos = current_pos.get_prior();
                  }
              }
          current_pos = current_pos.get_prior();
          }

//printf("start_node_max_index=%d\n",start_pos.in_node->get_max_index());
//if (start_pos.in_node->get_max_index() < min_match_size) printf("Fell off beginning of span 2\n");
      if (start_pos.in_node->get_max_index() < min_match_size)
          FAILURE;
       
//printf("c - searching string \"");
//for(ii=start_pos.char_index;ii<start_pos.in_node->get_max_index();ii++)
//printf("%c",*((char*)(start_pos.in_node->get_contents())+ii));
//printf("\"\n");
      found_off = tml_aegis_match_re(
                    ((char*)((start_pos.in_node->get_contents()))+start_pos.char_index),
                    start_pos.in_node->get_max_index()-start_pos.char_index+1,
                    pattern);
      if (found_off == -1)
          FAILURE;

//printf("found pattern @ offset %d\n",found_off);
      i = found_off;
      j = start_pos.in_node->get_max_index()-found_off+1;
      while( (j>0) &&
           ( (tmp_found_off = tml_aegis_match_re(
              ((char*)(start_pos.in_node->get_contents())+i),
               j,
               pattern)) != -1)
             ) {
            i += tmp_found_off;
            found_off = i;
            i++;
            j = start_pos.in_node->get_max_index()-found_off+1;
//printf("found at %d; checking @ %d; strlen = %d  \"",found_off,i,j);
//for(ii=0; ii<8; ii++)printf("%c",*((char*)(start_pos.in_node->get_contents()+found_off+ii)));
//printf("\"\n");
           }
      start_pos.char_index = found_off;
      found_str.set_start(start_pos);
      found_str.set_end(start_pos);
      SUCCESS;
    }
}
                                                                  
long                               // <>0 if exp is OK and pattern fits
     tml_aegis_compile_exp(             // compile using std regexp
       char *exp,                 // string holding ASCII pattern
       long   len,                 // length of pattern
       void *pattern,             // buffer for output pattern
       long   pattern_max,         // maximum output pattern size
       long   case_sens=1          // case sensitive?  (default: YUP)
       )
{
    if(tml_aegis_compile_re(exp, len, pattern, pattern_max, case_sens))
      SUCCESS;
    else
      FAILURE;
}
